package org.ebmwebsourcing.b2bgw.edifact.lexers_parsers.edifact_standard.code_element;
   
import java_cup.runtime.*;
      
%%////////////////////////////////////////////////////////////////////////

%class LexerCL
%public

%line
%column

%16bit

%cup
   
%{   
    private Symbol symbol(int type) {
        return new Symbol(type, yyline, yycolumn);
    }

    private Symbol  symbol(int type, Object value) {
        return new Symbol(type, yyline, yycolumn, value);
    }  
    
    private boolean begfile = false;    
    private boolean codeline = false;    
    private boolean begtag = false;
    private boolean desc = false;
    private boolean wasDesc = false;    
    
%}

newline = \r|\n|\r\n

multiline = (".\n\n") | (".\r\n\r\n") | (".\n") | (".\r\n")
   
anything = [A-Za-z_][A-Za-z_0-9]* | 
					[A-Za-Z_0-9_]+ | . | " "{4,15}
					
deletion =	"X    "
					
note = "Note:" ~"\n\n"

sentence = ([A-Z] [a-z-]+ ","? (" " ","? "("? [a-z-]+ ")"? ","? )+) | 
					([A-Z] [a-z/]+ ","? (" " "?", "("?  [a-z-]+ ")"? ","? )+) | 
					([A-Z] [a-z]+ ","? (" " ","? "("? [A-Z]? [a-z/-]+ ")"? ","? )+) |
					([A-Z]? "("? [a-z]+ ")"? ) |
					([A-Z]{2,15}) |
					([A-Z] " = " [A-Z] [a-z]+) |
					([A-Z] ".")+ |
					([A-Z]{2,15}) "/"? ([A-Z]{2,15}) |
					([0-9]{2,15}) ":" ([0-9]{2,15})

description = ("Desc: " ~".\n\n") | ("Desc: " ~".\r\n\r\n")

size = "Repr: " ~\n

codeval = ([0-9]{1,3} "  ") | ([A-Z]{1,6} " ") | ([0-9]{2} [A-Z] "  ")

codedef = [0-9]{4}


/////////////////////////////////////////////////////////////////
%%
  
   
<YYINITIAL> {

	{note}
		{}
		
	{deletion}
		{}		
		
	{multiline}
		{
			if (wasDesc){
				desc = false;
			}
		}

	{newline}
		{}

	"----------------------------------------------------------------------" 		
		{ 
			begfile = true;
			begtag = true;
			return symbol(SymCL.BEGTAG);
		}
		
	{codedef}		   
		{ 
			if (begtag){
				return symbol(SymCL.CODE, yytext());
			}		
		}

	{codeval}		   
		{ 
			if (begfile && !desc && !codeline) {
				codeline = true;
				return symbol(SymCL.CODEVAL, yytext());
			}
		} 		
	
	{size}		       
		{
			begtag = false;
			desc = false;
			codeline = false;
			return symbol(SymCL.SIZE, yytext());
		}

    {description}         
    	{
			return symbol(SymCL.DESC, yytext());
		} 			
                         
    {sentence}         
    	{
    		if (begtag){			
				return symbol(SymCL.DEFINITION, yytext());
			}
			else if (codeline){			
				codeline = false;
				desc = true;
				wasDesc = false;
				return symbol(SymCL.ENTITLE, yytext());
			}
			else if (desc){
				wasDesc = true;
				return symbol(SymCL.DESCVAL, yytext());			
			}
		}	
		
    {anything}            	{ 	}
    
}

//[^]                    { throw new Error("Illegal character <"+yytext()+">"); }
