package org.ebmwebsourcing.b2bgw.edifact.lexers_parsers.edifact_standard.message;
import java_cup.runtime.*;
import java.io.InputStreamReader;
      
%%////////////////////////////////////////////////////////////////////////

%class LexerMessage

%public
%line
%column
%16bit

%cup
   
%{   
    private Symbol symbol(int type) {
        return new Symbol(type, yyline, yycolumn);
    }

    private Symbol  symbol(int type, Object value) {
        return new Symbol(type, yyline, yycolumn, value);
    }  
    
    private boolean entitle = false;
    
    private boolean begmes = false;
    
   private boolean begstruct = false;
   
   private boolean seggroup = false;

    
%}

newline = \r|\n|\r\n
   
anything = [A-Za-z_][A-Za-z_0-9]* | 
					[A-Za-Z_0-9_]+ | . | " "{4,15}
					
note = ("Note:" ~".\n") | ("Note:" ~".\r\n")

sentence = ([A-Z] [a-z-]+ "é"? (" " "("? [a-z]+ "é"? ")"? )+) | 
					([A-Z] [a-z/]+ "é"? (" " "("?  [a-z]+ ")"? )+) | 
					([A-Z] [a-z]+ "é"? (" " "("? [a-z/]+ ")"? )+) 	| 
					"("? [A-Za-z/]{2,20} ")"? |
					([A-Z] " = " [A-Z] [a-z]+) |
					([A-Z] ".")+ | "é"

function = ("Functional definition" ~"Field of application") | ("Functional definition" ~"Field of application")

type = ("Message Type" ~"\n") | ("Message Type" ~"\r\n")

version = ("Version" ~"\n") | ("Version" ~"\r\n")

release = ("Release" ~"\n") | ("Release" ~"\r\n")

principles = ("Principles" ~"2.     REFERENCES") | ("Principles" ~"2.     REFERENCES")

begstructure = "4.3.1  Segment table"

index = [0-9]{4} "   "

state = "M " | "C "

codeseg = [A-Z]{3}

repeat = ("  " [0-9]{1,9} "  ") 

repeatl = ("  " [0-9]{1,9} "-")

group = "----- Segment group " ~"------"

level = ("-"+ "+"+) | ("-"+ "+" "|"+) | ("|"+)

begmes = ("----------------------------------------------------------------------\n\n") | ("----------------------------------------------------------------------\r\n\r\n")

/////////////////////////////////////////////////////////////////
%%
  
   
<YYINITIAL> {

	{note}
		{
		}
		
	{repeat}
		{
			if (begstruct && entitle) {
				entitle = false;
				seggroup = true;
				return symbol(SymMessage.REPEAT, yytext());
			}
		}		
		
	{repeatl}
		{
			if (begstruct && entitle) {
				entitle = false;
				seggroup = true;
				return symbol(SymMessage.REPEAT, yytext());
			}
		}				
		
	{group}
		{
			if (begstruct) {
				seggroup = true;
				return symbol(SymMessage.GROUP, yytext());
			}
		}		
		
	{level}
		{
			if (seggroup) {
				seggroup = false;
				return symbol(SymMessage.LEVEL, yytext());
			}
		}					

	{newline}
		{}

	{type}
		{ 
			return symbol(SymMessage.TYPE, yytext());
		}
		
	{version}
		{ 
			return symbol(SymMessage.VERSION, yytext());
		}		
		
	{release}
		{ 
			return symbol(SymMessage.RELEASE, yytext());
		}		
		

	{begmes}	
		{ 
			begmes = true;
			return symbol(SymMessage.BEGMES);
		}

	{begstructure}
		{ 
			if (begmes){
				begstruct = true;
				return symbol(SymMessage.BEGSTRUCT);
			}
		}

	{codeseg}		       
		{ 
			if (begstruct) {
				return symbol(SymMessage.CODESEG, yytext());
			}
		}		
	
	{index}		   
		{ 
			if (begstruct){
				entitle = true;
				return symbol(SymMessage.INDEX, yytext());
			}		
		}
		
    {function}         
    	{	
			if (begmes){
				return symbol(SymMessage.FUNCTION, yytext());
			}
		} 		
	 		
		
    {principles}         
    	{	
			if (begmes){
				return symbol(SymMessage.PRINCIPLES, yytext());
			}
		} 					
                         
    {sentence}         
    	{
			if (begstruct && entitle){			
				return symbol(SymMessage.ENTITLE, yytext());
			}
		} 
		
    {state}
    	{
    		if (begstruct && entitle){    		
				return symbol(SymMessage.STATE, yytext());
			}
		}
		
    {anything}         
    	{}
    
}

//[^]                    { throw new Error("Illegal character <"+yytext()+">"); }
