/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.axis2;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.wsdl.Definition;
import javax.wsdl.WSDLException;
import javax.xml.namespace.QName;

import org.apache.axis2.AxisFault;
import org.apache.axis2.Constants;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.description.AxisService;
import org.apache.axis2.description.Parameter;
import org.apache.axis2.engine.AxisConfiguration;
import org.apache.axis2.wsdl.WSDLConstants;
import org.objectweb.petals.component.common.PEtALSComponentSDKException;
import org.objectweb.petals.component.common.su.ServiceUnitListener;
import org.objectweb.petals.component.common.util.WSDLHelper;
import org.objectweb.petals.tools.jbicommon.descriptor.Consumes;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;
import org.w3c.dom.Document;

/**
 * A service unit listener used to register new service into Axis Engine during
 * SU deployment.
 * 
 * @author Christophe HAMERLING - eBMWebSourcing
 * 
 */
public class Axis2BCSUListener implements ServiceUnitListener {

    private ConfigurationContext axisConfigurationContext;

    private ComponentContext componentContext;

    private Logger logger;

    /**
     * Creates a new instance of {@link Axis2BCSUListener}
     * 
     * @param axisConfigurationContext
     */
    public Axis2BCSUListener(ConfigurationContext axisConfigurationContext,
        ComponentContext componentContext, Logger logger) {
        this.axisConfigurationContext = axisConfigurationContext;
        this.componentContext = componentContext;
        this.logger = logger;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.component.common.su.ServiceUnitListener#onSUDeployed(java.lang.String,
     *      java.lang.String,
     *      org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor)
     */
    public void onSUDeployed(String serviceUnitName, String suRootPath,
        JBIDescriptor descriptor) throws PEtALSComponentSDKException {
        logger.log(Level.INFO, "Registering a new AxisService for SU "
            + serviceUnitName);

        List<Consumes> consumesList = descriptor.getServices().getConsumes();
        for (Consumes consumes : consumesList) {
            registerAxisService(consumes);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.component.common.su.ServiceUnitListener#onSUStarted(java.lang.String)
     */
    public void onSUStarted(String serviceUnitName)
        throws PEtALSComponentSDKException {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.component.common.su.ServiceUnitListener#onSUStopped(java.lang.String)
     */
    public void onSUStopped(String serviceUnitName)
        throws PEtALSComponentSDKException {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.component.common.su.ServiceUnitListener#onSUUndeployed(java.lang.String,
     *      java.lang.String,
     *      org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor)
     */
    public void onSUUndeployed(String serviceUnitName, String suRootPath,
        JBIDescriptor descriptor) throws PEtALSComponentSDKException {

        // delete registered axis services
        List<Consumes> consumesList = descriptor.getServices().getConsumes();
        for (Consumes consumes : consumesList) {
            unregisterAxisService(consumes);
        }
    }

    /**
     * 
     * @param serviceQName
     * @throws PEtALSComponentSDKException
     */
    private void registerAxisService(Consumes consumes)
        throws PEtALSComponentSDKException {

        QName serviceQName = consumes.getServiceName();
        logger.log(Level.INFO, "Adding Axis service to list : "
            + serviceQName.getLocalPart());

        AxisConfiguration axisConfig = axisConfigurationContext
            .getAxisConfiguration();

        try {
            AxisService axisService = null;
            String jbiEndpoint = consumes.getEndpointName();

            // try to find if the axisService has already been registered
            axisService = axisConfig.getService(serviceQName.getLocalPart());

            if (axisService == null) {
                // create a new axisService, according to the JBI
                // servicedescription
                axisService = new AxisService(serviceQName.getLocalPart());
                axisService.setTargetNamespace(serviceQName.getNamespaceURI());
                axisService.setEndpoint(jbiEndpoint);

                axisService.addParameter(new Parameter(Constants.SERVICE_CLASS,
                    "PetalsReceiver"));

                // add parameter to handle ?wsdl requests
                Parameter useOriginalwsdl = new Parameter("useOriginalwsdl",
                    "true");
                axisService.addParameter(useOriginalwsdl);

                ServiceEndpoint endpoint = componentContext.getEndpoint(
                    serviceQName, jbiEndpoint);

                Definition def = null;
                Document document = componentContext
                    .getEndpointDescriptor(endpoint);
                if (document != null) {
                    try {
                        def = WSDLHelper.createDefinitionFromDocument(document);
                    } catch (WSDLException e) {
                        logger.log(Level.WARNING,
                            "WSDL Definition has not been initialized");
                    }
                }

                Parameter defParam = new Parameter(
                    WSDLConstants.WSDL_4_J_DEFINITION, def);
                axisService.addParameter(defParam);

                // add service
                axisConfig.addService(axisService);
            } else {
                logger.log(Level.FINE,
                    "Service is already registered in Axis2BC");
            }
        } catch (AxisFault e) {
            throw new PEtALSComponentSDKException(
                "Can not register Service into Axis context", e);
        } catch (JBIException e) {
            throw new PEtALSComponentSDKException(e);
        }
    }

    /**
     * Unregister the service from Axis
     * 
     * @param serviceQName
     * @throws PEtALSComponentSDKException
     */
    private void unregisterAxisService(Consumes consumes)
        throws PEtALSComponentSDKException {

        QName serviceQName = consumes.getServiceName();
        logger.log(Level.INFO, "Removing Axis service from list : "
            + serviceQName.getLocalPart());

        try {
            // register an axis service to axis engine
            AxisService axisService = axisConfigurationContext
                .getAxisConfiguration().getService(serviceQName.getLocalPart());

            if (axisService != null) {
                axisConfigurationContext.getAxisConfiguration().removeService(
                    axisService.getName());
            }

        } catch (AxisFault e) {
            throw new PEtALSComponentSDKException(
                "Can not remove service from Axis context", e);
        }
    }

}
