/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: Axis2BCListener.java 154 19 avr. 2006 wjoseph $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.axis2.listener.soap;

import java.io.ByteArrayInputStream;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.InOptionalOut;
import javax.jbi.messaging.MessageExchangeFactory;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamReader;
import javax.xml.transform.Source;
import javax.xml.transform.TransformerException;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.apache.axiom.soap.SOAPBody;
import org.apache.axiom.soap.SOAPEnvelope;
import org.apache.axiom.soap.SOAPFactory;
import org.apache.axis2.AxisFault;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.description.AxisService;
import org.apache.axis2.engine.AxisEngine;
import org.apache.axis2.receivers.AbstractMessageReceiver;
import org.apache.axis2.util.Utils;
import org.objectweb.petals.component.common.su.SimpleServiceUnitManager;
import org.objectweb.petals.tools.jbicommon.util.StringHelper;
import org.objectweb.petals.tools.jbicommon.util.XMLUtil;
import org.w3c.dom.Document;

/**
 * Dispatcher for JBI container
 * 
 * @version $Rev: 250 $ $Date: 2006-04-21 14:20:57 +0200 (ven, 21 avr 2006) $
 * @since Petals 1.0
 * @author alouis,wjoseph - EBM Websourcing
 * 
 */
public class PetalsReceiver extends AbstractMessageReceiver {

    protected static final String ERROR_SERVICE_NOT_FOUND = "Requested service and/or operation not found/available";

    protected static final String ERROR_GET_WSDL = "Error while retrieving the service WSDL";

    protected static final String ERROR_WRONG_MESSAGE_STATUS = "JBI message has wrong status";

    protected static final String ERROR_SERVICE_INVOKATION = "An error occured in service invokation";

    protected static final String ERROR_CREATE_RESPONSE_FROM_DOM = "Error creating a string from a DOM service description";

    protected static final String ERROR_CREATE_STRING_FROM_NM = "Error creating a string from normalized message";

    protected Axis2ReceiverUtil axis2ReceiverUtil;

    protected Logger log;

    protected ComponentContext context;

    protected DeliveryChannel channel;

    protected MessageExchangeFactory exchangeFactory;

    protected SimpleServiceUnitManager bindingSUM;

    /**
     * Creates a new instance of PetalsReceiver
     * 
     * @param context
     * @param channel
     * @param bindingSUM
     * @param log
     */
    public PetalsReceiver(ComponentContext context, DeliveryChannel channel,
        SimpleServiceUnitManager bindingSUM, Logger log) {
        super();
        this.axis2ReceiverUtil = new Axis2ReceiverUtil();
        this.channel = channel;
        this.context = context;
        this.log = log;
        this.bindingSUM = bindingSUM;
        this.exchangeFactory = channel.createExchangeFactory();
    }

    /**
     * Process the SOAP request. calls processWSDL() or process(), and send the
     * response
     */
    public final void receive(final MessageContext msgContext) throws AxisFault {
        log.log(Level.FINE, "PetalsReceiver - receive");

        MessageContext outMsgContext = Utils
            .createOutMessageContext(msgContext);
        outMsgContext.getOperationContext().addMessageContext(outMsgContext);

        ServiceEndpoint jbiEndpoint = findEndpoint(msgContext);

        process(jbiEndpoint, msgContext, outMsgContext);
        
        // send the response if required
        if (outMsgContext.getEnvelope() != null) {
            AxisEngine engine = new AxisEngine(msgContext.getOperationContext()
                .getServiceContext().getConfigurationContext());

            engine.send(outMsgContext);
        }
    }

    /**
     * Return a new SOAP envelop from the SOAPFactory of the given
     * MessageContext
     * 
     * @param msgContext -
     *            not null
     * @return a new SOAPEnvelope
     * @throws AxisFault
     */
    protected SOAPEnvelope createResponseEnvelope(MessageContext msgContext)
        throws AxisFault {
        SOAPFactory factory = getSOAPFactory(msgContext);
        return factory.createSOAPEnvelope();
    }

    /**
     * Return the JBI ServiceEndpoint according to : <br>{
     * axisService.getTargetNamespace() } axisService.getName() <br>
     * axisService.getEndpoint()
     * 
     * @param msgContext
     * @return the corresponding endpoint, null if no one corresponding
     */
    protected ServiceEndpoint findEndpoint(MessageContext msgContext) {
        ServiceEndpoint ep = null;

        AxisService axisService = msgContext.getAxisService();
        QName serviceName = new QName(axisService.getTargetNamespace(),
            axisService.getName());
        String endpointName = axisService.getEndpoint();

        ep = context.getEndpoint(serviceName, endpointName);

        return ep;
    }

    /**
     * Create a String result from a DOM document
     * 
     * @param document
     *            the DOM Document
     * @return a String representation of the DOM Document
     * @throws TransformerException
     */
    public String createStringFromDOMDocument(Document document)
        throws TransformerException {
        return XMLUtil.createStringFromDOMNode(document);
    }

    /**
     * Sends the SOAPEnvelope of the resquest message directly to a JBI service
     * and operation on the JBI container. The response message from the JBI
     * service invocation is transmitted as an outgoing message.
     * 
     * @param inMessage
     *            MessageContext for incoming message
     * @param outMessage
     *            MessageContect fot outgoing message
     * @throws AxisFault
     *             error while processing request, send the JBI Exchange,
     *             processing the JBI response, or if the JBI response is a
     *             Fault
     */
    public void process(ServiceEndpoint endpoint, MessageContext inContext,
        MessageContext outContext) throws AxisFault {

        log.log(Level.FINE, "PetalsReceiver - process SOAP request");

        // Retrieve message operation and service
        QName operationName = findSOAPOperation(inContext);

        // As the SOAP/HTTP request wait for a response,
        // use the InOptionalOut MEP
        InOptionalOut exchange = null;

        try {
            exchange = createInOptionalOut(operationName, endpoint, inContext);
            // TODO
            processAttachements(inContext, exchange.getInMessage());
        } catch (Exception e) {
            String errorMsg = "Error while transforming SOAP request to JBI MessageExchange";
            log.log(Level.SEVERE, errorMsg, e);
            throw new AxisFault(errorMsg, e);
        }

        // send the message exchange through JBI
        // As the SOAP/HTTP request wait for a response,
        // use the sendSynchrone MEP
        try {
            channel.sendSync(exchange);
        } catch (MessagingException e) {
            String errorMsg = "Error while sending message through JBI NMR.";
            log.log(Level.SEVERE, errorMsg, e);
            throw new AxisFault(errorMsg, e);
        }

        // process the response received from JBI NMR. The outcontext is updated
        processJbiNmrResponse(exchange, outContext, getSOAPFactory(inContext));
    }

    /**
     * Return the operation according to the WSAAction SOAP attribute
     * 
     * @param inContext
     *            SOAPrequest context
     * @return operation as QName, null if no WSAAction is specified
     */
    protected QName findSOAPOperation(MessageContext inContext) {
        String operation = inContext.getWSAAction();
        QName operationName = null;
        if (!StringHelper.isNullOrEmpty(operation)) {
            operationName = QName.valueOf(operation);
        }
        return operationName;
    }

    /**
     * Process the JBI response. <br>
     * According to the response status :
     * <li>
     * <ul>
     * do nothin for DONE,
     * </ul>
     * <ul>
     * throws an AxisFault for ERROR,
     * </ul>
     * <ul>
     * set outContext body to a FAULT,
     * </ul>
     * <ul>
     * set outContext body to the response.
     * </ul>
     * </li>
     * 
     * @param exchange
     *            JBI response
     * @param outMessage
     *            set to FAULT or OUT JBI response
     * @param factory
     *            soap factory used to create body response
     * @throws AxisFault
     *             with ERROR JBI status, or problem while creating response
     */
    protected void processJbiNmrResponse(InOptionalOut exchange,
        MessageContext outMessage, SOAPFactory factory) throws AxisFault {
        if (exchange.getStatus().equals(ExchangeStatus.DONE)) {
            // exchange DONE, nothing to do, no soap response

        } else if (exchange.getStatus().equals(ExchangeStatus.ERROR)) {
            // exchange ERROR: throws an axis fault
            throw new AxisFault(exchange.getError());

        } else if (exchange.getStatus().equals(ExchangeStatus.ACTIVE)) {
            if (exchange.getFault() != null) {
                // exchange FAULT : return a fault message
                try {
                    outMessage.setEnvelope(createResponseFromNormalizedMessage(
                        factory, exchange.getFault()));
                } catch (AxisFault e) {
                    String errorMsg = "Error while creating SOAP Fault response.";
                    log.log(Level.SEVERE, errorMsg, e);
                    throw new AxisFault(errorMsg, e);
                }

            } else {
                // exchange OUT : return the response
                // TODO attachment
                try {
                    outMessage.setEnvelope(createResponseFromNormalizedMessage(
                        factory, exchange.getOutMessage()));
                } catch (AxisFault e) {
                    String errorMsg = "Error while creating SOAP Fault response.";
                    log.log(Level.SEVERE, errorMsg, e);
                    throw new AxisFault(errorMsg, e);
                }
            }
            // close the messageExchange with the JBI NMR
            try {
                exchange.setStatus(ExchangeStatus.DONE);
                channel.send(exchange);
            } catch (MessagingException e) {
                String errorMsg = "Error while closing JBI MessageExchange.";
                log.log(Level.SEVERE, errorMsg, e);
                throw new AxisFault(errorMsg, e);
            }

        } else {
            throw new AxisFault(ERROR_WRONG_MESSAGE_STATUS);
        }

    }

    /**
     * TODO
     *
     * @param inMessage
     * @param nm
     */
    protected void processAttachements(MessageContext inMessage,
        NormalizedMessage nm) {
        /*
         * Attachments attachments = (Attachments) inMessage
         * .getProperty(MTOMConstants.ATTACHMENTS); if (attachments != null) {
         * IncomingAttachmentStreams incoming = attachments
         * .getIncomingAttachmentStreams(); int i = 0; while
         * (incoming.isReadyToGetNextStream()) { IncomingAttachmentInputStream
         * attachmentInpustStream = incoming .getNextStream(); if
         * (attachmentInpustStream != null) { MIMEHelper help = new
         * MIMEHelper(attachmentInpustStream,"");
         * 
         * DataSource ds = new ByteArrayDataSource( attachmentInpustStream,
         * null); DataHandler dh = new DataHandler(ds);
         * attachmentInpustStream.getContentId(); new
         * Attachments(attachmentInpustStream,null); nm.addAttachment("00" +
         * i++, dh); } else { break; } }
         * 
         * DataHandler dh = attachments.getDataHandler("001");
         * nm.addAttachment("001", dh); }
         */
    }

    /**
     * Creates a SOAP response from a NormalizedMessage
     * 
     * @param factory
     *            soap factory
     * @param nm2
     *            NormalizedMessage containing the response
     * @return a SOAPEnveloppe created from the nm NomalizedMessage content
     * @throws AxisFault
     */
    protected SOAPEnvelope createResponseFromNormalizedMessage(
        SOAPFactory factory, NormalizedMessage nm) throws AxisFault {

        SOAPEnvelope responseEnv = createSoapEnvelope(factory);

        /*
         * Create and fill the Soap body with the content of the Normalized
         * message
         */
        String response = null;
        if (nm != null) {
            try {
                response = axis2ReceiverUtil.createStringContent(nm);
            } catch (Exception e) {
                log.log(Level.SEVERE, ERROR_CREATE_STRING_FROM_NM);
                throw new AxisFault(ERROR_CREATE_STRING_FROM_NM, e);
            }
        } else {
            response = "<Response>Done</Response>";
        }
        createSoapBody(factory, responseEnv, response);
        return responseEnv;
    }

    /**
     * Create a new Axis 2 SOAP body with the right name spaces and service name
     * 
     * @param factory
     *            the soap factory
     * @param envelope
     *            a soap envelope
     * @param response
     *            the response
     * @return a new soap body
     */
    protected OMElement createSoapBody(SOAPFactory factory,
        SOAPEnvelope envelope, String response) throws AxisFault {
        OMElement bodyContent = null;
        ByteArrayInputStream inputStream = new ByteArrayInputStream(response
            .getBytes());
        try {
            XMLStreamReader parser = XMLInputFactory.newInstance()
                .createXMLStreamReader(inputStream);
            StAXOMBuilder builder = new StAXOMBuilder(factory, parser);
            bodyContent = (OMElement) builder.getDocumentElement();
        } catch (Exception e) {
            throw new AxisFault(
                "Error parsing the response from JBI service to a SOAPBody", e);
        }
        SOAPBody respBody = factory.createSOAPBody(envelope);
        respBody.addChild(bodyContent);
        return respBody;
    }

    /**
     * Create a new Axis 2 SOAP envelope with the right name spaces
     * 
     * @param factory
     *            the soap factory; there are ex
     * @param namespaces
     *            name spaces for the soap envelope
     * @return a new SOAP envelope
     */
    protected SOAPEnvelope createSoapEnvelope(SOAPFactory factory) {
        return factory.createSOAPEnvelope();
    }

    /**
     * Create an InOptionalOut exchange initialized with the given parameters.
     * 
     * @param operationName
     * @param endpoint
     *            jbi destination endpoint
     * @param inContext
     *            soap inContext
     * @return a new InOptionalOut exchange
     * @throws Exception
     */
    protected InOptionalOut createInOptionalOut(QName operationName,
        ServiceEndpoint endpoint, MessageContext inContext) throws Exception {
        InOptionalOut messageExchange = null;

        Source source = axis2ReceiverUtil.createSourceContent(inContext);

        messageExchange = exchangeFactory.createInOptionalOutExchange();
        NormalizedMessage nm = messageExchange.createMessage();
        nm.setContent(source);
        messageExchange.setInMessage(nm);

        messageExchange.setEndpoint(endpoint);
        messageExchange.setOperation(operationName);

        return messageExchange;
    }
}
