/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.engine.clock.sdk;

import java.net.URI;

import javax.jbi.JBIException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.objectweb.petals.component.common.HandlingException;
import org.objectweb.petals.component.common.MEPConstants;
import org.objectweb.petals.component.common.se.AbstractServiceEngine;
import org.objectweb.petals.component.common.util.MessageExchangeWrapper;
import org.objectweb.petals.tools.jbicommon.descriptor.Extensions;

public class ClockEngine extends AbstractServiceEngine {

	private Clock clock;

	private ServiceEndpoint endpoint;

	private static final String SERVICE = "ClockService";

	private static final String ENDPOINT = "ClockEndpoint";

	private static final QName SERVICE_NAME = new QName(
			"http://petals.objectweb.org/", SERVICE);

	@Override
	protected boolean onExchange(MessageExchangeWrapper exchange, Extensions extensions)
			throws HandlingException {

		// Get Operation name
		String opName = exchange.getOperationName();

		// Get Message Exchange Pattern
		URI pattern = exchange.getExchangePattern();

		if ("time".equalsIgnoreCase(opName)
				&& pattern.equals(MEPConstants.IN_OUT_PATTERN.value())) {

			String en = clock.time();

			exchange.setOutMessageContent(en);

		} else {
			throw new HandlingException("Operation not allowed ("
					+ exchange.getOperation().getLocalPart()
					+ ") with the following pattern ("
					+ exchange.getPattern().toString() + ")");
		}

		return true;
	}

	@Override
	public void start() throws JBIException {
		super.start();
		endpoint = getContext().activateEndpoint(SERVICE_NAME, ENDPOINT);
		clock = new ClockImpl(getLogger());
	}

	@Override
	public void stop() throws JBIException {
		super.stop();
		getContext().deactivateEndpoint(endpoint);
		clock = null;
	}

}
