/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.component.common.bc;

import org.objectweb.petals.component.common.AbstractComponent;
import org.objectweb.petals.component.common.HandlingException;
import org.objectweb.petals.component.common.su.BindingComponentServiceUnitManager;
import org.objectweb.petals.component.common.util.MessageExchangeWrapper;
import org.objectweb.petals.component.common.util.PetalsExtensionsUtil;
import org.objectweb.petals.tools.jbicommon.descriptor.Extensions;

/**
 * An abstract class that can be used to easily create a new JBI binding
 * components. It extends AbstractComponent to add binding component specific
 * features
 * 
 * @author ofabre,alouis
 * 
 */
public abstract class AbstractBindingComponent extends AbstractComponent {

    /**
     * An initialization class that handles the JBIListener and the
     * ExternalListenerManager
     */
    private BindingComponentInitializer initializer;

    @Override
    protected BindingComponentServiceUnitManager createServiceUnitManager() {

        return new BindingComponentServiceUnitManager(getContext(),
            getLogger(), this);
    }

    @Override
    protected boolean onExchange(final MessageExchangeWrapper exchange,
        final Extensions extensions) throws HandlingException {
        String externalAddress = PetalsExtensionsUtil
            .extractValueFromKeyValueExtension(extensions,
                PetalsExtensionsUtil.ADDRESS);

        JBIListener listener = initializer.getJbiListener();
        if (listener == null) {
            throw new HandlingException("No jbi Listener provided. "
                + "So provides node not allowed");
        }

        return listener.onJBIMessage(externalAddress, exchange, extensions);

    }

    @Override
    protected void init() {
        super.init();
        initializer = new BindingComponentInitializer();
        init(initializer);

    }

    /**
     * This method must be overidden to initialize JBIListener and
     * ExternalListenerManager handled by the
     * {@link BindingComponentInitializer}. This method is called at the end of
     * the component classical initialization. When this method is called, the
     * component's delivery channel, context, logger and service unit manager
     * are available.<br/> If this binding component must handle jbi messages,
     * you must provide your custom {@link JBIListener}.<br/> If this binding
     * component must handle messages from external service to internal jbi
     * components, you must provide your custom {@link ExternalListenerManager}.
     * 
     * @param initializer
     *            the {@link BindingComponentInitializer} to fill
     */
    protected abstract void init(BindingComponentInitializer initializer);

    /**
     * An initialization class that handles the JBIListener and the
     * ExternalListenerManager
     * 
     * @author ofabre
     * 
     */
    public class BindingComponentInitializer {

        private JBIListener jbiListener;

        private ExternalListenerManager externalListenerManager;

        public ExternalListenerManager getExternalListenerManager() {
            return externalListenerManager;
        }

        public void setExternalListenerManager(
            ExternalListenerManager externalListenerManager) {
            this.externalListenerManager = externalListenerManager;
        }

        public JBIListener getJbiListener() {
            return jbiListener;
        }

        public void setJbiListener(JBIListener jbiListener) {
            this.jbiListener = jbiListener;
        }

    }

    public BindingComponentInitializer getInitializer() {
        return initializer;
    }

}
