/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.component.common.su;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;
import javax.jbi.management.DeploymentException;

import org.objectweb.petals.component.common.PEtALSComponentSDKException;
import org.objectweb.petals.component.common.bc.AbstractBindingComponent;
import org.objectweb.petals.component.common.bc.ExternalListenerManager;
import org.objectweb.petals.tools.jbicommon.descriptor.Consumes;

/**
 * Like SimpleServiceUnitManager but start and stop listeners for external
 * addresses.
 * 
 * @author ofabre
 * 
 */
public class BindingComponentServiceUnitManager extends
    SimpleServiceUnitManager {

    private AbstractBindingComponent bindingComponent;

    public BindingComponentServiceUnitManager(ComponentContext context,
        Logger logger, AbstractBindingComponent bindingComponent) {
        super(bindingComponent, context, logger);
        this.bindingComponent = bindingComponent;
    }

    private List<String> getServiceUnitAddresses(final String serviceUnitName) {
        return new ArrayList<String>(getServiceUnitDatas().get(serviceUnitName)
            .getAddressToConsumes().keySet());
    }

    @Override
    public void start(final String serviceUnitName) throws DeploymentException {
        super.start(serviceUnitName);
        for (String address : getServiceUnitAddresses(serviceUnitName)) {
            try {
                Consumes consumes = getConsumesFromAddress(address);
                if (consumes == null) {
                    throw new DeploymentException(
                        "The Consumes node of the JBI.XML descriptor file is null.");
                }
                getExternalListenerManager().startListening(address,
                    consumes.getExtensions());
            } catch (PEtALSComponentSDKException e) {
                String msg = "Failed to start listening to address : "
                    + address;
                getLogger().severe(msg);
                throw new DeploymentException(msg, e);
            }
        }
    }

    private ExternalListenerManager getExternalListenerManager()
        throws DeploymentException {
        ExternalListenerManager externalListenerManager = bindingComponent
            .getInitializer().getExternalListenerManager();
        if (externalListenerManager == null) {
            throw new DeploymentException(
                "No external Listener manager provided. "
                    + "So consumes node not allowed");
        }
        return externalListenerManager;
    }

    @Override
    public void stop(final String serviceUnitName) throws DeploymentException {
        for (String address : getServiceUnitAddresses(serviceUnitName)) {
            try {
                getExternalListenerManager().stopListening(address);
            } catch (PEtALSComponentSDKException e) {
                String msg = "Failed to start listening to address : "
                    + address;
                getLogger().severe(msg);
                throw new DeploymentException(msg, e);
            }
        }
        super.stop(serviceUnitName);
    }

}
