/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.component.common.su;

import org.objectweb.petals.component.common.PEtALSComponentSDKException;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;

/**
 * This interface defines callback methods for ServiceUnit management. <br>
 * It takes place in the PEtALS abstract S.E. or Binding component framework. <br>
 * A <code>ServiceUnitHandler</code> object is set on the component using the
 * <code>setServiceUnitHandler()</code> method. <br>
 * This callback is not mandatory, but it gives extra management during the
 * Service Units life cycle.
 * 
 * @author alouis - EBM Websourcing
 * 
 */
public interface ServiceUnitListener {

    /**
     * Callback after a ServiceUnit has been deployed
     * 
     * @param serviceUnitName
     * @param suRootPath
     * @param descriptor
     */
    void onSUDeployed(final String serviceUnitName, final String suRootPath,
        final JBIDescriptor descriptor) throws PEtALSComponentSDKException;

    /**
     * Callback after a ServiceUnit has been started
     * 
     * @param serviceUnitName
     */
    void onSUStarted(final String serviceUnitName) throws PEtALSComponentSDKException;

    /**
     * Callback after a ServiceUnit has been stopped
     * 
     * @param serviceUnitName
     */
    void onSUStopped(final String serviceUnitName) throws PEtALSComponentSDKException;

    /**
     * Callback after a ServiceUnit has been undeployed
     * 
     * @param serviceUnitName
     * @param suRootPath
     */
    void onSUUndeployed(final String serviceUnitName, final String suRootPath,
        final JBIDescriptor descriptor) throws PEtALSComponentSDKException;

}
