package org.objectweb.petals.component.common.util;

import java.net.URI;
import java.util.Map;
import java.util.Set;

import javax.activation.DataHandler;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;

import org.objectweb.petals.component.common.HandlingException;

public interface MessageExchangeWrapper extends MessageExchange {

	/**
	 * Put a Source object in the out message of the message exchange. If the
	 * out normalized message is null, it is created here.
	 * 
	 * @param outContent
	 *            the content to put in the out message
	 * @throws HandlingException
	 *             if error occured during out message creation
	 */
	void setOutMessageContent(final String outContent) throws HandlingException;

	/**
	 * Get the content of the "in" message as a String
	 * 
	 * @return the String represention of the "in" message content
	 * @throws HandlingException
	 *             if error occured during String creation from Source or if
	 *             "in" message is null
	 */
	String getInMessageContent() throws HandlingException;

	/**
	 * Get all the input attachments in a {@link Set}
	 * 
	 * @param exchange
	 * @return
	 */
	Set<DataHandler> getInMessageAttachments() throws HandlingException;

	/**
	 * Get the message exchange Pattern
	 * 
	 * @return the URI pattern. Can't be null
	 * @throws HandlingException
	 *             if no message exchange pattern is set
	 */
	URI getExchangePattern() throws HandlingException;

	/**
	 * Get the operation name (local part of the operation QName)
	 * 
	 * @return the operation name
	 * @throws HandlingException
	 *             if no Operation QName is set
	 */
	String getOperationName() throws HandlingException;

	/**
	 * Get the endpoint name specified in the message exchange
	 * 
	 * @return the endpoint name
	 * @throws HandlingException
	 *             if no Endpoint is set
	 */
	String getEndpointName() throws HandlingException;

	/**
	 * Extract incoming NormalizedMessage from given MessageExchange
	 * 
	 * @return the incoming NormalizedMessage
	 * 
	 */
	NormalizedMessage getInMessage();

	/**
	 * Set the input message
	 * 
	 * @param msg
	 * @throws MessagingException
	 */
	void setInMessage(final NormalizedMessage msg) throws MessagingException;

	/**
	 * Extract outgoing NormalizedMessage from given MessageExchange
	 * 
	 * @return the outgoing NormalizedMessage
	 * 
	 */
	NormalizedMessage getOutMessage();

	/**
	 * Set the output message
	 * 
	 * @param msg
	 * @throws MessagingException
	 */
	void setOutMessage(final NormalizedMessage msg) throws MessagingException;
	
	/**
	 * Put a Map of attachements in the out message of the message exchange. If the
	 * out normalized message is null, it is created here.
	 * 
	 * @param attachements
	 *            the attachements to put in the out message
	 * @throws HandlingException
	 *             if error occured during out message creation
	 */
	void setOutMessageAttachements(Map<String, DataHandler> attachements)
	throws HandlingException;

}
