/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.component.common.util;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Set;

import javax.activation.DataHandler;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;

/**
 * Utilities for normalized message manipulation.
 * 
 * @author Christophe HAMERLING - eBMWebSourcing
 * 
 */
public class NormalizedMessageUtil {

    /**
     * Date format used to get unique file name
     */
    private static final String DATE_FORMAT = "ddMMyyHHmmssSSS";

    /**
     * Copy the attachments from source to destination
     * 
     * @param from
     * @param to
     * @throws MessagingException
     */
    public static void copyAttachments(final NormalizedMessage from,
        NormalizedMessage to) throws MessagingException {
        Set attachmentIds = from.getAttachmentNames();
        for (Object id : attachmentIds) {
            String attachName = (String) id;
            DataHandler dh = from.getAttachment(attachName);
            to.addAttachment(attachName, dh);
        }
    }

    /**
     * Copy the properties from source to destination.
     * 
     * @param from
     * @param to
     */
    public static void copyProperties(final NormalizedMessage from,
        NormalizedMessage to) {
        Set propertiesNames = from.getPropertyNames();
        for (Object propertyName : propertiesNames) {
            String name = (String) propertyName;
            Object value = from.getProperty(name);
            to.setProperty(name, value);
        }
    }

    /**
     * Copy the content from source to destination.
     * 
     * @param from
     * @param to
     */
    public static void copyContent(final NormalizedMessage from,
        NormalizedMessage to) throws MessagingException {
        to.setContent(from.getContent());
    }

    /**
     * Write the given attchment to a file. The file will have the same name as
     * the attachment one.
     * 
     * @param dh
     *            the attachement
     * @param dest
     *            the destination file
     */
    public static void writeAttachmentToFile(final DataHandler handler,
        final File destDir) {
        FileOutputStream fos = null;

        try {
            if (!destDir.exists())
                destDir.mkdirs();

            // output file
            File outFile = new File(destDir.getAbsolutePath(), handler
                .getName());

            // if file already exists, add a timestamp suffix
            String ext = "";
            if (outFile.exists()) {
                SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT);
                ext = sdf.format(new Date(System.currentTimeMillis()));
            }

            fos = new FileOutputStream(
                new File(outFile.getAbsoluteFile() + ext));

            handler.writeTo(fos);
            fos.flush();

        } catch (FileNotFoundException e) {
        } catch (IOException e) {
        } finally {
            try {
                fos.close();
            } catch (IOException e) {
            }
        }
    }

    /**
     * Write all the attachments of the normalized message to files. The file
     * name will be the same as the attachment name.
     * 
     * @param nm
     * @param outputDir
     */
    public static void writeAttachmentsToFiles(final NormalizedMessage nm,
        final File destDir) {
        
        Set attachments = nm.getAttachmentNames();
        for (Object attachmentId : attachments) {
            writeAttachmentToFile(nm.getAttachment((String) attachmentId),
                destDir);
        }
    }

}
