/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: PetalsDispatcherTest.java 154 18 mai 2006 wjoseph $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.component.common.util;

import java.io.File;

import javax.xml.namespace.QName;

import junit.framework.TestCase;

import org.w3c.dom.Document;

import org.objectweb.petals.component.common.util.WSDLHelper;

/**
 * 
 * @author wjoseph - EBM Websourcing
 * 
 */
public class WSDLHelperTest extends TestCase {

    File wsdlFile = null ;
    @Override
    protected void setUp() throws Exception {
        wsdlFile = new File("src" + File.separator + "test-data" + File.separator +"test.wsdl");
        if(!wsdlFile.exists()) {
            wsdlFile = new File("common" + File.separator + "src" + File.separator + "test-data" + File.separator +"test.wsdl");
            if(!wsdlFile.exists()) {
                wsdlFile = new File("components" + File.separator + "common" + File.separator + "src" + File.separator + "test-data" + File.separator +"test.wsdl");
            }
        }
    }

   public void testChangeServiceAddressInWSDL() {
       /*
        * Test Case 1
        */
       /*
        * Create Test parameters
        */
       String prefix = "http://myhost.com:myport/myserver/myservices/";
       QName serviceName = new QName("http://act.org/","EchoService");
       Document serviceDesc = WSDLHelper.createDocumentFromWSDL(wsdlFile);
       String oldAddress = null;
       String newAddress = null;
       try {
       oldAddress = WSDLHelper.getServiceAddressFromWSDLDocument(serviceDesc, serviceName);
       } catch (Exception e) {
           e.printStackTrace();
       }
    
       /*
        * Run Test Case 1
        */
       try {
           serviceDesc = WSDLHelper.changeServiceAddressInWSDL(serviceName, serviceDesc, prefix);
       } catch (Exception e) {
           e.printStackTrace();
           fail("Error during invokation");
       }
       try {
           newAddress = WSDLHelper.getServiceAddressFromWSDLDocument(serviceDesc, serviceName);
           } catch (Exception e) {
               e.printStackTrace();
           } 
       assertNotSame(oldAddress, newAddress);
       
   }
   
   public void testGetPortNameFromWSDLDocument() {
       /*
        * Test Case 1
        */
       /*
        * Create Test parameters 
        */
       QName serviceName = new QName("http://act.org/","EchoService");
       Document serviceDesc = WSDLHelper.createDocumentFromWSDL(wsdlFile);
       String portName = null;
       String port = "Echo";
       /*
        * Run Test Case 1
        */
       try {
           portName = WSDLHelper.getPortNameFromWSDLDocument(serviceDesc, serviceName);
       } catch (Exception e) {
           e.printStackTrace();
           fail("Error during invokation");
       }
       assertEquals(port, portName);
   }
   
   public void testGetServiceNameFromWSDLFile() {
       /*
        * Test Case 1
        */
       /*
        * Create Test parameters
        */
       QName serviceName = new QName("http://act.org/","EchoService");
       QName service = null;
       /*
        * Run Test Case 1
        */
       try {
           service = WSDLHelper.getServiceNameFromWSDLFile(wsdlFile)[0];
       } catch (Exception e) {
           e.printStackTrace();
           fail("Error during invokation");
       }
       assertEquals(serviceName,service);
   }
   
   public void testHasOperationNamed() {
       /*
        * Test Case 1
        */
       /*
        * Create Test Parameters
        */
       QName serviceName = new QName("http://act.org/","EchoService");
       Document serviceDesc = WSDLHelper.createDocumentFromWSDL(wsdlFile);
       String op = "echoString";
       boolean result = false;
       /*
        * Run Test Case 1
        */
       try {
           result = WSDLHelper.hasOperationNamed(serviceDesc, op, serviceName);
       } catch (Exception e) {
           e.printStackTrace();
           fail("Error during invokation");
       }
       assertTrue(result);
       
       /*
        * Test Case 2
        */
       /*
        * Create Test parameters
        */
       op = "fooOp";
       /*
        * Run test Case 2
        */
       try {
           result = WSDLHelper.hasOperationNamed(serviceDesc, op, serviceName);
       } catch (Exception e) {
           e.printStackTrace();
           fail("Error during invokation");
       }
       assertFalse(result);
   }
   
   public void testIsInOutOperation() {
       /*
        * Test Case 1
        */
       /*
        * Create Test Parameters
        */
       QName serviceName = new QName("http://act.org/","EchoService");
       Document serviceDesc = WSDLHelper.createDocumentFromWSDL(wsdlFile);
       String op = "echoString";
       boolean result = false;
       /*
        * Run Test Case 1
        */
       try {
           result = WSDLHelper.isInOutOperation(serviceDesc, op, serviceName);
       } catch (Exception e) {
           e.printStackTrace();
           fail("Error during invokation");
       }
       assertTrue(result);
       
       /*
        * Test Case 2
        */
       /*
        * Create Test parameters
        */
       op = "fooOp";
       /*
        * Run test Case 2
        */
       try {
           result = WSDLHelper.isInOutOperation(serviceDesc, op, serviceName);
       } catch (Exception e) {
           e.printStackTrace();
           fail("Error during invokation");
       }
       assertFalse(result);
   }
   
   public void testGetTargetNSFromWSDLDocument() {
       /*
        * Test Case 1
        */
       /*
        * Create Test parameters
        */
       Document serviceDesc = WSDLHelper.createDocumentFromWSDL(wsdlFile);
       String namespace = "http://act.org/";
       String nameSpace = null;
       /*
        * Run test Case 1
        */
       try {
            nameSpace = WSDLHelper.getTargetNSFromWSDLDocument(serviceDesc);
       } catch (Exception e) {
           e.printStackTrace();
           fail("Error during invokation");
       }
       assertEquals(namespace, nameSpace);
   }
}
