/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: CsvProcessorTest.java 1202 2006-11-06 20:20:41Z dutoo $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.engine.csv;

import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.verify;

import java.net.URI;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.logging.Logger;

import javax.activation.DataHandler;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;

import junit.framework.TestCase;

import org.easymock.IArgumentMatcher;
import org.easymock.classextension.EasyMock;

import org.objectweb.petals.component.common.MEPConstants;
import org.objectweb.petals.component.common.util.SourceHelper;
import org.objectweb.petals.component.common.util.StringHelper;
import org.objectweb.petals.engine.csv.Mock.MockDeliveryChannel;
import org.objectweb.petals.engine.csv.Mock.MockMessageExchange;
import org.objectweb.petals.engine.csv.Mock.MockServiceEndpoint;
import org.objectweb.petals.engine.csv.exception.CsvException;

/**
 * Tests the Csv processor
 * 
 * @version $Rev: 1069 $Date: ${date}
 * @since Petals 1.0
 * @author Marc Dutoo - Open Wide
 * 
 */
public class CsvProcessorTest extends TestCase {

    private String baseDir;

    private Logger logger;

    @Override
    public void setUp() throws Exception {

        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
        this.logger = EasyMock.createMock(Logger.class);
        super.setUp();
    }

    public void testCsvContentSingleValue() {
        testCsvContent("singlevalue",
            "<?xml version=\"1.0\" encoding=\"UTF-8\"?><data xmlns=\"http://petals.objectweb.org/components/csv/csvxml\"><row><cell>singlevalue</cell></row></data>");
    }

    public void testCsvContentSingleRow() {
        testCsvContent("singlerow1,singlerow2",
            "<?xml version=\"1.0\" encoding=\"UTF-8\"?><data xmlns=\"http://petals.objectweb.org/components/csv/csvxml\"><row><cell>singlerow1</cell><cell>singlerow2</cell></row></data>");
    }

    public void testCsvContentTwoRows() {
        testCsvContent(
            "singlerow1,singlerow2\nsecondrow1,secondrow2",
            "<?xml version=\"1.0\" encoding=\"UTF-8\"?><data xmlns=\"http://petals.objectweb.org/components/csv/csvxml\"><row><cell>singlerow1</cell><cell>singlerow2</cell></row>"
                + "<row><cell>secondrow1</cell><cell>secondrow2</cell></row></data>");
    }

    public void testCsvContent(String testCsvInput, String testXmlOutput) {
        MockDeliveryChannel mockDeliveryChannel = new MockDeliveryChannel();

        Properties csvProperties = new Properties();
        csvProperties.setProperty(CsvProcessor.CSV_INPUT_MODE_PROP,
            CsvProcessor.CSV_INPUT_MODE_CONTENT);
        Map<String, Properties> mapEndpointCsv = new HashMap<String, Properties>();
        mapEndpointCsv.put("myServiceName", csvProperties);
        CsvProcessor csvProcessor = new CsvProcessor(mockDeliveryChannel,
            logger, mapEndpointCsv);

        // now test :
        try {
            mockDeliveryChannel.createExchangeFactory()
                .createInOutExchange();
        } catch (MessagingException e) {
            e.printStackTrace();
        }
        final MockServiceEndpoint mockServiceEndpoint = createMock(MockServiceEndpoint.class);
        // expect(mockServiceEndpoint.getEndpointName()).andReturn("myEndpointName");
        expect(mockServiceEndpoint.getServiceName()).andReturn(
            new QName("myServiceName"));

        final QName myOperation = new QName(CsvProcessor.CSV_TO_XML_OPERATION);

        final NormalizedMessage mockInNormalizedMessage = createMock(NormalizedMessage.class);
        String myContent = testCsvInput;
        Source contentSource = SourceHelper.createContentSource(myContent);
        // NB. we can't merely use createSource(), since its transformation to
        // string
        // would fail without it being actual XML
        expect(mockInNormalizedMessage.getContent()).andReturn(contentSource);
        // NB. content transformation mode, so no attachment
        // Set<String> attachmentNames = new HashSet<String>();
        // String attachmentName1 = "myFirstAttachment";
        // attachmentNames.add(attachmentName1);
        // expect(mockInNormalizedMessage.getAttachmentNames()).andReturn(attachmentNames);
        // expect(mockInNormalizedMessage.getAttachment(attachmentName1)).andReturn(
        // new DataHandler(new ByteArrayDataSource(testXmlOutput)));

        final NormalizedMessage mockOutNormalizedMessage = createMock(NormalizedMessage.class);

        /*
         * MessageExchange mockMessageExchange =
         * createMock(MessageExchange.class);
         * expect(mockMessageExchange.getStatus()).andReturn(ExchangeStatus.ACTIVE);
         * expect(mockMessageExchange.getRole()).andReturn(MessageExchange.Role.PROVIDER);
         * expect(mockMessageExchange.getPattern()).andReturn(MEPConstants.IN_OUT_PATTERN).atLeastOnce(); //
         * asked for more than once
         * expect(mockMessageExchange.getFault()).andReturn(null); try {
         * expect(mockMessageExchange.createMessage()).andReturn(mockOutNormalizedMessage); }
         * catch (Exception e) { e.printStackTrace(); }
         * expect(mockMessageExchange.getEndpoint()).andReturn(mockServiceEndpoint);
         * expect(mockMessageExchange.getOperation()).andReturn(myOperation);
         * expect(mockMessageExchange.getMessage("in")).andReturn(mockInNormalizedMessage);
         */
        MessageExchange mockMessageExchange = new MockMessageExchange() {
            // NB. Fault already returns null;
            @Override
            public NormalizedMessage createMessage() throws MessagingException {
                return mockOutNormalizedMessage;
            }

            @Override
            public ServiceEndpoint getEndpoint() {
                return mockServiceEndpoint;
            }

            @Override
            public NormalizedMessage getMessage(String name) {
                return mockInNormalizedMessage;
            }

            @Override
            public QName getOperation() {
                return myOperation;
            }

            @Override
            public URI getPattern() {
                return MEPConstants.IN_OUT_PATTERN;
            }

            @Override
            public Role getRole() {
                return MessageExchange.Role.PROVIDER;
            }

            @Override
            public ExchangeStatus getStatus() {
                return ExchangeStatus.ACTIVE;
            }
        };

        try {
            mockOutNormalizedMessage.setContent(eqSource(SourceHelper
                .createSource(testXmlOutput)));// TODO check
            // expectLastCall().andAnswer(new IAnswer<void>() {
            mockMessageExchange.setMessage(mockOutNormalizedMessage, "out");
        } catch (MessagingException e) {
            e.printStackTrace();
        }

        replay(mockServiceEndpoint);
        replay(mockInNormalizedMessage);
        replay(mockOutNormalizedMessage);
        // replay(mockMessageExchange);

        try {
            // Function to test
            csvProcessor.process(mockMessageExchange);
        } catch (CsvException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }

        verify(mockServiceEndpoint);
        verify(mockInNormalizedMessage);
        verify(mockOutNormalizedMessage);
        // verify(mockMessageExchange);
    }

    public void testCsvAttachmentSingleValue() {
        testCsvAttachment("singlevalue",
            "<?xml version=\"1.0\" encoding=\"UTF-8\"?><data xmlns=\"http://petals.objectweb.org/components/csv/csvxml\"><row><cell>singlevalue</cell></row></data>");
    }

    public void testCsvAttachmentSingleRow() {
        testCsvAttachment("singlerow1,singlerow2",
            "<?xml version=\"1.0\" encoding=\"UTF-8\"?><data xmlns=\"http://petals.objectweb.org/components/csv/csvxml\"><row><cell>singlerow1</cell><cell>singlerow2</cell></row></data>");
    }

    public void testCsvAttachmentTwoRows() {
        testCsvAttachment(
            "singlerow1,singlerow2\nsecondrow1,secondrow2",
            "<?xml version=\"1.0\" encoding=\"UTF-8\"?><data xmlns=\"http://petals.objectweb.org/components/csv/csvxml\"><row><cell>singlerow1</cell><cell>singlerow2</cell></row>"
                + "<row><cell>secondrow1</cell><cell>secondrow2</cell></row></data>");
    }

    public void testCsvAttachment(String testCsvInput, String testXmlOutput) {
        MockDeliveryChannel mockDeliveryChannel = new MockDeliveryChannel();

        Properties csvProperties = new Properties();
        // NB. default CSV input mode is guessing
        Map<String, Properties> mapEndpointCsv = new HashMap<String, Properties>();
        mapEndpointCsv.put("myServiceName", csvProperties);
        CsvProcessor csvProcessor = new CsvProcessor(mockDeliveryChannel,
            logger, mapEndpointCsv);

        // now test :
        try {
            mockDeliveryChannel.createExchangeFactory()
                .createInOutExchange();
        } catch (MessagingException e) {
            e.printStackTrace();
        }

        MockServiceEndpoint mockServiceEndpoint = createMock(MockServiceEndpoint.class);
        // expect(mockServiceEndpoint.getEndpointName()).andReturn("myEndpointName");
        expect(mockServiceEndpoint.getServiceName()).andReturn(
            new QName("myServiceName"));

        QName myOperation = new QName(CsvProcessor.CSV_TO_XML_OPERATION);

        NormalizedMessage mockInNormalizedMessage = createMock(NormalizedMessage.class);
        // NB. attachment transformation mode, so no content
        // String myContent = "youpi";
        // Source contentSource = SourceHelper.createContentSource(myContent);
        // expect(mockInNormalizedMessage.getContent()).andReturn(contentSource);
        Set<String> attachmentNames = new HashSet<String>();
        String attachmentName1 = "myFirstAttachment";
        attachmentNames.add(attachmentName1);
        expect(mockInNormalizedMessage.getAttachmentNames()).andReturn(
            attachmentNames).times(2); // twice because of "GUESS" mode
        expect(mockInNormalizedMessage.getAttachment(attachmentName1))
            .andReturn(new DataHandler(new ByteArrayDataSource(testCsvInput)));// TODO

        NormalizedMessage mockOutNormalizedMessage = createMock(NormalizedMessage.class);

        MessageExchange mockMessageExchange = createMock(MessageExchange.class);
        expect(mockMessageExchange.getStatus())
            .andReturn(ExchangeStatus.ACTIVE);
        expect(mockMessageExchange.getRole()).andReturn(
            MessageExchange.Role.PROVIDER);
        expect(mockMessageExchange.getPattern()).andReturn(
            MEPConstants.IN_OUT_PATTERN).atLeastOnce();
        expect(mockMessageExchange.getFault()).andReturn(null);
        try {
            expect(mockMessageExchange.createMessage()).andReturn(
                mockOutNormalizedMessage);
        } catch (Exception e) {
            e.printStackTrace();
        }
        expect(mockMessageExchange.getEndpoint())
            .andReturn(mockServiceEndpoint);
        expect(mockMessageExchange.getOperation()).andReturn(myOperation);
        expect(mockMessageExchange.getMessage("in")).andReturn(
            mockInNormalizedMessage);

        try {
            mockOutNormalizedMessage.setContent(eqSource(SourceHelper
                .createSource(testXmlOutput)));
            // expectLastCall().andAnswer(new IAnswer<void>() {
            mockMessageExchange.setMessage(mockOutNormalizedMessage, "out");
        } catch (MessagingException e) {
            e.printStackTrace();
        }

        replay(mockServiceEndpoint);
        replay(mockInNormalizedMessage);
        replay(mockOutNormalizedMessage);
        replay(mockMessageExchange);

        try {
            // Function to test
            csvProcessor.process(mockMessageExchange);
        } catch (CsvException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }

        verify(mockServiceEndpoint);
        verify(mockInNormalizedMessage);
        verify(mockOutNormalizedMessage);
        verify(mockMessageExchange);
    }

    /**
     * Custom argument matcher for Source
     * 
     * @author mdutoo
     * 
     */
    public static class SourceArgumentMatcher implements IArgumentMatcher {
        private Source expectedSource;

        public SourceArgumentMatcher(Source expectedSource) {
            this.expectedSource = expectedSource;
        }

        public boolean matches(Object found) {
            if (!(found instanceof Source)) {
                return false;
            }
            Source foundSource = (Source) found;
            try {
                String foundSourceContent = SourceHelper
                    .createString(foundSource);
                String expectedSourceContent = SourceHelper
                    .createString(expectedSource);
                return StringHelper.equal(foundSourceContent,
                    expectedSourceContent);
            } catch (Exception e) {
                return false;
            }
        }

        public void appendTo(StringBuffer sbuf) {
            String expectedSourceContent;
            try {
                expectedSourceContent = SourceHelper
                    .createString(expectedSource);
            } catch (Exception e) {
                expectedSourceContent = "<error creating representation>";
            }
            sbuf.append("eqSource(with content\"");
            sbuf.append(expectedSourceContent);
            sbuf.append("\")");
        }

    }

    /**
     * Helps using the source argument matcher
     * 
     * @param in
     * @return
     */
    public static Source eqSource(Source in) {
        EasyMock.reportMatcher(new SourceArgumentMatcher(in));
        return null;
    }


    public void testXmlToCsvContentSingleValue() {
        testXmlToCsvContent("<myTable><row><aCell>singlevalue</aCell></row></myTable>",
            "<content>singlevalue\n</content>");
    }

    public void testXmlToCsvContentSingleRow() {
        testXmlToCsvContent("<data><row><cell>singlerow1</cell><cell>singlerow2</cell></row></data>",
            "<content>singlerow1,singlerow2\n</content>");
    }

    public void testXmlToCsvContentTwoRows() {
        testXmlToCsvContent(
            "<data><row><cell>singlerow1</cell><cell>singlerow2</cell></row>"
                + "<anotherRow><cell>secondrow1</cell><cell>secondrow2</cell></anotherRow></data>",
            "<content>singlerow1,singlerow2\r\nsecondrow1,secondrow2\n</content>");
    }

    public void testXmlToCsvContent(String testCsvInput, String testXmlOutput) {
        MockDeliveryChannel mockDeliveryChannel = new MockDeliveryChannel();

        Properties csvProperties = new Properties();
        csvProperties.setProperty(CsvProcessor.CSV_INPUT_MODE_PROP,
            CsvProcessor.CSV_INPUT_MODE_CONTENT);
        Map<String, Properties> mapEndpointCsv = new HashMap<String, Properties>();
        mapEndpointCsv.put("myServiceName", csvProperties);
        CsvProcessor csvProcessor = new CsvProcessor(mockDeliveryChannel,
            logger, mapEndpointCsv);

        // now test :
        try {
            mockDeliveryChannel.createExchangeFactory()
                .createInOutExchange();
        } catch (MessagingException e) {
            e.printStackTrace();
        }
        final MockServiceEndpoint mockServiceEndpoint = createMock(MockServiceEndpoint.class);
        // expect(mockServiceEndpoint.getEndpointName()).andReturn("myEndpointName");
        expect(mockServiceEndpoint.getServiceName()).andReturn(
            new QName("myServiceName"));

        final QName myOperation = new QName(CsvProcessor.XML_TO_CSV_OPERATION);

        final NormalizedMessage mockInNormalizedMessage = createMock(NormalizedMessage.class);
        String myContent = testCsvInput;
        Source contentSource = SourceHelper.createSource(myContent);
        // NB. we can't merely use createSource(), since its transformation to
        // string
        // would fail without it being actual XML
        expect(mockInNormalizedMessage.getContent()).andReturn(contentSource);
        // NB. content transformation mode, so no attachment
        // Set<String> attachmentNames = new HashSet<String>();
        // String attachmentName1 = "myFirstAttachment";
        // attachmentNames.add(attachmentName1);
        // expect(mockInNormalizedMessage.getAttachmentNames()).andReturn(attachmentNames);
        // expect(mockInNormalizedMessage.getAttachment(attachmentName1)).andReturn(
        // new DataHandler(new ByteArrayDataSource(testXmlOutput)));

        final NormalizedMessage mockOutNormalizedMessage = createMock(NormalizedMessage.class);

        /*
         * MessageExchange mockMessageExchange =
         * createMock(MessageExchange.class);
         * expect(mockMessageExchange.getStatus()).andReturn(ExchangeStatus.ACTIVE);
         * expect(mockMessageExchange.getRole()).andReturn(MessageExchange.Role.PROVIDER);
         * expect(mockMessageExchange.getPattern()).andReturn(MEPConstants.IN_OUT_PATTERN).atLeastOnce(); //
         * asked for more than once
         * expect(mockMessageExchange.getFault()).andReturn(null); try {
         * expect(mockMessageExchange.createMessage()).andReturn(mockOutNormalizedMessage); }
         * catch (Exception e) { e.printStackTrace(); }
         * expect(mockMessageExchange.getEndpoint()).andReturn(mockServiceEndpoint);
         * expect(mockMessageExchange.getOperation()).andReturn(myOperation);
         * expect(mockMessageExchange.getMessage("in")).andReturn(mockInNormalizedMessage);
         */
        MessageExchange mockMessageExchange = new MockMessageExchange() {
            // NB. Fault already returns null;
            @Override
            public NormalizedMessage createMessage() throws MessagingException {
                return mockOutNormalizedMessage;
            }

            @Override
            public ServiceEndpoint getEndpoint() {
                return mockServiceEndpoint;
            }

            @Override
            public NormalizedMessage getMessage(String name) {
                return mockInNormalizedMessage;
            }

            @Override
            public QName getOperation() {
                return myOperation;
            }

            @Override
            public URI getPattern() {
                return MEPConstants.IN_OUT_PATTERN;
            }

            @Override
            public Role getRole() {
                return MessageExchange.Role.PROVIDER;
            }

            @Override
            public ExchangeStatus getStatus() {
                return ExchangeStatus.ACTIVE;
            }
        };

        try {
            mockOutNormalizedMessage.setContent(eqSource(SourceHelper
                .createSource(testXmlOutput)));// TODO check
            // expectLastCall().andAnswer(new IAnswer<void>() {
            mockMessageExchange.setMessage(mockOutNormalizedMessage, "out");
        } catch (MessagingException e) {
            e.printStackTrace();
        }

        replay(mockServiceEndpoint);
        replay(mockInNormalizedMessage);
        replay(mockOutNormalizedMessage);
        // replay(mockMessageExchange);

        try {
            // Function to test
            csvProcessor.process(mockMessageExchange);
        } catch (CsvException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }

        verify(mockServiceEndpoint);
        verify(mockInNormalizedMessage);
        verify(mockOutNormalizedMessage);
        // verify(mockMessageExchange);
    }

}
