/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: CsvTest.java 1089 2006-09-14 19:57:55Z dutoo $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.engine.csv;

import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.verify;

import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.ObjectName;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.petals.component.common.serviceunitmanager.manager.PetalsServiceUnitManager;
import org.objectweb.petals.engine.csv.Csv;
import org.objectweb.petals.engine.csv.Mock.MockCsv;
import org.objectweb.petals.engine.csv.Mock.MockDeliveryChannel;
import org.objectweb.petals.engine.csv.Mock.MockServiceEndpoint;
import org.w3c.dom.Document;


/**
 * Tests the Csv component
 * @version	$Rev: 1069 $Date: ${date}
 * @since	Petals 1.0
 * @author Marc Dutoo - Open Wide
 *
 */
public class CsvTest extends TestCase {
    Logger logger;

    
    @Override
    public void setUp() throws Exception {
        logger = EasyMock.createMock(Logger.class);
    }

    
    public void testGetExtensionMBeanName() {
        // Run Test Case
        Csv csv = new Csv();
        ObjectName objectName = csv.getExtensionMBeanName();
        assertNull(objectName);
    }

    
    public void testGetServiceDescription() {
        // Create Mock Objects
        PetalsServiceUnitManager suManager = createMock(PetalsServiceUnitManager.class);
        MockCsv csv = new MockCsv();

        // Create test parameters
        Document document = null;
        ServiceEndpoint fooService = createMock(ServiceEndpoint.class);
        ServiceEndpoint barService = createMock(ServiceEndpoint.class);

        try {
            document = DocumentBuilderFactory.newInstance()
            .newDocumentBuilder().newDocument();
            expect(suManager.getServiceDescription(fooService)).andReturn(
                document).anyTimes();
            expect(suManager.getServiceDescription(barService)).andReturn(null)
            .anyTimes();
            expect(suManager.getServiceDescription(null)).andReturn(null)
            .anyTimes();
        } catch (ParserConfigurationException e2) {
            e2.printStackTrace();
        }
        replay(suManager);

        csv.setServiceUnitManager(suManager);
        // Run the test case 1
        // Asking service description for foo service
        Document doc = null;
        doc = csv.getServiceDescription(fooService);
        assertEquals(document, doc);

        // Run the test case 2
        // Asking service description for bar service
        doc = null;
        doc = csv.getServiceDescription(barService);
        assertNull("This service description should be null", doc);

        // Run the test case 3
        // Asking service description for null service
        doc = null;
        doc = csv.getServiceDescription(null);
        assertNull("This service description should be null", doc);

        verify(suManager);
    }

    
    public void testInit() {
        Csv csv = new Csv();
        ComponentContext componentContext = createMock(ComponentContext.class);

        // Create test parameters
        Logger logger = Logger.getLogger("", null);
        MockDeliveryChannel channel = new MockDeliveryChannel();

        // Init mocks
        try {
            expect(componentContext.getDeliveryChannel()).andReturn(channel);
            expect(componentContext.getLogger("", null)).andReturn(logger);
            expect(componentContext.getComponentName()).andReturn("");
        } catch (JBIException e) {
            e.printStackTrace();
        }
        replay(componentContext);

        // Run the Test
        try {
            csv.init(componentContext);
        } catch (JBIException e) {

        }
        verify(componentContext);
    }

    
    public void testIsExchangeWithConsumerOkay() {
        // Create Mock Objects
        MessageExchange msgExchange = createMock(MessageExchange.class);

        Csv csv = new Csv();
        csv.logger=logger;
        // Create test parameters
        ServiceEndpoint se = new MockServiceEndpoint();

        // Run the Test
        boolean result = csv.isExchangeWithConsumerOkay(se,
            msgExchange);
        assertEquals(result, true);
    }

    
    public void testIsExchangeWithProviderOkay() {
        // Create Mock Objects
        MessageExchange msgExchange = createMock(MessageExchange.class);

        Csv csv = new Csv();
        csv.logger=logger;
        // Create test parameters
        ServiceEndpoint se = new MockServiceEndpoint();

        // Run the Test
        boolean result = csv.isExchangeWithProviderOkay(se,
            msgExchange);
        assertEquals(result, false);
    }

}


