/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * Id : ${file_name} 154 ${date}  rbarraza 
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.filetransferbc.listeners;

import java.io.File;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.messaging.NormalizedMessage;

import org.objectweb.petals.binding.filetransferbc.FileTransferBCException;
import org.objectweb.petals.binding.filetransferbc.IOUtils;
import org.objectweb.petals.component.common.HandlingException;
import org.objectweb.petals.component.common.PEtALSComponentSDKException;
import org.objectweb.petals.component.common.bc.JBIListener;
import org.objectweb.petals.component.common.util.MessageExchangeWrapper;
import org.objectweb.petals.component.common.util.SourceHelper;
import org.objectweb.petals.tools.jbicommon.descriptor.Extensions;
import org.objectweb.petals.tools.jbicommon.util.StringHelper;

/**
 * The file transfer BC JBI listener. Writes a file to hard disk on incoming JBI
 * message.
 * 
 * @version $Rev: 250 $Date: {date}
 * @since Petals 1.0
 * @author alouis, Rocio BARRAZA, Christophe HAMERLING - eBMWebsourcing
 * 
 */
public class FileTransferBCJBIListener implements JBIListener {

    private final Logger logger;

    /**
     * Creates a new instance of {@link FileTransferBCJBIListener}
     * 
     * @param channel
     * @param logger
     * @param context
     * @param workDir
     */
    public FileTransferBCJBIListener(Logger logger) {
        this.logger = logger;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.component.common.bc.JBIListener#onJBIMessage(java.lang.String,
     *      javax.jbi.messaging.MessageExchange,
     *      org.objectweb.petals.tools.jbicommon.descriptor.Extensions)
     */
    public boolean onJBIMessage(String address, MessageExchangeWrapper exchange,
        Extensions extensions) throws HandlingException {
        boolean result = false;

        String content = null;
        NormalizedMessage nm = exchange.getInMessage();

        // get message content as string
        try {
            content = SourceHelper.createString(nm.getContent());
        } catch (PEtALSComponentSDKException e) {
            throw new HandlingException(e.getMessage());
        }

        // write content a file. The destDir is the endpoint
        // name
        File parentDir = new File(address);
        if (!parentDir.exists())
            parentDir.mkdirs();
        
        String destDir = exchange.getEndpoint().getEndpointName();
        File outputDir = new File(parentDir, destDir);
         
        logger.log(Level.INFO, "Writing data to directory : " + outputDir.getAbsolutePath());
        
        if (StringHelper.isNullOrEmpty(content)) {
            logger.log(Level.INFO, "content is null");

        } else {
            try {
                String fileName = null;
                if (exchange.getOperation() != null) {
                    fileName = exchange.getOperation().getLocalPart();
                } else {
                    fileName = "content";
                }

                IOUtils.writeStringToFile(content, outputDir, fileName);
            } catch (FileTransferBCException e) {
                throw new HandlingException(e);
            }
        }

        // write attachments to files
        Set attachIds = nm.getAttachmentNames();
        if (!attachIds.isEmpty()) {
            try {
                IOUtils.writeAttachmentsToFiles(nm, outputDir);
            } catch (FileTransferBCException e) {
                throw new HandlingException(e);
            }
        }

        return result;
    }
}
