/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.engine.helloworld;

import java.net.URI;
import java.util.logging.Level;

import javax.jbi.JBIException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.naming.NameAlreadyBoundException;
import javax.naming.NamingException;
import javax.xml.namespace.QName;

import org.objectweb.petals.component.common.HandlingException;
import org.objectweb.petals.component.common.MEPConstants;
import org.objectweb.petals.component.common.se.AbstractServiceEngine;
import org.objectweb.petals.component.common.util.MessageExchangeWrapper;
import org.objectweb.petals.component.common.util.WSDLHelper;
import org.objectweb.petals.tools.jbicommon.descriptor.Extensions;
import org.w3c.dom.Document;

public class HelloWorldEngine extends AbstractServiceEngine {

	private static final String PRINT_MESS = "printMessage";

	private static final String BIND_MESS = "bindMessage";

	private static final String SAY_HELLO = "sayHello";

	private Helloworld helloworld;

	private ServiceEndpoint endpoint;

	private static final String INTERFACE = "HelloworldInterface";

	private static final String SERVICE = "HelloworldService";

	private static final String ENDPOINT = "HelloworldEndpoint";

	private static final QName SERVICE_NAME = new QName(
			"http://petals.objectweb.org/", SERVICE);

	private static final QName INTERFACE_NAME = new QName(
			"http://petals.objectweb.org/", INTERFACE);

	private Document desc;

	@Override
	protected boolean onExchange(final MessageExchangeWrapper exchange,
			final Extensions extensions) throws HandlingException {

		String outContent = null;
		boolean outResponse = false;

		// Get Operation name
		String opName = exchange.getOperationName();

		// Get Message Exchange Pattern
		URI pattern = exchange.getExchangePattern();

		// Get in Message content
		String stringContent = exchange.getInMessageContent();

		if (PRINT_MESS.equalsIgnoreCase(opName)
				&& pattern.equals(MEPConstants.IN_ONLY_PATTERN.value())) {
			helloworld.printMessage(stringContent);
		} else if (BIND_MESS.equalsIgnoreCase(opName)
				&& pattern.equals(MEPConstants.IN_ONLY_PATTERN.value())) {
			helloworld.bindMessage(stringContent);
		} else if (SAY_HELLO.equalsIgnoreCase(opName)
				&& pattern.equals(MEPConstants.IN_OUT_PATTERN.value())) {
			outContent = helloworld.sayHello(stringContent);
			outResponse = true;
		} else {
			throw new HandlingException("Operation not allowed (" + opName
					+ ") with the following pattern (" + pattern + ")");
		}

		if (outResponse) {
			exchange.setOutMessageContent(outContent);
		}

		return outResponse;
	}

	@Override
	public void start() throws JBIException {
		super.start();
		// Register in the Helloworld entry Jndi context
		try {

			getContext().getNamingContext().createSubcontext(
					HelloworldImpl.HELLOWORLD_CONTEXT);

		} catch (NameAlreadyBoundException e) {
			getLogger().log(Level.INFO,
					"the HelloWorld jndi context is already bound.");
		} catch (NamingException e) {
			throw new JBIException(e);
		}

		desc = WSDLHelper.createLightWSDL20(INTERFACE_NAME, SERVICE_NAME,
				ENDPOINT);
		// Create a new Instance of the Helloworld implementation
		helloworld = new HelloworldImpl(getContext(), getLogger());

		// Activate a default endpoint
		endpoint = getContext().activateEndpoint(SERVICE_NAME, ENDPOINT);
	}

	@Override
	protected String getResourceBundleName() {
		return "hello";
	}

	@Override
	public void stop() throws JBIException {
		super.stop();
		getContext().deactivateEndpoint(endpoint);
		helloworld = null;
	}

	public Document getServiceDescription(final ServiceEndpoint endpoint) {
		return desc;
	}

}
