/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.engine.helloworld;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;
import javax.naming.NameNotFoundException;
import javax.naming.NamingException;

import org.objectweb.petals.tools.jbicommon.util.StringHelper;

public class HelloworldImpl implements Helloworld {

	public static final String HELLOWORLD_CONTEXT = "helloworldContext";

	public static final String HELLOWORLD_JNDI_ENTRY = "helloworldEntry";

	private final ComponentContext context;

	private Logger logger; // NOPMD

	public HelloworldImpl(ComponentContext context, Logger logger) {
		super();
		this.context = context;
		this.logger = logger;
	}

	public void bindMessage(final String message) {
		if (StringHelper.isNullOrEmpty(message)) {
			logger.log(Level.INFO, "hello.printNothing");
		} else {

			String jndiName = HELLOWORLD_CONTEXT + "/" + HELLOWORLD_JNDI_ENTRY;

			try {
				context.getNamingContext().unbind(jndiName);
				context.getNamingContext().bind(jndiName, message);
			} catch (NameNotFoundException e1) {
				// Context doesn't exist, I can add it
				try {
					context.getNamingContext().bind(jndiName, message);
				} catch (NamingException e) {
					logger.log(Level.WARNING, "hello.cantBind",
							new String[] { jndiName });
				}
			} catch (NamingException e1) {
				logger.log(Level.WARNING, "hello.cantBind",
						new String[] { jndiName });
			}
		}

	}

	public void printMessage(final String message) {
		if (StringHelper.isNullOrEmpty(message)) {
			logger.log(Level.INFO, "hello.printNothing");
		} else {
			logger.log(Level.INFO, "hello.received", new String[] { message });
		}
	}

	public String sayHello(final String message) {
		String response = "You told me: " + message;
		if (StringHelper.isNullOrEmpty(message)) {
			logger.log(Level.INFO, "hello.printNothing");
			response = "You told me nothing";
		} else {
			logger.log(Level.INFO, "hello.received", new String[] { message });
			response = "You told me: " + message;
		}
		return response;
	}

}
