/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SampleClientListener.java 16:40:52 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.engine.sampleclient;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;

import org.objectweb.petals.component.common.util.SourceHelper;
import org.objectweb.petals.engine.sampleclient.gui.Console;

/**
 * Message listener of the SampleClient component
 * 
 * @author alouis, ddesjardins - eBMWebsourcing
 */
public class SampleClientListener implements Runnable {

    private DeliveryChannel channel;

    private MessageExchange messageExchange;

    private boolean running = true;

    private Console console;

    private Logger logger;

    public SampleClientListener(DeliveryChannel channel, Console c, Logger log) {
        super();
        this.channel = channel;
        console = c;
        logger = log;
    }

    public void run() {

        mainLoop: while (running) {
            try {
                messageExchange = channel.accept();
                if (messageExchange != null) {
                    process(messageExchange);
                }
            } catch (MessagingException e) {
                logger.severe(this + " run()" + e.getMessage());
                continue mainLoop;
            }
        }

    }

    public void process(MessageExchange msg) {
        logger.log(Level.INFO, "process(" + msg + ")");

        String response = null;

        if (ExchangeStatus.DONE.equals(msg.getStatus())) {
            response = "receive a DONE status.";
        } else if (ExchangeStatus.ERROR.equals(msg.getStatus())) {
            response = "receive an ERROR status :"
                + msg.getError().getMessage();
        } else {
            try {
                if (msg.getMessage("OUT") != null)
                    response = SourceHelper.createString(msg.getMessage("OUT")
                        .getContent());
                else if (msg.getFault() != null)
                    response = "FAULT : "
                        + SourceHelper
                            .createString(msg.getFault().getContent());

                msg.setStatus(ExchangeStatus.DONE);

                channel.send(msg);
            } catch (Exception e) {
                logger.log(Level.SEVERE, e.getClass() + ":" + e.getMessage());
                response = e.getMessage();
            }
        }
        console.setResponse(response);
    }

    public void stopProcessing() {
        this.running = false;
    }

}
