/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: XlinkerBC.java 154 19 avr. 2006 wjoseph $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.xlinker;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.jbi.component.ComponentContext;
import javax.jbi.component.ComponentLifeCycle;
import javax.jbi.component.ServiceUnitManager;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.ObjectName;
import javax.xml.namespace.QName;

import org.objectweb.petals.binding.xlinker.handler.XlinkerServiceUnitHandler;
import org.objectweb.petals.binding.xlinker.servicemanager.XlinkerServiceUnitManager;
import org.objectweb.petals.component.common.util.ComponentLogger;
import org.objectweb.petals.component.common.util.XMLHelper;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Node;

/**
 * Xlinker binding Component
 * 
 * @version $Rev: 250 $ $Date: 2006-04-21 14:20:57 +0200 (ven, 21 avr 2006) $
 * @since Petals 1.0
 * @author alouis, wjoseph - EBM Websourcing
 * 
 */
public class XlinkerBC implements Component, ComponentLifeCycle {

    public static ComponentContext context;

    public static DeliveryChannel channel;
    
    protected Logger logger;
    
    protected XlinkerServiceUnitManager suManager;
    
    protected XlinkerBCListener listener;

    protected Thread listenerThread;

    /* (non-Javadoc)
     * @see javax.jbi.component.ComponentLifeCycle#getExtensionMBeanName()
     */
    public ObjectName getExtensionMBeanName() {
        return null;
    }

    /* (non-Javadoc)
     * @see javax.jbi.component.Component#getLifeCycle()
     */
    public ComponentLifeCycle getLifeCycle() {
        return this;
    }

    /**
     * Returns the running XlinkerBCListener
     * @return the running XlinkerBCListener
     */
    public XlinkerBCListener getListener() {
        return listener;
    }

    /* (non-Javadoc)
     * @see javax.jbi.component.Component#getServiceDescription(javax.jbi.servicedesc.ServiceEndpoint)
     */
    public Document getServiceDescription(ServiceEndpoint arg0) {
        return suManager.getServiceDescription(arg0);
    }

    /* (non-Javadoc)
     * @see javax.jbi.component.Component#getServiceUnitManager()
     */
    public ServiceUnitManager getServiceUnitManager() {
        logger
                .log(Level.FINE,
                        "XlinkerBC Component instance asked to provide a ServiceUnitManager.");
        return suManager;
    }

    /* (non-Javadoc)
     * @see javax.jbi.component.ComponentLifeCycle#init(javax.jbi.component.ComponentContext)
     */
    public void init(ComponentContext ctx) throws JBIException {
        XlinkerBC.context = ctx;
        Logger log = context.getLogger("", null);
        logger = createComponentLogger(log,context.getComponentName());
        logger.log(Level.INFO, "init()");
        logger.log(Level.FINE, "Initialize the XlinkerBC service");
        suManager = createServiceUnitManager();
    }

    /* (non-Javadoc)
     * @see javax.jbi.component.Component#isExchangeWithConsumerOkay(javax.jbi.servicedesc.ServiceEndpoint, javax.jbi.messaging.MessageExchange)
     */
    public boolean isExchangeWithConsumerOkay(ServiceEndpoint arg0,
            MessageExchange arg1) {
        return true;
    }

    /* (non-Javadoc)
     * @see javax.jbi.component.Component#isExchangeWithProviderOkay(javax.jbi.servicedesc.ServiceEndpoint, javax.jbi.messaging.MessageExchange)
     */
    public boolean isExchangeWithProviderOkay(ServiceEndpoint arg0,
            MessageExchange arg1) {
        return true;
    }

    /* (non-Javadoc)
     * @see javax.jbi.component.Component#resolveEndpointReference(org.w3c.dom.DocumentFragment)
     */
    public ServiceEndpoint resolveEndpointReference(DocumentFragment arg0) {
        ServiceEndpoint result = null;
        if (arg0 != null) {
            String prefix = arg0.getPrefix();
            if (prefix == null) {
                prefix = new String();
            } else {
                prefix += ":";
            }
            Node service = XMLHelper.findChild(arg0, prefix + "service", true);

            if (service != null) {
                String serviceRequested = XMLHelper.getAttributeValue(service,
                        "name");
                if (serviceRequested != null) {
                    result = suManager.findEndpointForService(QName.valueOf(serviceRequested));
                }
            }
        }
        return result;
    }

    /**
     * Stop the running listener 
     * (non-Javadoc)
     * @see javax.jbi.component.ComponentLifeCycle#shutDown()
     */
    public void shutDown() throws JBIException {
        logger.log(Level.INFO, "shutDown()");
        listener.stop();
    }

    /**
     * Create a listener thread dealing with JBI messages
     *  (non-Javadoc)
     * @see javax.jbi.component.ComponentLifeCycle#start()
     */
    public void start() throws JBIException {
        logger.log(Level.INFO, "start()");
        logger.log(Level.FINE, "Start the XlinkerBC service");
        XlinkerBC.channel = XlinkerBC.context.getDeliveryChannel();
        this.listener = new XlinkerBCListener(XlinkerBC.channel,
                XlinkerBC.context, suManager, logger);
        Thread.currentThread().setContextClassLoader(
                this.getClass().getClassLoader());
        listenerThread = new Thread(this.listener,context.getComponentName()+"-JBI listener thread");
        listenerThread.setContextClassLoader(this.getClass().getClassLoader());
        listenerThread.start();
        XlinkerServiceUnitHandler handler = createXlinkerHandler();
        suManager.addNewHandler(handler);
    }

    /* (non-Javadoc)
     * @see javax.jbi.component.ComponentLifeCycle#stop()
     */
    public void stop() throws JBIException {
        logger.log(Level.INFO, "stop()");
    }
    
    /**
     * Create a new componentLogger instance 
     * @param log instance of the component java.logging.util.Logger
     * @param componentName name of the component running
     * @return a new componentLogger
     */
    protected ComponentLogger createComponentLogger(Logger log,String componentName){
        return new ComponentLogger(log, log.getName(), log
            .getResourceBundleName(), componentName);
    }

    /**
     * Create a new Xlinker servuce unit manager
     * @return a new Service Unit manager
     * @throws JBIException if an exception occurs during instanciation
     */
    protected XlinkerServiceUnitManager createServiceUnitManager()
        throws JBIException {
        return new XlinkerServiceUnitManager(context, logger, this);
    }

    /**
     * Create a new XlinkerServiceUnitHandler
     * @return a new instance of XlinkerServiceUnitHandler 
     */
    protected XlinkerServiceUnitHandler createXlinkerHandler() {
        return new XlinkerServiceUnitHandler(context, logger);
    }
}
