/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: PetalsProvider.java 154 19 avr. 2006 wjoseph $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.xlinker.provider;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.activation.DataHandler;
import javax.jbi.JBIException;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.wsdl.Definition;
import javax.wsdl.WSDLException;
import javax.xml.namespace.QName;
import javax.xml.soap.AttachmentPart;
import javax.xml.soap.MessageFactory;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPMessage;
import javax.xml.transform.Source;

import org.act.soap.server.MessageContext;
import org.act.soap.server.provider.Provider;
import org.act.soap.server.provider.ProviderException;
import org.act.soap.server.provider.ServiceImplementationException;
import org.act.soap.util.SOAPUtils;
import org.objectweb.petals.binding.xlinker.XlinkerBC;
import org.objectweb.petals.binding.xlinker.utils.SOAPHelper;
import org.objectweb.petals.component.common.util.SourceHelper;
import org.objectweb.petals.component.common.util.WSDLHelper;
import org.w3c.dom.Document;

/**
 * @version $Rev: 250 $ $Date: 2006-04-21 14:20:57 +0200 (ven, 21 avr 2006) $
 * @since Petals 1.0
 * @author wjoseph - EBM Webousourcing
 * 
 */
public class PetalsProvider extends Provider {

    protected static final String ERROR_CREATE_SOURCE = "Error while creating source object from SOAP message ";

    protected static final String ERROR_PARSE_WSDL = "Error while parsing WSDL file ";

    protected static final String ERROR_CREATE_MSG_EXCH = "Error creating the JBI message exchange ";

    protected static final String ERROR_ADD_ATTACHMENT = "Error adding attachment to the normalized message ";

    protected static final String ERROR_CREATE_STRING_FROM_NM = "Error creating a string from normalized message ";

    protected static final String ERROR_SEND_MSG = "Error sending the message on the JBI container ";

    protected static final String ERROR_SERVICE_INVOKE = "An error occured in service invokation ";

    protected static final String ERROR_NO_ENDPOINT_FOR_SERVICE = "No endpoint was found for this service ";

    protected static final String ERROR_WRONG_MSG_STATUS = "Message status is not 'ACTIVE' ";

    protected static final String ERROR_PUT_CONTENT_IN_SOAP_ENVELOPE = "Error puting message content in soap envelope ";

    protected static final String ERROR_GET_ATTACHMENTS_FROM_SOAP_MESSAGE = "Error extracting attachments from soap message ";

    protected static final String ERROR_PUT_ATTACHMENTS_IN_SOAP_MESSAGE = "Error putting the attachments from JBI message in the soap message ";

    protected MessageFactory mMessageFactory;

    private Class mServiceClass;

    private Object mServiceObject;

    public PetalsProvider() {
        mMessageFactory = SOAPUtils.MESSAGEFACTORY_INSTANCE;
    }

    /**
     * Process a document style web-service call. The request soap message is
     * parsed and its body is sent to the JBI service. The response from this
     * service will be placed as the soap response message after a
     * transformation (non-Javadoc)
     * 
     * @see org.act.soap.server.provider.Provider#doDocumentProcess(org.act.soap.server.MessageContext)
     */
    @Override
    public void doDocumentProcess(MessageContext msgContext)
        throws ProviderException, ServiceImplementationException {
        try {
            String methodName = msgContext.getBindingOperation().getName();
            QName method = new QName(methodName);
            SOAPMessage requestMessage = msgContext.getRequestMessage();
            Definition serviceDef = msgContext.getWSDLDefinition();
            /**
             * TODO We need to know the service name here, we check in the
             * service definition and return the first name found which would
             * cause problem if there is more than one service defined
             */
            QName serviceName = getServiceName(msgContext);
            /**
             * Get all attachments from the message
             */
            List<DataHandler> dataHandlers = null;
            dataHandlers = getAttachmentsFromSOAPMessage(requestMessage);

            // Get the endpoint of this service
            ServiceEndpoint[] endpointList = getEndpointsForService(serviceName);
            ServiceEndpoint endpoint = null;
            if (endpointList.length > 0) {
                endpoint = endpointList[0];
            } else {
                throw new ProviderException(ERROR_NO_ENDPOINT_FOR_SERVICE + " "
                        + serviceName);
            }
            // Determines if the operation exists and if it is an inOnly or
            // inOut operation
            boolean found = hasOperationNamed(serviceDef, methodName,
                    serviceName);
            boolean inOut = isInOutOperation(serviceDef, methodName,
                    serviceName);
            MessageExchange msg = null;
            Source source = null;
            // Extract the content of the soap body and convert it to a source
            source = createSource(requestMessage);
            // Create the message exchange for the JBI container
            if (found && inOut) {
                msg = (InOut) createInOutMessageExchange(method, serviceName,
                        source);
            } else if (found && !inOut) {
                msg = (InOnly) createInOnlyMessageExchange(method, serviceName,
                        source);
            }
            msg.setEndpoint(endpoint);
            // Create the request message for JBI
            NormalizedMessage inMessage = null;
            inMessage = msg.getMessage("in");
            // Put all attachements in the JBI message
            int i = 0;
            for (DataHandler dh : dataHandlers) {
                try {
                    inMessage.addAttachment("" + i++, dh);
                } catch (MessagingException e) {
                    throw new ProviderException(ERROR_ADD_ATTACHMENT, e);
                }
            }
            // Sends the request to the JBI Service
            DeliveryChannel channel = getDeliveryChannel();
            try {
                channel.sendSync(msg, 0);
            } catch (Exception e) {
                throw new ProviderException(ERROR_SEND_MSG, e);
            }
            // Retrieve response message
            if ((inOut && msg.getStatus().equals(ExchangeStatus.ACTIVE))
                    || (!inOut && msg.getStatus().equals(ExchangeStatus.DONE))) {
                NormalizedMessage nm2 = msg.getFault();
                if (nm2 == null) {
                    nm2 = msg.getMessage("out");
                }
                // Create SOAP Message
                SOAPMessage responseMessage = createSOAPMessage();
                // Fill the Body of the response message
                SOAPEnvelope responseEnvelope = responseMessage.getSOAPPart()
                        .getEnvelope();
                createResponseFromNormalizedMessage(responseEnvelope, nm2);
                // Put all attachments in the response message
                putJBIAttachmentsInSOAPMessage(responseMessage, nm2);
                msgContext.setResponseMessage(responseMessage);
                if (inOut) {
                    try {
                        msg.setStatus(ExchangeStatus.DONE);
                    } catch (MessagingException e) {
                        throw new ProviderException(ERROR_WRONG_MSG_STATUS, e);
                    }
                    // Send the reponse with status DONE to the JBI service
                    // provider
                    try {
                        channel.send(msg);
                    } catch (MessagingException e) {
                        throw new ProviderException(ERROR_SEND_MSG, e);
                    }
                }
            } else {
                if (msg.getStatus().equals(ExchangeStatus.ERROR)) {
                    Exception error = msg.getError();
                    try {
                        msg.setStatus(ExchangeStatus.DONE);
                    } catch (MessagingException e) {
                        throw new ProviderException(ERROR_WRONG_MSG_STATUS, e);
                    }
                    // Send the reponse with status DONE to the service provider
                    try {
                        channel.send(msg);
                    } catch (MessagingException e) {
                        throw new ProviderException(ERROR_SEND_MSG, e);
                    }
                    throw new ProviderException(ERROR_SERVICE_INVOKE, error);
                } else {
                    throw new ProviderException(ERROR_WRONG_MSG_STATUS);
                }
            }
        } catch (SOAPException ex) {
            throw new ProviderException(org.act.soap.util.i18n.Message
                    .getMessage("javaSOAPException", ex.getMessage()), ex);
        }
    }

    /**
     * This method simply call the doDocumentProcess method with same arguments
     * (non-Javadoc)
     * 
     * @see org.act.soap.server.provider.Provider#doRPCProcess(org.act.soap.server.MessageContext)
     */
    @Override
    public void doRPCProcess(MessageContext msgContext)
        throws ProviderException, ServiceImplementationException {
        doDocumentProcess(msgContext);
    }

    /**
     * Return the service class's name
     * @return The service class's name
     */
    public String getServiceClassName() {
        return mServiceClass.getName();
    }
    
    /* (non-Javadoc)
     * @see org.act.soap.server.provider.Provider#init(org.act.soap.wsdd.Provider)
     */
    @Override
    public void init(org.act.soap.wsdd.Provider wsddProvider)
        throws ProviderException {
        String serviceClassName = "";
        try {
            serviceClassName = wsddProvider.getParameter("ClassName")
                    .getValue();
            ClassLoader classLoader = Thread.currentThread()
                    .getContextClassLoader();
            mServiceClass = classLoader.loadClass(serviceClassName);
            mServiceObject = mServiceClass.newInstance();
            Method initMethod = null;
            try {
                initMethod = mServiceClass.getMethod("init", new Class[0]);
                if (null != initMethod) {
                    initMethod.invoke(mServiceObject, new Object[0]);
                }
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        } catch (ClassNotFoundException ex) {
            throw new ProviderException(org.act.soap.util.i18n.Message
                    .getMessage("serviceClassNotFound", serviceClassName));
        } catch (Exception ex) {
            ex.printStackTrace();
            throw new ProviderException(
                    org.act.soap.util.i18n.Message.getMessage(
                            "serviceClassInvokeException", ex.getMessage()), ex);
        }
    }

    /**
     * Create a new InOnly MessageExchange with the given Operation, Service and
     * content
     * 
     * @param operationName
     *            the targeted operation QName
     * @param serviceName
     *            the targeted service QName
     * @param content
     *            the Source content
     * @return a new MessageExchange, must be non null
     * @throws MessagingException
     *             if MessageExchange and NormalizedMessage can't be created or
     *             filled or if delivery channel can't be retrieved
     */
    protected MessageExchange createInOnlyMessageExchange(QName operationName,
            QName serviceName, Source content) throws ProviderException {
        return createMessageExchange(operationName, serviceName, content, true);
    }

    /**
     * Create a new InOut MessageExchange with the given Operation, Service and
     * content
     * 
     * @param operationName
     *            the targeted operation QName
     * @param serviceName
     *            the targeted service QName
     * @param content
     *            the Source content
     * @return a new MessageExchange, must be non null
     * @throws MessagingException
     *             if MessageExchange and NormalizedMessage can't be created or
     *             filled or if delivery channel can't be retrieved
     */
    protected MessageExchange createInOutMessageExchange(QName operationName,
            QName serviceName, Source content) throws ProviderException {
        return createMessageExchange(operationName, serviceName, content, false);
    }

    /**
     * Create a new MessageExchange with the given Operation, Service and
     * content. MEP is InOnly if inOnly boolean is set to true, InOut if it's
     * set to false
     * 
     * @param operationName
     *            the targeted operation QName
     * @param serviceName
     *            the targeted service QName
     * @param content
     *            the Source content
     * @param inOnly
     *            boolean true --> MEP=InOnly, false --> MEP=InOut
     * @return a new MessageExchange, must be non null
     * @throws MessagingException
     *             if MessageExchange and NormalizedMessage can't be created or
     *             filled or if delivery channel can't be retrieved
     */
    protected MessageExchange createMessageExchange(QName operationName,
            QName serviceName, Source content, boolean inOnly)
        throws ProviderException {
        MessageExchange messageExchange = null;
        try {
            DeliveryChannel channel = getDeliveryChannel();
            if (channel != null) {
                if (inOnly) {
                    messageExchange = channel.createExchangeFactory()
                            .createInOnlyExchange();
                } else {
                    messageExchange = channel.createExchangeFactory()
                            .createInOutExchange();
                }
                NormalizedMessage nm = messageExchange.createMessage();
                nm.setContent(content);
                if (inOnly) {
                    ((InOnly) messageExchange).setInMessage(nm);
                } else {
                    ((InOut) messageExchange).setInMessage(nm);
                }
                messageExchange.setService(serviceName);
                messageExchange.setOperation(operationName);
            } else {
                throw new MessagingException(
                        "Can't retrieve the delivery channel of the component");
            }
        } catch (MessagingException e) {
            throw new ProviderException(ERROR_CREATE_MSG_EXCH, e);
        }
        return messageExchange;
    }

    /**
     * Creates a SOAP response from a NormalizedMessage
     * 
     * @param inMessage
     *            inMessage of the MessageExchange
     * @param nm2
     *            NormalizedMessage containing the response
     * @return a SOAPEnveloppe created from the nm2 NomalizedMessage content
     * @throws ProviderException
     * @throws AxisFault
     */
    protected SOAPEnvelope createResponseFromNormalizedMessage(
            SOAPEnvelope responseEnv, NormalizedMessage nm2)
        throws ProviderException {

        /**
         * Create and fill the Soap body with the content of the Normalized
         * message
         */
        String response = null;
        if (nm2 != null) {
            try {
                response = createString(nm2);
            } catch (Exception e) {
                throw new ProviderException(ERROR_CREATE_STRING_FROM_NM, e);
            }
        } else {
            response = "<Response>Done</Response>";
        }
        try {
            putContentInSOAPEnvelope(responseEnv, response);
        } catch (Exception e) {
            throw new ProviderException(ERROR_PUT_CONTENT_IN_SOAP_ENVELOPE, e);
        }

        return responseEnv;
    }

    /**
     * Create an empty soap message
     * 
     * @return A new SOAPMessage
     * @throws SOAPException
     *             if the new message could not be created
     */
    protected SOAPMessage createSOAPMessage() throws SOAPException {
        SOAPMessage responseMessage = mMessageFactory.createMessage();
        responseMessage
                .setProperty(SOAPMessage.CHARACTER_SET_ENCODING, "UTF-8");
        responseMessage.setProperty(SOAPMessage.WRITE_XML_DECLARATION, "true");
        return responseMessage;
    }

    /**
     * Create a new Source object containing the axis message Soap envelope
     * 
     * @param message
     *            the axis message
     * @return the source content
     */
    protected Source createSource(SOAPMessage message) throws ProviderException {
        String source = null;
        try {
            source = SOAPHelper.createStringFromSOAPEnvelope(message
                    .getSOAPPart().getEnvelope());
        } catch (Exception e) {
            throw new ProviderException(ERROR_CREATE_SOURCE, e);
        }
        return SourceHelper.createSource(source);
    }

    /**
     * Return the content of the normalized message as a String
     * 
     * @param nm
     *            the normalized message
     * @return a String
     * @throws Exception
     *             if an error during the transformation from source to String
     *             occured
     */
    protected String createString(NormalizedMessage nm) throws Exception {
        return SourceHelper.createString(nm.getContent());
    }

    /**
     * Queries the JBI container for ServiceEndpoint matching the service name
     * 
     * @param serviceName
     *            Name of the service to search
     * @return All matching ServiceEndpoint
     */
    protected ServiceEndpoint[] findEndpointsForService(String serviceName) {
        return XlinkerBC.context.getEndpointsForService(new QName(serviceName));
    }

    /**
     * Return all attachments contained in the soap message as a List of
     * DataHandlers
     * 
     * @param message
     *            The soap message
     * @return A list of DataHandler, may be empty, not null
     * @throws ProviderException
     *             if all attachments from soap message could not be retrieved
     */
    @SuppressWarnings("unchecked")
    protected List<DataHandler> getAttachmentsFromSOAPMessage(
            SOAPMessage message) throws ProviderException {
        List<DataHandler> dataHandlers = new ArrayList<DataHandler>();
        try {
            Iterator<AttachmentPart> attachmentsIterator = message
                    .getAttachments();
            while (attachmentsIterator.hasNext()) {
                AttachmentPart attachment = (AttachmentPart) attachmentsIterator
                        .next();
                dataHandlers.add(attachment.getDataHandler());
            }
        } catch (Exception ex) {
            throw new ProviderException(ERROR_GET_ATTACHMENTS_FROM_SOAP_MESSAGE);
        }
        return dataHandlers;
    }

    /**
     * Allow to retrieve the XlinkerBC Delivery Channel
     * 
     * @return the XlinkerBC Delivery Channel, null if it can't be retrieved
     * @throws MessagingException
     *             if a channel has already been opened, but not yet closed.
     */
    protected DeliveryChannel getDeliveryChannel() throws ProviderException {
        return XlinkerBC.channel;
    }

    /**
     * Retrieve the list of all endpoints that match the given serviceName
     * 
     * @param serviceName
     *            the QName of a service
     * @return an array of ServiceEndpoint that match serviceName
     */
    protected ServiceEndpoint[] getEndpointsForService(QName serviceName) {
        return XlinkerBC.context.getEndpointsForService(serviceName);
    }

    /**
     * Search the JBI container for services named like AxisService and return
     * the first service description found
     * 
     * @param service
     *            AxisService to look for
     * @return A dom document representation of the service description, or null
     *         if no service with this name found on JBI container
     * @throws JBIException
     */
    protected Document getServiceDescription(QName service) throws JBIException {
        Document serviceDesc = null;
        ServiceEndpoint[] serviceList = XlinkerBC.context
                .getEndpointsForService(service);
        if (serviceList.length > 0) {
            serviceDesc = XlinkerBC.context
                    .getEndpointDescriptor(serviceList[0]);
        }
        return serviceDesc;
    }

    /**
     * Return the service QName according to its definition contained in the
     * MessageContext
     * 
     * @param msgContext
     *            current MessageContext
     * @return The service QName
     */
    protected QName getServiceName(MessageContext msgContext) {
        return WSDLHelper.getServiceNameFromWSDLDefinition(msgContext
                .getWSDLDefinition())[0];
    }

    /**
     * Checks if the service description specifies an operation named
     * operationName
     * 
     * @param serviceDesc
     *            Service description to look into
     * @param operationName
     *            Name of operation to find
     * @return True if operation was found, or else false
     * @throws WSDLException
     */
    protected boolean hasOperationNamed(Definition serviceDef,
            String operationName, QName serviceName) {
        return WSDLHelper.hasOperationNamed(serviceDef, operationName,
                serviceName);
    }

    /**
     * Checks if the service description specifies an operation named
     * operationName and if this operation specifyes a response type
     * 
     * @param serviceDesc
     *            Service description to look into
     * @param operationName
     *            Name of operation to find
     * @return True if operation was found and is InOut, or else false
     * @return
     * @throws WSDLException
     */
    protected boolean isInOutOperation(Definition serviceDef,
            String operationName, QName serviceName) {
        return WSDLHelper.isInOutOperation(serviceDef, operationName,
                serviceName);
    }

    /**
     * Parse the response argument as a dom document and insert it in the body
     * of the soap envelope
     * 
     * @param responseEnv
     *            a soap envelope to be modified
     * @param response
     *            the string to insert in the envelope
     * @throws Exception
     *             if the response could not be parsed to an XML document or if
     *             an error occurs while updating the soap envelope
     */
    protected void putContentInSOAPEnvelope(SOAPEnvelope responseEnv,
            String response) throws Exception {
        SOAPHelper.putContentInSOAPEnvelope(responseEnv, response);
    }

    /**
     * Get attachments from a NormalizedMessage and place them in the soap
     * message
     * 
     * @param message
     *            soap message to receive attachments
     * @param nm
     *            NormalizedMessage containing attachements
     * @throws ProviderException
     *             if attachments from normalized message could not be retrieved
     *             or added to soap message
     */
    @SuppressWarnings("unchecked")
    protected void putJBIAttachmentsInSOAPMessage(SOAPMessage message,
            NormalizedMessage nm) throws ProviderException {
        if (nm != null) {
            try {
                Set<String> attachmentNames = nm.getAttachmentNames();
                for (String attName : attachmentNames) {
                    DataHandler dh = nm.getAttachment(attName);
                    AttachmentPart attachment = message
                            .createAttachmentPart(dh);
                    message.addAttachmentPart(attachment);
                }
                message.saveChanges();
            } catch (SOAPException e) {
                throw new ProviderException(
                        ERROR_PUT_ATTACHMENTS_IN_SOAP_MESSAGE);
            }
        }
    }

}
