/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SOAPHelper.java 154 2006-03-27 15:30:10Z wjoseph $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.xlinker.utils;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPEnvelope;

import org.objectweb.petals.component.common.util.XMLHelper;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

/**
 * @version	$Rev: 250 $ $Date: 2006-04-21 14:20:57 +0200 (ven, 21 avr 2006) $
 * @since	Petals 1.0
 * @author wjoseph - EBM Webousourcing
 *
 */
public final class SOAPHelper {

    /**
     * Default constructor
     */
    private SOAPHelper() {
    }

    /**
     * Make a String representation of the content of a SOAP envelope
     * @param envelope the soap envelope to be converted
     * @return a String representation of the soap envelope
     * @throws Exception if an error occurs during the parsing of the soap envelope
     */
    @SuppressWarnings("unchecked")
    public static String createStringFromSOAPEnvelope(SOAPEnvelope envelope)
        throws Exception {
        String result = new String();
        String end = null;
        List<String> prefixs = new ArrayList<String>();
        String namespaces = new String();
        SOAPBody body = envelope.getBody();
        Iterator<String> iterator = envelope.getNamespacePrefixes();
        Node node = body.getFirstChild();
        if (node != null) {
            while (node.getNodeType() != Node.ELEMENT_NODE) {
                node = node.getNextSibling();
            }
            NamedNodeMap attributes = node.getAttributes();
            while (iterator.hasNext()) {
                String prefix = iterator.next();
                String uri = envelope.getNamespaceURI(prefix);
                if (prefix != null && uri != null && !prefixs.contains(prefix)
                        && attributes.getNamedItem("xmlns:" + prefix) == null
                        && !uri.equalsIgnoreCase("http://schemas.xmlsoap.org/soap/envelope/")) {
                    namespaces += " xmlns:" + prefix + "=\"" + uri + "\" ";
                    prefixs.add(prefix);
                }
            }

            if (body != null) {
                iterator = body.getNamespacePrefixes();
                while (iterator.hasNext()) {
                    String prefix = iterator.next();
                    String uri = envelope.getNamespaceURI(prefix);
                    if (prefix != null
                            && uri != null
                            && !prefixs.contains(prefix)
                            && attributes.getNamedItem("xmlns:" + prefix) == null
                            && !uri.equalsIgnoreCase("http://schemas.xmlsoap.org/soap/envelope/")) {
                        namespaces += " xmlns:" + prefix + "=\"" + uri + "\" ";
                        prefixs.add(prefix);
                    }
                }
            }
            String nodeName = node.getNodeName();
            result = XMLHelper.createStringFromDOMNode(node);
            int idx3 = result.indexOf(nodeName);
            if (idx3 > -1) {
                result = result.substring(idx3 - 1);
            }
            int idx = result.indexOf('>');
            int idx2 = result.indexOf("/>");
            if (idx2 > 0 && idx2 < idx) {
                end = result.substring(0, idx2) + namespaces
                        + result.substring(idx2);
            } else {
                end = result.substring(0, idx) + namespaces
                        + result.substring(idx);
            }
        } else {
            end = new String();
        }
        return end;
    }

    /**
     * Put the response string in the body of the soap envelope
     * @param envelope the soap envelope to enhance
     * @param response the content to be set in the envelope's body, must be XML
     * @return the soap envelope with its new content
     * @throws Exception If the content is not valid XML
     */
    public static SOAPEnvelope putContentInSOAPEnvelope(SOAPEnvelope envelope,
            String response) throws Exception {
        SOAPBody body = null;
        body = envelope.getBody();
        Document doc = XMLHelper.createDocumentFromString(response);
        org.w3c.dom.Node node = doc.getFirstChild();
        while (node.getNodeType() != Node.ELEMENT_NODE) {
            node = node.getNextSibling();
        }
        Node importedNode = body.getOwnerDocument().importNode(node, true);
        body.appendChild(importedNode);
        return envelope;
    }
}
