/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: XlinkerBCListenerTest.java 154 18 mai 2006 wjoseph $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.xlinker;

import java.lang.reflect.Method;
import java.util.HashSet;
import java.util.Set;
import java.util.logging.Logger;

import javax.activation.DataHandler;
import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.InOptionalOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.messaging.RobustInOnly;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.wsdl.WSDLException;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.soap.SOAPConnection;
import javax.xml.soap.SOAPMessage;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;

import junit.framework.TestCase;

import org.act.soap.server.StandaloneWSServer;
import org.easymock.classextension.EasyMock;
import org.objectweb.petals.binding.xlinker.XlinkerBCListener;
import org.objectweb.petals.binding.xlinker.mock.MockDeliveryChannel;
import org.objectweb.petals.binding.xlinker.mock.MockFault;
import org.objectweb.petals.binding.xlinker.mock.MockInOnly;
import org.objectweb.petals.binding.xlinker.mock.MockInOptionalOut;
import org.objectweb.petals.binding.xlinker.mock.MockInOut;
import org.objectweb.petals.binding.xlinker.mock.MockMessageExchange;
import org.objectweb.petals.binding.xlinker.mock.MockNormalizedMessage;
import org.objectweb.petals.binding.xlinker.mock.MockRobustInOnly;
import org.objectweb.petals.binding.xlinker.mock.MockServiceEndpoint;
import org.objectweb.petals.binding.xlinker.mock.MockXlinkerBCListener;
import org.objectweb.petals.component.common.serviceunitmanager.manager.PetalsServiceUnitManager;
import org.w3c.dom.Document;

import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.reset;
import static org.easymock.classextension.EasyMock.verify;

/**
 * 
 * @author wjoseph - EBM Websourcing
 * 
 */
public class XlinkerBCListenerTest extends TestCase {

    public void testCreateRequestMessage() {
        /*
         * Test Case 1
         */
        /*
         * Create Mocks
         */
        PetalsServiceUnitManager sum = createMock(PetalsServiceUnitManager.class);
        DeliveryChannel channel = createMock(DeliveryChannel.class);
        ComponentContext context = createMock(ComponentContext.class);

        /*
         * Create Test parameters
         */
        Logger log = EasyMock.createMock(Logger.class);

        XlinkerBCListener xlinkerbcListener = new XlinkerBCListener(channel,
            context, sum, log);
        /*
         * Run Test Case 1
         */
        SOAPMessage result = null;
        try {
            result = xlinkerbcListener.createRequestMessage();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create a new soap message");
        }
        assertNotNull("A new SOAPMessage should have been created", result);
        assertEquals("The SOAPAction MimeHeader should be an empty string", "",
            result.getMimeHeaders().getHeader("SOAPAction")[0]);
    }

    public void testCreateSOAPConnection() {
        /*
         * Create Mocks
         */
        PetalsServiceUnitManager sum = createMock(PetalsServiceUnitManager.class);
        DeliveryChannel channel = createMock(DeliveryChannel.class);
        ComponentContext context = createMock(ComponentContext.class);

        /*
         * Create Test parameters
         */
        Logger log = EasyMock.createMock(Logger.class);

        XlinkerBCListener xlinkerbcListener = new XlinkerBCListener(channel,
            context, sum, log);
        /*
         * Run Test Case 1
         */
        SOAPConnection result = null;
        try {
            result = xlinkerbcListener.createSOAPConnection();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Exception raised during method invokation");
        }
        assertNotNull("A new SOAPConnection instance should have benn created",
            result);
    }

    public void testGetReceiver() {
        /*
         * Test Case 1
         */
        /*
         * Create test parameters
         */
        MockXlinkerBCListener xlinkerbcListener = new MockXlinkerBCListener();
        StandaloneWSServer server = createMock(StandaloneWSServer.class);
        xlinkerbcListener.receiver = server;
        Object result = null;
        /*
         * Run Test Case 1
         */
        result = xlinkerbcListener.getReceiver();
        assertEquals("The returned server is different", result, server);

    }

    public void testProcessExternalCall() {

        /*
         * Create Mocks
         */
        Method method1 = null;
        try {
            method1 = XlinkerBCListener.class.getDeclaredMethod(
                "hasOperationNamed", new Class[] {Document.class, String.class,
                    QName.class});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could retrieve method for mocks");
        }
        Method[] mockedMethods = new Method[] {method1};
        MockXlinkerBCListener xlinkerListener = createMock(
            MockXlinkerBCListener.class, mockedMethods);
        MessageExchange messageExchange = createMock(MockMessageExchange.class);
        Document serviceDesc = createMock(Document.class);
        QName serviceName = new QName("fooService");
        String operation = "fooOperation";

        /*
         * Test Case 1
         */
        try {
            expect(
                xlinkerListener.hasOperationNamed(serviceDesc, operation,
                    serviceName)).andReturn(false);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize mocks");
        }
        replay(xlinkerListener);
        /*
         * Run Test Case 1
         */
        try {
            xlinkerListener.processExternalCallSuper(messageExchange,
                serviceDesc, serviceName, operation);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during processExternalCall invocation");
        }
        assertTrue(xlinkerListener.sendErrorCalled);
        verify(xlinkerListener);

        /*
         * Test Case 2
         */
        reset(xlinkerListener);
        try {
            expect(
                xlinkerListener.hasOperationNamed(serviceDesc, operation,
                    serviceName)).andReturn(true);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize mocks");
        }
        replay(xlinkerListener);

        /*
         * Create Test parameters
         */
        mockedMethods = new Method[] {};
        MockDeliveryChannel channel = new MockDeliveryChannel();
        xlinkerListener.setChannel(channel);
        /*
         * Run Test Case 2
         */
        try {
            xlinkerListener.processExternalCallSuper(messageExchange,
                serviceDesc, serviceName, operation);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during processInternalCall invocation");
        }
        assertTrue(xlinkerListener.processOperationCalled);
        verify(xlinkerListener);

        /*
         * Test Case 3 A WSDLException will be thrown causing a
         * MessagingException to be thrown
         */
        reset(xlinkerListener);
        try {
            expect(
                xlinkerListener.hasOperationNamed(serviceDesc, operation,
                    serviceName)).andThrow(
                (new WSDLException("Testing exception", "Exception")));
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize mocks");
        }
        replay(xlinkerListener);

        /*
         * Create Test parameters
         */
        mockedMethods = new Method[] {};
        channel = new MockDeliveryChannel();
        xlinkerListener.setChannel(channel);
        /*
         * Run Test Case 3
         */
        try {
            xlinkerListener.processExternalCallSuper(messageExchange,
                serviceDesc, serviceName, operation);
            fail("No exception was raised");
        } catch (Exception e) {
            verify(xlinkerListener);
        }
    }

    public void testProcessInOnlyOperation() {
        // Create Mocks
        Method method1 = null;
        Method method2 = null;
        Method method3 = null;
        try {
            method1 = XlinkerBCListener.class.getDeclaredMethod(
                "getPortNameFromWSDLDocument", new Class[] {Document.class,
                    QName.class});
            method2 = XlinkerBCListener.class.getDeclaredMethod("createString",
                new Class[] {Source.class});
            method3 = XlinkerBCListener.class.getDeclaredMethod("callWS",
                new Class[] {QName.class, String.class, String.class,
                    Document.class, String.class, Set.class});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could retrieve method for mocks");
        }
        Method[] mockedMethods = new Method[] {method1, method2, method3};
        MockXlinkerBCListener xlinkerListener = createMock(
            MockXlinkerBCListener.class, mockedMethods);
        NormalizedMessage nm = createMock(MockNormalizedMessage.class);
        MessageExchange messageExchange = createMock(MockInOnly.class);
        Source source = new StreamSource();
        String portName = "fooServicePort";
        String opName = "fooOperation";
        QName serviceName = new QName("fooService");
        Set<DataHandler> attachments = new HashSet<DataHandler>();
        String inEnv = new String();
        Document serviceDesc = null;
        mockedMethods = new Method[] {};
        // MockDeliveryChannel channel =
        // createMock(MockDeliveryChannel.class,mockedMethods);
        MockDeliveryChannel channel = new MockDeliveryChannel();
        xlinkerListener.setChannel(channel);
        SOAPMessage message = createMock(SOAPMessage.class);
        try {
            expect(nm.getContent()).andReturn(source);
            expect(
                xlinkerListener.getPortNameFromWSDLDocument(serviceDesc,
                    serviceName)).andReturn(portName);
            expect(xlinkerListener.createString(source)).andReturn(inEnv);
            expect(
                xlinkerListener.callWS(serviceName, opName, portName,
                    serviceDesc, inEnv, attachments)).andReturn(message);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize mocks");
        }
        replay(nm);
        replay(xlinkerListener);

        // Run Test Case 1
        // The remote webservice should be called
        try {
            xlinkerListener.processInOnlyOperationSuper(messageExchange,
                opName, serviceName, serviceDesc, nm, attachments);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during call of processInOnlyOperation");
        }
        verify(nm);
        verify(xlinkerListener);
        assertTrue(channel.isSend());

        // Test Case 2
        reset(nm);
        reset(xlinkerListener);
        messageExchange = createMock(InOptionalOut.class);
        mockedMethods = new Method[] {};
        channel = createMock(MockDeliveryChannel.class, mockedMethods);
        xlinkerListener.setChannel(channel);
        try {
            expect(nm.getContent()).andReturn(source);
            expect(
                xlinkerListener.getPortNameFromWSDLDocument(serviceDesc,
                    serviceName)).andReturn(portName);
            expect(xlinkerListener.createString(source)).andReturn(inEnv);
            expect(
                xlinkerListener.callWS(serviceName, opName, portName,
                    serviceDesc, inEnv, attachments)).andReturn(message);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize mocks");
        }
        replay(nm);
        replay(xlinkerListener);

        // Run Test Case 2
        // The remote webservice should be called
        try {
            xlinkerListener.processInOnlyOperationSuper(messageExchange,
                opName, serviceName, serviceDesc, nm, attachments);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during call of processInOnlyOperation");
        }
        verify(nm);
        verify(xlinkerListener);
        assertTrue(channel.isSend());

        // Test Case 3
        reset(nm);
        reset(xlinkerListener);
        messageExchange = createMock(RobustInOnly.class);
        mockedMethods = new Method[] {};
        channel = createMock(MockDeliveryChannel.class, mockedMethods);
        xlinkerListener.setChannel(channel);
        try {
            expect(nm.getContent()).andReturn(source);
            expect(
                xlinkerListener.getPortNameFromWSDLDocument(serviceDesc,
                    serviceName)).andReturn(portName);
            expect(xlinkerListener.createString(source)).andReturn(inEnv);
            expect(
                xlinkerListener.callWS(serviceName, opName, portName,
                    serviceDesc, inEnv, attachments)).andReturn(message);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize mocks");
        }
        replay(nm);
        replay(xlinkerListener);

        // Run Test Case 3
        // The remote webservice should be called
        try {
            xlinkerListener.processInOnlyOperationSuper(messageExchange,
                opName, serviceName, serviceDesc, nm, attachments);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during call of processInOnlyOperation");
        }
        verify(nm);
        verify(xlinkerListener);
        assertTrue(channel.isSend());

        // Create Mocks
        messageExchange = createMock(MockInOut.class);

        // Create Test parameters
        xlinkerListener = new MockXlinkerBCListener();

        // Run Test Case 4
        try {
            xlinkerListener.processInOnlyOperationSuper(messageExchange,
                opName, serviceName, serviceDesc, nm, attachments);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during call of processInOnlyOperation");
        }
        assertTrue(xlinkerListener.sendErrorCalled);

        // Create Test parameter
        xlinkerListener = new MockXlinkerBCListener();

        // Run Test Case 5
        try {
            xlinkerListener.processInOnlyOperationSuper(null, opName,
                serviceName, serviceDesc, nm, attachments);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during call of processInOnlyOperation");
        }
        assertTrue(xlinkerListener.sendErrorCalled);
    }

    public void testProcessInOutOperation() {
        // Create Mocks
        Method method1 = null;
        Method method2 = null;
        Method method3 = null;
        Method method5 = null;
        try {
            method1 = XlinkerBCListener.class.getDeclaredMethod(
                "getPortNameFromWSDLDocument", new Class[] {Document.class,
                    QName.class});
            method2 = XlinkerBCListener.class.getDeclaredMethod("createString",
                new Class[] {Source.class});
            method3 = XlinkerBCListener.class.getDeclaredMethod("callWS",
                new Class[] {QName.class, String.class, String.class,
                    Document.class, String.class, Set.class});
            method5 = MockNormalizedMessage.class.getDeclaredMethod(
                "getContent", new Class[] {});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could retrieve method for mocks");
        }
        Method[] mockedMethods = new Method[] {method1, method2, method3};
        MockXlinkerBCListener xlinkerListener = createMock(
            MockXlinkerBCListener.class, mockedMethods);
        mockedMethods = new Method[] {method5};
        NormalizedMessage nm = createMock(MockNormalizedMessage.class,
            mockedMethods);
        mockedMethods = new Method[] {};
        MockInOut messageExchange = createMock(MockInOut.class, mockedMethods);
        Source source = new StreamSource();
        String portName = "fooServicePort";
        String opName = "fooOperation";
        QName serviceName = new QName("fooService");
        Set<DataHandler> attachments = new HashSet<DataHandler>();
        String inEnv = new String();
        SOAPMessage outMessage = createMock(SOAPMessage.class);
        Document serviceDesc = null;
        mockedMethods = new Method[] {};
        MockDeliveryChannel channel = createMock(MockDeliveryChannel.class,
            mockedMethods);
        xlinkerListener.setChannel(channel);
        try {
            expect(nm.getContent()).andReturn(source);
            expect(
                xlinkerListener.getPortNameFromWSDLDocument(serviceDesc,
                    serviceName)).andReturn(portName);
            expect(xlinkerListener.createString(source)).andReturn(inEnv);
            expect(
                xlinkerListener.callWS(serviceName, opName, portName,
                    serviceDesc, inEnv, attachments)).andReturn(outMessage);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize mocks");
        }
        replay(nm);
        replay(xlinkerListener);

        // Run Test Case 1
        // The remote webservice should be called
        try {
            xlinkerListener.processInOutOperationSuper(messageExchange, opName,
                serviceName, serviceDesc, nm, attachments);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during call of processInOnlyOperation");
        }
        verify(nm);
        verify(xlinkerListener);
        assertTrue(messageExchange.isSetMessage());
        assertTrue(channel.isSend());
        assertTrue(xlinkerListener.isFillNormalizedMessageFromSOAPMessage());

        // Test Case 2
        reset(nm);
        reset(xlinkerListener);
        MessageExchange messageExchange2 = createMock(MockInOptionalOut.class,
            mockedMethods);
        mockedMethods = new Method[] {};
        channel = createMock(MockDeliveryChannel.class, mockedMethods);
        xlinkerListener.setChannel(channel);
        try {
            expect(nm.getContent()).andReturn(source);
            expect(
                xlinkerListener.getPortNameFromWSDLDocument(serviceDesc,
                    serviceName)).andReturn(portName);
            expect(xlinkerListener.createString(source)).andReturn(inEnv);
            expect(
                xlinkerListener.callWS(serviceName, opName, portName,
                    serviceDesc, inEnv, attachments)).andReturn(outMessage);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize mocks");
        }
        replay(nm);
        replay(xlinkerListener);

        // Run Test Case 2
        // The remote webservice should be called
        try {
            xlinkerListener.processInOutOperationSuper(messageExchange, opName,
                serviceName, serviceDesc, nm, attachments);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during call of processInOnlyOperation");
        }
        verify(nm);
        verify(xlinkerListener);
        assertTrue(messageExchange.isSetMessage());
        assertTrue(channel.isSend());

        // Test Case 3

        // Create Mocks
        messageExchange2 = createMock(MockInOnly.class);

        // Create Test parameters
        xlinkerListener = new MockXlinkerBCListener();

        // Run Test Case 3
        try {
            xlinkerListener.processInOutOperationSuper(messageExchange2,
                opName, serviceName, serviceDesc, nm, attachments);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during call of processInOnlyOperation");
        }
        assertTrue(xlinkerListener.sendErrorCalled);

        // Test Case 4

        // Create Mocks
        messageExchange2 = createMock(RobustInOnly.class);

        // Create Test parameters
        xlinkerListener = new MockXlinkerBCListener();

        // Run Test Case 4
        try {
            xlinkerListener.processInOutOperationSuper(messageExchange2,
                opName, serviceName, serviceDesc, nm, attachments);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during call of processInOnlyOperation");
        }
        assertTrue(xlinkerListener.sendErrorCalled);

        // Test Case 5

        // Create Test parameter
        xlinkerListener = new MockXlinkerBCListener();

        // Run Test Case 5
        try {
            xlinkerListener.processInOutOperationSuper(null, opName,
                serviceName, serviceDesc, nm, attachments);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during call of processInOnlyOperation");
        }
        assertTrue(xlinkerListener.sendErrorCalled);
    }

    public void testProcessMessage() {
        // Test Case 1

        // Create Mocks
        Method method1 = null;
        Method method2 = null;
        Method method3 = null;
        Method method4 = null;
        Method method5 = null;
        try {
            method1 = MockMessageExchange.class.getDeclaredMethod("getStatus",
                new Class[] {});
            method2 = MockMessageExchange.class.getDeclaredMethod("getService",
                new Class[] {});
            method3 = MockMessageExchange.class.getDeclaredMethod(
                "getOperation", new Class[] {});
            method4 = MockMessageExchange.class.getDeclaredMethod(
                "getEndpoint", new Class[] {});
            method5 = PetalsServiceUnitManager.class.getDeclaredMethod(
                "getServiceDescription", new Class[] {ServiceEndpoint.class});

        } catch (Exception e) {
            e.printStackTrace();
            fail("Could retrieve method for mocks");
        }
        Method[] mockedMethods = new Method[] {method1, method2, method3,
            method4};
        MockMessageExchange messageExchange = createMock(
            MockMessageExchange.class, mockedMethods);
        mockedMethods = new Method[] {method5};
        PetalsServiceUnitManager sum = createMock(
            PetalsServiceUnitManager.class, mockedMethods);
        expect(messageExchange.getStatus()).andReturn(ExchangeStatus.ERROR);
        replay(messageExchange);
        MockXlinkerBCListener xlinkerListener = new MockXlinkerBCListener();

        // Run Test Case 1
        try {
            xlinkerListener.processMessage(messageExchange);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during processMessage invocation");
        }
        verify(messageExchange);
        assertTrue(xlinkerListener.sendDoneCalled);

        // Test Case 2
        reset(messageExchange);

        // Create Test parameters
        String service = "fooService";
        QName serviceName = new QName(service);
        String operation = "fooOperation";
        QName operationName = new QName(operation);
        Document axisServiceDesc = null;
        try {
            axisServiceDesc = DocumentBuilderFactory.newInstance()
                .newDocumentBuilder().newDocument();
        } catch (Exception e) {
            e.printStackTrace();
        }

        MockServiceEndpoint axisEndpoint = new MockServiceEndpoint(serviceName,
            "fooServiceEndpoint");
        expect(messageExchange.getStatus()).andReturn(ExchangeStatus.ACTIVE);
        expect(messageExchange.getOperation()).andReturn(operationName);
        expect(messageExchange.getService()).andReturn(serviceName);
        expect(messageExchange.getEndpoint()).andReturn(axisEndpoint)
            .anyTimes();
        expect(sum.getServiceDescription(axisEndpoint)).andReturn(
            axisServiceDesc);
        replay(sum);
        replay(messageExchange);

        // Run Test Case 2
        xlinkerListener.setServiceUnitManager(sum);
        try {
            xlinkerListener.processMessage(messageExchange);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during processMessage invocation");
        }
        verify(sum);
        verify(messageExchange);

        // Test Case 3
        reset(messageExchange);
        reset(sum);

        // Create Test parameters
        Document barServiceDesc = null;
        try {
            barServiceDesc = DocumentBuilderFactory.newInstance()
                .newDocumentBuilder().newDocument();
        } catch (Exception e) {
            e.printStackTrace();
        }

        ServiceEndpoint barEndpoint = new MockServiceEndpoint(new QName(
            "barService"), "barMockServiceEndpoint");
        expect(messageExchange.getStatus()).andReturn(ExchangeStatus.ACTIVE);
        expect(messageExchange.getOperation()).andReturn(operationName);
        expect(messageExchange.getService()).andReturn(serviceName);
        expect(messageExchange.getEndpoint()).andReturn(barEndpoint).anyTimes();
        expect(sum.getServiceDescription(barEndpoint))
            .andReturn(barServiceDesc);
        replay(sum);
        replay(messageExchange);

        // Run Test Case 3
        xlinkerListener.setServiceUnitManager(sum);
        try {
            xlinkerListener.processMessage(messageExchange);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during processMessage invocation");
        }
        assertTrue(xlinkerListener.processExternalCallCalled);
        verify(sum);
        verify(messageExchange);

        // Test Case 4
        reset(messageExchange);
        reset(sum);

        // Create Test parameters
        axisServiceDesc = null;

        expect(messageExchange.getStatus()).andReturn(ExchangeStatus.ACTIVE);
        expect(messageExchange.getOperation()).andReturn(operationName);
        expect(messageExchange.getService()).andReturn(serviceName);
        expect(messageExchange.getEndpoint()).andReturn(axisEndpoint);
        expect(sum.getServiceDescription(axisEndpoint)).andReturn(
            axisServiceDesc);
        replay(sum);
        replay(messageExchange);

        // Run Test Case 4
        xlinkerListener.setServiceUnitManager(sum);
        try {
            xlinkerListener.processMessage(messageExchange);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during processMessage invocation");
        }
        assertTrue(xlinkerListener.sendErrorCalled);
        verify(sum);
        verify(messageExchange);
    }

    public void testProcessOperation() {
        // Test Case 1

        // Create Mocks
        Method method1 = null;
        Method method2 = null;
        Method method3 = null;
        Method method4 = null;
        Method method5 = null;
        try {
            method1 = XlinkerBCListener.class.getDeclaredMethod(
                "getServiceAddressFromWSDLDocument", new Class[] {
                    Document.class, QName.class});
            method2 = MockMessageExchange.class.getDeclaredMethod(
                "getEndpoint", new Class[] {});
            method3 = MockMessageExchange.class.getDeclaredMethod("getMessage",
                new Class[] {String.class});
            method4 = XlinkerBCListener.class.getDeclaredMethod(
                "isInOutOperation", new Class[] {Document.class, String.class,
                    QName.class});
            method5 = XlinkerBCListener.class.getDeclaredMethod(
                "extractAttachments", new Class[] {NormalizedMessage.class});

        } catch (Exception e) {
            e.printStackTrace();
            fail("Could retrieve method for mocks");
        }
        Method[] mockedMethods = new Method[] {method1, method4, method5};
        MockXlinkerBCListener xlinkerListener = createMock(
            MockXlinkerBCListener.class, mockedMethods);
        mockedMethods = new Method[] {method2, method3};
        MessageExchange messageExchange = createMock(MockMessageExchange.class,
            mockedMethods);
        String targetURI = "http://host:port/service/uri";
        Document serviceDesc = createMock(Document.class);
        MockServiceEndpoint fooEndpoint = createMock(MockServiceEndpoint.class);
        String operation = "fooOperation";
        QName service = new QName("fooService");
        Set<DataHandler> attachments = new HashSet<DataHandler>();
        NormalizedMessage inMsg = createMock(MockNormalizedMessage.class);
        try {
            expect(
                xlinkerListener.getServiceAddressFromWSDLDocument(serviceDesc,
                    service)).andReturn(targetURI);
            expect(xlinkerListener.extractAttachments(inMsg)).andReturn(
                attachments);
            expect(
                xlinkerListener.isInOutOperation(serviceDesc, operation,
                    service)).andReturn(true);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize mocks");
        }
        expect(messageExchange.getEndpoint()).andReturn(fooEndpoint);
        expect(messageExchange.getMessage("IN")).andReturn(inMsg);
        replay(messageExchange);
        replay(xlinkerListener);

        // Create Test parameters
        Logger log = EasyMock.createMock(Logger.class);
        xlinkerListener.setLogger(log);

        // Run Test Case 1
        try {
            xlinkerListener.processOperationSuper(messageExchange, serviceDesc,
                operation, service);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during processOperation invocation");
        }
        assertTrue(xlinkerListener.processInOutOperationCalled);
        verify(xlinkerListener);
        verify(messageExchange);

        // Test Case 2
        mockedMethods = new Method[] {method1, method4, method5};
        xlinkerListener = createMock(MockXlinkerBCListener.class, mockedMethods);
        xlinkerListener.setLogger(log);
        reset(messageExchange);
        try {
            expect(
                xlinkerListener.getServiceAddressFromWSDLDocument(serviceDesc,
                    service)).andReturn(targetURI);
            expect(xlinkerListener.extractAttachments(inMsg)).andReturn(
                attachments);
            expect(
                xlinkerListener.isInOutOperation(serviceDesc, operation,
                    service)).andReturn(false);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize mocks");
        }
        expect(messageExchange.getEndpoint()).andReturn(fooEndpoint);
        expect(messageExchange.getMessage("IN")).andReturn(inMsg);
        replay(messageExchange);
        replay(xlinkerListener);

        // Run Test Case 2
        try {
            xlinkerListener.processOperationSuper(messageExchange, serviceDesc,
                operation, service);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during processOperation invocation");
        }
        assertTrue(xlinkerListener.processInOnlyOperationCalled);
        verify(xlinkerListener);
        verify(messageExchange);

        // Test Case 3
        reset(messageExchange);
        mockedMethods = new Method[] {method1, method4, method5};
        xlinkerListener = createMock(MockXlinkerBCListener.class, mockedMethods);
        xlinkerListener.setLogger(log);
        try {
            expect(
                xlinkerListener.getServiceAddressFromWSDLDocument(serviceDesc,
                    service)).andReturn(targetURI);
            expect(xlinkerListener.extractAttachments(null)).andReturn(
                attachments);
            expect(
                xlinkerListener.isInOutOperation(serviceDesc, operation,
                    service)).andReturn(false);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize mocks");
        }
        expect(messageExchange.getEndpoint()).andReturn(fooEndpoint);
        expect(messageExchange.getMessage("IN")).andReturn(null);
        replay(messageExchange);
        replay(xlinkerListener);

        // Run Test Case 3
        try {
            xlinkerListener.processOperationSuper(messageExchange, serviceDesc,
                operation, service);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during processOperation invocation");
        }
        assertTrue(xlinkerListener.sendFaultCalled);
        verify(xlinkerListener);
        verify(messageExchange);

        // Test Case 4
        reset(messageExchange);
        mockedMethods = new Method[] {method1, method4, method5};
        xlinkerListener = createMock(MockXlinkerBCListener.class, mockedMethods);
        xlinkerListener.setLogger(log);
        try {
            expect(
                xlinkerListener.getServiceAddressFromWSDLDocument(serviceDesc,
                    service)).andReturn(targetURI);
            expect(xlinkerListener.extractAttachments(null)).andReturn(
                attachments);
            expect(
                xlinkerListener.isInOutOperation(serviceDesc, operation,
                    service)).andReturn(true);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize mocks");
        }
        expect(messageExchange.getEndpoint()).andReturn(fooEndpoint);
        expect(messageExchange.getMessage("IN")).andReturn(null);
        replay(messageExchange);
        replay(xlinkerListener);

        // Run Test Case 4
        try {
            xlinkerListener.processOperationSuper(messageExchange, serviceDesc,
                operation, service);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during processOperation invocation");
        }
        assertTrue(xlinkerListener.sendFaultCalled);
        verify(xlinkerListener);
        verify(messageExchange);

    }

    public void testSendDone() {
        // Test Case 1
        MockXlinkerBCListener xlinkerListener = new MockXlinkerBCListener();
        Method[] mockedMethods = new Method[] {};
        MockDeliveryChannel channel = createMock(MockDeliveryChannel.class,
            mockedMethods);
        MockMessageExchange messageExchange = createMock(
            MockMessageExchange.class, mockedMethods);
        xlinkerListener.setChannel(channel);
        xlinkerListener.setMessageExchange(messageExchange);
        // Run Test Case 1
        try {
            xlinkerListener.sendDoneSuper(messageExchange);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during sendDone invocation");
        }
        assertTrue(channel.isSend());
    }

    public void testSendError() {
        // Test Case 1
        Method[] mockedMethods = new Method[] {};
        MockXlinkerBCListener xlinkerListener = new MockXlinkerBCListener();
        mockedMethods = new Method[] {};
        MockInOut messageExchange = createMock(MockInOut.class, mockedMethods);
        mockedMethods = new Method[] {};
        MockDeliveryChannel channel = createMock(MockDeliveryChannel.class,
            mockedMethods);
        String reason = "reason";
        Logger log = EasyMock.createMock(Logger.class);
        xlinkerListener.setMessageExchange(messageExchange);
        xlinkerListener.setChannel(channel);
        xlinkerListener.setLogger(log);
        // Run Test Case 1
        try {
            xlinkerListener.sendErrorSuper(messageExchange, reason, true);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during sendError invocation");
        }
        assertTrue(messageExchange.isSetStatus());
        assertTrue(messageExchange.isSetError());
        assertTrue(channel.isSend());

        // Test Case 2
        xlinkerListener = createMock(MockXlinkerBCListener.class, mockedMethods);
        MockInOptionalOut messageExchange2 = createMock(
            MockInOptionalOut.class, mockedMethods);
        channel = createMock(MockDeliveryChannel.class, mockedMethods);
        xlinkerListener.setChannel(channel);
        xlinkerListener.setLogger(log);
        // Run Test Case 2
        try {
            xlinkerListener.sendErrorSuper(messageExchange2, reason, true);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during sendError invocation");
        }
        assertTrue(messageExchange2.isSetStatus());
        assertTrue(messageExchange2.isSetError());
        assertTrue(channel.isSend());

        // Test Case 3
        xlinkerListener = createMock(MockXlinkerBCListener.class, mockedMethods);
        MockRobustInOnly messageExchange3 = createMock(MockRobustInOnly.class,
            mockedMethods);
        channel = createMock(MockDeliveryChannel.class, mockedMethods);
        xlinkerListener.setChannel(channel);
        xlinkerListener.setLogger(log);
        // Run Test Case 3
        try {
            xlinkerListener.sendErrorSuper(messageExchange3, reason, true);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during sendError invocation");
        }
        assertTrue(messageExchange3.isSetStatus());
        assertTrue(messageExchange3.isSetError());
        assertTrue(channel.isSend());

        // Test Case 4
        xlinkerListener = createMock(MockXlinkerBCListener.class, mockedMethods);
        MockInOnly messageExchange4 = createMock(MockInOnly.class,
            mockedMethods);
        channel = createMock(MockDeliveryChannel.class, mockedMethods);
        xlinkerListener.setChannel(channel);
        xlinkerListener.setLogger(log);
        // Run Test Case 4
        try {
            xlinkerListener.sendErrorSuper(messageExchange4, reason, true);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during sendError invocation");
        }
        assertTrue(xlinkerListener.sendDoneCalled);

        /*
         * Test Case 5
         */
        xlinkerListener = createMock(MockXlinkerBCListener.class, mockedMethods);
        messageExchange3 = createMock(MockRobustInOnly.class, mockedMethods);
        channel = createMock(MockDeliveryChannel.class, mockedMethods);
        xlinkerListener.setChannel(channel);
        xlinkerListener.setLogger(log);
        /*
         * Run Test Case 5
         */
        try {
            xlinkerListener.sendErrorSuper(messageExchange3, reason, false);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during sendError invocation");
        }
        assertTrue(messageExchange3.isSetStatus());
        assertTrue(messageExchange3.isSetError());
        assertTrue(channel.isSend());
    }

    public void testSendFault() {
        // Test Case 1
        Method method1 = null;
        Method method2 = null;
        try {
            method1 = XlinkerBCListener.class.getDeclaredMethod(
                "createSoapFault", new Class[] {Exception.class, String.class});
            method2 = MockInOut.class.getDeclaredMethod("createFault",
                new Class[] {});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could retrieve method for mocks");
        }
        Method[] mockedMethods = new Method[] {method1};
        MockXlinkerBCListener xlinkerListener = createMock(
            MockXlinkerBCListener.class, mockedMethods);
        mockedMethods = new Method[] {method2};
        MockInOut messageExchange = createMock(MockInOut.class, mockedMethods);
        MockDeliveryChannel channel = new MockDeliveryChannel();
        MockFault fault = new MockFault();
        String reason = "reason";
        String faultString = new String();

        Exception ex = new Exception();
        try {
            expect(xlinkerListener.createSoapFault(ex, reason)).andReturn(
                faultString);
            expect(messageExchange.createFault()).andReturn(fault);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize mocks");
        }
        replay(xlinkerListener);
        replay(messageExchange);
        xlinkerListener.setMessageExchange(messageExchange);
        xlinkerListener.setChannel(channel);
        // Run Test Case 1
        try {
            xlinkerListener.sendFaultSuper(messageExchange, reason, ex);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during sendFault invocation");
        }
        assertTrue(messageExchange.isSetStatus());
        assertTrue(messageExchange.isSetFault());
        assertTrue(channel.isSend());
        verify(xlinkerListener);
        verify(messageExchange);

        // Test Case 2
        mockedMethods = new Method[] {method1};
        xlinkerListener = createMock(MockXlinkerBCListener.class, mockedMethods);
        try {
            method2 = MockInOptionalOut.class.getDeclaredMethod("createFault",
                new Class[] {});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could retrieve method for mocks");
        }
        mockedMethods = new Method[] {method2};
        MockInOptionalOut messageExchange2 = createMock(
            MockInOptionalOut.class, mockedMethods);
        mockedMethods = new Method[] {};
        channel = new MockDeliveryChannel();
        try {
            expect(xlinkerListener.createSoapFault(ex, reason)).andReturn(
                faultString);
            expect(messageExchange2.createFault()).andReturn(fault);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize mocks");
        }
        replay(xlinkerListener);
        replay(messageExchange2);
        xlinkerListener.setMessageExchange(messageExchange2);
        xlinkerListener.setChannel(channel);
        // Run Test Case 2
        try {
            xlinkerListener.sendFaultSuper(messageExchange2, reason, ex);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during sendFault invocation");
        }
        assertTrue(messageExchange2.isSetStatus());
        assertTrue(messageExchange2.isSetFault());
        assertTrue(channel.isSend());
        verify(xlinkerListener);
        verify(messageExchange2);

        // Test Case 3
        mockedMethods = new Method[] {method1};
        xlinkerListener = createMock(MockXlinkerBCListener.class, mockedMethods);
        try {
            method2 = MockRobustInOnly.class.getDeclaredMethod("createFault",
                new Class[] {});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could retrieve method for mocks");
        }
        mockedMethods = new Method[] {method2};
        MockRobustInOnly messageExchange3 = createMock(MockRobustInOnly.class,
            mockedMethods);
        mockedMethods = new Method[] {};
        channel = createMock(MockDeliveryChannel.class, mockedMethods);
        try {
            expect(xlinkerListener.createSoapFault(ex, reason)).andReturn(
                faultString);
            expect(messageExchange3.createFault()).andReturn(fault);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize mocks");
        }
        replay(xlinkerListener);
        replay(messageExchange3);
        xlinkerListener.setMessageExchange(messageExchange3);
        xlinkerListener.setChannel(channel);

        // Run Test Case 3
        try {
            xlinkerListener.sendFaultSuper(messageExchange3, reason, ex);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during sendFault invocation");
        }
        assertTrue(messageExchange3.isSetStatus());
        assertTrue(messageExchange3.isSetFault());
        assertTrue(channel.isSend());
        verify(xlinkerListener);
        verify(messageExchange3);

        // Test Case 4
        mockedMethods = new Method[] {method1};
        xlinkerListener = createMock(MockXlinkerBCListener.class, mockedMethods);
        MessageExchange messageExchange4 = createMock(MockInOnly.class);

        // Run Test Case 4
        try {
            xlinkerListener.sendFaultSuper(messageExchange4, reason, ex);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during sendFault invocation");
        }
        assertTrue(xlinkerListener.sendDoneCalled);
    }

    public void testStop() {
        /*
         * Create Test parameters
         */
        StandaloneWSServer rcvr = createMock(StandaloneWSServer.class);
        MockXlinkerBCListener xlinkerListener = new MockXlinkerBCListener();
        xlinkerListener.setReceiver(rcvr);

        // Run Test Case
        xlinkerListener.stop();
        assertEquals(xlinkerListener.running, false);
    }

    public void testXlinkerBCListener() {
        // Test Case 1

        // Create Mocks
        PetalsServiceUnitManager sum = createMock(PetalsServiceUnitManager.class);
        DeliveryChannel channel = createMock(DeliveryChannel.class);
        ComponentContext context = createMock(ComponentContext.class);

        // Create Test parameters
        Logger log = EasyMock.createMock(Logger.class);

        // Run Test Case 1
        XlinkerBCListener xlinkerbcListener = new XlinkerBCListener(channel,
            context, sum, log);
        assertEquals(log, xlinkerbcListener.logger);
        assertEquals(context, xlinkerbcListener.context);
        assertEquals(channel, xlinkerbcListener.channel);
        assertEquals(sum, xlinkerbcListener.suManager);

        // Test Case 2
        xlinkerbcListener = null;
        // Run Test Case 2
        xlinkerbcListener = new XlinkerBCListener();
        assertNotNull(xlinkerbcListener);
    }

    @Override
    protected void setUp() throws Exception {

    }

}
