/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: PetalsDispatcherTest.java 154 18 mai 2006 wjoseph $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.xlinker;

import java.lang.reflect.Method;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.component.ServiceUnitManager;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.ObjectName;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.petals.binding.xlinker.XlinkerBC;
import org.objectweb.petals.binding.xlinker.XlinkerBCListener;
import org.objectweb.petals.binding.xlinker.handler.XlinkerServiceUnitHandler;
import org.objectweb.petals.binding.xlinker.mock.MockComponentContext;
import org.objectweb.petals.binding.xlinker.mock.MockDeliveryChannel;
import org.objectweb.petals.binding.xlinker.mock.MockMessageExchange;
import org.objectweb.petals.binding.xlinker.mock.MockServiceEndpoint;
import org.objectweb.petals.binding.xlinker.mock.MockXlinkerBC;
import org.objectweb.petals.binding.xlinker.servicemanager.XlinkerServiceUnitManager;
import org.objectweb.petals.component.common.util.ComponentLogger;
import org.w3c.dom.Document;

import static org.easymock.EasyMock.*;
import static org.easymock.classextension.EasyMock.*;

/**
 * 
 * @author wjoseph - EBM Websourcing
 * 
 */
public class XlinkerBCTest extends TestCase {

    public void testCreateServiceUnitManager() {
        /*
         * Test Case 1
         */
        MockXlinkerBC xlinkerbc = new MockXlinkerBC();
        MockComponentContext ctx = new MockComponentContext();
        Logger log = EasyMock.createMock(Logger.class);
        xlinkerbc.setLogger(log);
        xlinkerbc.setComponentContext(ctx);
        Object result = null;
        /*
         * Run Test Case 1
         */
        try {
            result = xlinkerbc.createServiceUnitManager();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create new service unit manager");
        }
        assertNotNull("The created ServiceUnitManager is null", result);
    }

    public void testCreateXlinkerHandler() {
        /*
         * Test Case 1
         */
        MockComponentContext ctx = createMock(MockComponentContext.class);
        MockXlinkerBC xlinkerbc = new MockXlinkerBC();
        /*
         * Create test parameters
         */
        Logger log = EasyMock.createMock(Logger.class);
        MockDeliveryChannel channel = new MockDeliveryChannel();
        try {
            expect(ctx.getDeliveryChannel()).andReturn(channel).anyTimes();
            expect(ctx.getInstallRoot()).andReturn("").anyTimes();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(ctx);
        xlinkerbc.setLogger(log);
        xlinkerbc.setComponentContext(ctx);
        XlinkerServiceUnitHandler result = null;
        /*
         * Run Test Case 1
         */
        try {
            result = xlinkerbc.createXlinkerHandler();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        verify(ctx);
        assertEquals("Component and handler loggers are different", result
            .getLogger(), log);
        assertEquals("Component and handler loggers are different", result
            .getContext(), ctx);
    }

    public void testGetExtensionMBeanName() {
        // Run Test Case
        XlinkerBC xlinkerbc = new XlinkerBC();
        ObjectName objectName = xlinkerbc.getExtensionMBeanName();
        assertNull(objectName);
    }

    public void testGetListener() {
        /*
         * Test Case 1
         */
        MockXlinkerBC xlinkerbc = new MockXlinkerBC();
        XlinkerBCListener listener = new XlinkerBCListener();
        xlinkerbc.setListener(listener);
        /*
         * Run Test Case 1
         */
        Object result = xlinkerbc.getListener();
        assertEquals("Both objects should be equal", result, listener);
    }

    public void testGetServiceDescription() {
        // Create mock object and set suManager in a new method

        // Create Mock Objects
        XlinkerServiceUnitManager suManager = createMock(XlinkerServiceUnitManager.class);
        MockXlinkerBC xlinkerbc = new MockXlinkerBC();

        // Create test parameters
        Document document = null;
        ServiceEndpoint fooService = new MockServiceEndpoint(new QName(
            "fooService"), "fooServiceEndpoint");
        ServiceEndpoint barService = new MockServiceEndpoint(new QName(
            "barService"), "barServiceEndpoint");
        try {
            document = DocumentBuilderFactory.newInstance()
                .newDocumentBuilder().newDocument();
            expect(suManager.getServiceDescription(fooService)).andReturn(
                document).anyTimes();
            expect(suManager.getServiceDescription(barService)).andReturn(null)
                .anyTimes();
            expect(suManager.getServiceDescription(null)).andReturn(null)
                .anyTimes();
        } catch (ParserConfigurationException e2) {
            e2.printStackTrace();
        }
        replay(suManager);

        xlinkerbc.setServiceUnitManager(suManager);
        // Run the test case 1
        // Asking service description for foo service
        Document doc = null;
        doc = xlinkerbc.getServiceDescription(fooService);
        assertEquals(document, doc);

        // Run the test case 2
        // Asking service description for bar service
        doc = null;
        doc = xlinkerbc.getServiceDescription(barService);
        assertNull("This service description should be null", doc);

        // Run the test case 3
        // Asking service description for null service
        doc = null;
        doc = xlinkerbc.getServiceDescription(null);
        assertNull("This service description should be null", doc);

        verify(suManager);
    }

    public void testGetServiceUnitManager() {
        // Create Mock Object
        XlinkerServiceUnitManager suManager = createMock(XlinkerServiceUnitManager.class);

        // Create test Parameters
        MockXlinkerBC xlinkerbc = new MockXlinkerBC();
        Logger log = EasyMock.createMock(Logger.class);
        xlinkerbc.setServiceUnitManager(suManager);
        xlinkerbc.setLogger(log);
        ServiceUnitManager result = null;

        // Run Test Case
        result = xlinkerbc.getServiceUnitManager();
        assertEquals(suManager, result);
    }

    public void testInit() {

        // Create Mock Objects
        Method method1 = null;
        Method method2 = null;
        Method method3 = null;
        ComponentContext componentContext = null;
        try {
            method1 = XlinkerBC.class.getDeclaredMethod(
                "createServiceUnitManager", new Class[] {});
            method2 = MockComponentContext.class.getDeclaredMethod("getLogger",
                new Class[] {String.class, String.class});
            method3 = XlinkerBC.class.getDeclaredMethod(
                    "createComponentLogger", new Class[] {Logger.class,String.class});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could retrieve mocked methods");
        }
        Method[] mockedMethods = new Method[] {method1, method3};
        XlinkerBC xlinkerbc = createMock(XlinkerBC.class, mockedMethods);
        mockedMethods = new Method[] {method2};
        componentContext = createMock(MockComponentContext.class, mockedMethods);
        XlinkerServiceUnitManager suManager = createMock(XlinkerServiceUnitManager.class);

        // Create test parameters
        Logger logger = EasyMock.createMock(Logger.class);
        ComponentLogger componentLogger = createMock(ComponentLogger.class);
        // Init mocks
        try {
            expect(componentContext.getLogger("", null)).andReturn(logger);
            expect(xlinkerbc.createServiceUnitManager()).andReturn(suManager);
            expect(xlinkerbc.createComponentLogger(logger, null)).andReturn(componentLogger);
        } catch (JBIException e) {
            e.printStackTrace();
        }
        replay(componentContext);
        replay(xlinkerbc);

        // Run the Test
        try {
            xlinkerbc.init(componentContext);
        } catch (JBIException e) {
            e.printStackTrace();
            fail("Error in XlinkerBC initialisation");
        }
        verify(componentContext);
        verify(xlinkerbc);
    }

    public void testIsExchangeWithConsumerOkay() {
        // Create Mock Objects
        MessageExchange msgExchange = createMock(MockMessageExchange.class);

        XlinkerBC xlinkerbc = new XlinkerBC();

        // Create test parameters
        QName serviceName = new QName("fooService");
        ServiceEndpoint se = new MockServiceEndpoint(serviceName,
            "fooServiceEndpoint");

        // Run the Test
        boolean result = xlinkerbc.isExchangeWithConsumerOkay(se, msgExchange);
        assertEquals(result, true);
    }

    public void testIsExchangeWithProviderOkay() {
        // Create Mock Objects
        MessageExchange msgExchange = createMock(MockMessageExchange.class);

        XlinkerBC xlinkerbc = new XlinkerBC();

        // Create test parameters
        QName serviceName = new QName("fooService");
        ServiceEndpoint se = new MockServiceEndpoint(serviceName,
            "fooServiceEndpoint");

        // Run the Test
        boolean result = xlinkerbc.isExchangeWithProviderOkay(se, msgExchange);
        assertEquals(result, true);
    }

    public void testStart() {
        String baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
        /*
         * Test Case 1
         */
        MockComponentContext ctx = createMock(MockComponentContext.class);
        MockXlinkerBC xlinkerbc = new MockXlinkerBC();
        /*
         * Create test parameters
         */
        Logger log = EasyMock.createMock(Logger.class);
        MockDeliveryChannel channel = new MockDeliveryChannel();
        XlinkerServiceUnitManager sum = null;
        try {
            sum = new XlinkerServiceUnitManager(ctx, log);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not instantiate XlinkerServiceUnitManager");
        }
        try {
            expect(ctx.getDeliveryChannel()).andReturn(channel).anyTimes();
            expect(ctx.getInstallRoot()).andReturn(baseDir).anyTimes();
            expect(ctx.getComponentName()).andReturn("toto").anyTimes();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(ctx);
        xlinkerbc.setLogger(log);
        xlinkerbc.setComponentContext(ctx);
        xlinkerbc.setServiceUnitManager(sum);
        /*
         * Run Test Case 1
         */
        try {
            xlinkerbc.start();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        verify(ctx);
        assertTrue("Listener thread not running", xlinkerbc.listenerThread
            .isAlive());
    }

    public void testStop() {
        XlinkerBC xlinkerbc = new XlinkerBC();
        Logger log = createNiceMock(Logger.class);
        
        xlinkerbc.logger = log;
        // Run the Test
        try {
            xlinkerbc.stop();
        } catch (JBIException e) {
            e.printStackTrace();
            fail("Error in XlinkerBC stop method");
        }
    }

    @Override
    protected void setUp() throws Exception {

    }

}
