/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: PetalsDispatcherTest.java 154 18 mai 2006 wjoseph $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.xlinker;

import java.io.File;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.InstallationContext;

import org.objectweb.petals.binding.xlinker.XlinkerBC;
import org.objectweb.petals.binding.xlinker.XlinkerBCBootstrap;
import org.objectweb.petals.binding.xlinker.mock.MockInstallationContext;

import junit.framework.TestCase;
import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.verify;

/**
 * 
 * @author wjoseph - EBM Websourcing
 * 
 */
public class XlinkerBootstrapImplTest extends TestCase {

    File testDataDir = null;

    public void testCleanUp() {

        XlinkerBCBootstrap xlinkerbootstrap = new XlinkerBCBootstrap();

        // Run the Test
        try {
            xlinkerbootstrap.cleanUp();
        } catch (JBIException e) {
            e.printStackTrace();
            fail("Error in XlinkerBCBootstrap clean up");
        }
    }

    public void testGetExtensionMBeanName() {
        // Create objects
        XlinkerBCBootstrap xlinkerbootstrap = new XlinkerBCBootstrap();

        // Run the Test
        Object result = null;
        result = xlinkerbootstrap.getExtensionMBeanName();
        assertNull("Result of invocation should be null", result);
    }

    public void testInit() {
        // Create Mock Objects
        InstallationContext installationContext = createMock(MockInstallationContext.class);
        // Init mocks
        expect(installationContext.getInstallRoot()).andReturn(
                testDataDir.getAbsolutePath());
        replay(installationContext);

        XlinkerBCBootstrap xlinkerbootstrap = new XlinkerBCBootstrap();
        // Run the Test
        try {
            xlinkerbootstrap.init(installationContext);
        } catch (JBIException e) {
            e.printStackTrace();
            fail("Error in XlinkerBCBootstrap init method");
        }

        assertEquals(xlinkerbootstrap.context, installationContext);
    }

    public void testOnInstall() {
        // Create Mock Objects
        InstallationContext installationContext = createMock(MockInstallationContext.class);
        // Init mocks
        expect(installationContext.getInstallRoot()).andReturn(
                testDataDir.getAbsolutePath() + File.separator + "onInstall");
        replay(installationContext);

        // Create test parameters

        XlinkerBCBootstrap xlinkerbootstrap = new XlinkerBCBootstrap();
        // Run the Test
        try {
            xlinkerbootstrap.init(installationContext);
        } catch (JBIException e) {
            e.printStackTrace();
            fail("Error in XlinkerBCBootstrap init method");
        }

        // Run the Test
        try {
            xlinkerbootstrap.onInstall();
        } catch (JBIException e) {
            e.printStackTrace();
            fail("Error in XlinkerBCBootstrap onInstall method");
        }
        verify(installationContext);
        File[] installDirfiles = new File(testDataDir,"onInstall").listFiles();
        boolean deployDirCreated = false;
        boolean workingDirCreated = false;
        for (File file : installDirfiles) {
            if (file.getName().equals("deploy")) {
                deployDirCreated = true;
            }
            if (file.getName().equals("working")) {
                workingDirCreated = true;
            }
        }
        assertTrue(workingDirCreated);
        assertTrue(deployDirCreated);
    }

    public void testOnUninstall() {
        File metaInfDir = new File(testDataDir.getAbsolutePath()
                + File.separator + "META-INF");
        metaInfDir.mkdir();
        // Create Mock Objects
        InstallationContext installationContext = createMock(MockInstallationContext.class);
        // Init mocks
        expect(installationContext.getInstallRoot()).andReturn(
                testDataDir.getAbsolutePath());
        replay(installationContext);

        // Create Test parameters
        File deployDir = new File(testDataDir.getAbsolutePath()
                + File.separator + "deploy");
        File workingDir = new File(testDataDir.getAbsolutePath()
                + File.separator + "working");
        deployDir.mkdir();
        workingDir.mkdir();
        XlinkerBCBootstrap xlinkerbootstrap = new XlinkerBCBootstrap();
        // Run the Test
        try {
            xlinkerbootstrap.init(installationContext);
            // xlinkerbootstrap.onInstall();
        } catch (JBIException e) {
            e.printStackTrace();
            fail("Error in XlinkerBCBootstrap init or onUninstall method");
        }

        // Run the Test
        try {
            xlinkerbootstrap.onUninstall();
        } catch (JBIException e) {
            e.printStackTrace();
            fail("Error in XlinkerBCBootstrap onuninstall method");
        }
        verify(installationContext);
        assertFalse(deployDir.exists());
        assertFalse(workingDir.exists());

        // Clean up
        for (File file : metaInfDir.listFiles()) {
            file.delete();
        }
        deployDir.delete();
        workingDir.delete();
        metaInfDir.delete();
    }

    public void testStop() {
        XlinkerBC xlinkerbc = new XlinkerBC();
        xlinkerbc.logger = Logger.getAnonymousLogger();
        // Run the Test
        try {
            xlinkerbc.stop();
        } catch (JBIException e) {
            e.printStackTrace();
            fail("Error in XlinkerBCBootstrap stop method");
        }
    }

    @Override
    protected void setUp() throws Exception {
        String baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
        
        testDataDir = new File(baseDir+"src"+File.separator+"test-data");
    }

}
