/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: XlinkerServiceUnitHandlerTest.java 154 18 mai 2006 wjoseph $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.xlinker.handler;

import java.io.File;
import java.lang.reflect.Method;
import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;
import javax.jbi.management.DeploymentException;
import javax.xml.namespace.QName;

import junit.framework.TestCase;

import org.act.soap.server.StandaloneWSServer;
import org.act.soap.server.deploy.DeployException;
import org.easymock.classextension.EasyMock;
import org.objectweb.petals.binding.xlinker.XlinkerBC;
import org.objectweb.petals.binding.xlinker.XlinkerBCListener;
import org.objectweb.petals.binding.xlinker.handler.XlinkerServiceUnitHandler;
import org.objectweb.petals.binding.xlinker.mock.MockAutoDeployer;
import org.objectweb.petals.binding.xlinker.mock.MockComponentContext;
import org.objectweb.petals.binding.xlinker.mock.MockXlinkerServiceUnitHandler;
import org.objectweb.petals.binding.xlinker.servicemanager.XlinkerServiceUnitManager;
import org.objectweb.petals.component.common.serviceunitmanager.manager.PetalsServiceUnitManager;

import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.verify;

/**
 * Tests for XlinkerServiceUnitHandler
 * 
 * @author wjoseph - EBM Websourcing
 * 
 */
public class XlinkerServiceUnitHandlerTest extends TestCase {

    private File testDataDir;

    @Override
    protected void setUp() throws Exception {
        String baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);

        testDataDir = new File(baseDir
            + "src" + File.separator + "test-data");
    }

    public void testXlinkerServiceUnitHandler() {
        /*
         * Test Case 1
         */
        XlinkerServiceUnitHandler sum = new XlinkerServiceUnitHandler();
        assertEquals(sum.getSERVICE_UNIT_TYPE(), "xlinker_service");

        /*
         * Test Case 2
         */

        /*
         * Create Test parameters
         */
        ComponentContext ctx = EasyMock.createMock(ComponentContext.class);
        Logger log = EasyMock.createMock(Logger.class);

        /*
         * Run Test Case 2
         */
        sum = new XlinkerServiceUnitHandler(ctx, log);
        assertEquals(ctx, sum.getContext());
        assertEquals(log, sum.getLogger());
        assertEquals(sum.getSERVICE_UNIT_TYPE(), "xlinker_service");
    }

    public void testDeployWsar() {
        /*
         * Test Case 1 The invokation goes without errors
         */
        XlinkerServiceUnitHandler sum = new XlinkerServiceUnitHandler();
        /*
         * Create mocks
         */
        Method method1 = null;
        Method method2 = null;
        Method method3 = null;
        try {
            method1 = XlinkerBC.class.getDeclaredMethod("getListener",
                new Class[] {});
            method2 = XlinkerBCListener.class.getDeclaredMethod("getReceiver",
                new Class[] {});
            method3 = StandaloneWSServer.class.getDeclaredMethod(
                "getAutoDeployer", new Class[] {});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error getting mocked methods ");
        }
        Method[] mockedMethods = new Method[] {method1};
        XlinkerBC xlink = createMock(XlinkerBC.class, mockedMethods);
        mockedMethods = new Method[] {method2};
        XlinkerBCListener listener = createMock(XlinkerBCListener.class,
            mockedMethods);
        mockedMethods = new Method[] {method3};
        StandaloneWSServer server = createMock(StandaloneWSServer.class,
            mockedMethods);
        MockAutoDeployer autoDeployer = new MockAutoDeployer();

        /*
         * Init mocks
         */
        expect(xlink.getListener()).andReturn(listener);
        expect(listener.getReceiver()).andReturn(server);
        expect(server.getAutoDeployer()).andReturn(autoDeployer);

        replay(xlink);
        replay(listener);
        replay(server);
        /*
         * Create Test parameters
         */
        File wsarFile = new File("");
        QName serviceName = new QName("fooService");

        /*
         * Run Test Case 1
         */
        try {
            sum.deployWsar(xlink, wsarFile, serviceName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        verify(xlink);
        verify(listener);
        verify(server);
        assertTrue(autoDeployer.isDeploy());

        /*
         * Test Case 2 An exception is raised during invokation
         */
        sum = new XlinkerServiceUnitHandler();
        /*
         * Create mocks
         */
        mockedMethods = new Method[] {method1};
        xlink = createMock(XlinkerBC.class, mockedMethods);
        mockedMethods = new Method[] {method2};
        listener = createMock(XlinkerBCListener.class, mockedMethods);
        mockedMethods = new Method[] {method3};
        server = createMock(StandaloneWSServer.class, mockedMethods);
        autoDeployer = new MockAutoDeployer() {
            public void deploy(String arg0, String arg1) throws DeployException {
                throw new DeployException("Testing exception");
            }
        };

        /*
         * Init mocks
         */
        expect(xlink.getListener()).andReturn(listener);
        expect(listener.getReceiver()).andReturn(server);
        expect(server.getAutoDeployer()).andReturn(autoDeployer);

        replay(xlink);
        replay(listener);
        replay(server);
        /*
         * Run Test Case 2
         */
        try {
            sum.deployWsar(xlink, wsarFile, serviceName);
            fail("No exception was raised");
        } catch (Exception e) {
            verify(xlink);
            verify(listener);
            verify(server);
        }
    }

    public void testUnDeployWsar() {
        /*
         * Test Case 1 The invokation goes without errors
         */
        XlinkerServiceUnitHandler sum = new XlinkerServiceUnitHandler();
        /*
         * Create mocks
         */
        Method method1 = null;
        Method method2 = null;
        Method method3 = null;
        try {
            method1 = XlinkerBC.class.getDeclaredMethod("getListener",
                new Class[] {});
            method2 = XlinkerBCListener.class.getDeclaredMethod("getReceiver",
                new Class[] {});
            method3 = StandaloneWSServer.class.getDeclaredMethod(
                "getAutoDeployer", new Class[] {});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error getting mocked methods ");
        }
        Method[] mockedMethods = new Method[] {method1};
        XlinkerBC xlink = createMock(XlinkerBC.class, mockedMethods);
        mockedMethods = new Method[] {method2};
        XlinkerBCListener listener = createMock(XlinkerBCListener.class,
            mockedMethods);
        mockedMethods = new Method[] {method3};
        StandaloneWSServer server = createMock(StandaloneWSServer.class,
            mockedMethods);
        MockAutoDeployer autoDeployer = new MockAutoDeployer();

        /*
         * Init mocks
         */
        expect(xlink.getListener()).andReturn(listener);
        expect(listener.getReceiver()).andReturn(server);
        expect(server.getAutoDeployer()).andReturn(autoDeployer);

        replay(xlink);
        replay(listener);
        replay(server);
        /*
         * Create Test parameters
         */
        QName serviceName = new QName("fooService");

        /*
         * Run Test Case 1
         */
        try {
            sum.undeployWsar(xlink, serviceName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        verify(xlink);
        verify(listener);
        verify(server);
        assertTrue(autoDeployer.isUndeploy());

        /*
         * Test Case 2 An exception is raised in the undeploy method
         */
        mockedMethods = new Method[] {method1};
        xlink = createMock(XlinkerBC.class, mockedMethods);
        mockedMethods = new Method[] {method2};
        listener = createMock(XlinkerBCListener.class, mockedMethods);
        mockedMethods = new Method[] {method3};
        server = createMock(StandaloneWSServer.class, mockedMethods);
        autoDeployer = new MockAutoDeployer() {
            public void undeploy(String arg0) throws DeployException {
                throw new DeployException("Testing exception7");
            }
        };

        /*
         * Init mocks
         */
        expect(xlink.getListener()).andReturn(listener);
        expect(listener.getReceiver()).andReturn(server);
        expect(server.getAutoDeployer()).andReturn(autoDeployer);

        replay(xlink);
        replay(listener);
        replay(server);

        /*
         * Run Test Case 2
         */
        try {
            sum.undeployWsar(xlink, serviceName);
            fail("No exception was raised");
        } catch (Exception e) {
            verify(xlink);
            verify(listener);
            verify(server);
        }
    }

    public void testGetWsarFile() {
        /*
         * Test Case 1 There is no .wsar file , the result should be null
         */

        /*
         * Create Test parameters
         */
        MockXlinkerServiceUnitHandler handler = new MockXlinkerServiceUnitHandler();
        File result = null;
        /*
         * Run Test Case 1
         */
        result = handler.getWsarFileSuper(testDataDir.getAbsolutePath() + File.separator + "getService2");
        assertNull("The result of invokation should be null", result);

        /*
         * Test Case 2 There is a .wsar file in the directory, the result should
         * be this file
         */

        /*
         * Create Test parameters
         */
        File wsarFile = new File(testDataDir.getAbsolutePath(), "getService" + File.separator +"foo.wsar");
        result = null;
       
        /*
         * Run Test Case 2
         */
        result = handler.getWsarFileSuper(testDataDir.getAbsolutePath() + File.separator + "getService");
        assertEquals("Returned File different from the one we created", result
            .getAbsolutePath(), wsarFile.getAbsolutePath());

    }

    public void testGetServiceName() {
        /*
         * Test Case 1 The jbi.xml file is present
         */
        /*
         * Create Test parameters
         */
        MockXlinkerServiceUnitHandler handler = new MockXlinkerServiceUnitHandler();
        QName serviceName = new QName("namespace", "fooService");
        QName result = null;
        /*
         * Run Test Case 1
         */
        try {
            result = handler.getServiceNameSuper(testDataDir.getAbsolutePath() + File.separator + "getService");
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("Wrong QName returned", serviceName, result);
        /*
         * Clean up
         */

        /*
         * Test Case 2 No jbi.xml, an exeption will be thrown
         */
        result = null;
        /*
         * Run Test Case 2
         */
        try {
            result = handler.getServiceNameSuper(testDataDir.getAbsolutePath() + File.separator + "getService2");
            fail("No exception was thrown");
        } catch (Exception e) {
        }
        assertNull("No QName should be returned", result);
    }

    public void testGetJbiDescriptor() {
        /*
         * Test Case 1 The jbi.xml file is present and should be returned
         */
        /*
         * Create Test parameters
         */
        MockXlinkerServiceUnitHandler handler = new MockXlinkerServiceUnitHandler();
        File jbiFile = new File(testDataDir.getAbsolutePath(), "getService" + File.separator + "META-INF"
            + File.separator + "jbi.xml");
        File result = null;
        /*
         * Run Test Case 1
         */
        result = handler.getJbiDescriptor(testDataDir.getAbsolutePath() + File.separator + "getService");
        assertEquals("Incorrect file found", result.getAbsolutePath(), jbiFile
            .getAbsolutePath());


        /*
         * Test Case 2 The jbi.xml is not present, null should be returned
         */
        /*
         * Run Test Case 2
         */
        result = handler.getJbiDescriptor(testDataDir.getAbsolutePath());
        assertNull("Wrong file found", result);

    }

    public void testValidateServicePackage() {
        /*
         * Test Case 1 The package is valid, the result should be true
         */

        /*
         * Create Test parameters
         */
        MockXlinkerServiceUnitHandler handler = new MockXlinkerServiceUnitHandler();
        boolean result = false;

        /*
         * Run Test Case 1
         */
        result = handler.validateServicePackage(testDataDir.getAbsolutePath()+ File.separator + "getService");

        assertTrue("The result of invokation should be true", result);

        /*
         * Clean up
         */

        /*
         * Test Case 2 The package is missing some files, the result should be
         * false
         */

        /*
         * Create Test parameters
         */
        handler = new MockXlinkerServiceUnitHandler();
        result = false;
        /*
         * Run Test Case 2
         */
        result = handler.validateServicePackage(testDataDir.getAbsolutePath() + File.separator + "getService2");
        assertFalse("The result of invokation should be false", result);

        /*
         * Test Case 3 The package contains extra files, the result should be
         * false
         */

        /*
         * Create Test parameters
         */
        handler = new MockXlinkerServiceUnitHandler();
        result = false;
        /*
         * Run Test Case 3
         */
        result = handler.validateServicePackage(testDataDir.getAbsolutePath() + File.separator + "getService4");
        assertFalse("The result of invokation should be false", result);

    }

    public void testDeploy() {
        /*
         * Test Case 1
         */
        /*
         * Create Test parameters
         */
        Logger log = EasyMock.createMock(Logger.class);
        MockComponentContext ctx = new MockComponentContext() {
            public String getComponentName() {
                return "FooComponent";
            }
        };
        MockXlinkerServiceUnitHandler handler = new MockXlinkerServiceUnitHandler(
            ctx, log);
        String result = null;
        /*
         * Run Test Case 1
         */
        try {
            result = handler.deploy("fooSU", "xlinker_service",testDataDir.getAbsolutePath()+ File.separator + "getService");
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertNotNull("The result should not be null", result);

        /*
         * Test Case 2
         */
        /*
         * Create Test parameters
         */
        log = EasyMock.createMock(Logger.class);
        ctx = new MockComponentContext() {
            public String getComponentName() {
                return "FooComponent";
            }
        };
        handler = new MockXlinkerServiceUnitHandler(ctx, log);
        result = null;

        /*
         * Run Test Case 2
         */
        try {
            result = handler.deploy("fooSU","xlinker_service",  testDataDir.getAbsolutePath()+ File.separator + "getService3");
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertNull("The result should be null", result);
    }

    public void testStart() {
        /*
         * Test Case 1 Everything goes well
         */
        MockComponentContext ctx = new MockComponentContext();
        Logger log = EasyMock.createMock(Logger.class);
        MockXlinkerServiceUnitHandler handler = new MockXlinkerServiceUnitHandler(
            ctx, log);
        XlinkerServiceUnitManager sum = createMock(XlinkerServiceUnitManager.class);
        String suName = "serviceUnitName";
        /*
         * Initialize Mocks
         */
        expect(sum.getXlinkerBC()).andReturn(null);
        replay(sum);
        /*
         * Run Test Case 1
         */
        try {
            handler.start(suName, (PetalsServiceUnitManager) sum);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertTrue(handler.isDeployWsar());
        assertTrue(handler.isGetServiceName());
        assertTrue(handler.isGetWsarFile());
        verify(sum);
        /*
         * Test Case 2 An exception is raised during deploy process
         */
        handler = new MockXlinkerServiceUnitHandler(ctx, log) {
            @Override
            protected QName getServiceName(String serviceUnitRootPath)
                throws DeploymentException {
                throw new DeploymentException(
                    "getServiceName testing exception");
            }
        };
        sum = createMock(XlinkerServiceUnitManager.class);
        /*
         * Initialize Mocks
         */
        expect(sum.getXlinkerBC()).andReturn(null);
        replay(sum);
        /*
         * Run Test Case 2
         */
        try {
            handler.start(suName, (PetalsServiceUnitManager) sum);
            fail("No exception was thrown");
        } catch (Exception e) {
            assertTrue(handler.isGetWsarFile());
            assertTrue(handler.isUndeployWsar());
            assertEquals("Exception is not the one expected",
                XlinkerServiceUnitHandler.FAILED_DEPLOY_SERVICE, e.getMessage());
            verify(sum);
        }

        /*
         * Test Case 3 An exception is raised during undeploy process
         */
        handler = new MockXlinkerServiceUnitHandler(ctx, log) {
            @Override
            protected QName getServiceName(String serviceUnitRootPath)
                throws DeploymentException {
                throw new DeploymentException(
                    "getServiceName testing exception");
            }

            @Override
            protected void undeployWsar(XlinkerBC xlink, QName serviceName)
                throws Exception {
                throw new DeploymentException("undeployWsar testing exception");
            }
        };
        sum = createMock(XlinkerServiceUnitManager.class);
        /*
         * Initialize Mocks
         */
        expect(sum.getXlinkerBC()).andReturn(null);
        replay(sum);
        /*
         * Run Test Case 3
         */
        try {
            handler.start(suName, (PetalsServiceUnitManager) sum);
            fail("No exception was thrown");
        } catch (Exception e) {
            assertTrue(handler.isGetWsarFile());
            assertEquals("Exception is not the one expected",
                XlinkerServiceUnitHandler.FAILED_DEPLOY_SERVICE, e.getMessage());
            verify(sum);
        }
    }
/*
    private void copy(File src, File dst) throws IOException {
        InputStream in = new FileInputStream(src);
        OutputStream out = new FileOutputStream(dst);

        // Transfer bytes from in to out
        byte[] buf = new byte[1024];
        int len;
        while ((len = in.read(buf)) > 0) {
            out.write(buf, 0, len);
        }
        in.close();
        out.close();
    }*/
}
