/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: XlinkerServiceUnitHandlerTest.java 154 18 mai 2006 wjoseph $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.xlinker.provider;

import java.lang.reflect.Method;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.activation.DataHandler;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.wsdl.BindingOperation;
import javax.wsdl.Definition;
import javax.xml.namespace.QName;
import javax.xml.soap.AttachmentPart;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPMessage;
import javax.xml.transform.Source;

import junit.framework.TestCase;

import org.act.soap.server.MessageContext;
import org.act.soap.server.provider.ProviderException;
import org.act.soap.util.SOAPUtils;
import org.apache.commons.collections.ArrayIterator;
import org.objectweb.petals.binding.xlinker.mock.MockDeliveryChannel;
import org.objectweb.petals.binding.xlinker.mock.MockInOnly;
import org.objectweb.petals.binding.xlinker.mock.MockInOut;
import org.objectweb.petals.binding.xlinker.mock.MockNormalizedMessage;
import org.objectweb.petals.binding.xlinker.mock.MockPetalsProvider;
import org.objectweb.petals.binding.xlinker.mock.MockServiceEndpoint;
import org.objectweb.petals.binding.xlinker.provider.PetalsProvider;
import org.objectweb.petals.component.common.util.SourceHelper;

import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.reset;
import static org.easymock.classextension.EasyMock.verify;

import com.ibm.wsdl.BindingOperationImpl;

/**
 * Tests for XlinkerServiceUnitHandler
 * 
 * @author wjoseph - EBM Websourcing
 * 
 */
public class PetalsProviderTest extends TestCase {

    public void testCreateMessageExchange() {
        /*
         * Test Case 1 We create an InOnly message exchange
         */

        /*
         * Create Mocks
         */
        Method method1 = null;
        try {
            method1 = PetalsProvider.class.getDeclaredMethod(
                    "getDeliveryChannel", new Class[] {});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked method");
        }
        Method[] mockedMethods = new Method[] {method1};
        PetalsProvider provider = createMock(PetalsProvider.class,
                mockedMethods);
        /*
         * Init test parameters
         */
        DeliveryChannel channel = new MockDeliveryChannel();

        QName opName = new QName("testOp");
        QName serviceName = new QName("testService");
        Source content = SourceHelper.createContentSource("testMessage");
        /*
         * Init mocks
         */
        try {
            expect(provider.getDeliveryChannel()).andReturn(channel).anyTimes();
        } catch (Exception e1) {
            e1.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(provider);
        /*
         * Run Test Case 1
         */
        MessageExchange messageExchange = null;
        try {
            messageExchange = provider.createInOnlyMessageExchange(opName,
                    serviceName, content);
        } catch (ProviderException e) {
            e.printStackTrace();
            fail("Error creating a new MessageExchange");
        }
        assertTrue("Message exchange must be an instance of InOnly",
                messageExchange instanceof InOnly);
        assertEquals("Wrong operation name", opName, messageExchange
                .getOperation());
        assertEquals("Wrong service name", serviceName, messageExchange
                .getService());
        assertEquals("Wrong content", content, ((InOnly) messageExchange)
                .getInMessage().getContent());

        /*
         * Test Exception throwing for null delivery channel
         */
        reset(provider);
        try {
            expect(provider.getDeliveryChannel()).andReturn(null).anyTimes();
        } catch (ProviderException e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(provider);

        /*
         * Run Test Case 2
         */
        try {
            messageExchange = provider.createInOnlyMessageExchange(opName,
                    serviceName, content);
            fail("Must throw an exception if delivery channel is null");
        } catch (ProviderException e) {
            // Do nothing its ok
        }
        
        /*
         * Test Case 3
         */
        reset(provider);
        /*
         * Init mocks
         */
        try {
            expect(provider.getDeliveryChannel()).andReturn(
                    channel).anyTimes();
        } catch (ProviderException e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(provider);

        /*
         * Run the test
         */
        messageExchange = null;
        try {
            messageExchange = provider.createInOutMessageExchange(
                    opName, serviceName, content);
        } catch (ProviderException e) {
            e.printStackTrace();
            fail("Error creating a new MessageExchange");
        }
        assertTrue("Message exchange must be an instance of InOut",
                messageExchange instanceof InOut);
        assertEquals("Wrong operation name", opName, messageExchange
                .getOperation());
        assertEquals("Wrong service name", serviceName, messageExchange
                .getService());
        assertEquals("Wrong content", content, ((InOut) messageExchange)
                .getInMessage().getContent());

        /*
         * Test Case 4
         * Test Exception throwing for null delivery channel
         */
        reset(provider);
        try {
            expect(provider.getDeliveryChannel()).andReturn(null)
                    .anyTimes();
        } catch (ProviderException e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(provider);

        /*
         * Run the test
         */
        try {
            messageExchange = provider.createInOutMessageExchange(
                    opName, serviceName, content);
            fail("Must throw an exception if delivery channel is null");
        } catch (ProviderException e) {
            // Do nothing its ok
        }
    }

    public void testCreateResponseFromNormalizedMessage() {
        /*
         * Test Case 1
         * Everything goes well
         */
        
        /*
         * Create Mocks 
         */
        Method method1 = null;
        try {
            method1 = PetalsProvider.class.getDeclaredMethod(
                    "createString", new Class[] {NormalizedMessage.class});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked method");
        } 
        Method[] mockedMethods = new Method[] {method1};
        MockPetalsProvider provider = createMock(MockPetalsProvider.class,
                mockedMethods);
        /*
         * Create Test parameters
         */
        SOAPEnvelope responseEnv = createMock(SOAPEnvelope.class);
        MockNormalizedMessage nm2 = new MockNormalizedMessage();
        String response = "ResponseString";
        /*
         * Initialize mocks
         */
        try {
            expect(provider.createString(nm2)).andReturn(response);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize mocks");
        }
        replay(provider);
        /*
         * Run Test Case 1
         */
        try {
            provider.createResponseFromNormalizedMessageSuper(responseEnv, nm2);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method involation");
        }
        assertTrue(provider.isPutContentInSOAPEnvelope());
        verify(provider);
        
        /*
         * Test Case 2
         * An exception is thrown during createString invokation
         */
        reset(provider);
        /*
         * Initialize mocks
         */
        try {
            expect(provider.createString(nm2)).andThrow(new Exception("testing exception"));
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize mocks");
        }
        replay(provider);
        /*
         * Run Test Case 2
         */
        try {
            provider.createResponseFromNormalizedMessageSuper(responseEnv, nm2);
            fail("No exception was raised");
        } catch (Exception e) {
            assertEquals("Exception message incorrect",e.getMessage(),PetalsProvider.ERROR_CREATE_STRING_FROM_NM);
            verify(provider);
        }
        
        /*
         * Test Case 3
         * An exception is thrown during putContentInSOAPEnvelope invokation
         */
        reset(provider);
        /*
         * Create Test parameters
         */
        response = "exception";
        /*
         * Initialize mocks
         */
        try {
            expect(provider.createString(nm2)).andReturn(response);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize mocks");
        }
        replay(provider);
        /*
         * Run Test Case 3
         */
        try {
            provider.createResponseFromNormalizedMessageSuper(responseEnv, nm2);
            fail("No exception was raised");
        } catch (Exception e) {
            assertEquals("Exception message incorrect",e.getMessage(),PetalsProvider.ERROR_PUT_CONTENT_IN_SOAP_ENVELOPE);
            verify(provider);
        }
        
        /*
         * Test Case 4
         * A null normalized message is given as parameter
         */
        
        /*
         * Create Mocks 
         */
        
        provider = new MockPetalsProvider();
        /*
         * Create Test parameters
         */
        responseEnv = createMock(SOAPEnvelope.class);
        nm2 = null;
        /*
         * Run Test Case 4
         */
        try {
            provider.createResponseFromNormalizedMessageSuper(responseEnv, nm2);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method involation");
        }
        assertTrue(provider.isPutContentInSOAPEnvelope());
    }
    
    public void testCreateSOAPMessage() {
        /*
         * Test Case 1
         */
        PetalsProvider provider = new PetalsProvider();
        /*
         * Create Test parameters
         */
        SOAPMessage result = null;
        /*
         * Run Test Case 1
         */
        try {
           result = provider.createSOAPMessage();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertNotNull("The result of invokation should not be null",result);
    }
    @SuppressWarnings("unchecked")
    public void testGetAttachmentsFromSOAPMessage() {
        /*
         * Test Case 1
         * The SOAPMessage has no attachment
         */
        /*
         * Create Mocks
         */
        Method method1 = null;
        try {
            method1 = SOAPMessage.class.getDeclaredMethod(
                    "getAttachments", new Class[] {});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked method");
        }
        Method[] mockedMethods = new Method[] {method1};
        SOAPMessage message = createMock(SOAPMessage.class,mockedMethods);
        Iterator<AttachmentPart> ite = new ArrayIterator();
        /*
         * Initialize Mocks
         */
        try {
            expect(message.getAttachments()).andReturn(ite);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize mocks");
        }
        replay(message);
        /*
         * Create Test parameters
         */
        PetalsProvider provider = new PetalsProvider();
        List<DataHandler> result = null;
        /*
         * Run Test Case 1
         */
        try {
            result = provider.getAttachmentsFromSOAPMessage(message);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertNotNull("The result of invokation should not be null",result);
        assertEquals("The attachments list should be empty",result.size(),0);
        verify(message);
        
        /*
         * Test Case 2
         * The SOAPMessage has one attachment
         */
        /*
         * Create Mocks
         */
        message = createMock(SOAPMessage.class,mockedMethods);
        method1 = null;
        Method method2 = null;
        Method method3 = null;
        try {
            method1 = ArrayIterator.class.getDeclaredMethod(
                    "hasNext", new Class[] {});
            method2 = ArrayIterator.class.getDeclaredMethod(
                    "next", new Class[] {});
            method3 = AttachmentPart.class.getDeclaredMethod(
                    "getDataHandler", new Class[] {});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked method");
        }
        mockedMethods = new Method[] {method1, method2}; 
        ite = createMock(ArrayIterator.class,mockedMethods);
        mockedMethods = new Method[] {method3};
        AttachmentPart attachment = createMock(AttachmentPart.class, mockedMethods);
        DataHandler dataHandler = createMock(DataHandler.class);
        /*
         * Initialize Mocks
         */
        try {
            expect(message.getAttachments()).andReturn(ite);
            expect(ite.hasNext()).andReturn(true);
            expect(ite.next()).andReturn(attachment);
            expect(ite.hasNext()).andReturn(false);
            expect(attachment.getDataHandler()).andReturn(dataHandler);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize mocks");
        }
        replay(message);
        replay(ite);
        replay(attachment);
        /*
         * Create Test parameters
         */
        provider = new PetalsProvider();
        result = null;
        /*
         * Run Test Case 1
         */
        try {
            result = provider.getAttachmentsFromSOAPMessage(message);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertNotNull("The result of invokation should not be null",result);
        assertEquals("The attachments list should not be empty",result.size(),1);
        assertEquals("The DataHandler is not the right one",result.get(0),dataHandler);
        verify(message);
        verify(ite);
        verify(attachment);
    }
    
    public void testDoRPCProcess() {
        /*
         * Test Case 1
         */
        /*
         * Create Test parameters
         */
        MockPetalsProvider provider = new MockPetalsProvider();
        MessageContext msgContext = createMock(MessageContext.class);
        /*
         * Run Test Case 1
         */
        try {
            provider.doRPCProcess(msgContext);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertTrue(provider.isDoDocumentProcess());
    }
    
    public void testDoDocumentProcess() {
        /*
         * Test Case 1 Everything goes well, the requested operation is InOut
         */
        /*
         * Create Mocks
         */
        Method method1 = null;
        Method method2 = null;
        Method method3 = null;
        Method method4 = null;
        Method method5 = null;
        Method method6 = null;
        Method method7 = null;
        Method method8 = null;
        Method method9 = null;
        try {
            method1 = MessageContext.class.getDeclaredMethod(
                    "getWSDLDefinition", new Class[] {});
            method2 = PetalsProvider.class.getDeclaredMethod("getServiceName",
                    new Class[] {MessageContext.class});
            method3 = PetalsProvider.class.getDeclaredMethod(
                    "getEndpointsForService", new Class[] {QName.class});
            method4 = PetalsProvider.class.getDeclaredMethod(
                    "hasOperationNamed", new Class[] {Definition.class,
                        String.class, QName.class});
            method5 = PetalsProvider.class.getDeclaredMethod(
                    "isInOutOperation", new Class[] {Definition.class,
                        String.class, QName.class});
            method6 = PetalsProvider.class.getDeclaredMethod("createSource",
                    new Class[] {SOAPMessage.class});
            method7 = PetalsProvider.class.getDeclaredMethod(
                    "createMessageExchange", new Class[] {QName.class,
                        QName.class, Source.class, boolean.class});
            method8 = PetalsProvider.class.getDeclaredMethod(
                    "getDeliveryChannel", new Class[] {});

        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mock methods");
        }
        Method[] mockedMethods = new Method[] {method1};
        MessageContext msgContext = createMock(MessageContext.class,
                mockedMethods);
        mockedMethods = new Method[] {method2, method3, method4, method5,
            method6, method7, method8, method9};
        MockPetalsProvider provider = createMock(MockPetalsProvider.class,
                mockedMethods);
        /*
         * Create Test parameters
         */
        provider.setMessageFactory(SOAPUtils.MESSAGEFACTORY_INSTANCE);
        String service = "fooService";
        String opName = "fooOperation";
        String interfaceName = "foo";
        QName opQName = new QName(opName);
        BindingOperation bindingOp = new BindingOperationImpl();
        bindingOp.setName(opName);
        msgContext.setBindingOperation(bindingOp);
        SOAPMessage requestMsg = null;
        QName serviceQName = new QName("namespaceURI", service);
        try {
            requestMsg = provider.createSOAPMessage();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create request soap Message");
        }
        msgContext.setRequestMessage(requestMsg);
        Definition serviceDef = createMock(Definition.class);
        MockServiceEndpoint endpoint = new MockServiceEndpoint(serviceQName,
                interfaceName);
        MockServiceEndpoint[] endpointList = new MockServiceEndpoint[] {endpoint};
        Source reqSource = createMock(Source.class);
        MockInOut msgExch = new MockInOut() {
            public ExchangeStatus getStatus() {
                return ExchangeStatus.ACTIVE;
            }
        };
        MockNormalizedMessage respJBIMsg = new MockNormalizedMessage();
        MockDeliveryChannel channel = new MockDeliveryChannel();
        /*
         * Initialize Mocks
         */
        try {
            msgExch.setOutMessage(respJBIMsg);
            expect(msgContext.getWSDLDefinition()).andReturn(serviceDef);
            expect(provider.getServiceName(msgContext)).andReturn(serviceQName);
            expect(provider.getEndpointsForService(serviceQName)).andReturn(
                    endpointList);
            expect(provider.hasOperationNamed(serviceDef, opName, serviceQName))
                    .andReturn(true);
            expect(provider.isInOutOperation(serviceDef, opName, serviceQName))
                    .andReturn(true);
            expect(provider.createSource(requestMsg)).andReturn(reqSource);
            expect(
                    provider.createMessageExchange(opQName, serviceQName,
                            reqSource, false)).andReturn(msgExch);
            expect(provider.getDeliveryChannel()).andReturn(channel).anyTimes();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error initializing the Mocks");
        }
        replay(msgContext);
        replay(provider);
        /*
         * Run Test Case 1
         */
        try {
            provider.doDocumentProcessSuper(msgContext);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        verify(msgContext);
        verify(provider);
        assertTrue("sendSync was not called ", channel.isSendSyncLong());
        assertTrue("send should was not called ", channel.isSend());
        assertTrue("createResponseFromNormalizedMessage was not called",
                provider.isCreateResponseFromNormalizedMessage());
        assertTrue("createResponseFromNormalizedMessage was not called",
                provider.isPutJBIAttachmentsInSOAPMessage());

        /*
         * Test Case 2 
         * Everything goes well, requested method is inonly
         */

        mockedMethods = new Method[] {method1};
        msgContext = createMock(MessageContext.class, mockedMethods);
        mockedMethods = new Method[] {method2, method3, method4, method5,
            method6, method7, method8, method9};
        provider = createMock(MockPetalsProvider.class, mockedMethods);
        /*
         * Create Test parameters
         */
        provider.setMessageFactory(SOAPUtils.MESSAGEFACTORY_INSTANCE);
        msgContext.setBindingOperation(bindingOp);
        requestMsg = null;
        try {
            requestMsg = provider.createSOAPMessage();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create request soap Message");
        }
        msgContext.setRequestMessage(requestMsg);
        serviceDef = createMock(Definition.class);
        MockInOnly msgExch2 = new MockInOnly() {
            public ExchangeStatus getStatus() {
                return ExchangeStatus.DONE;
            }
        };
        channel = new MockDeliveryChannel();
        /*
         * Initialize Mocks
         */
        try {
            expect(msgContext.getWSDLDefinition()).andReturn(serviceDef);
            expect(provider.getServiceName(msgContext)).andReturn(serviceQName);
            expect(provider.getEndpointsForService(serviceQName)).andReturn(
                    endpointList);
            expect(provider.hasOperationNamed(serviceDef, opName, serviceQName))
                    .andReturn(true);
            expect(provider.isInOutOperation(serviceDef, opName, serviceQName))
                    .andReturn(false);
            expect(provider.createSource(requestMsg)).andReturn(reqSource);
            expect(
                    provider.createMessageExchange(opQName, serviceQName,
                            reqSource, true)).andReturn(msgExch2);
            expect(provider.getDeliveryChannel()).andReturn(channel).anyTimes();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error initializing the Mocks");
        }
        replay(msgContext);
        replay(provider);
        /*
         * Run Test Case 2
         */
        try {
            provider.doDocumentProcessSuper(msgContext);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        verify(msgContext);
        verify(provider);
        assertTrue("sendSync was not called ", channel.isSendSyncLong());
        assertFalse("send should not be called ", channel.isSend());
        assertTrue("createResponseFromNormalizedMessage was not called",
                provider.isCreateResponseFromNormalizedMessage());
        assertTrue("createResponseFromNormalizedMessage was not called",
                provider.isPutJBIAttachmentsInSOAPMessage());

        /*
         * Test Case 3 
         * No endpoint for service is found
         */
        mockedMethods = new Method[] {method1};
        msgContext = createMock(MessageContext.class, mockedMethods);
        mockedMethods = new Method[] {method2, method3};
        provider = createMock(MockPetalsProvider.class, mockedMethods);
        /*
         * Create Test parameters
         */
        provider.setMessageFactory(SOAPUtils.MESSAGEFACTORY_INSTANCE);
        msgContext.setBindingOperation(bindingOp);
        requestMsg = null;
        try {
            requestMsg = provider.createSOAPMessage();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create request soap Message");
        }
        msgContext.setRequestMessage(requestMsg);
        // endpoint = new MockServiceEndpoint(serviceQName,interfaceName);
        endpointList = new MockServiceEndpoint[] {};
        /*
         * Initialize Mocks
         */
        try {
            msgExch.setOutMessage(respJBIMsg);
            expect(msgContext.getWSDLDefinition()).andReturn(serviceDef);
            expect(provider.getServiceName(msgContext)).andReturn(serviceQName);
            expect(provider.getEndpointsForService(serviceQName)).andReturn(
                    endpointList);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error initializing the Mocks");
        }
        replay(msgContext);
        replay(provider);
        /*
         * Run Test Case 3 
         */
        try {
            provider.doDocumentProcessSuper(msgContext);
            fail("No Exception was raised");
        } catch (Exception e) {
            verify(msgContext);
            verify(provider);
        }
        
        /*
         * Test Case 4
         * An error occurs during sendSync 
         */
        
        mockedMethods = new Method[] {method1};
        msgContext = createMock(MessageContext.class,
                mockedMethods);
        mockedMethods = new Method[] {method2, method3, method4, method5,
            method6, method7, method8, method9};
        provider = createMock(MockPetalsProvider.class,
                mockedMethods);
        /*
         * Create Test parameters
         */
        provider.setMessageFactory(SOAPUtils.MESSAGEFACTORY_INSTANCE);
        msgContext.setBindingOperation(bindingOp);
        requestMsg = null;
        endpointList = new MockServiceEndpoint[] {endpoint};
        try {
            requestMsg = provider.createSOAPMessage();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create request soap Message");
        }
        msgContext.setRequestMessage(requestMsg);
        msgExch = new MockInOut() {
            public ExchangeStatus getStatus() {
                return ExchangeStatus.ACTIVE;
            }
        };
        channel = new MockDeliveryChannel(){
            public boolean sendSync(MessageExchange exchange, long timeoutMS)
            throws MessagingException {
            throw new MessagingException("sendSync testing exception");
        }
        };
        /*
         * Initialize Mocks
         */
        try {
            msgExch.setOutMessage(respJBIMsg);
            expect(msgContext.getWSDLDefinition()).andReturn(serviceDef);
            expect(provider.getServiceName(msgContext)).andReturn(serviceQName);
            expect(provider.getEndpointsForService(serviceQName)).andReturn(
                    endpointList);
            expect(provider.hasOperationNamed(serviceDef, opName, serviceQName))
                    .andReturn(true);
            expect(provider.isInOutOperation(serviceDef, opName, serviceQName))
                    .andReturn(true);
            expect(provider.createSource(requestMsg)).andReturn(reqSource);
            expect(
                    provider.createMessageExchange(opQName, serviceQName,
                            reqSource, false)).andReturn(msgExch);
            expect(provider.getDeliveryChannel()).andReturn(channel).anyTimes();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error initializing the Mocks");
        }
        replay(msgContext);
        replay(provider);
        /*
         * Run Test Case 4
         */
        try {
            provider.doDocumentProcessSuper(msgContext);
            fail("No exception was raised");
        } catch (Exception e) {
            verify(msgContext);
            verify(provider);
        }
        
        
        /*
         * Test Case 5
         * An error occurs during setStatus to DONE 
         */
        
        mockedMethods = new Method[] {method1};
        msgContext = createMock(MessageContext.class,
                mockedMethods);
        mockedMethods = new Method[] {method2, method3, method4, method5,
            method6, method7, method8, method9};
        provider = createMock(MockPetalsProvider.class,
                mockedMethods);
        /*
         * Create Test parameters
         */
        provider.setMessageFactory(SOAPUtils.MESSAGEFACTORY_INSTANCE);
        msgContext.setBindingOperation(bindingOp);
        requestMsg = null;
        endpointList = new MockServiceEndpoint[] {endpoint};
        try {
            requestMsg = provider.createSOAPMessage();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create request soap Message");
        }
        msgContext.setRequestMessage(requestMsg);
        msgExch = new MockInOut() {
            public ExchangeStatus getStatus() {
                return ExchangeStatus.ACTIVE;
            }
            public void setStatus(ExchangeStatus status) throws MessagingException {
                throw new MessagingException("setStatus testing exception");
            }
        };
        channel = new MockDeliveryChannel();
        /*
         * Initialize Mocks
         */
        try {
            msgExch.setOutMessage(respJBIMsg);
            expect(msgContext.getWSDLDefinition()).andReturn(serviceDef);
            expect(provider.getServiceName(msgContext)).andReturn(serviceQName);
            expect(provider.getEndpointsForService(serviceQName)).andReturn(
                    endpointList);
            expect(provider.hasOperationNamed(serviceDef, opName, serviceQName))
                    .andReturn(true);
            expect(provider.isInOutOperation(serviceDef, opName, serviceQName))
                    .andReturn(true);
            expect(provider.createSource(requestMsg)).andReturn(reqSource);
            expect(
                    provider.createMessageExchange(opQName, serviceQName,
                            reqSource, false)).andReturn(msgExch);
            expect(provider.getDeliveryChannel()).andReturn(channel).anyTimes();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error initializing the Mocks");
        }
        replay(msgContext);
        replay(provider);
        /*
         * Run Test Case 5
         */
        try {
            provider.doDocumentProcessSuper(msgContext);
            fail("No exception was raised");
        } catch (Exception e) {
            verify(msgContext);
            verify(provider);
            assertTrue("sendSync was not called ", channel.isSendSyncLong());
            assertTrue("createResponseFromNormalizedMessage was not called",
                    provider.isCreateResponseFromNormalizedMessage());
            assertTrue("createResponseFromNormalizedMessage was not called",
                    provider.isPutJBIAttachmentsInSOAPMessage());
        }
        

        /*
         * Test Case 6
         * An error occurs during send of DONE 
         */
        
        mockedMethods = new Method[] {method1};
        msgContext = createMock(MessageContext.class,
                mockedMethods);
        mockedMethods = new Method[] {method2, method3, method4, method5,
            method6, method7, method8, method9};
        provider = createMock(MockPetalsProvider.class,
                mockedMethods);
        /*
         * Create Test parameters
         */
        provider.setMessageFactory(SOAPUtils.MESSAGEFACTORY_INSTANCE);
        msgContext.setBindingOperation(bindingOp);
        requestMsg = null;
        endpointList = new MockServiceEndpoint[] {endpoint};
        try {
            requestMsg = provider.createSOAPMessage();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create request soap Message");
        }
        msgContext.setRequestMessage(requestMsg);
        msgExch = new MockInOut() {
            public ExchangeStatus getStatus() {
                return ExchangeStatus.ACTIVE;
            }
        };
        channel = new MockDeliveryChannel(){
            public void send(MessageExchange exchange)
            throws MessagingException {
                throw new MessagingException("send testing exception");
        }
        };
        /*
         * Initialize Mocks
         */
        try {
            msgExch.setOutMessage(respJBIMsg);
            expect(msgContext.getWSDLDefinition()).andReturn(serviceDef);
            expect(provider.getServiceName(msgContext)).andReturn(serviceQName);
            expect(provider.getEndpointsForService(serviceQName)).andReturn(
                    endpointList);
            expect(provider.hasOperationNamed(serviceDef, opName, serviceQName))
                    .andReturn(true);
            expect(provider.isInOutOperation(serviceDef, opName, serviceQName))
                    .andReturn(true);
            expect(provider.createSource(requestMsg)).andReturn(reqSource);
            expect(
                    provider.createMessageExchange(opQName, serviceQName,
                            reqSource, false)).andReturn(msgExch);
            expect(provider.getDeliveryChannel()).andReturn(channel).anyTimes();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error initializing the Mocks");
        }
        replay(msgContext);
        replay(provider);
        /*
         * Run Test Case 6
         */
        try {
            provider.doDocumentProcessSuper(msgContext);
            fail("No exception was raised");
        } catch (Exception e) {
            verify(msgContext);
            verify(provider);
            assertTrue("sendSync was not called ", channel.isSendSyncLong());
            assertTrue("createResponseFromNormalizedMessage was not called",
                    provider.isCreateResponseFromNormalizedMessage());
            assertTrue("createResponseFromNormalizedMessage was not called",
                    provider.isPutJBIAttachmentsInSOAPMessage());
        }
        
        
        /*
         * Test Case 7
         * The JBI response has the status ERROR  
         */
        
        mockedMethods = new Method[] {method1};
        msgContext = createMock(MessageContext.class,
                mockedMethods);
        mockedMethods = new Method[] {method2, method3, method4, method5,
            method6, method7, method8, method9};
        provider = createMock(MockPetalsProvider.class,
                mockedMethods);
        /*
         * Create Test parameters
         */
        provider.setMessageFactory(SOAPUtils.MESSAGEFACTORY_INSTANCE);
        msgContext.setBindingOperation(bindingOp);
        requestMsg = null;
        endpointList = new MockServiceEndpoint[] {endpoint};
        try {
            requestMsg = provider.createSOAPMessage();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create request soap Message");
        }
        msgContext.setRequestMessage(requestMsg);
        msgExch = new MockInOut() {
            public ExchangeStatus getStatus() {
                return ExchangeStatus.ERROR;
            }
        };
        //MockNormalizedMessage respJBIMsg = new MockNormalizedMessage();
        channel = new MockDeliveryChannel();
        /*
         * Initialize Mocks
         */
        try {
            msgExch.setOutMessage(respJBIMsg);
            expect(msgContext.getWSDLDefinition()).andReturn(serviceDef);
            expect(provider.getServiceName(msgContext)).andReturn(serviceQName);
            expect(provider.getEndpointsForService(serviceQName)).andReturn(
                    endpointList);
            expect(provider.hasOperationNamed(serviceDef, opName, serviceQName))
                    .andReturn(true);
            expect(provider.isInOutOperation(serviceDef, opName, serviceQName))
                    .andReturn(true);
            expect(provider.createSource(requestMsg)).andReturn(reqSource);
            expect(
                    provider.createMessageExchange(opQName, serviceQName,
                            reqSource, false)).andReturn(msgExch);
            expect(provider.getDeliveryChannel()).andReturn(channel).anyTimes();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error initializing the Mocks");
        }
        replay(msgContext);
        replay(provider);
        /*
         * Run Test Case 7
         */
        try {
            provider.doDocumentProcessSuper(msgContext);
            fail("No exception was raised");
        } catch (Exception e) {
            verify(msgContext);
            verify(provider);
            assertTrue("sendSync was not called ", channel.isSendSyncLong());
            assertTrue("send should was not called ", channel.isSend());
        }
        
        
        /*
         * Test Case 8
         * The JBI response has the status DONE  
         */
        
        mockedMethods = new Method[] {method1};
        msgContext = createMock(MessageContext.class,
                mockedMethods);
        mockedMethods = new Method[] {method2, method3, method4, method5,
            method6, method7, method8, method9};
        provider = createMock(MockPetalsProvider.class,
                mockedMethods);
        /*
         * Create Test parameters
         */
        provider.setMessageFactory(SOAPUtils.MESSAGEFACTORY_INSTANCE);
        msgContext.setBindingOperation(bindingOp);
        requestMsg = null;
        endpointList = new MockServiceEndpoint[] {endpoint};
        try {
            requestMsg = provider.createSOAPMessage();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create request soap Message");
        }
        msgContext.setRequestMessage(requestMsg);
        msgExch = new MockInOut() {
            public ExchangeStatus getStatus() {
                return ExchangeStatus.DONE;
            }
        };
        //MockNormalizedMessage respJBIMsg = new MockNormalizedMessage();
        channel = new MockDeliveryChannel();
        /*
         * Initialize Mocks
         */
        try {
            msgExch.setOutMessage(respJBIMsg);
            expect(msgContext.getWSDLDefinition()).andReturn(serviceDef);
            expect(provider.getServiceName(msgContext)).andReturn(serviceQName);
            expect(provider.getEndpointsForService(serviceQName)).andReturn(
                    endpointList);
            expect(provider.hasOperationNamed(serviceDef, opName, serviceQName))
                    .andReturn(true);
            expect(provider.isInOutOperation(serviceDef, opName, serviceQName))
                    .andReturn(true);
            expect(provider.createSource(requestMsg)).andReturn(reqSource);
            expect(
                    provider.createMessageExchange(opQName, serviceQName,
                            reqSource, false)).andReturn(msgExch);
            expect(provider.getDeliveryChannel()).andReturn(channel).anyTimes();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error initializing the Mocks");
        }
        replay(msgContext);
        replay(provider);
        /*
         * Run Test Case 8
         */
        try {
            provider.doDocumentProcessSuper(msgContext);
            fail("No exception was raised");
        } catch (Exception e) {
            verify(msgContext);
            verify(provider);
            assertTrue("sendSync was not called ", channel.isSendSyncLong());
            //assertTrue("send should was not called ", channel.isSend());
        }
        
        
        /*
         * Test Case 9
         * The JBI response has the status ERROR  
         */
        
        mockedMethods = new Method[] {method1};
        msgContext = createMock(MessageContext.class,
                mockedMethods);
        mockedMethods = new Method[] {method2, method3, method4, method5,
            method6, method7, method8, method9};
        provider = createMock(MockPetalsProvider.class,
                mockedMethods);
        /*
         * Create Test parameters
         */
        provider.setMessageFactory(SOAPUtils.MESSAGEFACTORY_INSTANCE);
        msgContext.setBindingOperation(bindingOp);
        requestMsg = null;
        endpointList = new MockServiceEndpoint[] {endpoint};
        try {
            requestMsg = provider.createSOAPMessage();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create request soap Message");
        }
        msgContext.setRequestMessage(requestMsg);
        msgExch = new MockInOut() {
            public ExchangeStatus getStatus() {
                return ExchangeStatus.ERROR;
            }
            public void setStatus(ExchangeStatus status) throws MessagingException {
                throw new MessagingException("setStatus testing exception");
            }
        };
        //MockNormalizedMessage respJBIMsg = new MockNormalizedMessage();
        channel = new MockDeliveryChannel();
        /*
         * Initialize Mocks
         */
        try {
            msgExch.setOutMessage(respJBIMsg);
            expect(msgContext.getWSDLDefinition()).andReturn(serviceDef);
            expect(provider.getServiceName(msgContext)).andReturn(serviceQName);
            expect(provider.getEndpointsForService(serviceQName)).andReturn(
                    endpointList);
            expect(provider.hasOperationNamed(serviceDef, opName, serviceQName))
                    .andReturn(true);
            expect(provider.isInOutOperation(serviceDef, opName, serviceQName))
                    .andReturn(true);
            expect(provider.createSource(requestMsg)).andReturn(reqSource);
            expect(
                    provider.createMessageExchange(opQName, serviceQName,
                            reqSource, false)).andReturn(msgExch);
            expect(provider.getDeliveryChannel()).andReturn(channel).anyTimes();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error initializing the Mocks");
        }
        replay(msgContext);
        replay(provider);
        /*
         * Run Test Case 9
         */
        try {
            provider.doDocumentProcessSuper(msgContext);
            fail("No exception was raised");
        } catch (Exception e) {
            verify(msgContext);
            verify(provider);
            assertTrue("sendSync was not called ", channel.isSendSyncLong());
           // assertTrue("send should was not called ", channel.isSend());
        }
        
        /*
         * Test Case 10
         * An error occurs during the send  
         */
        
        mockedMethods = new Method[] {method1};
        msgContext = createMock(MessageContext.class,
                mockedMethods);
        mockedMethods = new Method[] {method2, method3, method4, method5,
            method6, method7, method8, method9};
        provider = createMock(MockPetalsProvider.class,
                mockedMethods);
        /*
         * Create Test parameters
         */
        provider.setMessageFactory(SOAPUtils.MESSAGEFACTORY_INSTANCE);
        msgContext.setBindingOperation(bindingOp);
        requestMsg = null;
        endpointList = new MockServiceEndpoint[] {endpoint};
        try {
            requestMsg = provider.createSOAPMessage();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create request soap Message");
        }
        msgContext.setRequestMessage(requestMsg);
        msgExch = new MockInOut() {
            public ExchangeStatus getStatus() {
                return ExchangeStatus.ERROR;
            }
        };
        //MockNormalizedMessage respJBIMsg = new MockNormalizedMessage();
        channel = new MockDeliveryChannel() {
            public void send(MessageExchange exchange) throws MessagingException {
                throw new MessagingException("send testing exception");
            }
        };
        /*
         * Initialize Mocks
         */
        try {
            msgExch.setOutMessage(respJBIMsg);
            expect(msgContext.getWSDLDefinition()).andReturn(serviceDef);
            expect(provider.getServiceName(msgContext)).andReturn(serviceQName);
            expect(provider.getEndpointsForService(serviceQName)).andReturn(
                    endpointList);
            expect(provider.hasOperationNamed(serviceDef, opName, serviceQName))
                    .andReturn(true);
            expect(provider.isInOutOperation(serviceDef, opName, serviceQName))
                    .andReturn(true);
            expect(provider.createSource(requestMsg)).andReturn(reqSource);
            expect(
                    provider.createMessageExchange(opQName, serviceQName,
                            reqSource, false)).andReturn(msgExch);
            expect(provider.getDeliveryChannel()).andReturn(channel).anyTimes();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error initializing the Mocks");
        }
        replay(msgContext);
        replay(provider);
        /*
         * Run Test Case 10
         */
        try {
            provider.doDocumentProcessSuper(msgContext);
            fail("No exception was raised");
        } catch (Exception e) {
            verify(msgContext);
            verify(provider);
            assertTrue("sendSync was not called ", channel.isSendSyncLong());
           // assertTrue("send should was not called ", channel.isSend());
        }
    }

    public void testPutJBIAttachmentsInSOAPMessage() {
        /*
         * Test Case 1
         * No attachment in JBI message
         */
        /*
         * Create Mocks
         */
        Method method1 = null;
        Method method2 = null;
        try {
            method1 = MockNormalizedMessage.class.getDeclaredMethod(
                    "getAttachmentNames", new Class[] {});
            method2 = MockNormalizedMessage.class.getDeclaredMethod(
                    "getAttachment", new Class[] {String.class});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked method");
        }
        Method[] mockedMethods = new Method[] {method1, method2};
        /*
         * Create Test parameters 
         */
        MockNormalizedMessage nm = new MockNormalizedMessage();
        PetalsProvider provider = new PetalsProvider();
        SOAPMessage message = null;
        try {
            message = provider.createSOAPMessage();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create a SOAPMessage");
        }
        /*
         * Run Test Case 1
         */
        try {
            provider.putJBIAttachmentsInSOAPMessage(message, nm);
        } catch (Exception e) {
            e.printStackTrace();
            fail ("Error during method invokation ");
        }
        assertFalse("The SOAP message should have no attachement",message.getAttachments().hasNext());
        
        /*
         * Test Case 2
         * One attachment in JBI message
         */
        /*
         * Create Mocks
         */
        nm = createMock(MockNormalizedMessage.class,mockedMethods);
        Set<String> attNames = new HashSet<String>();
        String name = "foo";
        attNames.add(name);
        DataHandler dh = createMock(DataHandler.class);
        /*
         * Initialize Mocks
         */
        expect(nm.getAttachmentNames()).andReturn(attNames);
        expect(nm.getAttachment(name)).andReturn(dh);
        replay(nm);
        /*
         * Create Test parameters 
         */
        provider = new PetalsProvider();
        message = null;
        try {
            message = provider.createSOAPMessage();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create a SOAPMessage");
        }
        /*
         * Run Test Case 1
         */
        try {
            provider.putJBIAttachmentsInSOAPMessage(message, nm);
        } catch (Exception e) {
            e.printStackTrace();
            fail ("Error during method invokation ");
        }
        verify(nm);
        assertTrue("The SOAP message should have attachement",message.getAttachments().hasNext());
        try {
        assertEquals("The attachment of SOAP message differs from the one in NormalizedMessage",((AttachmentPart)message.getAttachments().next()).getDataHandler(),dh);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error retrieving the attachement from SOAP message");
        }
    }
    
    @Override
    protected void setUp() throws Exception {
    }
}
