/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id : $ 
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.xquarebc.listeners;

import javax.xml.transform.Source;

/**
 * This interface represents a strategy for detecting "new data",
 * getting it (as XML) and updating what data is known.
 * 
 * Implementations are typically instanciated from a factory
 * each time new data must be detected and processed, so it may
 * (it is even advised) have a state.
 * 
 * @version $Rev: 250 $Date: {date}
 * @since Petals 1.0
 * @author Marc Dutoo - Open Wide
 *
 */
public interface NewDataStrategy {

    /**
     * Detects new data. May save the resulting information
     * for later use in getNewData() and updateDataKnown().
     * @return true if there is new data
     * @throws Exception
     */
    public abstract boolean detectNewData() throws Exception;

    /**
     * Queries for the detected new data and returns them as
     * a formatted XML source. 
     * @return
     * @throws Exception
     */
    public abstract Source getNewData() throws Exception;

    /**
     * Updates the persistent knwoledge of new data found so the
     * new time the strategy is used, it will be old data.
     * @throws Exception
     */
    public abstract void updateDataKnown() throws Exception;
}