/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id : $ 
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.xquarebc;

import java.io.File;

import javax.jbi.JBIException;
import javax.jbi.component.InstallationContext;

import junit.framework.TestCase;
import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.verify;

/**
 * 
 * Tests the XQuareBCBootstrapImpl
 * 
 * @version $Rev: 250 $Date: {date}
 * @since Petals 1.0
 * @author Marc Dutoo - Open Wide
 * 
 */
public class XQuareBCBootstrapImplTest extends TestCase {
    File installationRoot = null;

    private String baseDir;

    public void testCleanUp() {

        XQuareBCBootstrapImpl xquareBCBootstrap = new XQuareBCBootstrapImpl();

        // Run the Test
        try {
            xquareBCBootstrap.cleanUp();
        } catch (JBIException e) {
            e.printStackTrace();
            fail("Error in SMTPBCBootstrap clean up");
        }
    }

    public void testGetExtensionMBeanName() {
        // Create objects
        XQuareBCBootstrapImpl xquareBCBootstrap = new XQuareBCBootstrapImpl();

        // Run the Test
        Object result = null;
        result = xquareBCBootstrap.getExtensionMBeanName();
        assertNull("Result of invocation should be null", result);
    }

    public void testInit() {
        // Create Mock Objects
        InstallationContext installationContext = createMock(InstallationContext.class);
        // Init mocks
        expect(installationContext.getInstallRoot()).andReturn(
            installationRoot.getAbsolutePath());
        replay(installationContext);

        XQuareBCBootstrapImpl xquareBCBootstrap = new XQuareBCBootstrapImpl();
        // Run the Test
        try {
            xquareBCBootstrap.init(installationContext);
        } catch (JBIException e) {
            e.printStackTrace();
            fail("Error in XQuareBCBootstrapImpl init method");
        }

        assertEquals(xquareBCBootstrap.context, installationContext);
    }

    public void testOnInstall() {
        // Create Mock Objects
        InstallationContext installationContext = createMock(InstallationContext.class);
        // Init mocks
        XQuareBCBootstrapImpl xquareBCBootstrap = new XQuareBCBootstrapImpl();
        // Run the Test
        try {
            xquareBCBootstrap.init(installationContext);
        } catch (JBIException e) {
            e.printStackTrace();
            fail("Error in XQuareBCBootstrapImpl onInstall method");
        }
    }

    public void testOnUninstall() {
        // Create Mock Objects
        InstallationContext installationContext = createMock(InstallationContext.class);
        // Init mocks
        expect(installationContext.getInstallRoot()).andReturn(
            installationRoot.getAbsolutePath()).anyTimes();
        replay(installationContext);

        XQuareBCBootstrapImpl xquareBCBootstrap = new XQuareBCBootstrapImpl();
        // Run the Test
        try {
            xquareBCBootstrap.init(installationContext);
            xquareBCBootstrap.onInstall();
        } catch (JBIException e) {
            e.printStackTrace();
            fail("Error in XQuareBCBootstrapImpl init or onUninstall method");
        }

        // Run the Test
        try {
            xquareBCBootstrap.onUninstall();
        } catch (JBIException e) {
            e.printStackTrace();
            fail("Error in XQuareBCBootstrapImpl onuninstall method");
        }
        verify(installationContext);
        File certDir = new File(installationRoot.getAbsolutePath()
            + File.separator + "certificate");
        assertFalse(certDir.exists());
    }

    @Override
    protected void finalize() throws Throwable {
        File file = installationRoot.getParentFile();
        File[] files = file.listFiles();
        for (File files2 : files) {
            if (("bootstrap").equals(files2.getName())) {
                if (files2.listFiles().length == 0) {
                    files2.delete();
                }
            }
        }
        super.finalize();
    }

    @Override
    protected void setUp() throws Exception {
        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
        installationRoot = new File(baseDir + File.separator + "src"
            + File.separator + "test-data" + File.separator + "bootstrap");
        installationRoot.mkdirs();

    }

}
