/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id : $ 
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.xquarebc.listeners;

import java.io.FileInputStream;

/**
 * 
 * Tests the XQuareBCJBIProcessor
 * 
 * @version $Rev: 250 $Date: {date}
 * @since Petals 1.0
 * @author Marc Dutoo - Open Wide
 * 
 */
public class XQuareBCJBIProcessorTest extends XQuareBCTestTemplate {


    @Override
    public void setUp() throws Exception {
        super.setUp();
    }


    @Override
    protected void tearDown() throws Exception {
        super.tearDown();
    }
    
    

    /**
     * NB. The default XQuare sample doesn't work for this example,
     * its schema (auction.xsd) needs to be told no elements is allowed
     * (add minOccurs="0" to the USERS, ITEMS and BIDS elements)
     * @throws Exception
     */
    public void testInsertOneUser() throws Exception {
        cleanDatabase();
        String testContent = XQuareBCTestTemplate.getStringFromInputStream(new FileInputStream(auctionDir + "auction_test_insert_one_user-message.xml"));
        testOperation("insert", testContent, "<success/>", this.testXquareAuctionProperties);
        int userNb = getSelectRowNb("select * from users");
        assertEquals(1, userNb);
    }

    public void testInsertSmall() throws Exception {
        cleanDatabase();
        String testContent = XQuareBCTestTemplate.getStringFromInputStream(new FileInputStream(auctionDir + "auction_test_insert_small-message.xml"));
        testOperation("insert", testContent, "<success/>", this.testXquareAuctionProperties);
        int bidNb = getSelectRowNb("select * from bids");
        assertEquals(1, bidNb);
    }

    public void testInsertBig() throws Exception {
        cleanDatabase();
        String testContent = XQuareBCTestTemplate.getStringFromInputStream(new FileInputStream(auctionDir + "auction_test_insert_big-message.xml"));
        testOperation("insert", testContent, "<success/>", this.testXquareAuctionProperties);
        int userNb = getSelectRowNb("select * from users");
        assertEquals(6, userNb);
        int bidNb = getSelectRowNb("select * from items");
        assertEquals(8, bidNb);
        int itemNb = getSelectRowNb("select * from bids");
        assertEquals(16, itemNb);
    }

    
    public void testQuery() throws Exception {
        cleanDatabase();
        String bigDataContent = XQuareBCTestTemplate.getStringFromInputStream(new FileInputStream(auctionDir + "auction_test_insert_big-message.xml"));
        testOperation("insert", bigDataContent, "<success/>", this.testXquareAuctionProperties);
        
        String testContent = XQuareBCTestTemplate.getStringFromInputStream(new FileInputStream(auctionDir + "auction_test_query-message.xml"));
        String testResults = XQuareBCTestTemplate.getStringFromInputStream(new FileInputStream(auctionDir + "auction_test_query-results.xml"));
        testOperation("query", testContent, testResults, this.testXquareAuctionProperties);
        int userNb = getSelectRowNb("select * from users");
        assertEquals(6, userNb);
    }
    
    public void testStoredQuery() throws Exception {
        testXquareAuctionProperties.put(XQuareBCJBIProcessor.STORED_QUERY_PROP_PREFIX + 0,
            XQuareBCTestTemplate.getStringFromInputStream(new FileInputStream(auctionDir + "auction_test_stored_query_with_mod.xml")));
        
        cleanDatabase();
        String bigDataContent = XQuareBCTestTemplate.getStringFromInputStream(new FileInputStream(auctionDir + "auction_test_insert_big-message.xml"));
        testOperation("insert", bigDataContent, "<success/>", this.testXquareAuctionProperties);
        
        String testContent = XQuareBCTestTemplate.getStringFromInputStream(new FileInputStream(auctionDir + "auction_test_stored_query_with_mod-message.xml"));
        String testResults = XQuareBCTestTemplate.getStringFromInputStream(new FileInputStream(auctionDir + "auction_test_stored_query_with_mod-results.xml"));
        testOperation("storedQuery", testContent, testResults, this.testXquareAuctionProperties);
        int userNb = getSelectRowNb("select * from users");
        assertEquals(6, userNb);
    }
    

    public void testUpdateOneUser() throws Exception {
        // first inserting one user :
        testInsertOneUser();
        // then updating it :
        String testContent = XQuareBCTestTemplate.getStringFromInputStream(new FileInputStream(auctionDir + "auction_test_update_one_user-message.xml"));
        testOperation("insert", testContent, "<success/>", this.testXquareAuctionProperties);

        int userNb = getSelectRowNb("select * from users");
        assertEquals(1, userNb);
        int fRatingUserNb = getSelectRowNb("select * from users where rating='f'");
        assertEquals(1, fRatingUserNb);
    }
    

    public void testNewDataListener() throws Exception {
        // first inserting one user :
        testInsertOneUser();
        
        //TODO
        
        // then updating it :
        String testContent = XQuareBCTestTemplate.getStringFromInputStream(new FileInputStream(auctionDir + "auction_test_update_one_user-message.xml"));
        testOperation("insert", testContent, "<success/>", this.testXquareAuctionProperties);

        int userNb = getSelectRowNb("select * from users");
        assertEquals(1, userNb);
        int fRatingUserNb = getSelectRowNb("select * from users where rating='f'");
        assertEquals(1, fRatingUserNb);
    }


    
}
