/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id : $ 
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.xquarebc.listeners;

import java.io.File;
import java.io.FileInputStream;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;

import org.easymock.classextension.EasyMock;
import org.objectweb.petals.binding.xquarebc.XQuareBCException;
import org.objectweb.petals.binding.xquarebc.mock.MockDeliveryChannel;
import org.objectweb.petals.component.common.util.SourceHelper;
import org.objectweb.petals.component.common.util.WSDLHelper;
import org.w3c.dom.Document;

import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.verify;

/**
 * 
 * Tests the XQuareBCListener
 * 
 * @version $Rev: 250 $Date: {date}
 * @since Petals 1.0
 * @author Marc Dutoo - Open Wide
 * 
 */
public class XQuareBCListenerTest extends XQuareBCTestTemplate {

    private ComponentContext context;

    private MockDeliveryChannel channel;

    @Override
    public void setUp() throws Exception {
        super.setUp();
        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
        workDir = baseDir + "src" + File.separator + "test-data"
            + File.separator;
        this.logger = EasyMock.createMock(Logger.class);

        channel = new MockDeliveryChannel();

        context = createMock(ComponentContext.class);
        expect(context.getDeliveryChannel()).andReturn(channel).anyTimes();
        replay(context);
    }



    public void insertOneUser() throws Exception {
        cleanDatabase();
        String testContent = XQuareBCTestTemplate.getStringFromInputStream(new FileInputStream(auctionDir + "auction_test_insert_one_user-message.xml"));
        testOperation("insert", testContent, "<success/>", this.testXquareAuctionProperties);
        int userNb = getSelectRowNb("select * from users");
        assertEquals(1, userNb);
    }
    
    
    public void testHandleNewData() throws Exception {
        Method method1 = null;
        Method method2 = null;
        try {
            method1 = XQuareBCListener.class.getDeclaredMethod(
                "handleInOnlyMessage", new Class[] {Source.class, QName.class,
                    String.class});
            method2 = XQuareBCListener.class.getDeclaredMethod(
                "handleInOutMessage", new Class[] {Source.class, QName.class,
                    String.class});
        } catch (SecurityException e1) {
            e1.printStackTrace();
        } catch (NoSuchMethodException e1) {
            e1.printStackTrace();
        }
        Method[] mockedMethods = new Method[] {method1, method2};
        XQuareBCListener xquareListener = createMock(
            XQuareBCListener.class, mockedMethods);
        
        HashMap<String, Properties> serviceToPropertiesMap = new HashMap<String, Properties>();
        serviceToPropertiesMap.put("anyService", this.testXquareAuctionProperties);
        xquareListener.processor = new XQuareBCJBIProcessor(context, channel, logger, serviceToPropertiesMap);
        
        insertOneUser();

        String testContent = XQuareBCTestTemplate.getStringFromInputStream(new FileInputStream(auctionDir + "auction_test_listen_one_user-results.xml"));
        try {
            expect(
                xquareListener.handleInOutMessage(eqSource(SourceHelper.createSource(testContent)),
                    EasyMock.eq(QName.valueOf("service")), EasyMock.eq("operation"))).andReturn(true);
            expect(
                xquareListener.handleInOnlyMessage(eqSource(SourceHelper.createSource(testContent)),
                    EasyMock.eq(QName.valueOf("service")), EasyMock.eq("operation"))).andReturn(true);
        } catch (XQuareBCException e) {
            e.printStackTrace();
        }
        replay(xquareListener);

        boolean inOut = true;
        // it is an inOut operation
        xquareListener.handleNewData(inOut, QName.valueOf("service"),
            "operation", this.testXquareAuctionProperties);

        // it is an inOnly operation
        inOut = false;
        xquareListener.handleNewData(inOut, QName.valueOf("service"),
            "operation", this.testXquareAuctionProperties);
    }


    public void testProcess() {

        XQuareBCListener xquareListener = null;

        File wsdlFile = new File(workDir + "HelloworldService.wsdl");
        Document wsdlDoc = WSDLHelper.createDocumentFromWSDL(wsdlFile);

        ServiceEndpoint serviceEndpoint = createMock(ServiceEndpoint.class);
        ServiceEndpoint[] serviceEndpoints = {serviceEndpoint};

        ComponentContext context = createMock(ComponentContext.class);
        try {
            //expect(serviceEndpoint.getServiceName()).andReturn(
            //    new QName("{http://petals.objectweb.org/}HelloworldService"));
            expect(context.getEndpointsForService(EasyMock.eq(QName.valueOf("{http://petals.objectweb.org/}HelloworldService"))))
                .andReturn(serviceEndpoints);
            expect(context.getEndpointDescriptor(serviceEndpoint)).andReturn(wsdlDoc)
                .anyTimes();
        } catch (JBIException e) {
            e.printStackTrace();
        }
        replay(serviceEndpoint);
        replay(context);

        Map<String, Properties> mapDirConfig = new HashMap<String, Properties>();
        Properties props = new Properties();
        props.put("service.name", "MockService");
        props.put("operation.name", "inOnlyMessage");
        props.put("directory.destination", "destinationDir");
        mapDirConfig.put("MockService", props);
        xquareListener = new XQuareBCListener(
            context, channel, this.logger, null, null) {
            public void handleNewData(boolean inOut, QName service,
                String operation, Properties serviceProps) throws Exception {

            }
        };

        /*
         * Test case 1: operation inOnly
         */
        xquareListener.process(this.testXquareAuctionProperties);
        EasyMock.reset(context);
        EasyMock.reset(serviceEndpoint);
        mapDirConfig.clear();
        // assert 

        /*
         * Test case 2 :operation inOut
         */

        try {
            expect(context.getEndpointsForService(EasyMock.eq(QName.valueOf("{http://petals.objectweb.org/}HelloworldService"))))
                .andReturn(serviceEndpoints);
            expect(context.getEndpointDescriptor(serviceEndpoint)).andReturn(wsdlDoc)
                .anyTimes();
        } catch (JBIException e) {
            e.printStackTrace();
        }
        replay(serviceEndpoint);
        replay(context);

        props.put("operation.name", "inOutOperation");
        mapDirConfig.put("MockService", props);
        xquareListener.process(this.testXquareAuctionProperties);

        verify(context);
        verify(serviceEndpoint);
    }
}
