package org.objectweb.rentacar.centraloffice.services.reservation;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;

import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebService;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.namespace.QName;

import org.apache.log4j.Logger;
import org.objectweb.rentacar.centraloffice.services.CentralOfficeException;
import org.objectweb.rentacar.centraloffice.services.consultation.ConsultationImpl;
import org.objectweb.rentacar.persistance.bo.Agency;
import org.objectweb.rentacar.persistance.bo.AgencyCriteria;
import org.objectweb.rentacar.persistance.bo.AgencyVO;
import org.objectweb.rentacar.persistance.bo.Car;
import org.objectweb.rentacar.persistance.bo.Contact;
import org.objectweb.rentacar.persistance.bo.Reservation;
import org.objectweb.rentacar.persistance.dao.AgencyDAO;
import org.objectweb.rentacar.persistance.dao.CarDAO;
import org.objectweb.rentacar.persistance.dao.ContactDAO;
import org.objectweb.rentacar.persistance.dao.DAOException;
import org.objectweb.rentacar.persistance.dao.ReservationDAO;
import org.objectweb.rentacar.services.client.AgencyException_Exception;
import org.objectweb.rentacar.services.client.AgencyReservationServiceImpl;
import org.objectweb.rentacar.services.client.AgencyReservationServiceImplService;
import org.objectweb.rentacar.util.PropertiesLoader;
import org.objectweb.rentacar.util.PropertyLoadingException;

/**
 * 
 * @author ofabre
 *
 */
@WebService
public class ReservationServiceImpl {
	
	private Logger logger = Logger.getLogger(ReservationServiceImpl.class);
	
	/**
	 * Create a new reservation by invoking 
	 * the starting Agency create reservation web service. 
	 * An update of the central office database is made by 
	 * the agency web service.
	 * @param carId the reserved car Id
	 * @param start the starting date
	 * @param end the ending date
	 * @param customerId the customer Id
	 * @param startAgencyId the starting agency Id
	 * @param endAgencyId the ending agency Id
	 * @return the new reservation Id
	 * @throws CentralOfficeException
	 */
	@WebMethod
	public String createReservationOnCentralOffice(@WebParam(name="car")String carId, 
			@WebParam(name="start")XMLGregorianCalendar start, 
			@WebParam(name="end")XMLGregorianCalendar end, 
			@WebParam(name="customer")String customerId, 
			@WebParam(name="startAgency")String startAgencyId, 
			@WebParam(name="endAgency")String endAgencyId) 
	throws CentralOfficeException {
		
		URL wsdlLocation;
		logger.info("Central Office . createReservationOnCentralOffice : "+carId+
				", "+start.toString()+", "+end.toString()+
				", "+customerId+", "+startAgencyId+", "+endAgencyId);
		String agenciesPresent = null;
		try{
			agenciesPresent= PropertiesLoader.getProperty("central_office.properties", "agencies");
		}
		catch (PropertyLoadingException e){
			logger.error("Error while loading properties",e);
			throw new CentralOfficeException("Error while loading properties",e);
		}
		String reservationId;
		if(agenciesPresent != null && agenciesPresent.equals("1")){
			try {
				ConsultationImpl consultationImpl = new ConsultationImpl();
				// TODO : check nullity
				AgencyVO startAgency = consultationImpl.retreiveAgencies(new AgencyCriteria(startAgencyId, null, null))[0];
				wsdlLocation = new URL("http://"+startAgency.getHost()+":"+startAgency.getPort()+"/"+startAgency.getWarName()+"/reservation?wsdl");
			} catch (MalformedURLException e) {
				logger.debug("Error when creating wsdl URL", e);
				throw new CentralOfficeException("Error when creating wsdl URL", e);
			}
			QName serviceQname = new QName("{http://reservation.services.agency.rentacar.objectweb.org/}AgencyReservationServiceImplPort");
			
			AgencyReservationServiceImpl port;
			port = new AgencyReservationServiceImplService(wsdlLocation, serviceQname).getAgencyReservationServiceImplPort();
			
			
			try {
				reservationId = port.createReservation(carId, start, end, customerId, startAgencyId, endAgencyId);
			} catch (AgencyException_Exception e) {
				logger.error("Error when creating reservation for distant agency", e);
				throw new CentralOfficeException("Error when creating reservation for distant agency", e);
			}
		}
		else{
			logger.info("Central Office . createReservationInCentralDB : "+ customerId+", "+
					carId+", "+start.toString()+", "+end.toString()+", "+
					startAgencyId+", "+endAgencyId);
			AgencyDAO agencyDAO = AgencyDAO.getInstance();
			ContactDAO contactDAO = ContactDAO.getInstance();
			CarDAO carDAO = CarDAO.getInstance();
			
			Car car = null;
			Contact customer = null;
			Agency startingAgency = null;
			Agency endingAgency = null;
			try {
				car = (Car) carDAO.retrieveById(Car.class, carId);
				customer = (Contact) contactDAO.retrieveById(Contact.class, customerId);
				startingAgency = (Agency) agencyDAO.retrieveById(Agency.class, startAgencyId);
				endingAgency = (Agency) agencyDAO.retrieveById(Agency.class, endAgencyId);
			} catch (DAOException e) {
				logger.error("Error when retrieving object in database", e);
				throw new CentralOfficeException("Error when retrieving object in database", e);
			}
			Set<Car> cars = new HashSet<Car>();
			cars.add(car);
			Reservation reservationBO = new Reservation(customer, cars, start.toGregorianCalendar().getTime(), end.toGregorianCalendar().getTime(), startingAgency, endingAgency);
			try {
				reservationId = ReservationDAO.getInstance().create(reservationBO);
			} catch (DAOException e1) {
				logger.error("Error when creating a reservation in central office DB", e1);
				throw new CentralOfficeException("Error when creating a reservation in central office DB", e1);
			}
		}
		
		return reservationId;
		
	}
	
}
