package org.objectweb.rentacar.centraloffice.services.update;

import java.util.Date;
import java.util.HashSet;
import java.util.Set;

import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebService;

import org.apache.log4j.Logger;
import org.objectweb.rentacar.centraloffice.services.CentralOfficeException;
import org.objectweb.rentacar.persistance.bo.Agency;
import org.objectweb.rentacar.persistance.bo.Car;
import org.objectweb.rentacar.persistance.bo.Contact;
import org.objectweb.rentacar.persistance.bo.Reservation;
import org.objectweb.rentacar.persistance.dao.AgencyDAO;
import org.objectweb.rentacar.persistance.dao.CarDAO;
import org.objectweb.rentacar.persistance.dao.ContactDAO;
import org.objectweb.rentacar.persistance.dao.DAOException;
import org.objectweb.rentacar.persistance.dao.ReservationDAO;

/**
 * Services that allow to update central office database state.
 * @author ofabre
 *
 */
@WebService
public class UpdateCentralDBImpl {
	
	private Logger logger = Logger.getLogger(UpdateCentralDBImpl.class);
	
	/**
	 * Create a reservation in central office database.
	 * @param reservation The reservation to create
	 * @throws CentralOfficeException
	 */
	@WebMethod
	public void addReservationInCentralDB(
			@WebParam(name="customerId") String customerId, 
			@WebParam(name="carId") String carId, 
			@WebParam(name="startingDate") Date startingDate, 
			@WebParam(name="endingDate") Date endingDate, 
			@WebParam(name="startingAgencyId") String startingAgencyId, 
			@WebParam(name="endingAgencyId") String endingAgencyId) 
	throws CentralOfficeException{
		
		logger.info("Central Office . addReservationInCentralDB : "+ customerId+", "+
				carId+", "+startingDate.toString()+", "+endingDate.toString()+", "+
				startingAgencyId+", "+endingAgencyId);
		AgencyDAO agencyDAO = AgencyDAO.getInstance();
		ContactDAO contactDAO = ContactDAO.getInstance();
		CarDAO carDAO = CarDAO.getInstance();
		
		Car car = null;
		Contact customer = null;
		Agency startingAgency = null;
		Agency endingAgency = null;
		try {
			car = (Car) carDAO.retrieveById(Car.class, carId);
			customer = (Contact) contactDAO.retrieveById(Contact.class, customerId);
			startingAgency = (Agency) agencyDAO.retrieveById(Agency.class, startingAgencyId);
			endingAgency = (Agency) agencyDAO.retrieveById(Agency.class, endingAgencyId);
		} catch (DAOException e) {
			logger.error("Error when retrieving object in database", e);
			throw new CentralOfficeException("Error when retrieving object in database", e);
		}
		Set<Car> cars = new HashSet<Car>();
		cars.add(car);
		Reservation reservationBO = new Reservation(customer, cars, startingDate, endingDate, startingAgency, endingAgency);
		try {
			ReservationDAO.getInstance().create(reservationBO);
		} catch (DAOException e1) {
			logger.error("Error when creating a reservation in central office DB", e1);
			throw new CentralOfficeException("Error when creating a reservation in central office DB", e1);
		}
		
	}
	
}
