package org.objectweb.rentacar.persistance.bo;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;

import org.hibernate.annotations.GenericGenerator;
import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.CompareToBuilder;

/**
 * 
 * @author ofabre
 *
 */
@Entity
public class Address implements Comparable {
	
	private String addressId;
	
	private String address;
	
	private String city;
	
	private String zip;
	
	private String country;


	public Address() {
		super();
	}

	/**
	 * Class constructor
	 * @param address Number on the street and street name 
	 * @param city Name of the city
	 * @param zip Zip code of the city
	 * @param country Name of the country
	 */
	public Address(String address, String city, String zip, String country) {
		super();
		this.address = address;
		this.city = city;
		this.zip = zip;
		this.country = country;
	}
	
	/**
	 * Class constructor
	 * @param addressId Id of the address
	 * @param address Number on the street and street name 
	 * @param city Name of the city
	 * @param zip Zip code of the city
	 * @param country Name of the country
	 */
	public Address(String addressId, String address, String city, String zip, String country) {
		super();
		this.addressId = addressId;
		this.address = address;
		this.city = city;
		this.zip = zip;
		this.country = country;
	}
	
	/**
	 * Creates an Address object from an AddressVO object
	 * @param addressVO 
	 */
	public Address(AddressVO addressVO){
		super();
		this.addressId = addressVO.getAddressId();
		this.address = addressVO.getAddress();
		this.city = addressVO.getCity();
		this.zip = addressVO.getZip();
		this.country = addressVO.getCountry();
	}

	public String getAddress() {
		return address;
	}

	public void setAddress(String address) {
		this.address = address;
	}

	@Id	@GeneratedValue(generator = "system-uuid")
	@GenericGenerator(name="system-uuid", strategy = "uuid")
	public String getAddressId() {
		return addressId;
	}

	public void setAddressId(String addressId) {
		this.addressId = addressId;
	}

	public String getCity() {
		return city;
	}

	public void setCity(String city) {
		this.city = city;
	}

	public String getCountry() {
		return country;
	}

	public void setCountry(String country) {
		this.country = country;
	}

	public String getZip() {
		return zip;
	}

	public void setZip(String zip) {
		this.zip = zip;
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return new ToStringBuilder(this).append("country", this.country)
				.append("address", this.address).append("addressId",
						this.addressId).append("zip", this.zip).append("city",
						this.city).toString();
	}

	/**
	 * @see java.lang.Object#equals(Object)
	 */
	public boolean equals(Object object) {
		if (!(object instanceof Address)) {
			return false;
		}
		Address rhs = (Address) object;
		return new EqualsBuilder().append(this.addressId, rhs.addressId)
				.append(this.country, rhs.country).append(this.address,
						rhs.address).append(this.zip, rhs.zip).append(
						this.city, rhs.city).isEquals();
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return new HashCodeBuilder(-1574159667, 218892883).append(
				this.addressId).append(this.country).append(this.address)
				.append(this.zip).append(this.city).toHashCode();
	}

	/**
	 * @see java.lang.Comparable#compareTo(Object)
	 */
	public int compareTo(Object object) {
		Address myClass = (Address) object;
		return new CompareToBuilder().append(this.addressId, myClass.addressId)
				.append(this.country, myClass.country).append(this.address,
						myClass.address).append(this.zip, myClass.zip).append(
						this.city, myClass.city).toComparison();
	}
	
	

}
