package org.objectweb.rentacar.persistance.bo;

import java.util.HashSet;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.hibernate.annotations.GenericGenerator;

/**
 * 
 * @author ofabre
 * 
 */
@Entity
public class Agency {

	// Fields
	private String agencyId;
	
	private String host;
	
	private String port;
	
	private String warName;

	// Relations
	private Contact contact;

	private Set<Car> cars;

	public Agency() {
		super();
	}

	public Agency(Contact contact, Set<Car> cars) {
		super();
		this.contact = contact;
		this.cars = cars;
	}

	public Agency(String agencyId, Contact contact, Set<Car> cars) {
		super();
		this.agencyId = agencyId;
		this.contact = contact;
		this.cars = cars;
	}

	public Agency(AgencyVO agencyVO) {
		super();
		this.agencyId = agencyVO.getAgencyId();
		this.contact = new Contact(agencyVO.getContact());
		Set<Car> cars = new HashSet<Car>();
		for (CarVO car : agencyVO.getCars()) {
			cars.add(new Car(car));
		}
		this.cars = cars;
		this.host = agencyVO.getHost();
		this.port = agencyVO.getPort();
		this.warName = agencyVO.getWarName();
	}

	@Id
	@GeneratedValue(generator = "system-uuid")
	@GenericGenerator(name = "system-uuid", strategy = "uuid")
	public String getAgencyId() {
		return agencyId;
	}

	public void setAgencyId(String agencyId) {
		this.agencyId = agencyId;
	}

	@OneToMany(cascade = CascadeType.ALL)
	@JoinColumn(name = "agency_agencyId")
	public Set<Car> getCars() {
		return cars;
	}

	public void setCars(Set<Car> cars) {
		this.cars = cars;
	}

	@OneToOne(cascade = CascadeType.ALL)
	public Contact getContact() {
		return contact;
	}

	public void setContact(Contact contact) {
		this.contact = contact;
	}

	public String getHost() {
		return host;
	}

	public void setHost(String host) {
		this.host = host;
	}

	public String getPort() {
		return port;
	}

	public void setPort(String port) {
		this.port = port;
	}

	public String getWarName() {
		return warName;
	}

	public void setWarName(String warName) {
		this.warName = warName;
	}
	
	/**
	 * @see java.lang.Object#equals(Object)
	 */
	public boolean equals(Object object) {
		if (!(object instanceof Agency)) {
			return false;
		}
		Agency rhs = (Agency) object;
		return new EqualsBuilder().append(this.contact, rhs.contact).append(
				this.cars, rhs.cars).append(this.agencyId, rhs.agencyId)
				.append(this.host, rhs.host).append(this.port, rhs.port)
				.append(this.warName, rhs.warName).isEquals();
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return new HashCodeBuilder(1680302177, -561077521).append(this.contact)
				.append(this.cars).append(this.agencyId).append(this.host)
				.append(this.port).append(this.warName).toHashCode();
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return new ToStringBuilder(this).append("host", this.host).append(
				"agencyId", this.agencyId).append("contact", this.contact)
				.append("port", this.port).append("cars", this.cars).append(this.warName).toString();
	}

	

}
