package org.objectweb.rentacar.persistance.bo;

import java.util.Date;
import java.util.HashSet;
import java.util.Set;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;

/**
 * 
 * @author ofabre
 *
 */
public class ReservationVO {
	
	private String reservationId;
	
	private ContactVO customer;
	
	private Set<CarVO> cars;
	
	private Date startingDate;
	
	private Date endingDate;
	
	private AgencyVO startingAgency;
	
	private AgencyVO endingAgency;
	
	

	public ReservationVO() {
		super();
	}
	
	public ReservationVO(ContactVO customer, Set<CarVO> cars, Date startingDate, Date endingDate, AgencyVO startingAgency, AgencyVO endingAgency) {
		super();
		this.customer = customer;
		this.cars = cars;
		this.startingDate = startingDate;
		this.endingDate = endingDate;
		this.startingAgency = startingAgency;
		this.endingAgency = endingAgency;
	}

	public ReservationVO(String reservationId, ContactVO customer,  Set<CarVO> cars, Date startingDate, Date endingDate, AgencyVO startingAgency, AgencyVO endingAgency) {
		super();
		this.reservationId = reservationId;
		this.customer = customer;
		this.cars = cars;
		this.startingDate = startingDate;
		this.endingDate = endingDate;
		this.startingAgency = startingAgency;
		this.endingAgency = endingAgency;
	}
	
	public ReservationVO(Reservation reservation) {
		super();
		this.reservationId = reservation.getReservationId();
		this.customer = new ContactVO(reservation.getCustomer());
		Set<CarVO> cars = new HashSet<CarVO>();
		for (Car car : reservation.getCars()) {
			cars.add(new CarVO(car));
		}
		this.cars = cars;
		this.startingDate = reservation.getStartingDate();
		this.endingDate = reservation.getEndingDate();
		this.startingAgency = new AgencyVO(reservation.getStartingAgency());
		this.endingAgency = new AgencyVO(reservation.getEndingAgency());
	}

	public Set<CarVO> getCars() {
		return cars;
	}

	public void setCars(Set<CarVO> cars) {
		this.cars = cars;
	}

	public ContactVO getCustomer() {
		return customer;
	}

	public void setCustomer(ContactVO customer) {
		this.customer = customer;
	}

	public AgencyVO getEndingAgency() {
		return endingAgency;
	}

	public void setEndingAgency(AgencyVO endingAgency) {
		this.endingAgency = endingAgency;
	}

	public Date getEndingDate() {
		return endingDate;
	}

	public void setEndingDate(Date endingDate) {
		this.endingDate = endingDate;
	}

	public String getReservationId() {
		return reservationId;
	}

	public void setReservationId(String reservationId) {
		this.reservationId = reservationId;
	}

	public AgencyVO getStartingAgency() {
		return startingAgency;
	}

	public void setStartingAgency(AgencyVO startingAgency) {
		this.startingAgency = startingAgency;
	}

	public Date getStartingDate() {
		return startingDate;
	}

	public void setStartingDate(Date startingDate) {
		this.startingDate = startingDate;
	}

	/**
	 * @see java.lang.Object#equals(Object)
	 */
	public boolean equals(Object object) {
		if (!(object instanceof ReservationVO)) {
			return false;
		}
		ReservationVO rhs = (ReservationVO) object;
		return new EqualsBuilder().append(this.endingDate, rhs.endingDate)
				.append(this.cars, rhs.cars).append(this.startingDate, rhs.startingDate).append(
						this.reservationId, rhs.reservationId).append(
						this.customer, rhs.customer).append(this.endingAgency,
						rhs.endingAgency).append(this.startingAgency,
						rhs.startingAgency).isEquals();
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return new HashCodeBuilder(2130538209, 1222600301).append(
				this.endingDate).append(this.cars).append(
				this.startingDate).append(this.reservationId).append(
				this.customer).append(this.endingAgency).append(
				this.startingAgency).toHashCode();
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return new ToStringBuilder(this).append("startingDate",
				this.startingDate).append("car", this.cars).append(
				"reservationId", this.reservationId).append("customer",
				this.customer).append("startingAgency", this.startingAgency)
				.append("endingAgency",
						this.endingAgency)
				.append("endingDate", this.endingDate).toString();
	}

}
