package org.objectweb.rentacar.persistance.dao;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.hibernate.Session;
import org.objectweb.rentacar.persistance.bo.Car;
import org.objectweb.rentacar.persistance.util.HibernateUtil;

/**
 * @author ofabre
 */
public class CarDAO extends GenericDAO {

	private static CarDAO theInstance;

	private CarDAO() {

	}

	public static synchronized CarDAO getInstance() {
		if (null == theInstance) {
			theInstance = new CarDAO();
		}
		return theInstance;
	}

	/**
	 * Returns the list of reservations for the given period of time
	 * @param start Starting date for the search
	 * @param end Ending date for the search
	 * @return List of found reservation matching the criteria
	 * @throws DAOException
	 */
	public Set<Car> getReservedCarsByDates(Date start, Date end) throws DAOException {
		Session session = null;
		try {
			session = HibernateUtil.getSession();
		}
		catch (ExceptionInInitializerError e) {
			throw new DAOException("Error in Hibernate Session instanciation", e);
		}
		catch (Throwable e) {
			throw new DAOException("Error in Hibernate Session instanciation", e);
		}
		org.hibernate.Query q = session.createQuery("select cars from Reservation r where (" + "(r.endingDate <= :end AND r.endingDate >= :start) OR (r.startingDate <= :end AND r.startingDate >= :start)" + "OR (r.startingDate <= :start AND r.endingDate >= :end))");
		q.setDate("start", start);
		q.setDate("end", end);
		List<Car> cars = new ArrayList<Car>();
		for (Iterator iter = q.list().iterator(); iter.hasNext();) {
			cars.add((Car) iter.next());
		}
		Set<Car> result = new HashSet<Car>(cars);
		return result;
	}

	/**
	 * Returns the list of reservations for the given period of time
	 * @param start Starting date for the search
	 * @param end Ending date for the search
	 * @return Set of found reservation matching the criteria
	 * @throws DAOException
	 */
	public Set<Car> getAllAgencyCars() throws DAOException {
		/*
		 * Session session = null; try{ session = HibernateUtil.getSession(); }catch
		 * (ExceptionInInitializerError e){ throw new DAOException("Error in Hibernate
		 * Session instanciation", e); }catch (Throwable e){ throw new DAOException("Error
		 * in Hibernate Session instanciation", e); } org.hibernate.Query q =
		 * session.createQuery("select c from Car c "); List<Car> cars = q.list();
		 */
		List<Car> cars = Arrays.asList(retrieveAll(Car.class).toArray(new Car[10]));				
		Set<Car> result = new HashSet<Car>(cars);
		return result;
	}
}
