package org.objectweb.rentacar.persistance.dao;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.hibernate.Session;
import org.objectweb.rentacar.persistance.bo.Car;
import org.objectweb.rentacar.persistance.bo.Reservation;
import org.objectweb.rentacar.persistance.util.HibernateUtil;

/**
 * 
 * @author ofabre
 *
 */
public class ReservationDAO extends GenericDAO {
	
	private static ReservationDAO theInstance;
	
	private ReservationDAO(){
		
	}
	
	public static synchronized ReservationDAO getInstance() {
		if (null == theInstance) {
			theInstance = new ReservationDAO();
		}
		return theInstance;
	}


	/**
	 * Returns the list of reservations for the given period of time
	 * @param start Starting date for the search
	 * @param end Ending date for the search
	 * @return List of found reservation matching the criteria
	 * @throws DAOException
	 */
	public List<Reservation> getReservationByDates(Date start, Date end) throws DAOException {
		Session session = null;
		try{
			session =  HibernateUtil.getSession();
		}catch (ExceptionInInitializerError e){
			throw new DAOException("Error in Hibernate Session instanciation", e);
		}catch (Throwable e){
			throw new DAOException("Error in Hibernate Session instanciation", e);
		}
		org.hibernate.Query q = session.createQuery("select r from Reservation r where (" +
				"(r.endingDate <= :end AND r.endingDate >= :start) OR (r.startingDate <= :end AND r.startingDate >= :start))");
		q.setDate("start",start);
		q.setDate("end",end);
		List<Reservation >result = new ArrayList<Reservation>();
		for (Iterator iter = q.list().iterator();iter.hasNext();) {
			result.add((Reservation) iter.next());			
		}			
		return result;
	}
	
	/**
	 * Update the cars set of an reservation following the best practice given by Hibernate
	 * @param reservationId
	 * @param newCars
	 * @throws DAOExceptionAgency
	 */
	public void updateReservationCars(String reservationId, Set<Car> newCars) throws DAOException{
		
		Reservation reservation = null;
		
		reservation = (Reservation)retrieveById(Reservation.class, reservationId);
		
		reservation.getCars().clear();
		reservation.getCars().addAll(newCars);
		
		update(reservation);
	}
}
