/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.demo.mortgage.evalrate.sdk;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.objectweb.petals.component.common.HandlingException;
import org.objectweb.petals.tools.jbicommon.util.StringHelper;

/**
 * 
 * @author ofabre
 * 
 */
public class EvalRateImpl implements EvalRate {

	private Logger logger;

	public EvalRateImpl(Logger logger) {
		super();
		this.logger = logger;
	}

	public String evaluate(String profile) throws HandlingException {

		String msg = "";
		String response = "";

		if (StringHelper.isNullOrEmpty(profile)) {
			msg = "You must specify the profil of the mortgage subscriber";
			logger.log(Level.SEVERE, msg);
			throw new HandlingException(msg);
		}

		// Get the string content from the xml content
		String realProfile = profile
				.substring(profile.indexOf("<profile>") + 9, profile
						.indexOf("</profile>"));

		logger
				.info("Eval Rate Module called to evaluate mortgage rate based on the following profile : "
						+ realProfile);
		// Only "yes" profile is allowed for a simple Mortgage
		// Rate evaluation (first Mortgage process)
		if ("Yes".equalsIgnoreCase(realProfile)) {
			response = "5";
		}
		// For a Detailed Mortgage Rate evaluation, three different
		// profiles are allowed : Good, Medium and Risky
		else if ("Good".equalsIgnoreCase(realProfile)) {
			response = "4";
		} else if ("Medium".equalsIgnoreCase(realProfile)) {
			response = "5";
		} else if ("Risky".equalsIgnoreCase(realProfile)) {
			response = "6";
		}

		// Other profile are not allowed
		if ("".equals(response)) {
			msg = "Given mortgage profile isn't allowed: " + realProfile;
			logger.log(Level.SEVERE, "The EvalRate answers a fault: "
					+ response);
			throw new HandlingException(msg);
		} else {
			logger.log(Level.INFO, "The EvalRate answer : " + response);

			// Create xml content from string content
			response = "<response>" + response + "</response>";
		}
		return response;
	}

}
