/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.demo.mortgage.workflow.sdk;

import java.net.URI;

import javax.jbi.JBIException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.objectweb.petals.component.common.HandlingException;
import org.objectweb.petals.component.common.MEPConstants;
import org.objectweb.petals.component.common.se.AbstractServiceEngine;
import org.objectweb.petals.component.common.util.MessageExchangeWrapper;
import org.objectweb.petals.tools.jbicommon.descriptor.Extensions;

/**
 * 
 * @author ofabre
 * 
 */
public class MortgageWorkflowEngine extends AbstractServiceEngine {

	private ServiceEndpoint endpoint1;

	private ServiceEndpoint endpoint2;

	private static final String SERVICE = "MortgageWorkflow";

	private static final QName SERVICE_NAME = new QName(
			"http://petals.objectweb.org/", SERVICE);

	public static final String ENDPOINT1 = "simpleMortgageProfiler";

	public static final String ENDPOINT2 = "detailedMortgageProfiler";

	private MortgageWorkflow workflow;

	@Override
	protected boolean onExchange(MessageExchangeWrapper exchange,
			Extensions extensions) throws HandlingException {
		String outContent = null;
		boolean outResponse = false;

		// Get Operation name
		String opName = exchange.getOperationName();

		// Get Message Exchange Pattern
		URI pattern = exchange.getExchangePattern();

		// Get in Message content
		String stringContent = exchange.getInMessageContent();

		// Get Endpointname
		String endpointName = exchange.getEndpointName();

		if ("orchestrate".equalsIgnoreCase(opName)
				&& pattern.equals(MEPConstants.IN_OPTIONAL_OUT_PATTERN.value())) {
			try {
				outContent = workflow.orchestrate(endpointName, stringContent);
			} catch (Exception e) {
				throw new HandlingException(e);
			}
		} else {
			throw new HandlingException("Operation not allowed (" + opName
					+ ") with the following pattern (" + pattern + ")");
		}

		if (outContent != null) {
			exchange.setOutMessageContent(outContent);
			outResponse = true;
		} else {
			throw new HandlingException(opName
					+ " method returns a null result");
		}

		return outResponse;
	}

	@Override
	public void start() throws JBIException {
		super.start();
		workflow = new MortgageWorkflowImpl(getLogger(), getChannel());
		endpoint1 = getContext().activateEndpoint(SERVICE_NAME, ENDPOINT1);
		endpoint2 = getContext().activateEndpoint(SERVICE_NAME, ENDPOINT2);

	}

	@Override
	public void stop() throws JBIException {
		super.stop();
		workflow = null;
		getContext().deactivateEndpoint(endpoint1);
		getContext().deactivateEndpoint(endpoint2);
	}

}
