/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: Task.java 154 6 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.processor;

import java.util.HashMap;

/**
 * A Task is an execution unit, that can be executed or undo. Tasks can be used
 * by a processor that will execute them and reverse execution in case of fault.
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public interface Task {

    /**
     * Launch the task with the given context. Task can modify information
     * included in this context during execution.
     * 
     * @param context
     *            the execution context handling input and output information
     * @throws Exception
     *             if an error occurs during Task processing
     */
    void execute(HashMap context) throws Exception;

    /**
     * Reverse the task in order to retrieve the environ state before execution
     * of this task.
     * 
     * @param context
     *            the execution context handling input and output information
     * @throws Exception
     *             if an error occurs during Task unprocessing
     */
    void undo(HashMap context) throws Exception;
}
