/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 ? EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: ParameterCheckHelper.java,v 1.2 2005/07/22 10:24:27 wjoseph Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.util;

/**
 * Class managing parameters to checks there nullity or emptyness
 * 
 * @author wjoseph - eBMWebSourcing
 * 
 */
public final class ParameterCheckHelper {

    private static final String MUST_NOT_BE_EMPTY = " must not be empty";

    private static final String MUST_NOT_BE_NULL = " must not be null";

    private ParameterCheckHelper() {
        // avoid to be build
    }

    /**
     * Checks if the parameter is an empty string
     * 
     * @param parameter
     *            parameter to be tested
     * @param parameterName
     *            name of the parameter
     * @throws IllegalArgumentException
     *             if parameter is empty
     */
    public static void isEmptyParameter(String parameter, String parameterName)
        throws IllegalArgumentException {
        if (parameter != null
            && parameter.length() == 0) {
            throw new IllegalArgumentException(parameterName
                + MUST_NOT_BE_EMPTY);
        }
    }

    /**
     * Checks if the parameter is an empty string and log in error if it is
     * empty
     * 
     * @param parameter
     *            parameter to be tested
     * @param parameterName
     *            name of the parameter
     * @param log
     *            logger to use
     * @throws IllegalArgumentException
     *             if parameter is empty
     */
    public static void isEmptyParameterWithLog(String parameter,
        String parameterName, LoggingUtil log) throws IllegalArgumentException {
        if (parameter != null
            && parameter.length() == 0) {
            log.error(parameterName
                + MUST_NOT_BE_EMPTY);
            throw new IllegalArgumentException(parameterName
                + MUST_NOT_BE_EMPTY);
        }
    }

    /**
     * Checks if the parameter is null
     * 
     * @param parameter
     *            parameter to be tested
     * @param parameterName
     *            name of the parameter
     * @throws IllegalArgumentException
     *             if parameter is null
     */
    public static void isNullParameter(Object parameter, String parameterName)
        throws IllegalArgumentException {
        if (parameter == null) {
            throw new IllegalArgumentException(parameterName
                + MUST_NOT_BE_NULL);
        }
    }

    /**
     * Checks if the parameter is null and log in error if it is null
     * 
     * @param parameter
     *            parameter to be tested
     * @param parameterName
     *            name of the parameter
     * @param log
     *            logger to use
     * @throws IllegalArgumentException
     *             if parameter is null
     */
    public static void isNullParameterWithLog(Object parameter,
        String parameterName, LoggingUtil log) throws IllegalArgumentException {
        if (parameter == null) {
            log.error(parameterName
                + MUST_NOT_BE_NULL);
            throw new IllegalArgumentException(parameterName
                + MUST_NOT_BE_NULL);
        }
    }

}
