/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SystemUtil.java 446 2006-05-19 14:13:50Z ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.util;

import java.io.File;
import java.io.FileFilter;
import java.net.JarURLConnection;
import java.net.SocketException;
import java.net.URI;
import java.net.URL;

import org.objectweb.petals.PetalsException;

/**
 * This class is used to store the Petals configuration
 * 
 * @version $Rev: 446 $ $Date: 2006-05-19 16:13:50 +0200 (ven, 19 mai 2006) $
 * @since Petals 1.0
 * @author alouis, Rafael Marins
 * @author ddesjardins - eBMWebsourcing
 * @author Marc Dutoo - Open Wide
 */
public final class SystemUtil {

    /**
     * Petals install directory
     */
    protected static File petalsInstallDirectory;

    /**
     * Name of the container (may change if the name is already taken by another
     * container)
     */
    private static String containerName = "0";

    /**
     * uid of the container
     */
    private static String containerUID;

    /**
     * Is exchange validation enabled
     */
    private static String exchangeValidation;

    /**
     * IP of the host of the petals container
     */
    private static String host;

    /**
     * HTML adaptor port
     */
    private static String htmlPort;

    /**
     * JMX password
     */
    private static String jmxPassword;

    /**
     * JMX server port
     */
    private static String jmxPort;

    /**
     * JMX user
     */
    private static String jmxUser;

    /**
     * JNDI Factory
     */
    private static String jndiFactory;

    /**
     * JNDI port
     */
    private static String jndiPort;

    /**
     * Joram domain
     */
    private static String joramDomain;

    /**
     * Joram Domain Port
     */
    private static String joramDomainPort;

    /**
     * Joram ID
     */
    private static String joramId;

    /**
     * Joram password
     */
    private static String joramPassword;

    /**
     * Joram TCP Port
     */
    private static String joramTCPPort;

    /**
     * Joram User
     */
    private static String joramUser;

    /**
     * Tribe multicast IP
     */
    private static String tribeMulticastAddress;

    private SystemUtil() {
        super();
    }

    public static String getContainerName() {
        return containerName;
    }

    public static long getContainerUID() {
        if (containerUID == null) {
            try {
                containerUID = PropertyUtil.getProperty(
                    PropertyUtil.SERVER_PROPS, "container.uid");
            } catch (PetalsException e) {
                containerUID = "-1";
            }
        }
        return Long.parseLong(containerUID);
    }

    public static String getHost() {
        if (host == null) {

            try {
                host = PropertyUtil.getProperty(PropertyUtil.SERVER_PROPS,
                    "host");
            } catch (PetalsException e) {
                host = resolveLocalIP();
            }
        }
        return host;
    }

    public static String getHtmlPort() {
        if (htmlPort == null) {
            try {
                htmlPort = PropertyUtil.getProperty(PropertyUtil.SERVER_PROPS,
                    "html.port");
            } catch (PetalsException e) {
                htmlPort = "8082";
            }
        }
        return htmlPort;
    }

    public static String getJmxPassword() {
        if (jmxPassword == null) {
            try {
                jmxPassword = PropertyUtil.getProperty(
                    PropertyUtil.SERVER_PROPS, "jmx.pwd");
            } catch (PetalsException e) {
                jmxPassword = "";
            }
        }
        return jmxPassword;
    }

    public static String getJmxPort() {
        if (jmxPort == null) {
            try {
                jmxPort = PropertyUtil.getProperty(PropertyUtil.SERVER_PROPS,
                    "jmx.port");
            } catch (PetalsException e) {
                jmxPort = "8081";
            }
        }
        return jmxPort;
    }

    public static String getJmxUser() {
        if (jmxUser == null) {
            try {
                jmxUser = PropertyUtil.getProperty(PropertyUtil.SERVER_PROPS,
                    "jmx.user");
            } catch (PetalsException e) {
                jmxUser = "";
            }
        }
        return jmxUser;
    }

    public static String getJndiFactory() {
        if (jndiFactory == null) {
            try {
                jndiFactory = PropertyUtil.getProperty(
                    PropertyUtil.SERVER_PROPS, "java.naming.factory.initial");
            } catch (PetalsException e) {
                jndiFactory = "fr.dyade.aaa.jndi2.client.NamingContextFactory";
            }
        }
        return jndiFactory;
    }

    public static String getJndiPort() {
        if (jndiPort == null) {
            try {
                jndiPort = PropertyUtil.getProperty(PropertyUtil.SERVER_PROPS,
                    "java.naming.factory.port");
            } catch (PetalsException e) {
                jndiPort = "16400";
            }
        }
        return jndiPort;
    }

    public static String getJoramDomain() {
        if (joramDomain == null) {
            try {
                joramDomain = PropertyUtil.getProperty(
                    PropertyUtil.SERVER_PROPS, "joram.domain");
            } catch (PetalsException e) {
                joramDomain = "eBM";
            }
        }
        return joramDomain;
    }

    public static String getJoramDomainPort() {
        if (joramDomainPort == null) {
            try {
                joramDomainPort = PropertyUtil.getProperty(
                    PropertyUtil.SERVER_PROPS, "joram.domainport");
            } catch (PetalsException e) {
                joramDomainPort = "16200";
            }
        }
        return joramDomainPort;
    }

    public static String getJoramId() {
        if (joramId == null) {
            try {
                joramId = PropertyUtil.getProperty(PropertyUtil.SERVER_PROPS,
                    "joram.id");
            } catch (PetalsException e) {
                joramId = "0";
            }
        }
        return joramId;
    }

    public static String getJoramPassword() {
        if (joramPassword == null) {
            try {
                joramPassword = PropertyUtil.getProperty(
                    PropertyUtil.SERVER_PROPS, "joram.password");
            } catch (PetalsException e) {
                joramPassword = "petals";
            }
        }
        return joramPassword;
    }

    public static String getJoramTCPPort() {
        if (joramTCPPort == null) {
            try {
                joramTCPPort = PropertyUtil.getProperty(
                    PropertyUtil.SERVER_PROPS, "joram.tcpport");
            } catch (PetalsException e) {
                joramTCPPort = "16300";
            }
        }
        return joramTCPPort;
    }

    public static String getJoramUser() {
        if (joramUser == null) {
            try {
                joramUser = PropertyUtil.getProperty(PropertyUtil.SERVER_PROPS,
                    "joram.user");
            } catch (PetalsException e) {
                joramUser = "petals";
            }
        }
        return joramUser;
    }

    /**
     * Find the petals install directory
     * 
     * @return petals install directory
     * @throws RuntimeException
     *             PetalsInstallDirectory NotFound
     */
    public static File getPetalsInstallDirectory() {
        if (petalsInstallDirectory == null) {
            // first try to resolve install directory set in the environment
            File dir = resolveInstallDirectoryFromEnvironment();
            // otherwise, guess install directory using the framework

            if (dir == null) {
                dir = resolveInstallDirectoryFromFramework();
            }

            // Check that the path is correct :
            // search some /lib/petals....jar file
            if (isInstallDirectoryCorrect(dir)) {
                petalsInstallDirectory = dir;
            }
        }

        if (petalsInstallDirectory == null) {
            throw new RuntimeException(
                "Petals installation directory can not be found. "
                    + "Set a PETALS_HOME variable or use -Dpetals.home system variable.");
        }
        return petalsInstallDirectory;
    }

    public static String getTribeMulticastAddress() {
        if (tribeMulticastAddress == null) {
            try {
                tribeMulticastAddress = PropertyUtil.getProperty(
                    PropertyUtil.SERVER_PROPS, "tribe.multicast");
            } catch (PetalsException e) {
                tribeMulticastAddress = "224.7.65.23";
            }
        }
        return tribeMulticastAddress;
    }

    public static boolean isExchangeValidation() {
        if (exchangeValidation == null) {
            try {
                exchangeValidation = PropertyUtil.getProperty(
                    PropertyUtil.SERVER_PROPS, "exchange.validation", "false");
            } catch (PetalsException e) {
                exchangeValidation = "false";
            }
        }
        return Boolean.parseBoolean(exchangeValidation);
    }

    public static void setContainerName(String containerName) {
        SystemUtil.containerName = containerName;
    }

    public static void setContainerUID(long containerUID) {
        SystemUtil.containerUID = Long.toString(containerUID);
    }

    public static void setExchangeValidation(String exchangeValidation) {
        SystemUtil.exchangeValidation = exchangeValidation;
    }

    public static void setHost(String host) {
        SystemUtil.host = host;
    }

    public static void setHtmlPort(String htmlPort) {
        SystemUtil.htmlPort = htmlPort;
    }

    public static void setJmxPassword(String jmxPassword) {
        SystemUtil.jmxPassword = jmxPassword;
    }

    public static void setJmxPort(String jmxPort) {
        SystemUtil.jmxPort = jmxPort;
    }

    public static void setJmxUser(String jmxUser) {
        SystemUtil.jmxUser = jmxUser;
    }

    public static void setJndiFactory(String jndiFactory) {
        SystemUtil.jndiFactory = jndiFactory;
    }

    public static void setJndiPort(String jndiPort) {
        SystemUtil.jndiPort = jndiPort;
    }

    public static void setJoramDomain(String joramDomain) {
        SystemUtil.joramDomain = joramDomain;
    }

    public static void setJoramDomainPort(String joramPort) {
        SystemUtil.joramDomainPort = joramPort;
    }

    public static void setJoramId(String joramId) {
        SystemUtil.joramId = joramId;
    }

    public static void setJoramPassword(String joramPassword) {
        SystemUtil.joramPassword = joramPassword;
    }

    public static void setJoramTCPPort(String joramTCPPort) {
        SystemUtil.joramTCPPort = joramTCPPort;
    }

    public static void setJoramUser(String joramUser) {
        SystemUtil.joramUser = joramUser;
    }

    public static void setPetalsInstallDirectory(File petalsInstallDirectory) {
        SystemUtil.petalsInstallDirectory = petalsInstallDirectory;
    }

    public static void setTribeMulticastAddress(String tribeMulticastAddress) {
        SystemUtil.tribeMulticastAddress = tribeMulticastAddress;
    }

    /**
     * Resolve the "localhost" IP address to the real IP First, use
     * InetAddress.getLocalHost(). If the returned IP is a "127.0.xxx" like IP,
     * call the system command "ipconfig" or "ifconfig" and parse the result to
     * extract the IP.
     * 
     * @return the real IP address of the machine, or 127.0.0.1 if the
     *         resolution failed.
     */
    protected static String resolveLocalIP() {
        String host = "127.0.0.1";
        String hostIP = "";
        try {
            hostIP = NetworkUtil.getIPv4Address();
        } catch (SocketException e1) {
            // do nothing
        }
        if (hostIP.indexOf("/") > 1) {
            hostIP = hostIP.substring(hostIP.indexOf("/") + 1);
        }
        if (hostIP.startsWith("127.0")) {
            NetworkUtil.getAddressFromSystemCall();
        } else {
            host = hostIP;
        }
        return host;
    }

    /**
     * Check that the given directory contains at least one lib/petalsXXX.jar
     * file.
     * 
     * @param dir
     *            can be null
     * @return the install directory is ok or not
     */
    protected static boolean isInstallDirectoryCorrect(File dir) {
        boolean result = false;

        if (dir != null) {
            File libDir = new File(dir, "lib");

            if (libDir.exists()) {
                File[] libFiles = libDir.listFiles(new FileFilter() {
                    public boolean accept(File f) {
                        return f.getName().toLowerCase().startsWith("petals")
                            && f.getName().toLowerCase().endsWith(".jar");
                    }
                });

                result = libFiles != null && libFiles.length > 0;
            }
        }
        return result;
    }

    /**
     * Try to get the petals'home environment variable : petals.home system
     * property, or PETALS_HOME env variable
     * 
     * @return the related directory, or null
     */
    protected static File resolveInstallDirectoryFromEnvironment() {
        File directory = null;
        String petalsHome = System.getProperty("petals.home");
        if (petalsHome == null || petalsHome.trim().equals("")) {
            petalsHome = System.getenv("PETALS_HOME");
        }
        // not null or empty
        if (petalsHome != null && !petalsHome.trim().equals("")) {
            File theHome = new File(petalsHome);
            // exists and it's a directory
            if (theHome.exists() || theHome.isDirectory()) {
                directory = theHome;
            }
        }
        return directory;
    }

    /**
     * Try to find the install directory from the server.jar
     * 
     * @return the directory or null if a problem occured
     */
    protected static File resolveInstallDirectoryFromFramework() {
        // guess from the location of the server.jar using a marker
        // NB. the resource path uses '/' on Linux as well as on Windows
        URL url = SystemUtil.class.getClassLoader().getResource(
            "META-INF/LICENSE.txt");
        File directory = null;
        if (url != null) {
            try {
                if ("jar".equals(url.getProtocol())) {
                    JarURLConnection jarConnection = (JarURLConnection) url
                        .openConnection();
                    URL resourceJar = jarConnection.getJarFileURL();
                    // resolve parent directory for ${basedir}/bin/server.jar
                    URI baseURI = new URI(resourceJar.toString()).resolve("..");
                    directory = new File(baseURI);
                } else if ("file".equals(url.getProtocol())) {
                    directory = new File(url.toURI());
                }
            } catch (Exception ignored) {
                // ignored
            }
        } else {
            // Failed to load resource \"META-INF/petals-jar
            // installPath not found
        }
        return directory;
    }

}
