/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JNDIUtilTest.java 9:19:04 AM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.util;

import javax.naming.Context;
import javax.naming.Name;
import javax.naming.NameClassPair;
import javax.naming.NamingException;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.petals.util.mock.NamingEnumerationImpl;

/**
 * Test of the JNDIUtil
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class JNDIUtilTest extends TestCase {

    public void testIsBoundNameString() throws SecurityException,
        NoSuchMethodException, NamingException {
        Context context = EasyMock.createMock(Context.class);
        Name name = EasyMock.createMock(Name.class);
        NamingEnumerationImpl<NameClassPair> namingEnumerationImpl = new NamingEnumerationImpl<NameClassPair>(
            new NameClassPair[] {});

        EasyMock.expect(context.list(name.toString())).andReturn(
            namingEnumerationImpl);

        EasyMock.replay(context);
        EasyMock.replay(name);

        assertFalse(JNDIUtil.isBound(context, name, "name"));
    }

    public void testIsBoundNameName() throws SecurityException,
        NoSuchMethodException, NamingException {
        Context context = EasyMock.createMock(Context.class);
        Name name = EasyMock.createMock(Name.class);
        Name name1 = EasyMock.createMock(Name.class);
        NamingEnumerationImpl<NameClassPair> namingEnumerationImpl = new NamingEnumerationImpl<NameClassPair>(
            new NameClassPair[] {});

        EasyMock.expect(context.list(name.toString())).andReturn(
            namingEnumerationImpl);

        EasyMock.replay(context);
        EasyMock.replay(name);
        EasyMock.replay(name1);

        assertFalse(JNDIUtil.isBound(context, name, name1));
    }

    public void testIsBoundStringName() throws SecurityException,
        NoSuchMethodException, NamingException {
        Context context = EasyMock.createMock(Context.class);
        Name name1 = EasyMock.createMock(Name.class);
        NamingEnumerationImpl<NameClassPair> namingEnumerationImpl = new NamingEnumerationImpl<NameClassPair>(
            new NameClassPair[] {});

        EasyMock.expect(context.list("name")).andReturn(namingEnumerationImpl);

        EasyMock.replay(context);
        EasyMock.replay(name1);

        assertFalse(JNDIUtil.isBound(context, "name", name1));
    }

    public void testIsBoundStringString() throws SecurityException,
        NoSuchMethodException, NamingException {
        Context context = EasyMock.createMock(Context.class);
        NamingEnumerationImpl<NameClassPair> namingEnumerationImpl = new NamingEnumerationImpl<NameClassPair>(
            new NameClassPair[] {new NameClassPair("name1", "foo")});

        EasyMock.expect(context.list("name")).andReturn(namingEnumerationImpl);

        EasyMock.replay(context);

        assertTrue(JNDIUtil.isBound(context, "name", "name1"));
    }

    public void testIsBoundStringStringException() throws SecurityException,
        NoSuchMethodException, NamingException {
        Context context = EasyMock.createMock(Context.class);

        EasyMock.expect(context.list("name")).andThrow(new NamingException());

        EasyMock.replay(context);

        assertFalse(JNDIUtil.isBound(context, "name", "name1"));
    }
}
