/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: LoggingUtilTest.java 16:39:12 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.util;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * 
 * @author alouis, ddesjardins - eBMWebsourcing
 */
public class LoggingUtilTest extends TestCase {

    public void testError() {
        RuntimeException runtimeException = new RuntimeException();
        Logger logger = EasyMock.createMock(Logger.class);
        EasyMock.expect(logger.isOn()).andReturn(true).anyTimes();
        logger.log(BasicLevel.LEVEL_ERROR, "test logger", runtimeException);

        EasyMock.replay(logger);

        LoggingUtil loggingUtil = new LoggingUtil(logger);
        loggingUtil.logger = logger;
        loggingUtil.error("test logger", runtimeException);
    }

    public void testError1() {
        Logger logger = EasyMock.createMock(Logger.class);
        EasyMock.expect(logger.isOn()).andReturn(true).anyTimes();
        logger.log(BasicLevel.LEVEL_ERROR, "test logger");

        EasyMock.replay(logger);

        LoggingUtil loggingUtil = new LoggingUtil(logger);
        loggingUtil.error("test logger");
    }

    public void testWarning() {
        RuntimeException runtimeException = new RuntimeException();
        Logger logger = EasyMock.createMock(Logger.class);
        EasyMock.expect(logger.isOn()).andReturn(true).anyTimes();
        logger.log(BasicLevel.LEVEL_WARN, "test logger", runtimeException);

        EasyMock.replay(logger);

        LoggingUtil loggingUtil = new LoggingUtil(logger);
        loggingUtil.warning("test logger", runtimeException);
    }

    public void testWarning1() {
        Logger logger = EasyMock.createMock(Logger.class);
        EasyMock.expect(logger.isOn()).andReturn(true).anyTimes();
        logger.log(BasicLevel.LEVEL_WARN, "test logger");

        EasyMock.replay(logger);

        LoggingUtil loggingUtil = new LoggingUtil(logger);
        loggingUtil.warning("test logger");
    }

    public void testInfo() {
        RuntimeException runtimeException = new RuntimeException();
        Logger logger = EasyMock.createMock(Logger.class);
        EasyMock.expect(logger.isOn()).andReturn(true).anyTimes();
        logger.log(BasicLevel.LEVEL_INFO,
            "LoggingUtilTest.testInfo() test logger", runtimeException);

        EasyMock.replay(logger);

        LoggingUtil loggingUtil = new LoggingUtil(logger);
        loggingUtil.info("test logger", runtimeException);
    }

    public void testInfo1() {
        Logger logger = EasyMock.createMock(Logger.class);
        EasyMock.expect(logger.isOn()).andReturn(true).anyTimes();
        logger.log(BasicLevel.LEVEL_INFO,
            "LoggingUtilTest.testInfo1() test logger");

        EasyMock.replay(logger);

        LoggingUtil loggingUtil = new LoggingUtil(logger);
        loggingUtil.info("test logger");
    }

    public void testDebug() {
        Logger logger = EasyMock.createMock(Logger.class);
        EasyMock.expect(logger.isOn()).andReturn(true).anyTimes();
        logger.log(BasicLevel.LEVEL_DEBUG,
            "LoggingUtilTest.testDebug() test logger");

        EasyMock.replay(logger);

        LoggingUtil loggingUtil = new LoggingUtil(logger);
        loggingUtil.debug("test logger");
    }

    public void testStart() {
        Logger logger = EasyMock.createMock(Logger.class);
        EasyMock.expect(logger.isOn()).andReturn(true).anyTimes();
        logger.log(BasicLevel.LEVEL_DEBUG,
            "-START- LoggingUtilTest.testStart() test logger");

        EasyMock.replay(logger);

        LoggingUtil loggingUtil = new LoggingUtil(logger);
        loggingUtil.start("test logger");
    }

    public void testStart1() {
        Logger logger = EasyMock.createMock(Logger.class);
        EasyMock.expect(logger.isOn()).andReturn(true).anyTimes();
        logger.log(BasicLevel.LEVEL_DEBUG,
            "-START- LoggingUtilTest.testStart1()");

        EasyMock.replay(logger);

        LoggingUtil loggingUtil = new LoggingUtil(logger);
        loggingUtil.start();
    }

    public void testEnd() {
        Logger logger = EasyMock.createMock(Logger.class);
        EasyMock.expect(logger.isOn()).andReturn(true).anyTimes();
        logger.log(BasicLevel.LEVEL_DEBUG,
            "- END - LoggingUtilTest.testEnd() test logger");

        EasyMock.replay(logger);

        LoggingUtil loggingUtil = new LoggingUtil(logger);
        loggingUtil.end("test logger");
    }

    public void testEnd1() {
        Logger logger = EasyMock.createMock(Logger.class);
        EasyMock.expect(logger.isOn()).andReturn(true).anyTimes();
        logger
            .log(BasicLevel.LEVEL_DEBUG, "- END - LoggingUtilTest.testEnd1()");

        EasyMock.replay(logger);

        LoggingUtil loggingUtil = new LoggingUtil(logger);
        loggingUtil.end();
    }

    public void testCall() {
        Logger logger = EasyMock.createMock(Logger.class);
        EasyMock.expect(logger.isOn()).andReturn(true).anyTimes();
        logger
            .log(BasicLevel.LEVEL_DEBUG, "-CALL - LoggingUtilTest.testCall()");

        EasyMock.replay(logger);

        LoggingUtil loggingUtil = new LoggingUtil(logger);
        loggingUtil.call();
    }

    public void testCall1() {
        Logger logger = EasyMock.createMock(Logger.class);
        EasyMock.expect(logger.isOn()).andReturn(true).anyTimes();
        logger.log(BasicLevel.LEVEL_DEBUG,
            "-CALL - LoggingUtilTest.testCall1() test logger");

        EasyMock.replay(logger);

        LoggingUtil loggingUtil = new LoggingUtil(logger);
        loggingUtil.call("test logger");
    }

    public void testGetName() {
        Logger logger = EasyMock.createMock(Logger.class);

        EasyMock.expect(logger.getName()).andReturn("name");

        EasyMock.replay(logger);
        LoggingUtil loggingUtil = new LoggingUtil(logger);
        assertEquals(loggingUtil.getName(), "name");
    }

}
