/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: PropertyUtilTest.java 9:37:14 AM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.util;

import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URL;
import java.util.Properties;

import org.objectweb.petals.PetalsException;

import junit.framework.TestCase;

/**
 * Test of the PropertyUtil
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class PropertyUtilTest extends TestCase {

    public void testRetrieveJNDIProperties() {
        SystemUtil.setJndiPort("16400");
        SystemUtil.setJndiFactory("test");
        SystemUtil.setHost("127.0.0.1");
        Properties props = PropertyUtil.retrieveJNDIProperties();
        assertEquals(props.getProperty("java.naming.factory.host"), "127.0.0.1");
        assertEquals(props.getProperty("java.naming.factory.port"), "16400");
        assertEquals(props.getProperty("java.naming.factory.host"), "127.0.0.1");
    }

    public void testUpdateContainerProperties() throws IOException,
        PetalsException {
        URL url = PropertyUtil.class.getResource(PropertyUtil.SERVER_PROPS);
        Properties properties = new Properties();
        properties.load(url.openStream());
        properties.setProperty("joram.id", "-1");
        properties.store(new FileOutputStream(url.getFile()), "Test");
        PropertyUtil.updateContainerProperties("0", "16200", "16300", "8081",
            "8082", "16400", 1);
        assertEquals(PropertyUtil.getProperty(PropertyUtil.SERVER_PROPS,
            "joram.id"), "0");
    }

    public void testGetPropertyExceptionFile() {
        try {
            PropertyUtil.getProperty("test", "foo");
            fail();
        } catch (PetalsException e) {
            // do nothing
        }
    }
    
    public void testGetPropertyExceptionKey() {
        try {
            PropertyUtil.getProperty(PropertyUtil.SERVER_PROPS, "foo");
            fail();
        } catch (PetalsException e) {
            // do nothing
        }
    }
}
