/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ComponentInitialContext.java 154 14 sept. 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.component.context;

import java.util.Hashtable;

import javax.naming.Binding;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.Name;
import javax.naming.NameClassPair;
import javax.naming.NameParser;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;

/**
 * This class inherits from an InitialContext and delegates method calls to a
 * JNDI subcontext.
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class ComponentInitialContext extends InitialContext {

    public ComponentInitialContext(Context usersContext) throws NamingException {
        super();
        this.usersContext = usersContext;
    }

    protected Context usersContext;

    public Object addToEnvironment(String propName, Object propVal)
        throws NamingException {
        return usersContext.addToEnvironment(propName, propVal);
    }

    public void bind(Name name, Object obj) throws NamingException {
        usersContext.bind(name, obj);
    }

    public void bind(String name, Object obj) throws NamingException {
        usersContext.bind(name, obj);
    }

    public void close() throws NamingException {
        usersContext.close();
    }

    public Name composeName(Name name, Name prefix) throws NamingException {
        return usersContext.composeName(name, prefix);
    }

    public String composeName(String name, String prefix)
        throws NamingException {
        return usersContext.composeName(name, prefix);
    }

    public Context createSubcontext(Name name) throws NamingException {
        return usersContext.createSubcontext(name);
    }

    public Context createSubcontext(String name) throws NamingException {
        return usersContext.createSubcontext(name);
    }

    public void destroySubcontext(Name name) throws NamingException {
        usersContext.destroySubcontext(name);
    }

    public void destroySubcontext(String name) throws NamingException {
        usersContext.destroySubcontext(name);
    }

    public Hashtable<?, ?> getEnvironment() throws NamingException {
        return usersContext.getEnvironment();
    }

    public String getNameInNamespace() throws NamingException {
        return usersContext.getNameInNamespace();
    }

    public NameParser getNameParser(Name name) throws NamingException {
        return usersContext.getNameParser(name);
    }

    public NameParser getNameParser(String name) throws NamingException {
        return usersContext.getNameParser(name);
    }

    public NamingEnumeration<NameClassPair> list(Name name)
        throws NamingException {
        return usersContext.list(name);
    }

    public NamingEnumeration<NameClassPair> list(String name)
        throws NamingException {
        return usersContext.list(name);
    }

    public NamingEnumeration<Binding> listBindings(Name name)
        throws NamingException {
        return usersContext.listBindings(name);
    }

    public NamingEnumeration<Binding> listBindings(String name)
        throws NamingException {
        return usersContext.listBindings(name);
    }

    public Object lookup(Name name) throws NamingException {
        return usersContext.lookup(name);
    }

    public Object lookup(String name) throws NamingException {
        return usersContext.lookup(name);
    }

    public Object lookupLink(Name name) throws NamingException {
        return usersContext.lookupLink(name);
    }

    public Object lookupLink(String name) throws NamingException {
        return usersContext.lookupLink(name);
    }

    public void rebind(Name name, Object obj) throws NamingException {
        usersContext.rebind(name, obj);
    }

    public void rebind(String name, Object obj) throws NamingException {
        usersContext.rebind(name, obj);
    }

    public Object removeFromEnvironment(String propName) throws NamingException {
        return usersContext.removeFromEnvironment(propName);
    }

    public void rename(Name oldName, Name newName) throws NamingException {
        usersContext.rename(oldName, newName);
    }

    public void rename(String oldName, String newName) throws NamingException {
        usersContext.rename(oldName, newName);
    }

    public void unbind(Name name) throws NamingException {
        usersContext.unbind(name);
    }

    public void unbind(String name) throws NamingException {
        usersContext.unbind(name);
    }

}
