/** 
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: ComponentLifeCycle.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.jbi.component.lifecycle;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.management.ObjectName;

import org.objectweb.petals.jbi.component.context.ComponentContextImpl;
import org.objectweb.petals.jbi.component.thread.ComponentLifeCycleThread;
import org.objectweb.petals.jbi.management.systemstate.SystemState;
import org.objectweb.petals.jbi.messaging.DeliveryChannelImpl;
import org.objectweb.petals.tools.jbicommon.descriptor.ComponentDescription;
import org.objectweb.petals.util.LoggingUtil;

/**
 * Implementation of the <code>ComponentLifeCycle</code> mbean
 * 
 * @author Adrien LOUIS - EBM WebSourcing
 * @author wjoseph - eBM-WebSourcing
 */
public class ComponentLifeCycle extends LifeCycleAbstract implements
    ComponentLifeCycleMBean {

    /**
     * The component linked with the current component life cycle
     */
    protected Component component;

    /**
     * The component description 
     */
    private ComponentDescription componentDescription;

    /**
     * The component context
     */
    private ComponentContextImpl context;

    /**
     * 
     */
    private SystemState recoverySrv;
    
    /**
     * The component life cycle thread. Each life cycle have a thread to process
     * operation in the component ClassLoader
     */
    protected ComponentLifeCycleThread componentLifeCycleThread;

    /**
     * 
     * @param componentDescription
     * @param component
     * @param context
     * @param recoverySrv
     * @param mbeanName
     * @param log
     */
    public ComponentLifeCycle(ComponentDescription componentDescription,
        Component component, ComponentContextImpl context,
        SystemState recoverySrv, ObjectName mbeanName,
        ComponentLifeCycleThread componentLifeCycleThread, LoggingUtil log) {
        super(mbeanName, log);

        this.componentDescription = componentDescription;

        this.component = component;

        this.context = context;

        this.recoverySrv = recoverySrv;
        
        // set the jbiComponent ref to the contextImpl
        if (context != null) {
            context.setComponent(component);
        }

        // life cycle thread
        this.componentLifeCycleThread = componentLifeCycleThread;
    }

    @Override
    public void doInit() throws JBIException {
        log.start();
        componentLifeCycleThread.doInit(context);
        log.end();
    }

    @Override
    public void doShutdown() throws JBIException {
        log.start();
        componentLifeCycleThread.doShutdown();

        // close the Channel if required
        DeliveryChannelImpl channel = context.getDeliveryChannelImpl();
        if (channel != null && channel.isOpened())
            channel.close();
        
        log.end();
    }

    @Override
    public void doStart() throws JBIException {
        log.start();
        componentLifeCycleThread.doStart();
        log.end();
    }

    @Override
    public void doStop() throws JBIException {
        log.start();
        componentLifeCycleThread.doStop();
        log.end();
    }

    /**
     * @see javax.jbi.management.ComponentLifeCycleMBean#getExtensionMBeanName()
     */
    public ObjectName getExtensionMBeanName() throws JBIException {
        log.start();
        ObjectName on = componentLifeCycleThread.getExtensionMBeanName();
        log.end();
        return on;
    }

    /**
     * 
     * @return
     */
    public String getName() {
        return componentDescription.getIdentification().getName();
    }

    @Override
    public synchronized void setState(String state) throws JBIException {
        super.setState(state);
        try {
            recoverySrv.updateComponentLifeCycleState(getName(), state);
        } catch (Exception e) {
            String msg = "Component state can't be persisted for recovery";
            log.error(msg, e);
            throw new JBIException(msg, e);
        }
    }

    /* getters */
    
    public Component getComponent() {
        return component;
    }

    public ComponentContextImpl getComponentContext() {
        return context;
    }

    public ComponentDescription getComponentDescription() {
        return componentDescription;
    }

}
