/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: InstallationService.java 154 2006-03-27 15:30:10Z ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.component.lifecycle;

import javax.jbi.JBIException;
import javax.jbi.component.ServiceUnitManager;

import org.objectweb.petals.jbi.component.event.StateChangeFailedEvent;
import org.objectweb.petals.jbi.component.event.StateChangedEvent;
import org.objectweb.petals.jbi.management.service.LifeCycleManagerService;
import org.objectweb.petals.tools.jbicommon.descriptor.ServiceUnit;
import org.objectweb.petals.util.LoggingUtil;

/**
 * This class manage the life cycle of deployed service units
 * 
 * @author Olivier Fabre - EBM Websourcing
 * 
 */
public class ServiceUnitLifeCycle extends LifeCycleAbstract {

    private LifeCycleManagerService managerService;

    private ServiceUnit serviceUnit;

    private String suName;

    private String serviceUnitRootPath;

    public ServiceUnitLifeCycle(ServiceUnit serviceUnit,
            LifeCycleManagerService managerService, String serviceUnitRootPath,
            LoggingUtil log) {
        super(null, log);
        this.managerService = managerService;
        this.serviceUnit = serviceUnit;
        this.serviceUnitRootPath = serviceUnitRootPath;
        this.suName = serviceUnit.getIdentification().getName();
    }

    /**
     * Returns the ServiceUnitManager associated with the service unit's
     * targeted component
     * 
     * @return null if there's no Service Unit Manager
     */
    public ServiceUnitManager getServiceUnitManager() {
        ServiceUnitManager result = null;
        ComponentLifeCycle clc = managerService.getComponentByName(serviceUnit
                .getTargetComponentName());
        if (clc != null) {
            result = clc.getComponent().getServiceUnitManager();
        }
        return result;
    }

    public String getServiceUnitRootPath() {
        return serviceUnitRootPath;
    }

    public String getSuName() {
        return suName;
    }

    /**
     * Do nothing because there's no Service Unit MBean registered in JMX !
     */
    @Override
    public synchronized void stateChanged(StateChangedEvent event) {

    }

    /**
     * Do nothing because there's no Service Unit MBean registered in JMX !
     */
    @Override
    public synchronized void stateChangeFailed(StateChangeFailedEvent event) {

    }

    @Override
    public void doInit() throws JBIException {
        getServiceUnitManager().init(suName, serviceUnitRootPath);
    }

    @Override
    public void doStart() throws JBIException {
        getServiceUnitManager().start(suName);
    }

    @Override
    public void doStop() throws JBIException {
        getServiceUnitManager().stop(suName);
    }

    @Override
    public void doShutdown() throws JBIException {
        getServiceUnitManager().shutDown(suName);
    }

}
