/** 
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------  
 */
package org.objectweb.petals.jbi.component.thread;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.jbi.component.ComponentContext;
import javax.management.ObjectName;

/**
 * The component life cycle thread. Each component life cycle have its own
 * thread to proces operation that requires own ClassLoader.
 * 
 * <b>NOTE : This must implements the javax.jbi.component.ComponentLifeCycle
 * interface but due to the use of Thread and the start/stop methods it is
 * impossible to do this. LET'S SEE IF FUTURE JAVA VERSION ALLOWS TO DO THINGS
 * LIKE THAT</b>
 * 
 * @author Christophe Hamerling - EBM WebSourcing
 * 
 */
public class ComponentLifeCycleThread extends AbstractThread {

    /**
     * The component
     */
    private Component component;

    /**
     * The component context
     */
    private ComponentContext context;

    /**
     * 
     */
    private ObjectName objectName;

    /*
     * STATICS : The actions to process
     */
    public static final int INIT = 0;

    public static final int START = 1;

    public static final int STOP = 2;

    public static final int SHUTDOWN = 3;

    public static final int MBEANNAME = 4;

    /**
     * Creates a new instance of ComponentLifeCycle
     * 
     * @param component
     * @param context
     */
    public ComponentLifeCycleThread(Component component) {
        super();
        this.component = component;
        this.setName(component.toString() + "-life Cycle Thread");
    }

    /**
     * Excute the task.
     * 
     * @return -1 if the current thread needs to be shutdowned
     */
    protected int doTask(int action) {
        int result = 0;
        try {
            switch (action) {
                case INIT:
                    getComponent().getLifeCycle().init(context);
                    break;

                case START:
                    getComponent().getLifeCycle().start();
                    break;

                case STOP:
                    getComponent().getLifeCycle().stop();
                    break;

                case SHUTDOWN:
                    getComponent().getLifeCycle().shutDown();
                    break;

                case MBEANNAME:
                    objectName = getComponent().getLifeCycle()
                        .getExtensionMBeanName();
                    break;

                case SHUTDOWNTHREAD:
                    result = -1;
                    break;

                default:
                    // NOTHING
                    break;
            }

        } catch (JBIException jbie) {
            jbiException = jbie;
        }

        return result;
    }

    /**
     * @return the component
     */
    public Component getComponent() {
        return component;
    }

    /**
     * Start life cycle step
     * 
     */
    public void doStart() throws JBIException {
        execute(START);

        JBIException jbie = getJbiException();
        if (jbie != null) {
            throw jbie;
        }
    }

    /**
     * Stop life cycle step
     * 
     */
    public void doStop() throws JBIException {
        execute(STOP);

        JBIException jbie = getJbiException();
        if (jbie != null) {
            throw jbie;
        }
    }

    /**
     * Shutdown life cycle step
     * 
     */
    public void doShutdown() throws JBIException {
        execute(SHUTDOWN);

        JBIException jbie = getJbiException();
        if (jbie != null) {
            throw jbie;
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.ComponentLifeCycle#init(javax.jbi.component.ComponentContext)
     */
    public void doInit(ComponentContext context) throws JBIException {
        this.context = context;
        execute(INIT);

        JBIException jbie = getJbiException();
        if (jbie != null) {
            throw jbie;
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.ComponentLifeCycle#getExtensionMBeanName()
     */
    public ObjectName getExtensionMBeanName() {
        execute(MBEANNAME);
        return objectName;
    }
}