/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: InstalledDirectoryScanner.java,v 1.2 2006/04/07 10:24:27 ofabre Exp $
 * --------------------------------------------------------------------------  
 */
package org.objectweb.petals.jbi.management.autoload;

import java.io.File;
import java.io.FilenameFilter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.TimerTask;

/**
 * Periodicly check if new Install or Deploy elements are presents in the
 * autoload directories.
 * 
 * @author ofabre
 * 
 */
public class InstalledDirectoryScanner extends TimerTask {

    /**
     * filter for autoloading. Return true if the file is an archive : JAR or
     * ZIP.
     * 
     * @author alouis
     * 
     */
    private class ArchiveFileFilter implements FilenameFilter {

        /**
         * @see java.io.FilenameFilter#accept(java.io.File, java.lang.String)
         */
        public boolean accept(File dir, String name) {

            String lcName = name.toLowerCase();

            if (lcName.endsWith("jar") || lcName.endsWith("zip")) {
                return true;
            }
            return false;
        }

    }

    ArchiveFileFilter archiveFilter;

    AutoLoaderImpl autoLoader;

    File installedDir;

    List<File> previousFileList = new ArrayList<File>();

    public InstalledDirectoryScanner(AutoLoaderImpl autoLoader,
            File installedDir) {
        this.installedDir = installedDir;

        this.autoLoader = autoLoader;

        archiveFilter = new ArchiveFileFilter();
    }

    public void run() {
        
        List<File> newFileList = null;
        File[] files = installedDir.listFiles(archiveFilter);
        if(files != null){
            newFileList = Arrays.asList(files);
        }else{
            newFileList = new ArrayList<File>();
        }
        List<File> filesToUninstall = new ArrayList<File>();

        for (File file : previousFileList) {
            if (!newFileList.contains(file)) {
                filesToUninstall.add(file);
            }
        }

        if (filesToUninstall != null && !filesToUninstall.isEmpty()) {
            autoLoader.uninstall(filesToUninstall);
        }

        previousFileList.clear();
        previousFileList.addAll(newFileList);
    }
}
