/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: DeploymentTaskFactory.java 154 6 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.deployment;

import org.objectweb.petals.jbi.management.deployment.deploy.AllConnectionRegistrationTask;
import org.objectweb.petals.jbi.management.deployment.deploy.AllSUDeploymentTask;
import org.objectweb.petals.jbi.management.deployment.deploy.CopyArchiveToInstalledDirTask;
import org.objectweb.petals.jbi.management.deployment.deploy.CreateAndRegisterSULifeCycleTask;
import org.objectweb.petals.jbi.management.deployment.deploy.DescriptorAndURIRetrievalTask;
import org.objectweb.petals.jbi.management.deployment.deploy.SAAndSUExtractionTask;
import org.objectweb.petals.jbi.management.deployment.deploy.SAAndSUInstallRootRetrievalTask;
import org.objectweb.petals.jbi.management.deployment.deploy.SADeploymentSuccessTask;
import org.objectweb.petals.jbi.management.deployment.deploy.SALifeCycleRegistrationTask;
import org.objectweb.petals.jbi.management.deployment.deploy.SAPackageCheckingTask;
import org.objectweb.petals.jbi.management.deployment.deploy.SAStateHolderCreationTask;
import org.objectweb.petals.jbi.management.deployment.deploy.SUToComponentDeploymentTask;
import org.objectweb.petals.jbi.management.deployment.deploy.XMLDeploymentResultCreationTask;
import org.objectweb.petals.jbi.management.deployment.undeploy.AllConnectionsRemovalTask;
import org.objectweb.petals.jbi.management.deployment.undeploy.AllSUUndeploymentTask;
import org.objectweb.petals.jbi.management.deployment.undeploy.DeleteArchiveFromInstalledDirTask;
import org.objectweb.petals.jbi.management.deployment.undeploy.SACacheRemovalTask;
import org.objectweb.petals.jbi.management.deployment.undeploy.SAFolderRemovalTask;
import org.objectweb.petals.jbi.management.deployment.undeploy.SALifeCycleRetrievalTask;
import org.objectweb.petals.jbi.management.deployment.undeploy.SALifeCycleUnregistrationTask;
import org.objectweb.petals.jbi.management.deployment.undeploy.SAStateHolderRemovalTask;
import org.objectweb.petals.jbi.management.deployment.undeploy.XMLUndeploymentResultCreationTask;
import org.objectweb.petals.jbi.management.service.EndpointService;
import org.objectweb.petals.jbi.management.service.LifeCycleManagerService;
import org.objectweb.petals.jbi.management.service.PackageHandler;
import org.objectweb.petals.jbi.management.systemstate.SystemState;
import org.objectweb.petals.repository.RepositoryService;
import org.objectweb.petals.util.LoggingUtil;

/**
 * This Factory creates Deployment Tasks.
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class DeploymentTaskFactory {

    /**
     * Package Handler
     */
    protected PackageHandler packageHandler;

    protected EndpointService endpointService;

    protected DeploymentServiceImpl deploymentService;

    /**
     * JBI Container :: Recovery Service
     */
    protected SystemState recoverySrv;

    /**
     * Platform Component :: Repository Service
     */
    protected RepositoryService repositorySrv;

    /**
     * logger wrapper
     */
    protected LoggingUtil log;

    /**
     * JBI Container :: JMX Agent Admin
     */
    protected LifeCycleManagerService managerService;

    protected DescriptorAndURIRetrievalTask descriptorAndURIRetrievalTask;

    protected XMLDeploymentResultCreationTask resultCreationTask;

    protected SAPackageCheckingTask packageCheckingTask;

    protected SAAndSUExtractionTask extractionTask;

    protected AllSUDeploymentTask allSUDeploymentTask;

    protected SALifeCycleRegistrationTask saLifeCycleRegistrationTask;

    protected AllConnectionRegistrationTask allConnectionRegistrationTask;

    protected SAStateHolderCreationTask stateHolderCreationTask;

    protected CopyArchiveToInstalledDirTask copyArchiveToInstalledDirTask;

    protected SADeploymentSuccessTask deploymentSuccessTask;

    protected SUToComponentDeploymentTask suToComponentDeploymentTask;

    protected CreateAndRegisterSULifeCycleTask createAndRegisterSULifeCycleTask;

    protected SAAndSUInstallRootRetrievalTask saAndSUInstallRootRetrievalTask;

    protected AllConnectionsRemovalTask allConnectionsRemovalTask;

    protected AllSUUndeploymentTask allSUUndeploymentTask;

    protected SACacheRemovalTask saCacheRemovalTask;

    protected SAFolderRemovalTask saFolderRemovalTask;
    
    protected DeleteArchiveFromInstalledDirTask deleteArchiveFromInstalledDirTask;

    protected SALifeCycleRetrievalTask saLifeCycleRetrievalTask;

    protected SALifeCycleUnregistrationTask saLifeCycleUnregistrationTask;

    protected SAStateHolderRemovalTask saStateHolderRemovalTask;

    protected XMLUndeploymentResultCreationTask xmlUndeploymentResultCreationTask;

    public DeploymentTaskFactory(PackageHandler packageHandler,
            EndpointService endpointService,
            DeploymentServiceImpl deploymentService, SystemState recoverySrv,
            RepositoryService repositorySrv, LoggingUtil log,
            LifeCycleManagerService managerService) {
        super();
        this.packageHandler = packageHandler;
        this.endpointService = endpointService;
        this.deploymentService = deploymentService;
        this.recoverySrv = recoverySrv;
        this.repositorySrv = repositorySrv;
        this.log = log;
        this.managerService = managerService;

        /*
         * Create Tasks
         */
        descriptorAndURIRetrievalTask = new DescriptorAndURIRetrievalTask(
                packageHandler);

        resultCreationTask = new XMLDeploymentResultCreationTask();

        packageCheckingTask = new SAPackageCheckingTask(log, deploymentService);

        extractionTask = new SAAndSUExtractionTask(packageHandler,
                repositorySrv, log);

        allSUDeploymentTask = new AllSUDeploymentTask(recoverySrv,
                repositorySrv, managerService, deploymentService, log);

        saLifeCycleRegistrationTask = new SALifeCycleRegistrationTask(
                managerService, log);

        allConnectionRegistrationTask = new AllConnectionRegistrationTask(
                endpointService, log);

        stateHolderCreationTask = new SAStateHolderCreationTask(recoverySrv,
                repositorySrv, log);

        copyArchiveToInstalledDirTask = new CopyArchiveToInstalledDirTask(log);

        deploymentSuccessTask = new SADeploymentSuccessTask(deploymentService);

        suToComponentDeploymentTask = new SUToComponentDeploymentTask(log,
                managerService);

        createAndRegisterSULifeCycleTask = new CreateAndRegisterSULifeCycleTask(
                log, managerService);

        saAndSUInstallRootRetrievalTask = new SAAndSUInstallRootRetrievalTask(
                repositorySrv, packageHandler);

        allConnectionsRemovalTask = new AllConnectionsRemovalTask(
                endpointService);

        allSUUndeploymentTask = new AllSUUndeploymentTask(managerService, log);

        saCacheRemovalTask = new SACacheRemovalTask(deploymentService);

        saFolderRemovalTask = new SAFolderRemovalTask();

        saLifeCycleRetrievalTask = new SALifeCycleRetrievalTask(managerService,
                log);

        saLifeCycleUnregistrationTask = new SALifeCycleUnregistrationTask(
                managerService);

        saStateHolderRemovalTask = new SAStateHolderRemovalTask(recoverySrv);

        xmlUndeploymentResultCreationTask = new XMLUndeploymentResultCreationTask();
        
        deleteArchiveFromInstalledDirTask = new DeleteArchiveFromInstalledDirTask();

    }

    public AllConnectionRegistrationTask getAllConnectionRegistrationTask() {
        return allConnectionRegistrationTask;
    }

    public AllSUDeploymentTask getAllSUDeploymentTask() {
        return allSUDeploymentTask;
    }

    public CopyArchiveToInstalledDirTask getCopyArchiveToInstalledDirTask() {
        return copyArchiveToInstalledDirTask;
    }

    public CreateAndRegisterSULifeCycleTask getCreateAndRegisterSULifeCycleTask() {
        return createAndRegisterSULifeCycleTask;
    }

    public SADeploymentSuccessTask getDeploymentSuccessTask() {
        return deploymentSuccessTask;
    }

    public DescriptorAndURIRetrievalTask getDescriptorAndURIRetrievalTask() {
        return descriptorAndURIRetrievalTask;
    }

    public SAAndSUExtractionTask getExtractionTask() {
        return extractionTask;
    }

    public SAPackageCheckingTask getPackageCheckingTask() {
        return packageCheckingTask;
    }

    public XMLDeploymentResultCreationTask getResultCreationTask() {
        return resultCreationTask;
    }

    public SALifeCycleRegistrationTask getSaLifeCycleRegistrationTask() {
        return saLifeCycleRegistrationTask;
    }

    public SAStateHolderCreationTask getStateHolderCreationTask() {
        return stateHolderCreationTask;
    }

    public SUToComponentDeploymentTask getSuToComponentDeploymentTask() {
        return suToComponentDeploymentTask;
    }

    public AllConnectionsRemovalTask getAllConnectionsRemovalTask() {
        return allConnectionsRemovalTask;
    }

    public AllSUUndeploymentTask getAllSUUndeploymentTask() {
        return allSUUndeploymentTask;
    }

    public SACacheRemovalTask getSaCacheRemovalTask() {
        return saCacheRemovalTask;
    }

    public SAFolderRemovalTask getSaFolderRemovalTask() {
        return saFolderRemovalTask;
    }

    public SALifeCycleRetrievalTask getSaLifeCycleRetrievalTask() {
        return saLifeCycleRetrievalTask;
    }

    public SALifeCycleUnregistrationTask getSaLifeCycleUnregistrationTask() {
        return saLifeCycleUnregistrationTask;
    }

    public SAStateHolderRemovalTask getSaStateHolderRemovalTask() {
        return saStateHolderRemovalTask;
    }

    public XMLUndeploymentResultCreationTask getXmlUndeploymentResultCreationTask() {
        return xmlUndeploymentResultCreationTask;
    }

    public SAAndSUInstallRootRetrievalTask getSaAndSUInstallRootRetrievalTask() {
        return saAndSUInstallRootRetrievalTask;
    }

	public DeleteArchiveFromInstalledDirTask getDeleteArchiveFromInstalledDirTask() {
		return deleteArchiveFromInstalledDirTask;
	}

}
