/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: DeploymentUtils.java 154 6 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.deployment;

import java.io.File;
import java.net.URI;

import org.objectweb.petals.jbi.management.autoload.AutoLoaderImpl;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;
import org.objectweb.petals.tools.jbicommon.descriptor.ServiceUnit;

public class DeploymentUtils {

    /**
     * Retrieve the service assembly name found in the SA production element of
     * the given descriptor
     * 
     * @param descriptor
     *            the service assembly jbi descriptor
     * @return the service assembly name
     */
    public static String getServiceAssemblyName(JBIDescriptor descriptor) {
        return descriptor.getServiceAssembly().getIdentification().getName();
    }

    /**
     * Retrieve the service unit name found in the SU production element
     * 
     * @param serviceUnit
     *            the service unit production element
     * @return the service unit name
     */
    public static String getServiceUnitName(ServiceUnit serviceUnit) {
        return serviceUnit.getIdentification().getName();
    }

    /**
     * Retrieve the service unit targeted component found in the SU production
     * element
     * 
     * @param serviceUnit
     *            the service unit production element
     * @return the service unit targeted component
     */
    public static String getServiceUnitTargetedComponent(ServiceUnit serviceUnit) {
        return serviceUnit.getTargetComponentName();
    }

    /**
     * Get the file according to the archive URI
     * 
     * @param archiveURI
     *            URI of the archive
     * @return a File pointing on the archive
     */
    public static File getArchiveFile(URI archiveURI) {
        return new File(archiveURI);
    }

    /**
     * Get a file of the given name in the "installed directory"
     * 
     * @param file
     *            the file to look for
     * @return a File pointing on the file name in the "installed directory"
     */
    public static File getInstalledArchive(File file) {
        return new File(AutoLoaderImpl.getInstalledDirectory(), file.getName());
    }

}
