/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SALifeCycleRegistrationTask.java 154 6 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.deployment.deploy;

import java.util.HashMap;

import org.objectweb.petals.jbi.component.lifecycle.ServiceAssemblyLifeCycle;
import org.objectweb.petals.jbi.management.deployment.DeploymentContextConstants;
import org.objectweb.petals.jbi.management.deployment.DeploymentUtils;
import org.objectweb.petals.jbi.management.service.LifeCycleManagerService;
import org.objectweb.petals.jbi.management.service.ManagementException;
import org.objectweb.petals.processor.Task;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;
import org.objectweb.petals.util.LoggingUtil;

/**
 * This task registers ServiceAssemblyLifeCycle into manager service map.
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class SALifeCycleRegistrationTask implements Task {

    /**
     * JBI Container :: JMX Agent Admin
     */
    protected LifeCycleManagerService managerService;

    /**
     * logger wrapper
     */
    protected LoggingUtil log;

    public SALifeCycleRegistrationTask(LifeCycleManagerService managerService,
            LoggingUtil log) {
        super();
        this.managerService = managerService;
        this.log = log;
    }

    public void execute(HashMap context) throws Exception {
        JBIDescriptor descriptor = (JBIDescriptor) context
                .get(DeploymentContextConstants.SA_DESCRIPTOR);

        ServiceAssemblyLifeCycle lifeCycle = (ServiceAssemblyLifeCycle) context
                .get(DeploymentContextConstants.SA_LIFECYCLE);

        managerService.registerServiceAssembly(DeploymentUtils
                .getServiceAssemblyName(descriptor), lifeCycle);

    }

    public void undo(HashMap context) {
        JBIDescriptor descriptor = (JBIDescriptor) context
                .get(DeploymentContextConstants.SA_DESCRIPTOR);

        try {
            managerService.unregisterServiceAssembly(DeploymentUtils
                    .getServiceAssemblyName(descriptor));
        } catch (ManagementException e) {
            String msg = "Failed to revert a SALifeCycleRegistrationTask";
            log.error(msg, e);
        }
    }

}
