/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SAStateHolderCreationTask.java 154 6 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.deployment.deploy;

import java.io.File;
import java.net.URI;
import java.util.HashMap;

import javax.jbi.management.LifeCycleMBean;

import org.objectweb.petals.jbi.management.deployment.DeploymentContextConstants;
import org.objectweb.petals.jbi.management.deployment.DeploymentUtils;
import org.objectweb.petals.jbi.management.systemstate.SystemState;
import org.objectweb.petals.processor.Task;
import org.objectweb.petals.repository.RepositoryService;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;
import org.objectweb.petals.util.LoggingUtil;

/**
 * This task creates state holder for service assembly
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class SAStateHolderCreationTask implements Task {

    /**
     * JBI Container :: Recovery Service
     */
    protected SystemState recoverySrv;

    /**
     * Platform Component :: Repository Service
     */
    protected RepositoryService repositorySrv;

    /**
     * logger wrapper
     */
    protected LoggingUtil log;

    public SAStateHolderCreationTask(SystemState recoverySrv,
            RepositoryService repositorySrv, LoggingUtil log) {
        super();
        this.recoverySrv = recoverySrv;
        this.repositorySrv = repositorySrv;
        this.log = log;
    }

    public void execute(HashMap context) throws Exception {
        JBIDescriptor descriptor = (JBIDescriptor) context
                .get(DeploymentContextConstants.SA_DESCRIPTOR);

        URI saArchiveURI = (URI) context
                .get(DeploymentContextConstants.ARCHIVE_URI);

        String saName = DeploymentUtils.getServiceAssemblyName(descriptor);

        File file = DeploymentUtils.getArchiveFile(saArchiveURI);
        File installedArchive = DeploymentUtils.getInstalledArchive(file);

        recoverySrv.createServiceAssemblyStateHolder(saName, repositorySrv
                .getServiceAssemblyInstallRoot(saName).toURL().toString(),
                installedArchive.toURL().toString(), LifeCycleMBean.SHUTDOWN);

    }

    public void undo(HashMap context) {
        JBIDescriptor descriptor = (JBIDescriptor) context
                .get(DeploymentContextConstants.SA_DESCRIPTOR);

        try {
            recoverySrv.deleteServiceAssemblyStateHolder(DeploymentUtils
                    .getServiceAssemblyName(descriptor));
        } catch (Exception e) {
            String msg = "Failed to revert a SAStateHolderCreationTask";
            log.error(msg, e);
        }

    }

}
