/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: XMLDeploymentResultCreationTask.java 154 6 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.deployment.deploy;

import java.util.HashMap;

import org.objectweb.petals.jbi.management.deployment.DeploymentContextConstants;
import org.objectweb.petals.jbi.management.deployment.DeploymentUtils;
import org.objectweb.petals.jbi.management.service.util.XMLResult;
import org.objectweb.petals.jbi.management.service.util.XMLResult.CauseFramework;
import org.objectweb.petals.jbi.management.service.util.XMLResult.MessageType;
import org.objectweb.petals.jbi.management.service.util.XMLResult.TaskResult;
import org.objectweb.petals.processor.Task;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;

/**
 * This task create framework task result. As we decide to create xml task
 * result only when deployment of the SA success, xml result only handles xml
 * component messages. All exceptions (caused by the deployment framework or
 * thrown by a component during the deployment of an su) are thrown and they
 * will stop the deployment process. It's a small violation of the
 * specification. Used by both DeploymentService and SystemState Recovery.
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class XMLDeploymentResultCreationTask implements Task {

    @SuppressWarnings("unchecked")
    public void execute(HashMap context) throws Exception {
        JBIDescriptor descriptor = (JBIDescriptor) context
                .get(DeploymentContextConstants.SA_DESCRIPTOR);

        XMLResult xmlResult = new XMLResult("deploy");
        createFrameworkTaskResult(DeploymentUtils
                .getServiceAssemblyName(descriptor), xmlResult);

        /*
         * Fill context
         */
        context.put(DeploymentContextConstants.XML_RESULT, xmlResult);

    }

    /**
     * Create the framework task result of the xml result message. Framework
     * task result is always a success because we decide that in other cases, an
     * exception is thrown.
     * 
     * @param saName
     *            the service assembly name
     * @param result
     *            framework task result
     */
    protected void createFrameworkTaskResult(String saName, XMLResult result) {
        result.addFrameworkTaskResult(TaskResult.SUCCESS, MessageType.INFO,
                "Successfully deployed service assembly : {1}",
                new String[] {saName}, CauseFramework.YES);
    }

    public void undo(HashMap context) {
        // Nothing to do

    }

}
