/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: AllConnectionsRemovalTask.java 154 10 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.deployment.undeploy;

import java.util.HashMap;
import java.util.List;

import javax.xml.namespace.QName;

import org.objectweb.petals.jbi.component.lifecycle.ServiceAssemblyLifeCycle;
import org.objectweb.petals.jbi.management.deployment.DeploymentContextConstants;
import org.objectweb.petals.jbi.management.service.EndpointService;
import org.objectweb.petals.processor.Task;
import org.objectweb.petals.tools.jbicommon.descriptor.Connection;
import org.objectweb.petals.tools.jbicommon.descriptor.ServiceAssembly;

/**
 * This task remove all connections from the registry.
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class AllConnectionsRemovalTask implements Task {

    protected EndpointService endpointService;

    public AllConnectionsRemovalTask(EndpointService endpointService) {
        super();
        this.endpointService = endpointService;
    }

    public void execute(HashMap context) throws Exception {

        ServiceAssemblyLifeCycle lifeCycle = (ServiceAssemblyLifeCycle) context
                .get(DeploymentContextConstants.SA_LIFECYCLE);

        removeConnections(lifeCycle);

    }

    /**
     * Remove all connections registered during the SA deployment process
     * 
     * @param saLifeCycle
     *            the service assembly life cycle
     * @throws Exception
     *             if problem occurs during the deletion of a specific
     *             connection
     */
    protected void removeConnections(ServiceAssemblyLifeCycle saLifeCycle)
        throws Exception {
        ServiceAssembly saDesc = saLifeCycle.getServiceAssembly();
        List<Connection> connections = saDesc.getConnections();
        if (connections != null) {
            for (Connection connection : connections) {
                QName consInterface = connection.getConsumerInterfaceName();
                QName consService = connection.getConsumerServiceName();
                String consEndpoint = connection.getConsumerEndpointName();
                QName provService = connection.getProviderServiceName();
                String provEndpoint = connection.getProviderEndpointName();

                if (provEndpoint != null && provService != null) {
                    if (consInterface != null) {
                        endpointService.deleteConnection(consInterface,
                                provService, provEndpoint);
                    } else if (consService != null && consEndpoint != null) {
                        endpointService.deleteConnection(consService,
                                consEndpoint, provService, provEndpoint);
                    }
                }
            }
        }
    }

    public void undo(HashMap context) throws Exception {
        // Nothing to do

    }

}
