/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SALifeCycleRetrievalTask.java 154 10 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.deployment.undeploy;

import java.util.HashMap;

import org.objectweb.petals.jbi.component.lifecycle.ServiceAssemblyLifeCycle;
import org.objectweb.petals.jbi.management.deployment.DeploymentContextConstants;
import org.objectweb.petals.jbi.management.deployment.ServiceAssemblyDataHandler;
import org.objectweb.petals.jbi.management.service.LifeCycleManagerService;
import org.objectweb.petals.processor.Task;
import org.objectweb.petals.util.LoggingUtil;

/**
 * Retrieve service assembly lifecycle.
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class SALifeCycleRetrievalTask implements Task {

	/**
	 * JBI Container :: JMX Agent Admin
	 */
	protected LifeCycleManagerService managerService;

	/**
	 * logger wrapper
	 */
	protected LoggingUtil log;

	public SALifeCycleRetrievalTask(LifeCycleManagerService managerService,
			LoggingUtil log) {
		super();
		this.managerService = managerService;
		this.log = log;
	}

	@SuppressWarnings("unchecked")
	public void execute(HashMap context) throws Exception {
		ServiceAssemblyDataHandler handler = (ServiceAssemblyDataHandler) context
				.get(DeploymentContextConstants.SA_DATA_HANDLER);

		ServiceAssemblyLifeCycle saLifeCycle = retrieveServiceAssemblyLifeCycle(handler
				.getName());

		/*
		 * Fill context
		 */
		context.put(DeploymentContextConstants.SA_LIFECYCLE, saLifeCycle);

	}

	/**
	 * Retrieve the {@link ServiceAssemblyLifeCycle} matching the given service
	 * assembly name.
	 * 
	 * @param saName
	 *            the service assembly name
	 * @return the retrieved {@link ServiceAssemblyLifeCycle}
	 * @throws Exception
	 *             if no service assembly can be retrieved
	 */
	protected ServiceAssemblyLifeCycle retrieveServiceAssemblyLifeCycle(
			String saName) throws Exception {
		ServiceAssemblyLifeCycle saLifeCycle = managerService
				.getServiceAssemblyByName(saName);
		if (saLifeCycle == null) {
			String msg = "The service assembly (" + saName
					+ ") isn't deployed into the JBI environment.";
			log.error(msg);
			throw new Exception(msg);
		}
		return saLifeCycle;
	}

	public void undo(HashMap context) throws Exception {
		// TODO Auto-generated method stub

	}

}
