package org.objectweb.petals.jbi.management.service;

import java.util.Map;

import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.objectweb.petals.jbi.component.lifecycle.ComponentLifeCycle;
import org.objectweb.petals.jbi.component.lifecycle.Installer;
import org.objectweb.petals.jbi.component.lifecycle.LifeCycleAbstract;
import org.objectweb.petals.jbi.component.lifecycle.ServiceAssemblyLifeCycle;

/**
 * Manager is the entry point for register and unregister elements in the JMX
 * environment (elements are JBI binding/engine components and systemServices
 * components). Related MBeans are created when registering and unregistering
 * elements.
 * 
 * <code>Manager</code> owns the JMX management services offered by the JBI
 * container to administrative clients. Management services are : - AdminService
 * (referenced in the JMX server as "admin"); - InstallationService (referenced
 * in the JMX server as "installation"); - DeploymentService (referenced in the
 * JMX server as "deployment");
 * 
 * <code>Manager</code> is the central point for accessing the mBeans
 * associated to the JBI container elements.
 * 
 * <code>ManagementException</code> exceptions are thrown when problems for
 * accessing the JMX server occur.
 * 
 * @author ofabre
 * 
 */
public interface LifeCycleManagerService {

    ObjectName getAdminServiceMBeanName() throws ManagementException;

    /**
     * Find the {@link LifeCycleAbstract} of a JBI Installable Component by its
     * unique name.
     * 
     * @param name
     *            the name of the engine or binding component; must be non-null
     *            and non-empty
     * @return the {@link LifeCycleAbstract} of the component, or
     *         <code>null</code> if there is no such component with the given
     *         <code>name</code>
     */
    ComponentLifeCycle getComponentByName(String name);

    ObjectName getDeploymentServiceMBeanName() throws ManagementException;

    ObjectName getInstallationServiceMBeanName() throws ManagementException;

    ObjectName getInstallerByName(String name);

    MBeanNamesImpl getMBeanNames();

    MBeanServer getMBeanServer();

    ServiceAssemblyLifeCycle getServiceAssemblyByName(String saName);

    Map<ObjectName, ComponentLifeCycle> getBindingCompoLifeCycles();

    Map<ObjectName, ComponentLifeCycle> getEngineCompoLifeCycles();

    Map<ObjectName, Installer> getInstallers();

    Map<String, ServiceAssemblyLifeCycle> getServiceAssemblyLifeCycles();

    Map<ObjectName, LifeCycleAbstract> getSystemServices();

    ObjectName registerBindingComponent(ComponentLifeCycle lifeCycle)
        throws ManagementException;

    ObjectName registerEngineComponent(ComponentLifeCycle lifeCycle)
        throws ManagementException;

    ObjectName registerInstaller(Installer installerMBean)
        throws ManagementException;

    void registerServiceAssembly(String saName,
        ServiceAssemblyLifeCycle serviceAssemblyLifeCycle);

    void unregisterBindingComponent(ObjectName name) throws ManagementException;

    void unregisterEngineComponent(ObjectName name) throws ManagementException;

    void unregisterInstaller(ObjectName name) throws ManagementException;

    void unregisterServiceAssembly(String saName) throws ManagementException;
}
