/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): Adrien LOUIS
 * --------------------------------------------------------------------------
 * $Id: Router.java,v 1.2 2006/03/17 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.jbi.routing;

import org.objectweb.petals.jbi.component.context.ComponentContextImpl;
import org.objectweb.petals.jbi.messaging.MessageExchangeImpl;

/**
 * Routes messages from the DeliveryChannel to their destinations. Performs an
 * itinerary resolution, adding for example a transformatino service before
 * delivering the message to its recipient, and perform an address resolution,
 * finding the endpoint satisfying the the destination information (finding an
 * endpoint for a specified service). Resolutions are based on defined rules.
 * 
 * Routes messages from the Transporter to the correct DeliveryChannel
 * 
 * @author Adrien LOUIS - EBM WebSourcing
 */
public interface Router {

    /**
     * Send the message into the NMR
     * 
     * @param source
     * @param exchange
     * @param timeOut timeout of the message (in ms). 0 for no timeout
     * @throws RoutingException
     */
    void send(ComponentContextImpl source, MessageExchangeImpl exchange,
        long timeOut) throws RoutingException;

    /**
     * Push this MessageExchange, received from the Transporter, to the correct
     * deliveryChannel queue.
     * 
     * @param exchange
     * @throws RoutingException
     */
    void receive(MessageExchangeImpl exchange) throws RoutingException;
}
