/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2006 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: Serializer.java,v 0.3 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.jbi.transport;

import javax.jbi.JBIException;
import javax.jms.Message;
import javax.jms.Session;

import org.objectweb.petals.jbi.messaging.MessageExchangeImpl;

/**
 * A <code>Serializer</code> for the transport ensure the mapping between
 * <code>MessageExchangeImpl</code> and <code>jms.Message</code>.
 * 
 * @version $Rev: 477 $ $Date: 2006-05-29 17:18:07 +0200 (lun., 29 mai 2006) $
 * @since Petals 1.0
 * @author alouis
 */
public interface Serializer {
    /**
     * Transform a <code>MessageExchangeImpl</code> into a
     * <code>jms.Message</code>
     * 
     * @param jbi
     *            the exchange, must be non null
     * @param Session
     *            the session used to create jms message, mus be non null
     * @return a jms message representing the given exchange
     * @throws TransportException TODO
     */
    Message jbi2jms(MessageExchangeImpl jbi, Session session)
        throws TransportException;

    /**
     * Transform a <code>jms.Message</code> into a
     * <code>MessageExchangeImpl</code>
     * 
     * @param jms
     *            the message to transform, must be non null
     * @return a messageExchange retrieved from the jms message
     * @throws JBIException
     *             error during the transformation
     */
    MessageExchangeImpl jms2jbi(Message jms) throws JBIException;
}
