/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ContainerInformation.java 16:40:52 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.kernel.admin;

import java.io.Serializable;

/**
 * This class contains all the informations of a container
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class ContainerInformation implements Serializable {

    /**
     * Container state
     *
     * @author ddesjardins - eBMWebsourcing
     */
    public static enum ContainerState {
        /**
         * Started state
         */
        STARTED,
        /**
         * Stopped state
         */
        STOPPED
    };

    /**
     * 
     */
    private static final long serialVersionUID = 1L;

    /**
     * Host of the petals container
     */
    private String host;

    /**
     * Port of the html adaptor
     */
    private String htmlPort;

    /**
     * Login to connect to the JMX server
     */
    private String jmxLogin;

    /**
     * Password to connect to the JMX server
     */
    private String jmxPassword;

    /**
     * Port of the JMX server
     */
    private String jmxPort;

    /**
     * JNDI Factory
     */
    private String jndiFactory;

    /**
     * JNDI port
     */
    private String jndiPort;

    /**
     * Name of the Joram domain
     */
    private String joramDomain;

    /**
     * Port of the Joram domain
     */
    private String joramDomainPort;

    /**
     * Id of the Joram server
     */
    private String joramId;

    /**
     * Login to connect to the Joram server
     */
    private String joramLogin;

    /**
     * Password to connect to the Joram server
     */
    private String joramPassword;

    /**
     * Port of the Joram TCP topic
     */
    private String joramTCPPort;

    /**
     * State of the container
     */
    private ContainerState state = ContainerState.STOPPED;

    /**
     * Container name
     */
    private String name;

    /**
     * Container uid
     */
    private long uid;

    public long getUid() {
        return uid;
    }

    public void setUid(long uid) {
        this.uid = uid;
    }

    public String getHost() {
        return host;
    }

    public String getHtmlPort() {
        return htmlPort;
    }

    public String getJmxLogin() {
        return jmxLogin;
    }

    public String getJmxPassword() {
        return jmxPassword;
    }

    public String getJmxPort() {
        return jmxPort;
    }

    public String getJndiFactory() {
        return jndiFactory;
    }

    public String getJndiPort() {
        return jndiPort;
    }

    public String getJoramDomain() {
        return joramDomain;
    }

    public String getJoramDomainPort() {
        return joramDomainPort;
    }

    public String getJoramId() {
        return joramId;
    }

    public String getJoramLogin() {
        return joramLogin;
    }

    public String getJoramPassword() {
        return joramPassword;
    }

    public String getJoramTCPPort() {
        return joramTCPPort;
    }

    public String getName() {
        return name;
    }

    public void setHost(String host) {
        this.host = host;
    }

    public void setHtmlPort(String htmlPort) {
        this.htmlPort = htmlPort;
    }

    public void setJmxLogin(String jmxLogin) {
        this.jmxLogin = jmxLogin;
    }

    public void setJmxPassword(String jmxPassword) {
        this.jmxPassword = jmxPassword;
    }

    public void setJmxPort(String jmxPort) {
        this.jmxPort = jmxPort;
    }

    public void setJndiFactory(String jndiFactory) {
        this.jndiFactory = jndiFactory;
    }

    public void setJndiPort(String jndiPort) {
        this.jndiPort = jndiPort;
    }

    public void setJoramDomain(String joramDomain) {
        this.joramDomain = joramDomain;
    }

    public void setJoramDomainPort(String joramPort) {
        this.joramDomainPort = joramPort;
    }

    public void setJoramId(String joramId) {
        this.joramId = joramId;
    }

    public void setJoramLogin(String joramLogin) {
        this.joramLogin = joramLogin;
    }

    public void setJoramPassword(String joramPassword) {
        this.joramPassword = joramPassword;
    }

    public void setJoramTCPPort(String joramTCPPort) {
        this.joramTCPPort = joramTCPPort;
    }

    public void setName(String name) {
        this.name = name;
    }

    public ContainerState getState() {
        return state;
    }

    public void setState(ContainerState state) {
        this.state = state;
    }

    public boolean isStarted() {
        return state.equals(ContainerState.STARTED);
    }

    public boolean isStopped() {
        return state.equals(ContainerState.STOPPED);
    }

    public String toString() {
        return "Container " + joramId + " (isRunning " + isStarted()
            + ") host: " + host + " Joram: " + joramDomainPort + " JMX: "
            + jmxPort + " JNDI: " + jndiPort + " HTML: " + htmlPort;
    }
}
