/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: PetalsAdminServiceMBean.java 16:40:52 alouis $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.kernel.admin.service;

import java.util.Collection;
import java.util.Map;


/**
 * This class is a manager of JMX server. It is used to get a connection to a
 * JMX Server on a distant container.
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public interface PetalsAdminServiceMBean {

    public static final String CONF_HOST="host";
    public static final String CONF_HTML_PORT="htmlPort";
    public static final String CONF_JMX_LOGIN="jmxLogin";
    public static final String CONF_JMX_PASSWORD="jmxPassword";
    public static final String CONF_JMX_PORT="jmxPort";
    public static final String CONF_JNDI_FACTORY="jndiFactory";
    public static final String CONF_JNDI_PORT="jndiPort";
    public static final String CONF_JORAM_DOMAIN="joramDomain";
    public static final String CONF_JORAM_DOMAIN_PORT="joramDomainPort";
    public static final String CONF_JORAM_ID="joramId";
    public static final String CONF_JORAM_LOGIN="joramLogin";
    public static final String CONF_JORAM_PASSWORD="joramPassword";
    public static final String CONF_JORAM_TCP_PORT="joramTCPPort";
    public static final String CONF_NAME="name";
    public static final String CONF_STATE="state";
    public static final String CONF_UID="uid";
    
    /**
     * Unload the container from the Petals global environment.
     * All ServiceAssemblies, Components and Share Libraries are
     * stopped, shutdown, and uninstalled.
     * All registered Endpoints are deactivated.
     * Then the container is removed from the domain.
     * @throws Exception
     */
    void shutdownContainer() throws Exception;
    
    /**
     * Stop the container.
     * The container is still visible in the Petals global environment, 
     * but marked as 'stopped'.
     * The registered Endpoints are still visible,too. 
     * @throws Exception
     */
    void stopContainer() throws Exception;
   
    /**
     * Retrieve the collection of all the Started servers configuration from this
     * domain.
     * 
     * @return Collection of started server information (Map key;value)
     */
    public Collection<Map<String,String>> getAllStartedServersConfiguration();
    
    /**
     * Retrieve the collection of all the servers configuration from this
     * domain.
     * 
     * @return Collection of server information (Map key;value)
     */
    public Collection<Map<String,String>> getAllServersConfiguration();

 
}
