/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MX4JHtmlAdaptor.java 454 2006-05-23 13:51:29Z ddesjardins $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.kernel.mx4j;

import javax.management.Attribute;
import javax.management.InstanceNotFoundException;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import mx4j.tools.adaptor.http.HttpAdaptor;
import mx4j.tools.adaptor.http.HttpAdaptorMBean;
import mx4j.tools.adaptor.http.XSLTProcessor;

import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.fraclet.annotation.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.LifeCycleType;
import org.objectweb.fractal.fraclet.annotation.Monolog;
import org.objectweb.fractal.fraclet.annotation.Requires;
import org.objectweb.fractal.jmx.agent.AdminAttributes;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.petals.util.SystemUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * This is the Fractal JMX adaptor for use with MX4J tools.
 * 
 * @version $Rev: 454 $ $Date: 2006-05-23 15:51:29 +0200 (mar., 23 mai 2006) $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 * @author ddesjardins - eBMWebsourcing
 * @author Anass OUAZZANI - eBMWebsourcing
 */
@FractalComponent
public class MX4JHtmlAdaptor {

    private static final String ADAPTOR = "MX4JHtmlAdaptor";

    /**
     * HTTP Adaptor
     */
    protected HttpAdaptorMBean adaptor;

    /**
     * AdminAttributes
     */
    @Requires(name="adminAtt",signature=org.objectweb.fractal.jmx.agent.AdminAttributes.class)
    protected AdminAttributes adminAttributes;

    /**
     * Logger
     */
    protected LoggingUtil log;

    /**
     * Logger
     */
    @Monolog(name="logger")
    protected Logger logger;

    private ObjectName oName;

    /**
     * @see LifeCycleController#startFc()
     */
    @LifeCycle(on=LifeCycleType.START)
    public void start() {
    	log=new LoggingUtil(null);
        log.start();
        try {
            // Retreive the port and host properties
            int port = Integer.parseInt(SystemUtil.getHtmlPort());
            // TODO : configure fine grained JMW access for security
            // String host = SystemUtil.getHost();
            if (adaptor != null && adaptor.isActive()
                    && adaptor.getPort() == port) {
                return;
            }
            // unregister & stops
            if (adaptor != null) {
                adaptor.stop();
                try {
                    adminAttributes.getRawMBeanServer().unregisterMBean(
                            getObjectName());
                } catch (InstanceNotFoundException ignore) {
                    // Do nothing
                }
            }
            adaptor = new HttpAdaptor();
            adaptor.setPort(port);
            adaptor.setHost("0.0.0.0");

            adaptor.addCommandProcessor("petals", new PetalsCommandProcessor());
            adaptor.addCommandProcessor("monitoring",
                    new MonitorCommandProcessor());

            adminAttributes.getRawMBeanServer().registerMBean(adaptor,
                    getObjectName());

            ObjectName processorName = new ObjectName(
                    "Server:name=XSLTProcessor");
            XSLTProcessor processor = new XSLTProcessor();

            processor.setDefaultPage("petals");
            processor
                    .setPathInJar("org/objectweb/petals/kernel/fractal/mx4j/xsl");

            processor.setUseCache(false);
            adminAttributes.getRawMBeanServer().registerMBean(processor,
                    processorName);
            adminAttributes.getRawMBeanServer().setAttribute(getObjectName(),
                    new Attribute("ProcessorName", processorName));

            adaptor.start();
        } catch (Exception e) {
            log.error("Fail to start HTML adaptor", e);
        }
        log.end();
    }

    /**
     * @see LifeCycleController#stopFc()
     */
    @LifeCycle(on=LifeCycleType.STOP)
    public void stop() {
        log.start();
        if (adaptor != null) {
            adaptor.stop();
            try {
                adminAttributes.getRawMBeanServer().unregisterMBean(
                        getObjectName());
            } catch (Exception ignore) {
                log.warning("Error during MBean unregistering: "
                        + ignore.getMessage(), ignore);
            }
        }
        log.end();
    }

    // -------------------------------------------------------------------------
    // Private implementation methods
    // -------------------------------------------------------------------------

    private ObjectName getObjectName() throws MalformedObjectNameException {
        if (oName == null) {
            String id = '@' + Integer.toHexString(this.hashCode());
            oName = new ObjectName(ADAPTOR + id + ":type=html,port="
                    + adaptor.getPort());
        }
        return oName;
    }
}
