/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: CommandReader.java 1:04:18 PM alouis $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.system.launch;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;

import javax.naming.NamingException;

import org.apache.commons.io.FileUtils;

import org.objectweb.petals.kernel.admin.service.PetalsAdminServiceMBean;
import org.objectweb.petals.util.JNDIUtil;
import org.objectweb.petals.util.SystemUtil;

/**
 * Used to read command from the console
 * 
 * @author alouis - eBMWebsourcing
 */
public class CommandReader {

    protected static int STOP_CODE = 0;

    protected static int ERROR_CODE = -1;

    protected static int OK_CODE = 1;

    protected File currentPath;

    protected File installPath;

    protected File installedPath;

    protected File uninstalledPath;

    protected PetalsAdminServiceMBean petalsService;
    
    public CommandReader(PetalsAdminServiceMBean petalsService) {
        currentPath = new File(".");
        File petalsPath = SystemUtil.getPetalsInstallDirectory();
        installPath = new File(petalsPath, "install");
        installedPath = new File(petalsPath, "installed");
        uninstalledPath = new File(petalsPath, "uninstalled");
        this.petalsService = petalsService;
    }

    /**
     * Read command line. This method ends with an "exit" command line
     * 
     */
    public void read() {
        System.out.println("Petals prompt. Tape 'help' for help.");
        BufferedReader br = new BufferedReader(new InputStreamReader(System.in));

        int code=1;

        while (code>0) {
            try {
                System.out.println();
                System.out.print("> ");
                code = processCommandLine(br.readLine());
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        System.out.println("exiting Petals...");
        System.exit(code);
    }

    /**
     * Process the command, and return a code if "exit" command, return -1
     * 
     * @param command
     * @return
     */
    protected int processCommandLine(String command) {
        if (command != null) {
            if ("help".equalsIgnoreCase(command)
                || "h".equalsIgnoreCase(command)) {
                return help();
            } else if (command.toLowerCase().startsWith("hotdeploy")) {
                return hotdeploy(command.substring(9, command.length()).trim());
            } else if (command.toLowerCase().startsWith("hd")) {
                return hotdeploy(command.substring(2, command.length()).trim());
            } else if (command.toLowerCase().startsWith("hotundeploy")) {
                return hotundeploy(command.substring(11, command.length()).trim());
            } else if (command.toLowerCase().startsWith("hu")) {
                return hotundeploy(command.substring(2, command.length()).trim());
            } else if (command.toLowerCase().startsWith("setpath")) {
                return setpath(command.substring(7, command.length()).trim());
            } else if (command.toLowerCase().startsWith("sp")) {
                return setpath(command.substring(2, command.length()).trim());
            } else if ("path".equalsIgnoreCase(command)
                || "p".equalsIgnoreCase(command)) {
                return path();
            } else if ("jndi".equalsIgnoreCase(command)
                || "d".equalsIgnoreCase(command)) {
                return jndi();
            } else if ("stop".equalsIgnoreCase(command)
                || "q".equalsIgnoreCase(command)) {
                return stop();
            } else if ("exit".equalsIgnoreCase(command)
                    || "x".equalsIgnoreCase(command)) {
                    return shutdown();
            } else {
                System.out.println("Unknown command");
            }
        }
        return OK_CODE;
    }

    /**
     * Help
     * 
     * @return 1
     */
    protected int help() {
        System.out.println("commands:");
        System.out.println("---------");
        System.out
            .println(" - hotdeploy [ZIP filePath] (or hd): install and start a component, a service assembly or a share library");
        System.out
            .println(" - hotundeploy [ZIP fileName] (or hu): shutdown and uninstall a component, a service assembly or a share library");
        System.out.println(" - path (or p)     : display current file system path");
        System.out
            .println(" - setpath [Path] (or sp): change current file system path");
        System.out.println(" - jndi (or d)     : display the naming directory");
        System.out.println();
        System.out.println(" - help (or h)     : this help");
        System.out.println(" - stop (or q)     : stop Petals");
        System.out.println(" - shutdown (or x) : shutdown Petals (remove from the Petals network");
        return OK_CODE;
    }

    /**
     * show current path
     * 
     * @return 1
     */
    protected int path() {
        System.out.println("current path is :");
        System.out.println(currentPath.getAbsolutePath());
        return OK_CODE;
    }

    /**
     * set current path
     * 
     * @return 1
     */
    protected int setpath(String newPath) {
        currentPath = new File(newPath);
        System.out.println("new current path is :");
        System.out.println(currentPath.getAbsolutePath());
        return OK_CODE;
    }

    /**
     * hot deploy
     * 
     * @return 1
     */
    protected int hotdeploy(String zip) {
        File source = new File(zip);
        if (!source.exists()) {
            source = new File(currentPath, zip);
        }

        if (!source.exists()) {
            System.out.println("File not found : " + source.getAbsolutePath());
        } else {
            try {
                FileUtils.copyFileToDirectory(source, installPath);
                Thread.sleep(1000);
            } catch (IOException e) {
                System.out
                    .println("File found, but can not copy and deploy file");
                e.printStackTrace();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
        return OK_CODE;
    }

    /**
     * hot deploy
     * 
     * @return 1
     */
    protected int hotundeploy(String zip) {
        File source = new File(installedPath, zip);

        if (!source.exists()) {
            System.out.println("File to undeploy not found : "
                + source.getAbsolutePath());
        } else {
            File uninstalledFile = new File(uninstalledPath, zip);
            source.renameTo(uninstalledFile);
        }
        return OK_CODE;
    }

    /**
     * display the naming directory
     * 
     * @return 1
     */
    protected int jndi() {
        try {
            System.out.println(JNDIUtil.browseJNDI(null));
        } catch (NamingException e) {
            e.printStackTrace();
        }
        return OK_CODE;
    }
    
    /**
     * Stop the container, nothing to perform
     * @return
     */
    protected int stop(){
        System.out.println("quit Petals... (stop the server)");
        return STOP_CODE;
    }
    
    /**
     * Shutdown the container, remove it from
     * the network 
     * @return
     */
    protected int shutdown(){
        try {
            System.out.println("exit Petals... (shutdown the server)");
            petalsService.shutdownContainer();
            return STOP_CODE;
        } catch (Exception e) {
            e.printStackTrace();
            return ERROR_CODE;
        }
    }

}
