/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MBeanHelper.java 16:40:52 alouis $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.system.launch;

import java.io.IOException;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;
import java.util.Set;

import javax.jbi.management.DeploymentServiceMBean;
import javax.jbi.management.InstallationServiceMBean;
import javax.management.MBeanServer;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.jmx.agent.AdminAttributes;
import org.objectweb.fractal.jmx.agent.Introspector;
import org.objectweb.fractal.util.Fractal;

import org.objectweb.petals.jbi.management.service.AdminServiceMBean;
import org.objectweb.petals.kernel.admin.service.PetalsAdminServiceMBean;
import org.objectweb.petals.util.SystemUtil;


/**
 * This class helps the Launcher to register MBean 
 * 
 * @author alouis
 * 
 */
public class MBeanHelper {
    public static void registerMBeans(Component petalsComposite) throws Exception {

        String componentName = "agent-jmx";
        ContentController contentController = Fractal
            .getContentController(petalsComposite);
        Component agent = FractalHelper.getComponentByName(contentController, componentName);

        String interfaceName = "admin";
        Introspector.CURRENCY_TIME_LIMIT = "-1";
        interfaceName = "attribute-controller";
        AdminAttributes adminAttributes = (AdminAttributes) agent
            .getFcInterface(interfaceName);

        MBeanServer jmxServer = adminAttributes.getRawMBeanServer();

        // register the fractal lifecycle controller
        LifeCycleController lcc = Fractal
            .getLifeCycleController(petalsComposite);

        jmxServer.registerMBean(Introspector.createMBean(lcc), new ObjectName(
            "FC/Petals@" + lcc.hashCode() + ":itf=lifecycle-controller"));

        // register JBI AdminMBean
        Component jbi = FractalHelper.getComponentByName(contentController, "jbi");
        ContentController contentControllerForJBI = Fractal
            .getContentController(jbi);
        registerComponent(contentControllerForJBI, "admin-impl", jmxServer,
            AdminServiceMBean.class, "Petals:name=Admin,type=service");

        // register JBI DeployMBean
        registerComponent(contentControllerForJBI, "deployment-impl",
            jmxServer, DeploymentServiceMBean.class,
            "Petals:name=Deployment,type=service");

        // register JBI InstallMBean
        registerComponent(contentControllerForJBI, "installation-impl",
            jmxServer, InstallationServiceMBean.class,
            "Petals:name=Installation,type=service");

        // register Petals Admin
        registerComponent(contentController, "petalsadmin-service", jmxServer,
            PetalsAdminServiceMBean.class,
            "Petals:name=PetalsAdmin,type=service");
    }

    /**
     * Register a fractal component-service to the jmx server. Assume the
     * interface name of the fractal component is "service"
     * 
     * @param contentController
     *            contentcontroller where the fractal comp can be found
     * @param fractalName
     *            the name of the fractal comp
     * @param jmxServer
     *            the jmx server to register to
     * @param mbeanClass
     *            the interface of the mbean
     * @param mbeanName
     *            the name of the mbean
     * @throws Exception
     */
    protected static void registerComponent(ContentController contentController,
        String fractalName, MBeanServer jmxServer, Class mbeanClass,
        String mbeanName) throws Exception {
        Component systemState = FractalHelper.getComponentByName(contentController,
            fractalName);
        Object object = systemState.getFcInterface("service");

        jmxServer.registerMBean(Introspector.createMBean(object, mbeanClass),
            new ObjectName(mbeanName));
    }
    
    
    /**
     * Return the JMX server by using the server.properties entries
     * 
     * @return local jmx server
     * @throws IOException
     */
    public static MBeanServerConnection retrieveJMXServer() throws IOException {
        // connect to the JMX server
        // with server.properties entries
        String jmxHost = SystemUtil.getHost();
        String jmxPort = SystemUtil.getJmxPort();
        String username = SystemUtil.getJmxUser();
        String password = SystemUtil.getJmxPassword();

        JMXConnector connector = null;
        Map<String, Object> args = new HashMap<String, Object>();
        if (!"".equals(username)) {
            if (password == null) {
                password = "";
            }
            String[] credentials = new String[] {username, password};
            args.put("jmx.remote.credentials", credentials);
        }
        String urlStr = "service:jmx:rmi:///jndi/rmi://" + jmxHost + ":"
            + jmxPort + "/management/rmi-jmx-connector";
        JMXServiceURL url = new JMXServiceURL(urlStr);
        connector = retrieveJMXConnector(url, args);
        return connector.getMBeanServerConnection();
    }

    /**
     * Return the MBean name from the jmx server with the specified pattern
     * 
     * @param pattern
     * @param jmxServer
     * @return the objectname, null if no object match
     * @throws MalformedObjectNameException
     * @throws NullPointerException
     * @throws IOException
     */
    public static ObjectName retrieveServiceMBean(String serviceName,
        MBeanServerConnection jmxServer) throws MalformedObjectNameException,
        NullPointerException, IOException {

        ObjectName result = null;

        // crate the search filter
        Hashtable<String, String> attributes = new Hashtable<String, String>();
        attributes.put("name", serviceName);
        attributes.put("type", "service");
        ObjectName objName = new ObjectName("Petals", attributes);
        
        Set objNames = jmxServer.queryNames(objName, null);
        if (objNames != null && objNames.size() == 1) {
            result = (ObjectName) objNames.iterator().next();
        }
        return result;
    }

    /**
     * @see JMXConnectorFactory#connect(JMXServiceURL, Map)
     */
    protected static JMXConnector retrieveJMXConnector(JMXServiceURL url,
        Map<String, ?> args) throws IOException {
        return JMXConnectorFactory.connect(url, args);
    }


   }
