<?xml version="1.0"?>
<!--
	Copyright (C) The MX4J Contributors.
	All rights reserved.
	
	This software is distributed under the terms of the MX4J License version 1.0.
	See the terms of the MX4J License in the documentation provided with this software.
	
	Author: Carlos Quiroz (tibu@users.sourceforge.net)
	Revision: $Revision: 76 $
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" indent="yes" encoding="ISO-8859-1" />
	<xsl:include href="common.xsl" />
	<xsl:include href="xalan-ext.xsl" />

	<xsl:param name="html.stylesheet">stylesheet.css</xsl:param>
	<xsl:param name="html.stylesheet.type">text/css</xsl:param>
	<xsl:param name="head.title">serverbydomain.title</xsl:param>

	<!-- Invoked when a query error is produced -->
	<xsl:template match="Domain" name="error">
		<xsl:for-each select="Exception">
			<tr>
				<td class="serverbydomain_domainline" colspan="5">
					<xsl:call-template name="str">
						<xsl:with-param name="id">serverbydomain.error.query</xsl:with-param>
						<xsl:with-param name="p0">
							<xsl:value-of select="@errorMsg" />
						</xsl:with-param>
					</xsl:call-template>
				</td>
			</tr>
		</xsl:for-each>
	</xsl:template>

	<!-- invoked for each monitors -->
	<xsl:template match="monitors" name="n_monitors">
		<tr width="100%">
			<td class="petals_domainline" colspan="5">Monitors</td>
		</tr>
		<xsl:for-each select="monitors/MBean">
			<xsl:sort data-type="text" order="ascending" select="@objectname" />
			<xsl:variable name="classtype">
				<xsl:if test="(position() mod 2)=1">darkline</xsl:if>
				<xsl:if test="(position() mod 2)=0">clearline</xsl:if>
			</xsl:variable>
			<xsl:variable name="objectname">
				<xsl:call-template name="uri-encode">
					<xsl:with-param name="uri" select="@objectname" />
				</xsl:call-template>
			</xsl:variable>
			<tr class="{$classtype}" width="100%">
				<td width="90%" align="left" class="petals_row">
					<a href="mbean?objectname={$objectname}">
						<xsl:value-of select="@objectname" />
					</a>
				</td>
			</tr>
		</xsl:for-each>
	</xsl:template>
	
	<!-- invoked for each Component -->
	<xsl:template match="supervision" name="n_admin">
		<tr width="100%">
			<td class="petals_domainline" colspan="5">Supervision</td>
		</tr>
		<xsl:for-each select="supervision/MBean">
			<xsl:sort data-type="text" order="ascending" select="@objectname" />
			<xsl:variable name="classtype">
				<xsl:if test="(position() mod 2)=1">darkline</xsl:if>
				<xsl:if test="(position() mod 2)=0">clearline</xsl:if>
			</xsl:variable>
			<xsl:variable name="objectname">
				<xsl:call-template name="uri-encode">
					<xsl:with-param name="uri" select="@objectname" />
				</xsl:call-template>
			</xsl:variable>
			<tr class="{$classtype}" width="100%">
				<td width="90%" align="left" class="petals_row">
					<a href="mbean?objectname={$objectname}">
						<xsl:value-of select="@objectname" />
					</a>
				</td>
			</tr>
		</xsl:for-each>
	</xsl:template>

	<!-- Main template -->
	<xsl:template match="monitoring">
		<html>
			<xsl:call-template name="head" />
			<body>
				<table width="100%" cellpadding="0" cellspacing="0" border="0">
					<tr width="100%">
						<td>
							<xsl:call-template name="toprow" />
							<xsl:call-template name="tabs">
								<xsl:with-param name="selection">monitoring</xsl:with-param>
							</xsl:call-template>
							<xsl:variable name="query">
								<xsl:call-template name="str">
									<xsl:with-param name="id">serverbydomain.server.query</xsl:with-param>
								</xsl:call-template>
							</xsl:variable>
							<table width="100%" cellpadding="0" cellspacing="0" border="0">

								<tr>
									<td class="page_title">
										<xsl:call-template name="str">
											<xsl:with-param name="id">monitoring.title</xsl:with-param>
										</xsl:call-template>
									</td>
									<form action="serverbydomain">
										<td align="right" class="page_title">
											<xsl:call-template name="str">
												<xsl:with-param name="id">serverbydomain.server.filter</xsl:with-param>
											</xsl:call-template>
											<input type="text" name="querynames" value="*:*" />
											<input type="submit" value="{$query}" />
										</td>
									</form>
								</tr>
							</table>
							<table width="100%" cellpadding="0" cellspacing="0" border="0">
								<!--xsl:call-template name="n_interceptors" /-->
								<!--<xsl:call-template name="n_monitors" />-->
								<xsl:call-template name="n_admin" />
							</table>
							<xsl:call-template name="bottom" />
						</td>
					</tr>
				</table>
			</body>
		</html>
	</xsl:template>
</xsl:stylesheet>

