/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ComponentLifeCycleTest.java 10:44:49 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.component.lifecycle;

import javax.jbi.JBIException;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.petals.jbi.component.context.ComponentContextImpl;
import org.objectweb.petals.jbi.component.lifecycle.mock.MockComponentDescriptionBuilder;
import org.objectweb.petals.jbi.component.thread.ComponentLifeCycleThread;
import org.objectweb.petals.jbi.management.systemstate.SystemStateImpl;
import org.objectweb.petals.jbi.messaging.mock.MockLogger;
import org.objectweb.petals.jbi.routing.mock.ComponentMock;
import org.objectweb.petals.tools.jbicommon.descriptor.ComponentDescription;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;

import static org.easymock.classextension.EasyMock.createMock;

/**
 * Test of the ComponentLifeCycle
 * 
 * @author ddesjardins - eBMWebsourcing
 * @author wjoseph - eBMWebSourcing
 */
public class ComponentLifeCycleTest extends TestCase {

    private ComponentLifeCycle componentLifeCycle;
    
    private ComponentLifeCycleThread lifeCycleThread;

    private ComponentMock componentMock;

    private ComponentDescription componentDescription;

    public void testDoInit() throws JBIException {
        componentLifeCycle.doInit();
        assertNotNull(componentMock.getContext());
    }

    public void testDoShutdown() throws JBIException {
        componentLifeCycle.doShutdown();
        assertEquals(componentMock.getState(), "Shutdown");
    }

    public void testDoStart() throws JBIException {
        componentLifeCycle.doStart();
        assertEquals(componentMock.getState(), "Started");
    }

    public void testDoStop() throws JBIException {
        componentLifeCycle.doStop();
        assertEquals(componentMock.getState(), "Stopped");
    }

    public void testGetComponent() {
        assertEquals(componentLifeCycle.getComponent(), componentMock);
    }

    public void testGetComponentDescription() {
        assertEquals(componentLifeCycle.getComponentDescription(),
            componentDescription);
    }

    public void testGetExtensionMBeanName()
        throws MalformedObjectNameException, NullPointerException, JBIException {
        assertEquals(componentLifeCycle.getExtensionMBeanName(),
            new ObjectName("test@test:name=test"));
    }

    public void testGetName() {
        assertEquals(componentLifeCycle.getName(), "testComponentDescription");
    }

    public void testGetComponentContext() {
        /*
         * Test Case 1
         */
        ComponentContextImpl context = new ComponentContextImpl();
        Logger logger = new MockLogger();
        ObjectName name = null;
        try {
            name = new ObjectName("test@test:name=test");
        } catch (Exception e) {
            fail("Could not create Test parameters");
        }
        
        lifeCycleThread = new ComponentLifeCycleThread(componentMock);
        lifeCycleThread.start();
        
        componentLifeCycle = new ComponentLifeCycle(null, null, context, null,
            name, lifeCycleThread, new LoggingUtil(logger));
        /*
         * Run Test Case 1
         */
        ComponentContextImpl result = componentLifeCycle.getComponentContext();
        assertEquals("Wrong invokation result", result, context);
    }

    @Override
    protected void setUp() throws MalformedObjectNameException,
        NullPointerException {
        componentDescription = MockComponentDescriptionBuilder
            .buildComponentDescription();
        componentMock = new ComponentMock();
        ComponentContextImpl componentContextImpl = new ComponentContextImpl();
        SystemStateImpl systemStateImpl = createMock(SystemStateImpl.class);
        Logger logger = new MockLogger();
        lifeCycleThread = new ComponentLifeCycleThread(componentMock);
        lifeCycleThread.start();
        componentLifeCycle = new ComponentLifeCycle(componentDescription,
            componentMock, componentContextImpl, systemStateImpl,
            new ObjectName("test@test:name=test"), lifeCycleThread, new LoggingUtil(logger));
    }

    public void testSetState() throws Exception {
        /*
         * Test Case 1 The state is properly set
         */
        /*
         * Create Test parameters
         */
        String state = LifeCycleMBean.STARTED;
        /*
         * Run Test Case 1
         */
        try {
            componentLifeCycle.setState(state);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("State was not properly set", componentLifeCycle
            .getCurrentState(), state);

        /*
         * Test Case 2 An exception is raised, the state could not be persisted
         */
        componentDescription = MockComponentDescriptionBuilder
            .buildComponentDescription();
        componentMock = new ComponentMock();
        ComponentContextImpl componentContextImpl = new ComponentContextImpl();
        SystemStateImpl systemStateImpl = new SystemStateImpl() {
            public void updateComponentLifeCycleState(String componentName,
                String lifecycleState) throws Exception {
                throw new Exception(
                    "updateComponentLifeCycleState testing exception");
            }
        };
        componentLifeCycle = new ComponentLifeCycle(componentDescription,
            componentMock, componentContextImpl, systemStateImpl,
            new ObjectName("test@test:name=test"), null, EasyMock
                .createMock(LoggingUtil.class));
        /*
         * Run Test Case 2
         */
        try {
            componentLifeCycle.setState(state);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }
    }
}
