/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ServiceAssemblyLifeCycleTest.java 12:11:14 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.component.lifecycle;

import java.io.File;

import javax.jbi.JBIException;
import javax.management.ObjectName;

import junit.framework.TestCase;

import org.objectweb.petals.jbi.component.context.ComponentContextImpl;
import org.objectweb.petals.jbi.component.lifecycle.mock.IdentificationMock;
import org.objectweb.petals.jbi.component.lifecycle.mock.MockComponentDescriptionBuilder;
import org.objectweb.petals.jbi.component.lifecycle.mock.MockServiceUnitBuilder;
import org.objectweb.petals.jbi.component.lifecycle.mock.ServiceAssemblyMock;
import org.objectweb.petals.jbi.component.lifecycle.mock.ServiceUnitManagerMock;
import org.objectweb.petals.jbi.management.service.LifeCycleManagerImpl;
import org.objectweb.petals.jbi.management.systemstate.SystemStateImpl;
import org.objectweb.petals.jbi.messaging.mock.MockLogger;
import org.objectweb.petals.jbi.routing.mock.ComponentMock;
import org.objectweb.petals.repository.RepositoryImpl;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * Test of the ServiceAssemblyLifeCycle
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class ServiceAssemblyLifeCycleTest extends TestCase {

    private ServiceAssemblyLifeCycle serviceAssemblyLifeCycle;

    private ServiceUnitLifeCycle serviceUnitLifeCycle;

    public void setUp() throws Exception {
        String baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);

        Logger logger = new MockLogger();
        ServiceAssemblyMock serviceAssemblyMock = new ServiceAssemblyMock();
        IdentificationMock identification = new IdentificationMock();
        identification.setName("testSA");
        serviceAssemblyMock.setIdentification(identification);
        serviceAssemblyLifeCycle = new ServiceAssemblyLifeCycle(new ObjectName(
            "test@test:name=test"), new LoggingUtil(logger),
            serviceAssemblyMock, new RepositoryImpl(), new SystemStateImpl(),
            new File(baseDir + "target").toURI());

        LifeCycleManagerImpl lifeCycleManagerImpl = new LifeCycleManagerImpl();

        ComponentMock componentMock = new ComponentMock();
        ServiceUnitManagerMock serviceUnitManagerMock = new ServiceUnitManagerMock();
        serviceUnitManagerMock.deploy("testServiceUnit", "none");
        componentMock.setServiceUnitManager(serviceUnitManagerMock);

        ComponentLifeCycle componentLifeCycle = new ComponentLifeCycle(
            MockComponentDescriptionBuilder.buildComponentDescription(),
            componentMock, new ComponentContextImpl(), new SystemStateImpl(),
            new ObjectName("test@test:name=test"), null, new LoggingUtil(logger));

        lifeCycleManagerImpl.getBindingCompoLifeCycles().put(
            new ObjectName("test@test:name=test"), componentLifeCycle);
        lifeCycleManagerImpl.start();

        serviceUnitLifeCycle = new ServiceUnitLifeCycle(MockServiceUnitBuilder
            .buildServiceUnit(), lifeCycleManagerImpl, baseDir + File.separator
            + "target", new LoggingUtil(logger));
    }

    public void testRegisterSUIntoSA() throws JBIException {
        serviceAssemblyLifeCycle.registerSUIntoSA("testSU",
            serviceUnitLifeCycle);
        assertEquals(serviceAssemblyLifeCycle.getServiceUnitsLifeCycles().get(
            "testSU"), serviceUnitLifeCycle);
    }

    public void testDoInit() throws JBIException {
        serviceAssemblyLifeCycle.doInit();
    }

    public void testDoShutdownFailed() throws JBIException {
        testRegisterSUIntoSA();
        try {
            serviceAssemblyLifeCycle.doShutdown();
            fail();
        } catch (Exception e) {

        }
    }

    public void testDoShutdown() throws JBIException {
        testRegisterSUIntoSA();
        serviceUnitLifeCycle.state = "Stopped";
        serviceAssemblyLifeCycle.doShutdown();
        assertEquals(serviceAssemblyLifeCycle.getServiceUnitsLifeCycles().get(
            "testSU").getCurrentState(), "Shutdown");
    }

    public void testDoStop() throws JBIException {
        testRegisterSUIntoSA();
        serviceUnitLifeCycle.state = "Started";
        serviceAssemblyLifeCycle.doStop();
        assertEquals(serviceAssemblyLifeCycle.getServiceUnitsLifeCycles().get(
            "testSU").getCurrentState(), "Stopped");
    }

    public void testDoStart() throws JBIException {
        testRegisterSUIntoSA();
        serviceUnitLifeCycle.state = "Stopped";
        serviceAssemblyLifeCycle.doStart();
        assertEquals(serviceAssemblyLifeCycle.getServiceUnitsLifeCycles().get(
            "testSU").getCurrentState(), "Started");
    }

}
