/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ServiceUnitManagerMock.java 11:33:24 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.component.lifecycle.mock;

import java.util.HashMap;
import java.util.Map;

import javax.jbi.JBIException;
import javax.jbi.component.ServiceUnitManager;
import javax.jbi.management.DeploymentException;

/**
 * Mock object of the ServiceUnitManager
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class ServiceUnitManagerMock implements ServiceUnitManager {

    private Map<String, ServiceUnitMock> units = new HashMap<String, ServiceUnitMock>();

    public String deploy(String serviceUnitName, String serviceUnitRootPath)
        throws DeploymentException {
        ServiceUnitMock serviceUnit = new ServiceUnitMock();
        IdentificationMock identification = new IdentificationMock();
        identification.setName(serviceUnitName);
        serviceUnit.setIdentification(identification);
        units.put(serviceUnitName, serviceUnit);
        return "Success";
    }

    public void init(String serviceUnitName, String serviceUnitRootPath)
        throws DeploymentException {
    }

    public void shutDown(String serviceUnitName) throws DeploymentException {
        try {
            units.get(serviceUnitName).shutDown();
        } catch (JBIException e) {
            throw new DeploymentException(e);
        }
    }

    public void start(String serviceUnitName) throws DeploymentException {
        try {
            units.get(serviceUnitName).start();
        } catch (JBIException e) {
            throw new DeploymentException(e);
        }
    }

    public void stop(String serviceUnitName) throws DeploymentException {
        try {
            units.get(serviceUnitName).stop();
        } catch (JBIException e) {
            throw new DeploymentException(e);
        }
    }

    public String undeploy(String serviceUnitName, String serviceUnitRootPath)
        throws DeploymentException {
        return null;
    }

    public Map<String, ServiceUnitMock> getUnits() {
        return units;
    }

}
