package org.objectweb.petals.jbi.component.thread;

import junit.framework.TestCase;

public class AbstractThreadTest extends TestCase {

    /**
     * The abstract thread to test
     */
    protected AbstractThread threadTest;

    /**
     * If the thread is not started, can not submit a task. The thread throws an
     * exception
     * 
     */
    public void testSubmissionFailure() {
        threadTest.execute(0);
        assertNotNull(threadTest.getJbiException());
    }

    /**
     * Test multiple task submitter in separate threads. 
     * 
     */
    public void testMultipleConsumers() {
        int loopA = 10;
        long sleepA = 100;
        int loopB = 5;
        long sleepB = 120;

        threadTest.start();

        ConsumerThread consumerA = new ConsumerThread(threadTest, loopA, sleepA);
        ConsumerThread consumerB = new ConsumerThread(threadTest, loopB, sleepB);

        consumerA.start();
        consumerB.start();

        // wait for tasks to complete
        try {
            consumerA.join(loopA * sleepA * 2);
            consumerB.join(loopB * sleepB * 2);
        } catch (InterruptedException e) {
            fail("Task submission failure");
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see junit.framework.TestCase#setUp()
     */
    protected void setUp() throws Exception {
        super.setUp();

        /*
         * Create the thread test class overriding the doTask method
         */
        threadTest = new AbstractThread() {
            @Override
            protected int doTask(int action) {
                int result = 0;
                switch (action) {
                    case AbstractThread.SHUTDOWNTHREAD:
                        result = -1;
                        break;

                    default:
                        result = action;
                        break;
                }
                return result;
            }
        };
    }

    /**
     * 
     * @author Christophe Hamerling - EBM WebSourcing
     * 
     */
    protected class ConsumerThread extends Thread {

        private AbstractThread abstractThread;

        private long sleepTime;

        private int loop;

        /**
         * Creates a new instance of ConsumerThread
         * 
         * @param abstractThread
         * @param loop
         * @param sleepTime
         */
        public ConsumerThread(AbstractThread abstractThread, int loop,
            long sleepTime) {
            this.abstractThread = abstractThread;
            this.loop = loop;
            this.sleepTime = sleepTime;
        }

        /*
         * (non-Javadoc)
         * 
         * @see java.lang.Thread#run()
         */
        @Override
        public void run() {
            for (int i = 0; i < loop; i++) {
                abstractThread.execute(i);
                try {
                    Thread.sleep(sleepTime);
                } catch (InterruptedException e) {
                }
            }
        }
    }

}
