/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.component.thread;

import javax.jbi.JBIException;
import javax.jbi.component.Bootstrap;
import javax.jbi.component.InstallationContext;
import javax.management.ObjectName;

import junit.framework.TestCase;

import org.objectweb.petals.jbi.component.context.InstallationContextImpl;
import org.objectweb.petals.jbi.component.lifecycle.Installer;

import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.expectLastCall;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.reset;
import static org.easymock.classextension.EasyMock.verify;

/**
 * Installer thread unit test
 * 
 * @author Christophe Hamerling - EBM WebSourcing
 */
public class InstallerThreadTest extends TestCase {

    private BootstrapThread installerThread;

    private Installer installerMock;

    private InstallationContext installContextMock;

    private Bootstrap bootstrapMock;

    /**
     * Test cleanUp
     * 
     */
    public void testCleanUp() throws JBIException {
        installerThread.start();

        reset(bootstrapMock);
        bootstrapMock.cleanUp();
        expectLastCall();
        replay(bootstrapMock);

        try {
            installerThread.cleanUp();
        } catch (JBIException e) {
            fail(e.getMessage());
        }
        verify(bootstrapMock);
    }

    /**
     * Test init
     * 
     */
    public void testInit() throws JBIException {
        installerThread.start();
        
        installContextMock = createMock(InstallationContextImpl.class);

        reset(bootstrapMock);
        
        bootstrapMock.init(installContextMock);
        expectLastCall();
        replay(bootstrapMock);

        try {
            installerThread.init(installContextMock);
        } catch (JBIException e) {
            fail(e.getMessage());
        }
        verify(bootstrapMock);
    }

    /**
     * Test onInstall
     * 
     */
    public void testOnInstall() throws JBIException {
        installerThread.start();

        reset(bootstrapMock);
        bootstrapMock.onInstall();
        expectLastCall();
        replay(bootstrapMock);

        try {
            installerThread.onInstall();
        } catch (JBIException e) {
            fail(e.getMessage());
        }
        verify(bootstrapMock);
    }

    /**
     * Test onUninstall
     * 
     */
    public void testOnUninstall() throws JBIException {
        installerThread.start();

        reset(bootstrapMock);
        bootstrapMock.onUninstall();
        expectLastCall();
        replay(bootstrapMock);

        try {
            installerThread.onUninstall();
        } catch (JBIException e) {
            fail(e.getMessage());
        }
        verify(bootstrapMock);
    }

    /**
     * Test getExtensionMBeanName
     * 
     */
    public void testGetExtensionMBeanName() {
        installerThread.start();
        
        reset(bootstrapMock);
        ObjectName objectName = createMock(ObjectName.class);
        expect(bootstrapMock.getExtensionMBeanName()).andReturn(objectName);
        replay(bootstrapMock);

        ObjectName oName = installerThread.getExtensionMBeanName();
        assertEquals(oName, objectName);
        verify(bootstrapMock);
    }
    
    public void testMultipleCalls(){
        installerThread.start();
        
        try {
            installerThread.onInstall();
            installerThread.onUninstall();
            installerThread.onInstall();
            installerThread.onUninstall();
            installerThread.onInstall();
            installerThread.onUninstall();
        } catch (JBIException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see junit.framework.TestCase#setUp()
     */
    protected void setUp() throws Exception {
        super.setUp();

        installerMock = createMock(Installer.class);
        bootstrapMock = createMock(Bootstrap.class);
        expect(installerMock.getComponentName()).andReturn(null);
        replay(installerMock);

        installerThread = new BootstrapThread(installerMock, bootstrapMock);
    }

}
