/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: EndpointServiceImplTest.java 16:39:45 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.service;

import javax.jbi.JBIException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.jbi.management.service.mock.MockDistributedJMXServer;
import org.objectweb.petals.jbi.management.service.mock.MockRegistry;
import org.objectweb.petals.jbi.messaging.mock.MockLogger;
import org.objectweb.petals.jbi.registry.ConsumerEndpoint;
import org.objectweb.petals.jbi.registry.ExternalEndpoint;
import org.objectweb.petals.jbi.registry.InternalEndpoint;
import org.objectweb.petals.kernel.admin.DistributedJMXServerFactoryImpl;
import org.objectweb.petals.util.LoggingUtil;

/**
 * Test of the EndpointServiceImpl
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class EndpointServiceImplTest extends TestCase {

    private EndpointServiceImpl endpointServiceImp;

    private MockRegistry registry;

    public void setUp() {
        endpointServiceImp = new EndpointServiceImpl();
        registry = new MockRegistry();
        endpointServiceImp.registry = registry;
        // Attach the log
        endpointServiceImp.log = new LoggingUtil(new MockLogger());
    }

    public void testActivateEndpoint() throws JBIException {
        endpointServiceImp.activateEndpoint(new QName("service"), "endpoint",
            new ConsumerEndpoint("compo", "0"));
        assertTrue(registry.isRegisterInternalEndpoint());
    }

    public void testActivateEndpointAddressNull() {
        try {
            endpointServiceImp.activateEndpoint(new QName("serv"), "endpoint",
                null);
            fail();
        } catch (Exception e) {
            // Do nothing
        }
    }

    public void testActivateEndpointEndpointNull() {
        try {
            endpointServiceImp.activateEndpoint(new QName("serv"), null, null);
            fail();
        } catch (Exception e) {
            // Do nothing
        }
    }

    public void testActivateEndpointServiceNull() {
        try {
            endpointServiceImp.activateEndpoint(null, null, null);
            fail();
        } catch (Exception e) {
            // Do nothing
        }
    }

    public void testCreateConnection() throws JBIException {
        endpointServiceImp.createConnection(new QName("serv"), "ep", new QName(
            "toserv"), "toep");
        assertTrue(registry.isRegisterConnection());
    }

    public void testCreateConnectionEndpointNull() {
        try {
            endpointServiceImp.createConnection(new QName("serv"), null, null,
                null);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testCreateConnectionInter() throws JBIException {
        endpointServiceImp.createConnection(new QName("inter"), new QName(
            "toserv"), "toendpoint");
        assertTrue(registry.isRegisterConnection());
    }

    public void testCreateConnectionInterInterNull() {
        try {
            endpointServiceImp.createConnection(null, null, null);
            fail();
        } catch (Exception e) {
            // Do nothing
        }
    }

    public void testCreateConnectionInterToEndpointNull() {
        try {
            endpointServiceImp.createConnection(new QName("inter"), new QName(
                "toserv"), null);
            fail();
        } catch (Exception e) {
            // Do nothing
        }
    }

    public void testCreateConnectionInterToServNull() {
        try {
            endpointServiceImp.createConnection(new QName("inter"), null, null);
            fail();
        } catch (Exception e) {
            // Do nothing
        }
    }

    public void testCreateConnectionServNull() {
        try {
            endpointServiceImp.createConnection(null, null, null, null);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testCreateConnectionToEndpointNull() {
        try {
            endpointServiceImp.createConnection(new QName("serv"), "ep",
                new QName("toserv"), null);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testCreateConnectionToServNull() {
        try {
            endpointServiceImp.createConnection(new QName("serv"), "ep", null,
                null);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testDeactivateEndpoint() throws JBIException {
        InternalEndpoint internalEndpoint = new InternalEndpoint(new QName(
            "service"), "endpoint", "compo", "0", null);
        endpointServiceImp.deactivateEndpoint(internalEndpoint);
        assertTrue(registry.isDeregisterInternalEndpoint());
    }

    public void testDeactivateEndpointNullException() {
        try {
            endpointServiceImp.deactivateEndpoint(null);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testDeleteConnection() throws JBIException {
        endpointServiceImp.deleteConnection(new QName("serv"), "ep", new QName(
            "toserv"), "toep");
        assertTrue(registry.isDeregisterConnection());
    }

    public void testDeleteConnectionEndpointNull() {
        try {
            endpointServiceImp.deleteConnection(new QName("serv"), null, null,
                null);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testDeleteConnectionInter() throws JBIException {
        endpointServiceImp.deleteConnection(new QName("inter"), new QName(
            "toserv"), "toep");
        assertTrue(registry.isDeregisterConnection());
    }

    public void testDeleteConnectionInterInterNull() {
        try {
            endpointServiceImp.deleteConnection(null, null, null);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testDeleteConnectionInterToEndpointNull() {
        try {
            endpointServiceImp.deleteConnection(new QName("inter"), new QName(
                "toserv"), null);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testDeleteConnectionInterToServNull() {
        try {
            endpointServiceImp.deleteConnection(new QName("inter"), null, null);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testDeleteConnectionServNull() {
        try {
            endpointServiceImp.deleteConnection(null, null, null, null);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testDeleteConnectionToEndpointNull() {
        try {
            endpointServiceImp.deleteConnection(new QName("serv"), "ep",
                new QName("toserv"), null);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testDeleteConnectionToServNull() {
        try {
            endpointServiceImp.deleteConnection(new QName("serv"), "ep", null,
                null);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testDeregisterExternaIEndpoint() throws JBIException {
        InternalEndpoint internalEndpoint = new InternalEndpoint(new QName(
            "service"), "endpoint", "compo", "0", null);
        endpointServiceImp.deregisterExternalEndpoint(internalEndpoint);
        assertTrue(registry.isDeregisterExternalEndpoint());
    }

    public void testDeregisterExternalEndpointNullException() {
        try {
            endpointServiceImp.deregisterExternalEndpoint(null);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testGetEndpoint() throws Exception {
        endpointServiceImp.getEndpoint(new QName("service"), "name");
        assertTrue(registry.isGetInternalEndpoint());
    }

    public void testGetEndpointDescriptorForEndpoint() throws JBIException {
        InternalEndpoint internalEndpoint = new InternalEndpoint(new QName(
            "serv"), "endpoint", "compo", "0", endpointServiceImp);
        endpointServiceImp.getEndpointDescriptorForEndpoint(internalEndpoint);
    }

    public void testGetEndpointDescriptorForEndpointNull() {
        try {
            endpointServiceImp.getEndpointDescriptorForEndpoint(null);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testGetEndpointEndpointNull() {
        try {
            endpointServiceImp.getEndpoint(new QName("serv"), null);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testGetEndpointServiceNull() {
        try {
            endpointServiceImp.getEndpoint(null, null);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testGetExternalEndpointsForInterface() throws Exception {
        endpointServiceImp
            .getExternalEndpointsForInterface(new QName("service"));
        assertTrue(registry.isGetExternalEndpointsForInterface());
    }

    public void testGetExternalEndpointsForInterfaceNull() {
        try {
            endpointServiceImp.getExternalEndpointsForInterface(null);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testGetExternalEndpointsForService() throws Exception {
        endpointServiceImp.getExternalEndpointsForService(new QName("service"));
        assertTrue(registry.isGetExternalEndpointsForService());
    }

    public void testGetExternalEndpointsForServiceNull() {
        try {
            endpointServiceImp.getExternalEndpointsForService(null);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testGetInterfacesForEndpoint() throws PetalsException {
        DistributedJMXServerFactoryImpl distributedJMXServerFactoryImpl = EasyMock
            .createMock(DistributedJMXServerFactoryImpl.class);
        MockDistributedJMXServer distributedJMXServer = new MockDistributedJMXServer(
            null);
        EasyMock.expect(
            distributedJMXServerFactoryImpl.createDistributedJMXServer("0"))
            .andReturn(distributedJMXServer).anyTimes();
        EasyMock.replay(distributedJMXServerFactoryImpl);
        endpointServiceImp.serverManager = distributedJMXServerFactoryImpl;

        InternalEndpoint internalEndpoint = new InternalEndpoint(new QName(
            "http://127.0.0.1:8080/axis/Helloworld.jws", "HelloworldService"),
            "endpoint", "compo", "0", endpointServiceImp);

        QName[] interfs = endpointServiceImp
            .getInterfacesForEndpoint(internalEndpoint);
        assertTrue(distributedJMXServer.isGetAdminServiceMBeanName());
        assertTrue(distributedJMXServer.isInvoke());
        assertTrue(registry.isValidateEndpoint());
        assertEquals(1, interfs.length);
    }

    public void testGetInterfacesForEndpointBadServiceName()
        throws PetalsException {
        DistributedJMXServerFactoryImpl distributedJMXServerFactoryImpl = EasyMock
            .createMock(DistributedJMXServerFactoryImpl.class);
        MockDistributedJMXServer distributedJMXServer = new MockDistributedJMXServer(
            null);
        EasyMock.expect(
            distributedJMXServerFactoryImpl.createDistributedJMXServer("0"))
            .andReturn(distributedJMXServer).anyTimes();
        EasyMock.replay(distributedJMXServerFactoryImpl);
        endpointServiceImp.serverManager = distributedJMXServerFactoryImpl;

        InternalEndpoint internalEndpoint = new InternalEndpoint(new QName(
            "HelloworldService"), "endpoint", "compo", "0", endpointServiceImp);

        QName[] interfs = endpointServiceImp
            .getInterfacesForEndpoint(internalEndpoint);
        assertTrue(distributedJMXServer.isGetAdminServiceMBeanName());
        assertTrue(distributedJMXServer.isInvoke());
        assertEquals(0, interfs.length);
    }

    public void testGetInterfacesForEndpointNull() {
        try {
            endpointServiceImp.getInterfacesForEndpoint(null);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testGetInternalEndpointsForInterface() {
        ServiceEndpoint[] endpoints = endpointServiceImp
            .getInternalEndpointsForInterface(new QName("inter"));
        assertEquals(1, endpoints.length);
    }

    public void testGetInternalEndpointsForInterfaceNotRegistered() {
        ServiceEndpoint[] endpoints = endpointServiceImp
            .getInternalEndpointsForInterface(new QName("inter1"));
        assertEquals(0, endpoints.length);
        assertTrue(registry.isGetInternalEndpointsForInterface());
    }

    public void testGetInternalEndpointsForServiceNull() {
        try {
            endpointServiceImp.getInternalEndpointsForService(null);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testGetInternalEndpointsForService() {
        ServiceEndpoint[] endpoints = endpointServiceImp
            .getInternalEndpointsForService(new QName("serv"));
        assertEquals(1, endpoints.length);
        assertTrue(registry.isGetInternalEndpointsForService());
    }

    public void testIsExchangeWithConsumerOkayForComponentNull() {
        try {
            endpointServiceImp.isExchangeWithConsumerOkayForComponent(null,
                null);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testIsExchangeWithProviderOkayForComponentNull() {
        try {
            endpointServiceImp.isExchangeWithProviderOkayForComponent(null,
                null);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testIsExchangeWithConsumerOkayForComponent()
        throws PetalsException {
        DistributedJMXServerFactoryImpl distributedJMXServerFactoryImpl = EasyMock
            .createMock(DistributedJMXServerFactoryImpl.class);
        MockDistributedJMXServer distributedJMXServer = new MockDistributedJMXServer(
            null);
        EasyMock.expect(
            distributedJMXServerFactoryImpl.createDistributedJMXServer("0"))
            .andReturn(distributedJMXServer).anyTimes();
        EasyMock.replay(distributedJMXServerFactoryImpl);
        endpointServiceImp.serverManager = distributedJMXServerFactoryImpl;

        InternalEndpoint internalEndpoint = new InternalEndpoint(new QName(
            "serv"), "endpoint", "compo", "0", endpointServiceImp);
        boolean test = endpointServiceImp
            .isExchangeWithConsumerOkayForComponent(internalEndpoint, null);
        assertTrue(distributedJMXServer.isGetAdminServiceMBeanName());
        assertTrue(distributedJMXServer.isInvoke());
        assertTrue(test);
    }

    public void testIsExchangeWithProviderOkayForComponent()
        throws PetalsException {
        DistributedJMXServerFactoryImpl distributedJMXServerFactoryImpl = EasyMock
            .createMock(DistributedJMXServerFactoryImpl.class);
        MockDistributedJMXServer distributedJMXServer = new MockDistributedJMXServer(
            null);
        EasyMock.expect(
            distributedJMXServerFactoryImpl.createDistributedJMXServer("0"))
            .andReturn(distributedJMXServer).anyTimes();
        EasyMock.replay(distributedJMXServerFactoryImpl);
        endpointServiceImp.serverManager = distributedJMXServerFactoryImpl;

        InternalEndpoint internalEndpoint = new InternalEndpoint(new QName(
            "serv"), "endpoint", "compo", "0", endpointServiceImp);
        boolean test = endpointServiceImp
            .isExchangeWithProviderOkayForComponent(internalEndpoint, null);
        assertTrue(distributedJMXServer.isGetAdminServiceMBeanName());
        assertTrue(distributedJMXServer.isInvoke());
        assertTrue(test);
    }

    public void testRegisterExternalEndpointNull() {
        try {
            endpointServiceImp.registerExternalEndpoint(null);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testRegisterExternalEndpoint() throws JBIException {
        ExternalEndpoint externalEndpoint = new ExternalEndpoint(
            new ConsumerEndpoint("compo", "0"), "compo", "0");
        endpointServiceImp.registerExternalEndpoint(externalEndpoint);
        assertTrue(registry.isRegisterExternalEndpoint());
    }

    public void testStop() throws IllegalLifeCycleException {
        endpointServiceImp.stop();
    }

}
