/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MockDistributedJMXServer.java 11:34:05 AM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.service.mock;

import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.ReflectionException;
import javax.management.remote.JMXConnector;

import org.easymock.classextension.EasyMock;
import org.objectweb.petals.kernel.admin.DistributedJMXServer;
import org.objectweb.petals.tools.jbicommon.util.XMLUtil;

/**
 * Mock object of the DistributedJMXServer
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class MockDistributedJMXServer extends DistributedJMXServer {

    private boolean getAdminServiceMBeanName;

    private boolean invoke;

    public MockDistributedJMXServer(MBeanServerConnection connection) {
        super(EasyMock.createMock(JMXConnector.class));
    }

    public ObjectName getAdminServiceMBeanName() throws IOException,
        MalformedObjectNameException, NullPointerException {
        getAdminServiceMBeanName = true;
        return null;
    }

    public Object invoke(ObjectName name, String operationName,
        Object[] params, String[] signature) throws InstanceNotFoundException,
        MBeanException, ReflectionException, IOException {
        invoke = true;
        if (operationName.equals(DistributedJMXServer.GET_SERVICE_DESCRITION)) {
            String baseDir = this.getClass().getResource(".").toString();
            baseDir = baseDir.substring(0, baseDir.indexOf("target"));
            baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
            DataInputStream dis = new DataInputStream(new FileInputStream(
                baseDir + "src" + File.separator + "test-data" + File.separator
                    + "wsdl" + File.separator + "HelloworldService.wsdl"));
            byte[] bytes = new byte[dis.available()];
            dis.readFully(bytes);
            dis.close();
            return XMLUtil.createDocumentFromString(new String(bytes));
        } else if (operationName.equals(DistributedJMXServer.IS_OK_WITH_CONS)) {
            return Boolean.TRUE;
        } else if (operationName.equals(DistributedJMXServer.IS_OK_WITH_PROV)) {
            return Boolean.TRUE;
        }
        return null;
    }

    public boolean isGetAdminServiceMBeanName() {
        return getAdminServiceMBeanName;
    }

    public boolean isInvoke() {
        return invoke;
    }

    public void setGetAdminServiceMBeanName(boolean getAdminServiceMBeanName) {
        this.getAdminServiceMBeanName = getAdminServiceMBeanName;
    }

    public void setInvoke(boolean invoke) {
        this.invoke = invoke;
    }

}
