/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SystemStateImplTest.java 5:04:24 PM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.systemstate;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.petals.jbi.management.service.ManagementException;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * Tests of the SystemStateImpl
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class SystemStateImplTest extends TestCase {

    public void testCreateComponentStateHolder() throws Exception {
        SystemStateImpl systemStateImpl = new SystemStateImpl();
        StateStorage stateStorage = EasyMock.createMock(StateStorage.class);
        ComponentState componentState = EasyMock
                .createMock(ComponentState.class);

        EasyMock.expect(
                stateStorage.createComponentStateHolder("compo", "installURL",
                        "zipURL", "installState", "lifecycleState")).andReturn(
                componentState);

        EasyMock.replay(stateStorage);

        systemStateImpl.stateStorage = stateStorage;

        assertEquals(componentState, systemStateImpl
                .createComponentStateHolder("compo", "installURL", "zipURL",
                        "installState", "lifecycleState"));
    }

    public void testDeleteComponentStateHolder() throws Exception {
        SystemStateImpl systemStateImpl = new SystemStateImpl();
        StateStorage stateStorage = EasyMock.createMock(StateStorage.class);
        ComponentState componentState = EasyMock
                .createMock(ComponentState.class);

        EasyMock.expect(
                stateStorage.deleteComponentStateHolder("componentName"))
                .andReturn(componentState);

        EasyMock.replay(stateStorage);

        systemStateImpl.stateStorage = stateStorage;

        assertEquals(componentState, systemStateImpl
                .deleteComponentStateHolder("componentName"));
    }

    public void testCreateServiceAssemblyStateHolder() throws Exception {
        SystemStateImpl systemStateImpl = new SystemStateImpl();
        StateStorage stateStorage = EasyMock.createMock(StateStorage.class);
        ServiceAssemblyState serviceAssemblyState = EasyMock
                .createMock(ServiceAssemblyState.class);

        EasyMock.expect(
                stateStorage.createServiceAssemblyStateHolder("saName",
                        "installURL", "zipURL", "lifecycleState")).andReturn(
                serviceAssemblyState);

        EasyMock.replay(stateStorage);

        systemStateImpl.stateStorage = stateStorage;

        assertEquals(serviceAssemblyState, systemStateImpl
                .createServiceAssemblyStateHolder("saName", "installURL",
                        "zipURL", "lifecycleState"));
    }

    public void testDeleteServiceAssemblyStateHolder() throws Exception {
        SystemStateImpl systemStateImpl = new SystemStateImpl();
        StateStorage stateStorage = EasyMock.createMock(StateStorage.class);
        ServiceAssemblyState serviceAssemblyState = EasyMock
                .createMock(ServiceAssemblyState.class);

        EasyMock
                .expect(stateStorage.deleteServiceAssemblyStateHolder("saName"))
                .andReturn(serviceAssemblyState);

        EasyMock.replay(stateStorage);

        systemStateImpl.stateStorage = stateStorage;

        assertEquals(serviceAssemblyState, systemStateImpl
                .deleteServiceAssemblyStateHolder("saName"));
    }

    public void testCreateSharedLibraryStateHolder() throws Exception {
        SystemStateImpl systemStateImpl = new SystemStateImpl();
        StateStorage stateStorage = EasyMock.createMock(StateStorage.class);
        SharedLibraryState sharedLibraryState = EasyMock
                .createMock(SharedLibraryState.class);

        EasyMock.expect(
                stateStorage.createSharedLibraryStateHolder("slName",
                        "installURL", "zipURL")).andReturn(sharedLibraryState);

        EasyMock.replay(stateStorage);

        systemStateImpl.stateStorage = stateStorage;

        assertEquals(sharedLibraryState, systemStateImpl
                .createSharedLibraryStateHolder("slName", "installURL",
                        "zipURL"));
    }

    public void testDeleteSharedLibraryStateHolder() throws Exception {
        SystemStateImpl systemStateImpl = new SystemStateImpl();
        StateStorage stateStorage = EasyMock.createMock(StateStorage.class);
        SharedLibraryState sharedLibraryState = EasyMock
                .createMock(SharedLibraryState.class);

        EasyMock.expect(stateStorage.deleteSharedLibraryStateHolder("slName"))
                .andReturn(sharedLibraryState);

        EasyMock.replay(stateStorage);

        systemStateImpl.stateStorage = stateStorage;

        assertEquals(sharedLibraryState, systemStateImpl
                .deleteSharedLibraryStateHolder("slName"));
    }

    public void testRecoverAllComponent() throws ManagementException {
        SystemStateImpl systemStateImpl = new SystemStateImpl();
        SystemRecovery systemRecovery = EasyMock
                .createMock(SystemRecovery.class);

        EasyMock.expect(systemRecovery.recoverAllComponent()).andReturn(true);

        EasyMock.replay(systemRecovery);

        systemStateImpl.systemRecovery = systemRecovery;

        systemRecovery.recoverAllComponent();
    }

    public void testRecoverAllServiceAssembly() throws ManagementException {
        SystemStateImpl systemStateImpl = new SystemStateImpl();
        SystemRecovery systemRecovery = EasyMock
                .createMock(SystemRecovery.class);

        EasyMock.expect(systemRecovery.recoverAllServiceAssembly()).andReturn(
                true);

        EasyMock.replay(systemRecovery);

        systemStateImpl.systemRecovery = systemRecovery;

        systemRecovery.recoverAllServiceAssembly();
    }

    public void testRecoverAllSharedLibrary() throws ManagementException {
        SystemStateImpl systemStateImpl = new SystemStateImpl();
        SystemRecovery systemRecovery = EasyMock
                .createMock(SystemRecovery.class);

        EasyMock.expect(systemRecovery.recoverAllSharedLibrary()).andReturn(
                true);

        EasyMock.replay(systemRecovery);

        systemStateImpl.systemRecovery = systemRecovery;

        systemRecovery.recoverAllSharedLibrary();
    }

    public void testRecoverAllEntities() throws SecurityException,
        NoSuchMethodException, ManagementException {
        SystemStateImpl systemStateImpl = EasyMock.createMock(
                SystemStateImpl.class, new Method[] {
                    SystemStateImpl.class.getDeclaredMethod(
                            "recoverAllSharedLibrary", new Class[] {}),
                    SystemStateImpl.class.getDeclaredMethod(
                            "recoverAllComponent", new Class[] {}),
                    SystemStateImpl.class.getDeclaredMethod(
                            "recoverAllServiceAssembly", new Class[] {})});
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);

        loggingUtil.start();
        EasyMock.expect(systemStateImpl.recoverAllComponent()).andReturn(true);
        EasyMock.expect(systemStateImpl.recoverAllServiceAssembly()).andReturn(
                true);
        EasyMock.expect(systemStateImpl.recoverAllSharedLibrary()).andReturn(
                true);
        loggingUtil.end();

        EasyMock.replay(systemStateImpl);
        EasyMock.replay(loggingUtil);

        systemStateImpl.log = loggingUtil;

        systemStateImpl.recoverAllEntities();
    }

    public void testRecoverAllEntitiesException() throws SecurityException,
        NoSuchMethodException, ManagementException {
        SystemStateImpl systemStateImpl = EasyMock.createMock(
                SystemStateImpl.class, new Method[] {
                    SystemStateImpl.class.getDeclaredMethod(
                            "recoverAllSharedLibrary", new Class[] {}),
                    SystemStateImpl.class.getDeclaredMethod(
                            "recoverAllComponent", new Class[] {}),
                    SystemStateImpl.class.getDeclaredMethod(
                            "recoverAllServiceAssembly", new Class[] {})});
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);
        ManagementException managementException = new ManagementException(
                "test error");

        loggingUtil.start();
        EasyMock.expect(systemStateImpl.recoverAllSharedLibrary()).andThrow(
                managementException);
        loggingUtil.error("Entities can't be correctly recovered !",
                managementException);
        loggingUtil.end();

        EasyMock.replay(systemStateImpl);
        EasyMock.replay(loggingUtil);

        systemStateImpl.log = loggingUtil;

        try {
            systemStateImpl.recoverAllEntities();
            fail("recoverAllEntities must throw an exception if SystemStateImpl.recoverAllSharedLibrary throws an exception");
        } catch (ManagementException e) {
            // DO nothing it's ok
        }
    }

    public void testRetrieveAllComponentStates() throws ManagementException {
        SystemStateImpl systemStateImpl = new SystemStateImpl();
        StateStorage stateStorage = EasyMock.createMock(StateStorage.class);
        List<ComponentState> compos = new ArrayList<ComponentState>();

        EasyMock.expect(stateStorage.retrieveAllComponentStates()).andReturn(
                compos);

        EasyMock.replay(stateStorage);

        systemStateImpl.stateStorage = stateStorage;

        assertEquals(systemStateImpl.retrieveAllComponentStates(), compos);
    }

    public void testRetrieveAllServiceAssemblyStates()
        throws ManagementException {
        SystemStateImpl systemStateImpl = new SystemStateImpl();
        StateStorage stateStorage = EasyMock.createMock(StateStorage.class);
        List<ServiceAssemblyState> serv = new ArrayList<ServiceAssemblyState>();

        EasyMock.expect(stateStorage.retrieveAllServiceAssemblyStates())
                .andReturn(serv);

        EasyMock.replay(stateStorage);

        systemStateImpl.stateStorage = stateStorage;

        assertEquals(systemStateImpl.retrieveAllServiceAssemblyStates(), serv);
    }

    public void testRetrieveAllSharedLibraryStates() throws ManagementException {
        SystemStateImpl systemStateImpl = new SystemStateImpl();
        StateStorage stateStorage = EasyMock.createMock(StateStorage.class);
        List<SharedLibraryState> shList = new ArrayList<SharedLibraryState>();

        EasyMock.expect(stateStorage.retrieveAllSharedLibraryStates())
                .andReturn(shList);

        EasyMock.replay(stateStorage);

        systemStateImpl.stateStorage = stateStorage;

        assertEquals(systemStateImpl.retrieveAllSharedLibraryStates(), shList);
    }


    public void testUpdateComponentInstallationState() throws Exception {
        SystemStateImpl systemStateImpl = new SystemStateImpl();
        StateStorage stateStorage = EasyMock.createMock(StateStorage.class);

        stateStorage.updateComponentInstallationState("compo", "installState");

        EasyMock.replay(stateStorage);

        systemStateImpl.stateStorage = stateStorage;

        systemStateImpl.updateComponentInstallationState("compo",
                "installState");
    }

    public void testUpdateComponentLifeCycleState() throws Exception {
        SystemStateImpl systemStateImpl = new SystemStateImpl();
        StateStorage stateStorage = EasyMock.createMock(StateStorage.class);

        stateStorage.updateComponentLifeCycleState("compo", "lifecycleState");

        EasyMock.replay(stateStorage);

        systemStateImpl.stateStorage = stateStorage;

        systemStateImpl
                .updateComponentLifeCycleState("compo", "lifecycleState");
    }

    public void testUpdateServiceAssemblyState() throws Exception {
        SystemStateImpl systemStateImpl = new SystemStateImpl();
        StateStorage stateStorage = EasyMock.createMock(StateStorage.class);

        stateStorage.updateServiceAssemblyState("saName", "lifecycleState");

        EasyMock.replay(stateStorage);

        systemStateImpl.stateStorage = stateStorage;

        systemStateImpl.updateServiceAssemblyState("saName", "lifecycleState");
    }

    public void testShutdown() {
        SystemStateImpl systemStateImpl = new SystemStateImpl();
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);

        loggingUtil.start();
        loggingUtil.end();

        EasyMock.replay(loggingUtil);
        systemStateImpl.log = loggingUtil;
        systemStateImpl.logger = EasyMock.createMock(Logger.class);

        systemStateImpl.stop();
        assertNull(systemStateImpl.logger);
    }
}
