/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MessageExchangeFactoryImplTest.java 11:21:38 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.messaging;

import java.net.URI;

import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.MessageExchange.Role;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.petals.jbi.component.context.ComponentContextImpl;
import org.objectweb.petals.jbi.messaging.mock.MockLogger;
import org.objectweb.petals.jbi.registry.InternalEndpoint;
import org.objectweb.petals.jbi.routing.RouterImpl;
import org.objectweb.util.monolog.api.Logger;

/**
 * Test of the MessageExchangeFactoryImpl
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class MessageExchangeFactoryImplTest extends TestCase {

    private MessageExchangeFactoryImpl factoryImpl;

    public void setUp() {
        ComponentContextImpl componentContextImpl = new ComponentContextImpl();
        RouterImpl routerImpl = new RouterImpl();
        Logger logger = new MockLogger();
        DeliveryChannelImpl deliveryChannelImpl = new DeliveryChannelImpl(
            componentContextImpl, routerImpl, logger);
        factoryImpl = new MessageExchangeFactoryImpl(deliveryChannelImpl, null,
            logger);
    }

    public void testSetDefaultInterfaceName() {
        factoryImpl.setDefaultInterfaceName(new QName("defaultinterfacename"));
        assertEquals(factoryImpl.defaultInterfaceName, new QName(
            "defaultinterfacename"));
    }

    public void testSetDefaultServiceEndpoint() {
        InternalEndpoint internalEndpoint = new InternalEndpoint(new QName(
            "servicename"), "endpoint", "compo", null, null);
        factoryImpl.setDefaultServiceEndpoint(internalEndpoint);
        assertEquals(factoryImpl.defaultServiceEndpoint, internalEndpoint);
    }

    public void testSetDefaultServiceName() {
        factoryImpl.setDefaultServiceName(new QName("servicename"));
        assertEquals(factoryImpl.defaultServiceName, new QName("servicename"));
    }

    public void testCreateExchange() throws MessagingException {
        factoryImpl.setDefaultServiceName(new QName("servicename"));
        MessageExchangeImpl messageExchangeImpl = factoryImpl.createExchange();
        assertEquals(messageExchangeImpl.getService(), new QName("servicename"));
    }

    public void testCreateExchange1() throws MessagingException {
        MessageExchange messageExchange = factoryImpl.createExchange(new QName(
            "servicename1"), new QName("operation1"));
        assertEquals(messageExchange.getService(), new QName("servicename1"));
        assertEquals(messageExchange.getOperation(), new QName("operation1"));
    }

    public void testCreateExchange2() throws MessagingException {
        MessageExchange messageExchange = factoryImpl
            .createExchange(MessageExchangeImpl.IN_ONLY_PATTERN);
        assertEquals(messageExchange.getPattern(),
            MessageExchangeImpl.IN_ONLY_PATTERN);
    }

    public void testCreateInOnlyExchange() throws MessagingException {
        MessageExchange messageExchange = factoryImpl.createInOnlyExchange();
        assertEquals(messageExchange.getPattern(),
            MessageExchangeImpl.IN_ONLY_PATTERN);
    }

    public void testCreateInOptionalOutExchange() throws MessagingException {
        MessageExchange messageExchange = factoryImpl
            .createInOptionalOutExchange();
        assertEquals(messageExchange.getPattern(),
            MessageExchangeImpl.IN_OPTIONAL_OUT_PATTERN);
    }

    public void testCreateInOutExchange() throws MessagingException {
        MessageExchange messageExchange = factoryImpl.createInOutExchange();
        assertEquals(messageExchange.getPattern(),
            MessageExchangeImpl.IN_OUT_PATTERN);
    }

    public void testCreateRobustInOnlyExchange() throws MessagingException {
        MessageExchange messageExchange = factoryImpl
            .createRobustInOnlyExchange();
        assertEquals(messageExchange.getPattern(),
            MessageExchangeImpl.ROBUST_IN_ONLY_PATTERN);
    }

    public void testCreateExchangeException() {
        try {
            factoryImpl.createExchange(URI
                .create("http://www.ebmwebsourcing.org/"));
            fail();
        } catch (MessagingException e) {
            // do nothing
        }
    }

    public void testCreateExchangeDecoratorROBUST_IN_ONLY_PATTERN()
        throws MessagingException {
        MessageExchangeImpl messageExchangeImpl = EasyMock
            .createMock(MessageExchangeImpl.class);

        EasyMock.expect(messageExchangeImpl.getRole()).andReturn(Role.CONSUMER)
            .anyTimes();
        messageExchangeImpl.setPattern(MessageExchangeImpl.IN_ONLY_PATTERN);
        messageExchangeImpl
            .setPattern(MessageExchangeImpl.ROBUST_IN_ONLY_PATTERN);
        EasyMock.expect(messageExchangeImpl.getPattern()).andReturn(
            MessageExchangeImpl.ROBUST_IN_ONLY_PATTERN);

        EasyMock.replay(messageExchangeImpl);

        MessageExchange messageExchange = factoryImpl.createExchangeDecorator(
            messageExchangeImpl, MessageExchangeImpl.ROBUST_IN_ONLY_PATTERN);
        assertEquals(messageExchange.getPattern(),
            MessageExchangeImpl.ROBUST_IN_ONLY_PATTERN);
    }

    public void testCreateExchangeDecoratorIN_OUT_PATTERN()
        throws MessagingException {
        MessageExchangeImpl messageExchangeImpl = EasyMock
            .createMock(MessageExchangeImpl.class);

        EasyMock.expect(messageExchangeImpl.getRole()).andReturn(Role.CONSUMER)
            .anyTimes();
        messageExchangeImpl.setPattern(MessageExchangeImpl.IN_OUT_PATTERN);
        messageExchangeImpl.setPattern(MessageExchangeImpl.IN_OUT_PATTERN);
        EasyMock.expect(messageExchangeImpl.getPattern()).andReturn(
            MessageExchangeImpl.IN_OUT_PATTERN);

        EasyMock.replay(messageExchangeImpl);

        MessageExchange messageExchange = factoryImpl.createExchangeDecorator(
            messageExchangeImpl, MessageExchangeImpl.IN_OUT_PATTERN);
        assertEquals(messageExchange.getPattern(),
            MessageExchangeImpl.IN_OUT_PATTERN);
    }

    public void testCreateExchangeDecoratorIN_OPTIONAL_OUT_PATTERN()
        throws MessagingException {
        MessageExchangeImpl messageExchangeImpl = EasyMock
            .createMock(MessageExchangeImpl.class);

        EasyMock.expect(messageExchangeImpl.getRole()).andReturn(Role.CONSUMER)
            .anyTimes();
        messageExchangeImpl.setPattern(MessageExchangeImpl.IN_OUT_PATTERN);
        messageExchangeImpl
            .setPattern(MessageExchangeImpl.IN_OPTIONAL_OUT_PATTERN);
        EasyMock.expect(messageExchangeImpl.getPattern()).andReturn(
            MessageExchangeImpl.IN_OPTIONAL_OUT_PATTERN);

        EasyMock.replay(messageExchangeImpl);

        MessageExchange messageExchange = factoryImpl.createExchangeDecorator(
            messageExchangeImpl, MessageExchangeImpl.IN_OPTIONAL_OUT_PATTERN);
        assertEquals(messageExchange.getPattern(),
            MessageExchangeImpl.IN_OPTIONAL_OUT_PATTERN);
    }

    public void testCreateExchangeDecoratorBadPattern()
        throws MessagingException {
        MessageExchangeImpl messageExchangeImpl = EasyMock
            .createMock(MessageExchangeImpl.class);

        EasyMock.expect(messageExchangeImpl.getRole()).andReturn(Role.CONSUMER);

        EasyMock.replay(messageExchangeImpl);

        assertNotNull(factoryImpl.createExchangeDecorator(messageExchangeImpl,
            URI.create("http://www.ebmwebsourcing.org/")));
    }

    public void testSetDefaultMessageExchangeProperties() {
        ServiceEndpoint serviceEndpoint = EasyMock
            .createMock(ServiceEndpoint.class);
        factoryImpl.defaultInterfaceName = new QName("inter");
        factoryImpl.defaultServiceEndpoint = serviceEndpoint;
        factoryImpl.defaultServiceName = new QName("serv");
        MessageExchangeImpl messageExchangeImpl = EasyMock
            .createMock(MessageExchangeImpl.class);

        messageExchangeImpl.setInterfaceName(new QName("inter"));
        messageExchangeImpl.setService(new QName("serv"));
        messageExchangeImpl.setEndpoint(serviceEndpoint);
        
        EasyMock.replay(messageExchangeImpl);

        factoryImpl.setDefaultMessageExchangeProperties(messageExchangeImpl);
    }
}
