/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: NormalizedMessageImplTest.java 10:12:25 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.messaging;

import java.io.File;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import javax.activation.DataHandler;
import javax.activation.FileDataSource;
import javax.jbi.messaging.MessagingException;
import javax.security.auth.Subject;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.petals.util.SourceHelper;

/**
 * Test of the NormalizedMessageImpl
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class NormalizedMessageImplTest extends TestCase {

    private NormalizedMessageImpl normalizedMessageImpl;

    private FileDataSource source;

    private String baseDir;

    public void setUp() {
        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
        source = new FileDataSource(new File(baseDir + "src" + File.separator
            + "test-data" + File.separator + "filesource.txt"));
        normalizedMessageImpl = new NormalizedMessageImpl();
    }

    public void testAddAttachment() throws IOException {
        try {
            normalizedMessageImpl
                .addAttachment("test", new DataHandler(source));
        } catch (MessagingException e) {
            fail();
        }
    }

    public void testGetAttachment() throws IOException {
        testAddAttachment();
        DataHandler dataHandler = normalizedMessageImpl.getAttachment("test");
        String content = (String) dataHandler.getContent();
        assertEquals(content, "File used to test the normalized message");
    }

    public void testGetContent() throws IOException, MessagingException {
        normalizedMessageImpl = new NormalizedMessageImpl();
        Source xml = new StreamSource(new File(baseDir + "src" + File.separator
            + "test-data" + File.separator + "descriptors" + File.separator
            + "sharedLibraryInstallationDescriptor.xml"));
        normalizedMessageImpl.setContent(xml);
        StreamSource source = (StreamSource) normalizedMessageImpl.getContent();
        assertEquals(source, xml);
    }

    public void testGetAttachmentNames() throws IOException {
        testAddAttachment();
        assertEquals(normalizedMessageImpl.getAttachmentNames().iterator()
            .next(), "test");
    }

    public void testRemoveAttachment() throws IOException, MessagingException {
        testAddAttachment();
        normalizedMessageImpl.removeAttachment("test");
        assertNull(normalizedMessageImpl.getAttachment("test"));
    }

    public void testRemoveAttachment1() throws IOException, MessagingException {
        testAddAttachment();
        try {
            normalizedMessageImpl.removeAttachment("foo");
            fail();
        } catch (MessagingException e) {

        }
    }

    public void testGetPropertyNames() throws IOException {
        normalizedMessageImpl = new NormalizedMessageImpl();
        normalizedMessageImpl.setProperty("test", "value");
        assertEquals(
            normalizedMessageImpl.getPropertyNames().iterator().next(), "test");
    }

    public void testGetSecuritySubject() {
        Subject subject = new Subject();
        normalizedMessageImpl.setSecuritySubject(subject);
        assertEquals(normalizedMessageImpl.getSecuritySubject(), subject);
    }

    public void testSetProperty() {
        normalizedMessageImpl.properties.put("test", EasyMock
            .createMock(DataHandler.class));
        normalizedMessageImpl.setProperty("test", null);
        assertNull(normalizedMessageImpl.attachments.get("test"));
    }

    public void testReadObjectDelegate() throws Exception {
        ObjectInputStream objectInputStream = EasyMock
            .createMock(ObjectInputStream.class);

        objectInputStream.defaultReadObject();
        EasyMock.expect(objectInputStream.available()).andReturn(4);
        EasyMock.expect(objectInputStream.readUTF()).andReturn("<test/>");
        EasyMock.expect(objectInputStream.readInt()).andReturn(0);

        EasyMock.replay(objectInputStream);

        normalizedMessageImpl.readObjectDelegate(objectInputStream);
        assertEquals(SourceHelper.createString(normalizedMessageImpl.content),
            "<test/>");
    }

    public void testWriteObjectDelegate() throws IOException {
        normalizedMessageImpl.content = SourceHelper.createSource("<foo/>");
        ObjectOutputStream objectOutputStream = EasyMock
            .createMock(ObjectOutputStream.class);

        objectOutputStream.defaultWriteObject();
        objectOutputStream.writeUTF("<foo/>");
        objectOutputStream.writeInt(0);
        
        EasyMock.replay(objectOutputStream);
        
        normalizedMessageImpl.writeObjectDelegate(objectOutputStream);
    }
}
