/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: AddressResolverTest.java 15:49:03 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.routing;

import java.lang.reflect.Method;

import javax.jbi.JBIException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.petals.jbi.component.context.ComponentContextImpl;
import org.objectweb.petals.jbi.component.lifecycle.mock.MockJBIDescriptorBuilder;
import org.objectweb.petals.jbi.component.mock.MockEndpointService;
import org.objectweb.petals.jbi.management.service.EndpointServiceImpl;
import org.objectweb.petals.jbi.management.service.LifeCycleManagerImpl;
import org.objectweb.petals.jbi.messaging.MessageExchangeImpl;
import org.objectweb.petals.jbi.registry.ConsumerEndpoint;
import org.objectweb.petals.jbi.registry.InternalEndpoint;
import org.objectweb.petals.jbi.routing.mock.ComponentMock;
import org.objectweb.petals.jbi.routing.mock.MBeanNamesMock;
import org.objectweb.petals.jbi.routing.mock.MBeanServerMock;

/**
 * Test the AddressResolver
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class AddressResolverTest extends TestCase {

    private MessageExchangeImpl messageExchangeImpl;

    private AddressResolver addressResolver;

    private ComponentContextImpl componentContextImpl;
    
    private LifeCycleManagerImpl lifeCycleManagerImpl;
    
    private MockEndpointService mockEndpointService;


    public void setUp() throws Exception {
        lifeCycleManagerImpl = new LifeCycleManagerImpl();

        // Set up the connection resolver
        addressResolver = new AddressResolver(mockEndpointService);
        mockEndpointService = new MockEndpointService();
        addressResolver.endpointService = mockEndpointService;
        componentContextImpl = new ComponentContextImpl(
            MockJBIDescriptorBuilder.buildComponentDescriptor(), "install",
            "work", new MBeanNamesMock(), new MBeanServerMock(),
            lifeCycleManagerImpl, new ConsumerEndpoint("test", "petals1"),
            null, null, null, null);
        messageExchangeImpl = new MessageExchangeImpl(null);
    }

    /**
     * Test everything null
     * 
     * @throws Exception
     */
    public void testResolveAddressEverythingNull() throws Exception {
        messageExchangeImpl.setEndpoint(null);
        addressResolver.resolveAddress(componentContextImpl,
            messageExchangeImpl);
        assertNull(messageExchangeImpl.getEndpoint());
    }

    /**
     * Test service name null
     * 
     * @throws Exception
     */
    public void testResolveAddressEverythingNull1() throws Exception {
        addressResolver.resolveAddress(componentContextImpl,
            messageExchangeImpl);
        assertNull(messageExchangeImpl.getEndpoint());
    }

    /**
     * Test endpoint null and service name does not exist
     */
    public void testResolveAddressWithUnregisteredService() {
        componentContextImpl.setComponent(new ComponentMock());
        messageExchangeImpl.setEndpoint(null);
        messageExchangeImpl.setService(new QName("foo"));
        try {
            addressResolver.resolveAddress(componentContextImpl,
                messageExchangeImpl);
            fail();
        } catch (Exception e) {

        }
    }

    /**
     * Test endpoint null and service name exists
     * 
     * @throws JBIException
     */
//    public void testResolveAddressWithExistingService() throws JBIException {
//        componentContextImpl.setComponent(new ComponentMock());
//        messageExchangeImpl.setEndpoint(null);
//        messageExchangeImpl.setService(new QName("service1"));
//        addressResolver.resolveAddress(componentContextImpl,
//            messageExchangeImpl);
//        assertNotNull(messageExchangeImpl.getEndpoint());
//    }

    /**
     * Test endpoint null and service name null and interface name not null (the
     * endpoint is resolved using the provider endpoint and service name)
     * 
     * @throws JBIException
     */
//    public void testResolveAddressWithAnInterfaceName() throws Exception {
//        componentContextImpl.setComponent(new ComponentMock());
//        messageExchangeImpl.setEndpoint(null);
//        messageExchangeImpl.setService(null);
//        messageExchangeImpl.setInterfaceName(new QName("interface1"));
//        addressResolver.resolveAddress(componentContextImpl,
//            messageExchangeImpl);
//        assertNotNull(messageExchangeImpl.getEndpoint());
//    }

    /**
     * Test endpoint not null and service name not null and interface name null
     * 
     * @throws JBIException
     */
//    public void testResolveAddressWithAEndpointAndAService() throws Exception {
//        InternalEndpoint internalEndpoint = new InternalEndpoint(new QName(
//            "test"), "endpoint:test", componentContextImpl.getComponentName(),
//            null, null);
//        componentContextImpl.setComponent(new ComponentMock());
//        messageExchangeImpl.setEndpoint(internalEndpoint);
//        messageExchangeImpl.setService(new QName("servicename"));
//        messageExchangeImpl.setInterfaceName(null);
//        addressResolver.resolveAddress(componentContextImpl,
//            messageExchangeImpl);
//        assertNotNull(messageExchangeImpl.getEndpoint());
//    }

    public void testresolveAddressEndpointNotNullServiceNotNullButNoConnection()
        throws SecurityException, NoSuchMethodException, RoutingException {
        addressResolver = EasyMock.createMock(AddressResolver.class,
            new Method[] {AddressResolver.class.getDeclaredMethod(
                "resolveLinkedAddressForEndpoint", new Class[] {QName.class,
                    String.class})});
        ComponentContextImpl componentContextImpl = EasyMock
            .createMock(ComponentContextImpl.class);
        MessageExchangeImpl messageExchangeImpl = EasyMock
            .createMock(MessageExchangeImpl.class);
        EndpointServiceImpl endpointServiceImpl = EasyMock
            .createMock(EndpointServiceImpl.class);

        EasyMock.expect(messageExchangeImpl.getEndpoint()).andReturn(
            new InternalEndpoint(new QName("service"), "endpoint", "compo",
                "0", null));
        EasyMock.expect(messageExchangeImpl.getService()).andReturn(
            new QName("service"));
        EasyMock.expect(messageExchangeImpl.getInterfaceName()).andReturn(null);
        EasyMock.expect(
            addressResolver.resolveLinkedAddressForEndpoint(
                new QName("service"), "endpoint")).andReturn(null);
        EasyMock.expect(
            endpointServiceImpl.getEndpoint(new QName("service"), "endpoint"))
            .andReturn(null);

        EasyMock.replay(messageExchangeImpl);
        EasyMock.replay(componentContextImpl);
        EasyMock.replay(endpointServiceImpl);
        EasyMock.replay(addressResolver);

        addressResolver.endpointService = endpointServiceImpl;

        try {
            addressResolver.resolveAddress(componentContextImpl,
                messageExchangeImpl);
            fail();
        } catch (RoutingException e) {
            // do nothing
        }
    }

    public void testresolveAddressInterfaceNotNullButNoConnection()
        throws SecurityException, NoSuchMethodException, RoutingException {
        addressResolver = EasyMock
            .createMock(AddressResolver.class,
                new Method[] {AddressResolver.class.getDeclaredMethod(
                    "resolveLinkedAddressForInterface",
                    new Class[] {QName.class})});
        ComponentContextImpl componentContextImpl = EasyMock
            .createMock(ComponentContextImpl.class);
        MessageExchangeImpl messageExchangeImpl = EasyMock
            .createMock(MessageExchangeImpl.class);
        EndpointServiceImpl endpointServiceImpl = EasyMock
            .createMock(EndpointServiceImpl.class);

        EasyMock.expect(messageExchangeImpl.getEndpoint()).andReturn(null);
        EasyMock.expect(messageExchangeImpl.getService()).andReturn(null);
        EasyMock.expect(messageExchangeImpl.getInterfaceName()).andReturn(
            new QName("interface"));
        EasyMock.expect(
            addressResolver.resolveLinkedAddressForInterface(new QName(
                "interface"))).andReturn(null);
        EasyMock.expect(
            endpointServiceImpl.getInternalEndpointsForInterface(new QName(
                "interface"))).andReturn(null);

        EasyMock.replay(messageExchangeImpl);
        EasyMock.replay(componentContextImpl);
        EasyMock.replay(endpointServiceImpl);
        EasyMock.replay(addressResolver);

        addressResolver.endpointService = endpointServiceImpl;

        try {
            addressResolver.resolveAddress(componentContextImpl,
                messageExchangeImpl);
            fail();
        } catch (RoutingException e) {
            // do nothing
        }
    }

    public void testResolveLinkedAddressForInterfaceNull() {
        EndpointServiceImpl endpointServiceImpl = EasyMock
            .createMock(EndpointServiceImpl.class);

        EasyMock.expect(
            endpointServiceImpl.getInternalEndpointsForInterface(new QName(
                "test"))).andReturn(new ServiceEndpoint[0]);

        EasyMock.replay(endpointServiceImpl);

        addressResolver.endpointService = endpointServiceImpl;
        assertNull(addressResolver.resolveLinkedAddressForInterface(new QName(
            "test")));
    }
}