/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: RouterImplTest.java 14:57:44 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.routing;

import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.MessageExchange.Role;
import javax.xml.namespace.QName;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.petals.jbi.component.context.ComponentContextImpl;
import org.objectweb.petals.jbi.component.lifecycle.ComponentLifeCycle;
import org.objectweb.petals.jbi.management.service.LifeCycleManagerImpl;
import org.objectweb.petals.jbi.messaging.DeliveryChannelImpl;
import org.objectweb.petals.jbi.messaging.MessageExchangeImpl;
import org.objectweb.petals.jbi.registry.ConsumerEndpoint;
import org.objectweb.petals.jbi.registry.InternalEndpoint;
import org.objectweb.petals.jbi.routing.mock.MockAddressResolver;
import org.objectweb.petals.jbi.routing.mock.MockComponentContext;
import org.objectweb.petals.jbi.routing.mock.MockDeliveryChannelImpl;
import org.objectweb.petals.jbi.routing.mock.MockEndpointService;
import org.objectweb.petals.jbi.routing.mock.MockTransporter;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * Tests of the RouterImpl
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class RouterImplTest extends TestCase {

    private RouterImpl routerImpl;

    private MockTransporter transporter;

    private MockAddressResolver addressResolver;

    private Logger logger;

    private MockEndpointService endpointService;

    private LifeCycleManagerImpl lifeCycleManagerImpl;

    public void setUp() {
        logger = EasyMock.createMock(Logger.class);
        routerImpl = new RouterImpl();
        endpointService = new MockEndpointService();
        lifeCycleManagerImpl = new LifeCycleManagerImpl();
        routerImpl.manager = lifeCycleManagerImpl;
        addressResolver = new MockAddressResolver(null);
        routerImpl.addressResolver = addressResolver;
        routerImpl.log = EasyMock.createMock(LoggingUtil.class);
        transporter = new MockTransporter();
        routerImpl.transporter = transporter;
        routerImpl.endpointService = endpointService;
    }

    public void testSendToProvider() throws RoutingException {
        ComponentContextImpl componentContextImpl = new ComponentContextImpl();
        DeliveryChannelImpl deliveryChannelImpl = new DeliveryChannelImpl(
            componentContextImpl, routerImpl, logger);
        MessageExchangeImpl messageExchangeImpl = new MessageExchangeImpl(null);
        messageExchangeImpl.setEndpoint(new InternalEndpoint(new QName("test"),
            "endpoint:test", "compo", "0", null));
        messageExchangeImpl.setService(new QName("test"));
        componentContextImpl.setDeliveryChannel(deliveryChannelImpl);

        routerImpl.send(componentContextImpl, messageExchangeImpl,0);

        assertTrue(addressResolver.isResolveAddress());
        assertTrue(transporter.isSend());
        assertEquals(messageExchangeImpl.getRole(), Role.PROVIDER);
    }

    /**
     * Send to consumer
     * 
     * @throws Exception
     */
    public void testSendToConsumer() throws Exception {
        ComponentContextImpl componentContextImpl = new ComponentContextImpl();
        DeliveryChannelImpl deliveryChannelImpl = new DeliveryChannelImpl(
            componentContextImpl, routerImpl, logger);
        MessageExchangeImpl messageExchangeImpl = new MessageExchangeImpl(
            new ConsumerEndpoint("compo", "0"));
        messageExchangeImpl.setEndpoint(new InternalEndpoint(new QName("test"),
            "endpoint:test", "compo", "0", null));
        messageExchangeImpl.setService(new QName("test"));
        messageExchangeImpl.setRole(MessageExchange.Role.PROVIDER);

        componentContextImpl.setDeliveryChannel(deliveryChannelImpl);

        routerImpl.send(componentContextImpl, messageExchangeImpl,0);

        assertFalse(addressResolver.isResolveAddress());
        assertTrue(transporter.isSend());
        assertEquals(messageExchangeImpl.getRole(), Role.CONSUMER);
    }

    public void testStop() throws Exception {
        // Bind the logger
        routerImpl.start();
        routerImpl.stop();
    }

    public void testReceiveProvider() throws RoutingException {
        MessageExchangeImpl exchangeImpl = new MessageExchangeImpl(
            new ConsumerEndpoint("compo", "0"));
        exchangeImpl.setRole(Role.PROVIDER);
        exchangeImpl.setEndpoint(new InternalEndpoint(new QName("serv"),
            "endpoint", "compo", "0", null));

        lifeCycleManagerImpl = EasyMock.createMock(LifeCycleManagerImpl.class);

        ComponentLifeCycle componentLifeCycle = EasyMock
            .createMock(ComponentLifeCycle.class);

        MockComponentContext componentContext = new MockComponentContext();

        MockDeliveryChannelImpl deliveryChannelImpl = new MockDeliveryChannelImpl(
            componentContext, null, null);

        componentContext.setDeliveryChannelImpl(deliveryChannelImpl);

        EasyMock.expect(componentLifeCycle.getComponentContext()).andReturn(
            componentContext);

        EasyMock.expect(lifeCycleManagerImpl.getComponentByName("compo"))
            .andReturn(componentLifeCycle).anyTimes();
        routerImpl.manager = lifeCycleManagerImpl;

        EasyMock.replay(componentLifeCycle);
        EasyMock.replay(lifeCycleManagerImpl);

        routerImpl.receive(exchangeImpl);
        assertTrue(deliveryChannelImpl.isPush());
    }

    public void testReceiveConsumer() throws RoutingException {
        MessageExchangeImpl exchangeImpl = new MessageExchangeImpl(
            new ConsumerEndpoint("compo", "0"));
        exchangeImpl.setRole(Role.CONSUMER);
        exchangeImpl.setEndpoint(new InternalEndpoint(new QName("serv"),
            "endpoint", "compo", "0", null));

        lifeCycleManagerImpl = EasyMock.createMock(LifeCycleManagerImpl.class);

        ComponentLifeCycle componentLifeCycle = EasyMock
            .createMock(ComponentLifeCycle.class);

        MockComponentContext componentContext = new MockComponentContext();

        MockDeliveryChannelImpl deliveryChannelImpl = new MockDeliveryChannelImpl(
            componentContext, null, null);

        componentContext.setDeliveryChannelImpl(deliveryChannelImpl);

        EasyMock.expect(componentLifeCycle.getComponentContext()).andReturn(
            componentContext);

        EasyMock.expect(lifeCycleManagerImpl.getComponentByName("compo"))
            .andReturn(componentLifeCycle).anyTimes();
        routerImpl.manager = lifeCycleManagerImpl;

        EasyMock.replay(componentLifeCycle);
        EasyMock.replay(lifeCycleManagerImpl);

        routerImpl.receive(exchangeImpl);
        assertTrue(deliveryChannelImpl.isPush());
    }

    public void testSendToConsumerException() throws RoutingException {
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);
        ComponentContextImpl componentContextImpl = EasyMock
            .createMock(ComponentContextImpl.class);
        MessageExchangeImpl messageExchangeImpl = new MessageExchangeImpl(
            new ConsumerEndpoint("compo", "0"));

        loggingUtil.start();

        EasyMock.replay(loggingUtil);
        EasyMock.replay(componentContextImpl);

        transporter.setThrowsException(true);
        routerImpl.log = loggingUtil;

        try {
            routerImpl
                .sendToConsumer(componentContextImpl, messageExchangeImpl,0);
            fail();
        } catch (RoutingException e) {
            // do nothing
        }
        assertEquals(messageExchangeImpl.getRole(),
            MessageExchange.Role.PROVIDER);
    }

    public void testSendToProviderException() throws RoutingException {
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);
        ComponentContextImpl componentContextImpl = EasyMock
            .createMock(ComponentContextImpl.class);
        MessageExchangeImpl messageExchangeImpl = new MessageExchangeImpl(
            new ConsumerEndpoint("compo", "0"));
        messageExchangeImpl.setEndpoint(new InternalEndpoint(new QName(
            "service"), "endpoint", "compo", "0", null));

        loggingUtil.start();

        EasyMock.replay(loggingUtil);
        EasyMock.replay(componentContextImpl);

        transporter.setThrowsException(true);
        routerImpl.log = loggingUtil;

        try {
            routerImpl
                .sendToProvider(componentContextImpl, messageExchangeImpl,0);
            fail();
        } catch (RoutingException e) {
            // do nothing
        }
        assertEquals(messageExchangeImpl.getRole(),
            MessageExchange.Role.CONSUMER);
    }

    public void testReceiveException() throws MessagingException {
        MessageExchangeImpl messageExchangeImpl = new MessageExchangeImpl(
            new ConsumerEndpoint("compo", "0"));
        // messageExchangeImpl.setEndpoint(new InternalEndpoint(new QName(
        // "service"), "endpoint", "compo", "0", null));

        LifeCycleManagerImpl lifeCycleManagerImpl = EasyMock
            .createMock(LifeCycleManagerImpl.class);
        ComponentLifeCycle componentLifeCycle = EasyMock
            .createMock(ComponentLifeCycle.class);
        ComponentContextImpl componentContextImpl = EasyMock
            .createMock(ComponentContextImpl.class);

        EasyMock.expect(lifeCycleManagerImpl.getComponentByName("compo"))
            .andReturn(componentLifeCycle);
        EasyMock.expect(componentLifeCycle.getComponentContext()).andReturn(
            componentContextImpl);
        
        MockDeliveryChannelImpl mockDeliveryChannelImpl = new MockDeliveryChannelImpl(
            new MockComponentContext(), null, null);
        mockDeliveryChannelImpl.setThrowException(true);
        EasyMock.expect(componentContextImpl.getDeliveryChannelImpl()).andReturn(
            mockDeliveryChannelImpl);

        EasyMock.replay(lifeCycleManagerImpl);
        EasyMock.replay(componentContextImpl);
        EasyMock.replay(componentLifeCycle);

        
        routerImpl.manager = lifeCycleManagerImpl;

        try {
            routerImpl.receive(messageExchangeImpl);
            fail();
        } catch (RoutingException e) {
            // do nothing
        }
    }
}
