/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JoramConnectionTest.java 12:34:54 PM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.transport;

import java.util.concurrent.ArrayBlockingQueue;

import javax.jms.Connection;
import javax.jms.JMSException;
import javax.jms.Session;
import javax.naming.NamingException;

import org.easymock.EasyMock;

import junit.framework.TestCase;

import org.objectweb.petals.jbi.transport.mock.MockInitialContext;

/**
 * Tests of the JoramConnection
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class JoramConnectionTest extends TestCase {

    protected MockInitialContext initialContext;

    protected JoramConnection joramConnection;

    public void setUp() throws NamingException {
        initialContext = new MockInitialContext();
        joramConnection = new JoramConnection(1, 16200, "petals", "petals", "127.0.0.1",
            initialContext);
    }
/*
    public void testGetQueueConnectionFactory() {
        QueueConnectionFactory queueConnectionFactory = EasyMock
            .createMock(QueueConnectionFactory.class);
        joramConnection.qcf = queueConnectionFactory;
        assertEquals(queueConnectionFactory, joramConnection
            .getQueueConnectionFactory());
    }

    public void testInit() throws Exception {
        JoramConnection joramConnection = EasyMock.createMock(JoramConnection.class,
            new Method[] {
                JoramConnection.class.getDeclaredMethod("connectAdminModule",
                    new Class[] {}),
                JoramConnection.class.getDeclaredMethod("disconnectAdminModule",
                    new Class[] {}),
                JoramConnection.class.getDeclaredMethod(
                    "setupQueueConnectionFactory", new Class[] {}),
                JoramConnection.class.getDeclaredMethod("setupConnectionFactory",
                    new Class[] {}),
                JoramConnection.class
                    .getDeclaredMethod("setupQueue", new Class[] {})});
        QueueConnectionFactory queueConnectionFactory = EasyMock
            .createMock(QueueConnectionFactory.class);
        Queue queue = EasyMock.createMock(Queue.class);

        joramConnection.connectAdminModule();
        EasyMock.expect(joramConnection.setupConnectionFactory()).andReturn(
            EasyMock.createMock(ConnectionFactory.class)).anyTimes();
        EasyMock.expect(joramConnection.setupQueueConnectionFactory()).andReturn(
            queueConnectionFactory).anyTimes();
        EasyMock.expect(joramConnection.setupQueue()).andReturn(queue).anyTimes();
        joramConnection.disconnectAdminModule();

        EasyMock.replay(joramConnection);

        joramConnection.init();

        assertEquals(joramConnection.qcf, queueConnectionFactory);
        assertEquals(joramConnection.queue, queue);
    }

    public void testInitException() throws SecurityException,
        NoSuchMethodException, ConnectException, UnknownHostException,
        AdminException, NamingException {
        JoramConnection joramConnection = EasyMock.createMock(JoramConnection.class,
            new Method[] {
                JoramConnection.class.getDeclaredMethod("connectAdminModule",
                    new Class[] {}),
                JoramConnection.class.getDeclaredMethod("disconnectAdminModule",
                    new Class[] {}),
                JoramConnection.class.getDeclaredMethod(
                    "setupQueueConnectionFactory", new Class[] {}),
                JoramConnection.class.getDeclaredMethod("setupConnectionFactory",
                    new Class[] {}),
                JoramConnection.class
                    .getDeclaredMethod("setupQueue", new Class[] {})});

        joramConnection.connectAdminModule();
        EasyMock.expect(joramConnection.setupConnectionFactory()).andThrow(
            new NamingException(
                "problem while the setup of the Connection factory"))
            .anyTimes();

        EasyMock.replay(joramConnection);

        try {
            joramConnection.init();
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    @SuppressWarnings("unchecked")
    public void testSetupConnectionFactory() throws ConnectException,
        NamingException, SecurityException, NoSuchMethodException {
        InitialContext initialContext = EasyMock
            .createMock(InitialContext.class);
        JoramConnection joramConnection = EasyMock.createMock(JoramConnection.class,
            new Method[] {JoramConnection.class.getDeclaredMethod(
                "createConnectionFactory", new Class[] {})});
        MockNamingEnumeration<NameClassPair> list1 = new MockNamingEnumeration<NameClassPair>();
        ConnectionFactory connectionFactory = EasyMock
            .createMock(ConnectionFactory.class);

        EasyMock.expect(initialContext.list("/")).andReturn(list1).anyTimes();
        EasyMock.expect(joramConnection.createConnectionFactory()).andReturn(
            connectionFactory).anyTimes();
        initialContext.bind("JoramConnectionFactory", connectionFactory);

        EasyMock.replay(initialContext);
        EasyMock.replay(joramConnection);

        joramConnection.ictx = initialContext;

        assertEquals(joramConnection.setupConnectionFactory(), connectionFactory);
    }

    @SuppressWarnings("unchecked")
    public void testSetupConnectionFactoryBound() throws ConnectException,
        NamingException, SecurityException, NoSuchMethodException {
        InitialContext initialContext = EasyMock
            .createMock(InitialContext.class);
        JoramConnection joramConnection = EasyMock.createMock(JoramConnection.class,
            new Method[] {JoramConnection.class.getDeclaredMethod(
                "createConnectionFactory", new Class[] {})});
        MockNamingEnumeration<NameClassPair> list1 = new MockNamingEnumeration<NameClassPair>();
        List<NameClassPair> names = new ArrayList<NameClassPair>();
        names.add(new NameClassPair("JoramConnectionFactory",
            ConnectionFactory.class.getName()));
        list1.setElems(names);

        ConnectionFactory connectionFactory = EasyMock
            .createMock(ConnectionFactory.class);

        EasyMock.expect(initialContext.list("/")).andReturn(list1).anyTimes();
        EasyMock.expect(joramConnection.createConnectionFactory()).andReturn(
            connectionFactory).anyTimes();
        EasyMock.expect(initialContext.lookup(JoramConnection.CONNECTION_FACTORY))
            .andReturn(connectionFactory).anyTimes();

        EasyMock.replay(initialContext);
        EasyMock.replay(joramConnection);

        joramConnection.ictx = initialContext;

        assertEquals(joramConnection.setupConnectionFactory(), connectionFactory);
    }

    @SuppressWarnings( {"unchecked", "static-access"})
    public void testSetupQueue() throws Exception {
        InitialContext initialContext = EasyMock
            .createMock(InitialContext.class);
        MockNamingEnumeration<NameClassPair> list1 = new MockNamingEnumeration<NameClassPair>();
        Queue queue = EasyMock.createMock(Queue.class);
        JoramConnection.class.getDeclaredMethods();

        JoramConnection joramConnection = EasyMock.createMock(JoramConnection.class,
            new Method[] {JoramConnection.class.getDeclaredMethod("createQueue",
                new Class[] {String.class})});

        EasyMock.expect(initialContext.list("/")).andReturn(list1).anyTimes();
        EasyMock.expect(joramConnection.createQueue("JoramConnection-0"))
            .andReturn(queue).anyTimes();

        initialContext.bind("JoramConnection-0", queue);
        queue.setFreeReading();
        queue.setFreeWriting();

        EasyMock.replay(initialContext);
        EasyMock.replay(queue);
        EasyMock.replay(joramConnection);

        joramConnection.id = 0;
        joramConnection.ictx = initialContext;

        assertEquals(joramConnection.setupQueue(), queue);
    }

    @SuppressWarnings( {"unchecked", "static-access"})
    public void testSetupQueueBound() throws Exception {
        InitialContext initialContext = EasyMock
            .createMock(InitialContext.class);
        MockNamingEnumeration<NameClassPair> list1 = new MockNamingEnumeration<NameClassPair>();
        Queue queue = EasyMock.createMock(Queue.class);
        JoramConnection.class.getDeclaredMethods();
        List<NameClassPair> names = new ArrayList<NameClassPair>();
        names.add(new NameClassPair("JoramConnection-0", ConnectionFactory.class
            .getName()));
        list1.setElems(names);

        JoramConnection joramConnection = EasyMock.createMock(JoramConnection.class,
            new Method[] {JoramConnection.class.getDeclaredMethod("createQueue",
                new Class[] {String.class})});

        EasyMock.expect(initialContext.list("/")).andReturn(list1).anyTimes();
        EasyMock.expect(joramConnection.createQueue("JoramConnection-0"))
            .andReturn(queue).anyTimes();
        EasyMock.expect(initialContext.lookup("JoramConnection-0")).andReturn(queue)
            .anyTimes();

        initialContext.bind("JoramConnection-0", queue);
        queue.setFreeReading();
        queue.setFreeWriting();

        EasyMock.replay(initialContext);
        EasyMock.replay(queue);
        EasyMock.replay(joramConnection);

        joramConnection.id = 0;
        joramConnection.ictx = initialContext;

        assertEquals(joramConnection.setupQueue(), queue);
    }

    @SuppressWarnings( {"unchecked", "static-access"})
    public void testSetupQueueConnectionFactoryBound() throws Exception {
        InitialContext initialContext = EasyMock
            .createMock(InitialContext.class);
        MockNamingEnumeration<NameClassPair> list1 = new MockNamingEnumeration<NameClassPair>();
        QueueConnectionFactory queueConnectionFactory = EasyMock
            .createMock(QueueConnectionFactory.class);
        List<NameClassPair> names = new ArrayList<NameClassPair>();
        names.add(new NameClassPair(JoramConnection.QUEUE_CONNECTION_FACTORY,
            ConnectionFactory.class.getName()));
        list1.setElems(names);

        JoramConnection joramConnection = EasyMock.createMock(JoramConnection.class,
            new Method[] {JoramConnection.class.getDeclaredMethod(
                "createQueueConnectionFactory", new Class[0])});

        EasyMock.expect(initialContext.list("/")).andReturn(list1).anyTimes();
        EasyMock.expect(
            initialContext.lookup(JoramConnection.QUEUE_CONNECTION_FACTORY))
            .andReturn(queueConnectionFactory).anyTimes();

        EasyMock.replay(initialContext);
        EasyMock.replay(joramConnection);
        EasyMock.replay(queueConnectionFactory);

        joramConnection.id = 0;
        joramConnection.ictx = initialContext;

        assertEquals(joramConnection.setupQueueConnectionFactory(),
            queueConnectionFactory);
    }

    @SuppressWarnings( {"unchecked", "static-access"})
    public void testSetupQueueConnectionFactory() throws Exception {
        InitialContext initialContext = EasyMock
            .createMock(InitialContext.class);
        MockNamingEnumeration<NameClassPair> list1 = new MockNamingEnumeration<NameClassPair>();
        QueueConnectionFactory queueConnectionFactory = EasyMock
            .createMock(QueueConnectionFactory.class);
        JoramConnection.class.getDeclaredMethods();

        JoramConnection joramConnection = EasyMock.createMock(JoramConnection.class,
            new Method[] {JoramConnection.class.getDeclaredMethod(
                "createQueueConnectionFactory", new Class[0])});

        EasyMock.expect(initialContext.list("/")).andReturn(list1).anyTimes();
        EasyMock.expect(joramConnection.createQueueConnectionFactory()).andReturn(
            queueConnectionFactory).anyTimes();
        
        initialContext.bind(JoramConnection.QUEUE_CONNECTION_FACTORY,
            queueConnectionFactory);

        EasyMock.replay(initialContext);
        EasyMock.replay(joramConnection);
        EasyMock.replay(queueConnectionFactory);

        joramConnection.id = 0;
        joramConnection.ictx = initialContext;

        assertEquals(joramConnection.setupQueueConnectionFactory(),
            queueConnectionFactory);
    }

    public void testStartListening() throws JMSException {
        MessageListener messageListener = EasyMock
            .createMock(MessageListener.class);
        QueueConnectionFactory queueConnectionFactory = EasyMock
            .createMock(QueueConnectionFactory.class);
        QueueConnection queueConnection = EasyMock
            .createMock(QueueConnection.class);
        QueueSession queueSession = EasyMock.createMock(QueueSession.class);
        QueueReceiver queueReceiver = EasyMock.createMock(QueueReceiver.class);

        EasyMock.expect(
            queueConnectionFactory.createQueueConnection("petals", "petals"))
            .andReturn(queueConnection).anyTimes();
        EasyMock.expect(queueConnection.createQueueSession(true, 0)).andReturn(
            queueSession).anyTimes();
        EasyMock.expect(queueSession.createReceiver(null)).andReturn(
            queueReceiver).anyTimes();
        queueReceiver.setMessageListener(messageListener);
        queueConnection.start();

        EasyMock.replay(messageListener);
        EasyMock.replay(queueConnectionFactory);
        EasyMock.replay(queueConnection);
        EasyMock.replay(queueSession);
        EasyMock.replay(queueReceiver);

        joramConnection.qcf = queueConnectionFactory;

        joramConnection.startListening(messageListener);
    }
*/
    public void testStop() throws JMSException, NamingException {
        Session session = EasyMock.createMock(Session.class);
        Connection conn= EasyMock.createMock(Connection.class);

        session.close();
        conn.stop();

        EasyMock.replay(session);
        EasyMock.replay(conn);

        joramConnection.sessionsPool = new ArrayBlockingQueue<Session>(1);
        joramConnection.session = session;
        joramConnection.connection = conn;
        joramConnection.stop();
    }
}
