/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: DistributedJMXServerManagerImplTest.java 14:10:53 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.kernel.admin;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NameClassPair;
import javax.naming.NamingException;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.petals.jbi.registry.mock.MockContext;
import org.objectweb.petals.kernel.admin.ContainerInformation.ContainerState;
import org.objectweb.petals.kernel.admin.util.ServerUtil;
import org.objectweb.petals.kernel.registry.jndi.NamingEnumerationImpl;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.Monolog;

/**
 * Test of the DistributedJMXServerManagerImpl
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class DistributedJMXServerManagerImplTest extends TestCase {

    private DistributedJMXServerFactoryImpl distributedJMXServerManagerImpl = new DistributedJMXServerFactoryImpl();

    public void setUp() throws Exception {
        distributedJMXServerManagerImpl.log = new LoggingUtil(Monolog
            .initialize().getLogger("test"));
    }

    public void testCreateDistributedJMXServer() throws Exception {
        ServerUtil.start(9999);
        MockContext containers = new MockContext();
        ContainerInformation containerInformation = new ContainerInformation();
        containerInformation.setHost("localhost");
        containerInformation.setJmxLogin("");
        containerInformation.setJmxPassword("");
        containerInformation.setName("0");
        containerInformation.setJmxPort("9999");
        containers.bind("0", containerInformation);
        distributedJMXServerManagerImpl.containersContext = containers;

        assertNotNull(distributedJMXServerManagerImpl
            .createDistributedJMXServer("0"));
        ServerUtil.stop();
    }

    

    public void testIsContainerStarted() throws NamingException {
        DistributedJMXServerFactoryImpl distributedJMXServerFactoryImpl = new DistributedJMXServerFactoryImpl();
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);
        InitialContext initialContext = EasyMock
            .createMock(InitialContext.class);
        Context contContext = EasyMock.createMock(Context.class);
        ContainerInformation containerInformation = new ContainerInformation();
        containerInformation.setState(ContainerState.STARTED);

        loggingUtil.start();
        EasyMock.expect(initialContext.list("containers")).andReturn(
            new NamingEnumerationImpl<NameClassPair>(
                new NameClassPair[] {new NameClassPair("test", "foo")}));
        EasyMock.expect(contContext.lookup("test")).andReturn(
            containerInformation);
        loggingUtil.end();

        EasyMock.replay(loggingUtil);
        EasyMock.replay(contContext);
        EasyMock.replay(initialContext);

        distributedJMXServerFactoryImpl.containersContext = contContext;
        distributedJMXServerFactoryImpl.log = loggingUtil;
        distributedJMXServerFactoryImpl.rootContext = initialContext;

        assertTrue(distributedJMXServerFactoryImpl.isContainerStarted("test"));
    }
}
