/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MX4J_HtmlAdaptorTest.java 15:05:03 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.kernel.mx4j;

import junit.framework.TestCase;
import mx4j.tools.adaptor.http.HttpAdaptorMBean;

import org.easymock.classextension.EasyMock;
import org.objectweb.fractal.jmx.agent.AdminAttributes;
import org.objectweb.petals.kernel.mx4j.mock.MockMBeanServer;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.petals.util.SystemUtil;

/**
 * Test the MX4JHtmlAdaptor
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class MX4JHtmlAdaptorTest extends TestCase {

    protected MX4JHtmlAdaptor adaptor;

    public void setUp() {
        adaptor = new MX4JHtmlAdaptor();
    }

    public void testStartStop() {
        AdminAttributes adminAttributes = EasyMock
            .createMock(AdminAttributes.class);
        MockMBeanServer mockMBeanServer = new MockMBeanServer();
        EasyMock.expect(adminAttributes.getRawMBeanServer()).andReturn(
            mockMBeanServer).anyTimes();
        EasyMock.replay(adminAttributes);
        adaptor.adminAttributes = adminAttributes;
        adaptor.log = EasyMock.createMock(LoggingUtil.class);
        adaptor.start();
        assertTrue(mockMBeanServer.isRegisterMBean());
        adaptor.stop();
        assertTrue(mockMBeanServer.isUnregisterMBean());
    }

    public void testStartFcAlreadyStarted() {
        SystemUtil.setHtmlPort("8082");

        HttpAdaptorMBean httpAdaptorMBean = EasyMock
            .createMock(HttpAdaptorMBean.class);

        EasyMock.expect(httpAdaptorMBean.isActive()).andReturn(Boolean.TRUE);
        EasyMock.expect(httpAdaptorMBean.getPort()).andReturn(8082);

        EasyMock.replay(httpAdaptorMBean);

        adaptor.adaptor = httpAdaptorMBean;
        adaptor.log = EasyMock.createMock(LoggingUtil.class);

        adaptor.start();
    }

    public void testStartStopAdaptor() {
        HttpAdaptorMBean httpAdaptorMBean = EasyMock
            .createMock(HttpAdaptorMBean.class);
        LoggingUtil log = EasyMock.createMock(LoggingUtil.class);
        AdminAttributes adminAttributes = EasyMock
            .createMock(AdminAttributes.class);
        RuntimeException runtimeException = new RuntimeException("test error");

        EasyMock.expect(httpAdaptorMBean.isActive()).andReturn(Boolean.FALSE);
        httpAdaptorMBean.stop();
        log.start();
        EasyMock.expect(adminAttributes.getRawMBeanServer()).andThrow(
            runtimeException);
        log.error("Fail to start HTML adaptor", runtimeException);
        log.end();

        EasyMock.replay(log);
        EasyMock.replay(httpAdaptorMBean);
        EasyMock.replay(adminAttributes);

        adaptor.adaptor = httpAdaptorMBean;
        adaptor.log = log;
        adaptor.adminAttributes = adminAttributes;

        adaptor.start();
    }

    public void testStopException() {
        AdminAttributes adminAttributes = EasyMock
            .createMock(AdminAttributes.class);
        LoggingUtil log = EasyMock.createMock(LoggingUtil.class);
        RuntimeException runtimeException = new RuntimeException(
            "test exception");

        log.start();
        EasyMock.expect(adminAttributes.getRawMBeanServer()).andThrow(
            runtimeException);
        log.warning("Error during MBean unregistering: test exception",
            runtimeException);
        log.end();

        EasyMock.replay(adminAttributes);
        EasyMock.replay(log);

        adaptor.adaptor = EasyMock.createMock(HttpAdaptorMBean.class);
        adaptor.log = log;
        adaptor.adminAttributes = adminAttributes;

        adaptor.stop();
    }

}
