/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: RegistryServerTest.java 4:12:33 PM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.kernel.registry;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.petals.kernel.registry.msg.response.ExceptionResponse;
import org.objectweb.petals.util.LoggingUtil;

/**
 * Tests of the RegistryServer
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class RegistryServerTest extends TestCase {

    public void testProcessResponseException() {
        RegistryServer server = new RegistryServer();
        ExceptionResponse exceptionResponse = new ExceptionResponse(
            new RuntimeException(), 0, 0);
        assertNull(server.processResponse(exceptionResponse));
    }

    public void testSetData() {
        RegistryServer server = new RegistryServer();
        Map<String, Map<String, Object>> map = new HashMap<String, Map<String, Object>>();
        server.setData(map);
        assertEquals(server.getData(), map);
    }

    public void testStart() throws SecurityException, NoSuchMethodException,
        IllegalLifeCycleException {
        RegistryServer server = EasyMock.createMock(RegistryServer.class,
            new Method[] {RegistryServer.class.getDeclaredMethod("startServer",
                new Class[0])});
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);

        loggingUtil.call();
        server.startServer();

        EasyMock.replay(loggingUtil);
        EasyMock.replay(server);

        server.log = loggingUtil;

        server.start();
    }

    public void testStop() throws SecurityException, NoSuchMethodException,
        IllegalLifeCycleException {
        RegistryServer server = EasyMock.createMock(RegistryServer.class,
            new Method[] {RegistryServer.class.getDeclaredMethod("stopServer",
                new Class[0])});
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);

        loggingUtil.start();
        server.stopServer();
        loggingUtil.end();

        EasyMock.replay(loggingUtil);
        EasyMock.replay(server);

        server.log = loggingUtil;

        server.stop();
    }

}
