/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: NamingContextImplTest.java 11:11:06 AM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.kernel.registry.jndi;

import java.util.Hashtable;

import javax.naming.CompositeName;
import javax.naming.InvalidNameException;
import javax.naming.Name;
import javax.naming.NamingException;

import org.objectweb.petals.kernel.registry.jndi.mock.MockJNDIConnection;

import junit.framework.TestCase;

/**
 * Tests of the NamingContextImpl
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class NamingContextImplTest extends TestCase {

    protected NamingContextImpl namingContextImpl;

    protected MockJNDIConnection connection;

    public void setUp() {
        connection = new MockJNDIConnection(9999, "127.0.0.1");
        namingContextImpl = new NamingContextImpl(
            new Hashtable<String, Object>(), connection, "test");
    }

    public void testAddToEnvironment() throws NamingException {
        namingContextImpl.addToEnvironment("key", "value");
        assertEquals(namingContextImpl.env.get("key"), "value");
    }

    public void testBindName() throws InvalidNameException, NamingException {
        namingContextImpl.bind(new CompositeName("foo"), "bar");
        assertTrue(connection.isBind());
    }

    public void testBindString() throws NamingException {
        namingContextImpl.bind("foo", "bar");
        assertTrue(connection.isBind());
    }

    public void testClose() throws NamingException {
        namingContextImpl.close();
        assertNull(namingContextImpl.jndiConnection);
    }

    public void testComposeNameName() throws InvalidNameException,
        NamingException {
        Name name = namingContextImpl.composeName(new CompositeName("foo"),
            new CompositeName("bar"));
        assertEquals(name, new CompositeName("barfoo"));
    }

    public void testComposeNameNameException() throws InvalidNameException,
        NamingException {
        try {
            namingContextImpl.composeName((CompositeName) null, null);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testComposeNameNameException1() throws InvalidNameException,
        NamingException {
        try {
            namingContextImpl.composeName(new CompositeName("foo"), null);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testComposeNameString() throws InvalidNameException,
        NamingException {
        String name = namingContextImpl.composeName("foo", "/bar/");
        assertEquals(name, "/bar/foo");
    }

    public void testComposeNameStringException() throws InvalidNameException,
        NamingException {
        try {
            namingContextImpl.composeName((String) null, null);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testComposeNameStringException1() throws InvalidNameException,
        NamingException {
        try {
            namingContextImpl.composeName("foo", null);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testCreateSubcontextName() throws InvalidNameException,
        NamingException {
        namingContextImpl.createSubcontext(new CompositeName("test"));
        assertTrue(connection.isCreateSubcontext());
    }

    public void testCreateSubcontextString() throws InvalidNameException,
        NamingException {
        namingContextImpl.createSubcontext("test");
        assertTrue(connection.isCreateSubcontext());
    }

    public void testDestroySubcontextName() throws InvalidNameException,
        NamingException {
        namingContextImpl.destroySubcontext(new CompositeName("test"));
        assertTrue(connection.isDestroySubcontext());
    }

    public void testDestroySubcontextString() throws InvalidNameException,
        NamingException {
        namingContextImpl.destroySubcontext("test");
        assertTrue(connection.isDestroySubcontext());
    }

    public void testGetEnvironment() throws NamingException {
        assertEquals(namingContextImpl.getEnvironment().size(), 0);
    }

    public void testGetNameInNamespace() throws NamingException {
        assertEquals(namingContextImpl.getNameInNamespace(), "test");
    }

    public void testGetNameParserName() throws InvalidNameException,
        NamingException {
        assertNotNull(namingContextImpl
            .getNameParser(new CompositeName("test")));
    }

    public void testGetNameParserString() throws InvalidNameException,
        NamingException {
        assertNotNull(namingContextImpl.getNameParser("test"));
    }

    public void testListName() throws InvalidNameException, NamingException {
        namingContextImpl.list(new CompositeName("test"));
        assertTrue(connection.isList());
    }

    public void testListString() throws InvalidNameException, NamingException {
        namingContextImpl.list("test");
        assertTrue(connection.isList());
    }

    public void testListBindingsName() throws InvalidNameException,
        NamingException {
        namingContextImpl.listBindings(new CompositeName("test"));
        assertTrue(connection.isListBindings());
    }

    public void testListBindingsString() throws InvalidNameException,
        NamingException {
        namingContextImpl.listBindings("test");
        assertTrue(connection.isListBindings());
    }

    public void testLookupName() throws InvalidNameException, NamingException {
        namingContextImpl.lookup(new CompositeName("test"));
        assertTrue(connection.isLookup());
    }

    public void testLookupString() throws InvalidNameException, NamingException {
        namingContextImpl.lookup("test");
        assertTrue(connection.isLookup());
    }

    public void testLookupLinkName() throws InvalidNameException,
        NamingException {
        namingContextImpl.lookupLink(new CompositeName("test"));
        assertTrue(connection.isLookupLink());
    }

    public void testLookupLinkString() throws InvalidNameException,
        NamingException {
        namingContextImpl.lookupLink("test");
        assertTrue(connection.isLookupLink());
    }

    public void testRebindName() throws InvalidNameException, NamingException {
        namingContextImpl.rebind(new CompositeName("test"), "foo");
        assertTrue(connection.isRebind());
    }

    public void testRebindString() throws InvalidNameException, NamingException {
        namingContextImpl.rebind("test", "foo");
        assertTrue(connection.isRebind());
    }

    public void testRemoveFromEnvironment() throws InvalidNameException,
        NamingException {
        namingContextImpl.env.put("key", "test");
        namingContextImpl.removeFromEnvironment("key");
        assertNull(namingContextImpl.env.get("key"));
    }

    public void testRenameName() throws InvalidNameException, NamingException {
        namingContextImpl.rename(new CompositeName("test"), new CompositeName(
            "test1"));
        assertTrue(connection.isRename());
    }

    public void testRenameString() throws InvalidNameException, NamingException {
        namingContextImpl.rename("test", "test1");
        assertTrue(connection.isRename());
    }

    public void testUnbindName() throws InvalidNameException, NamingException {
        namingContextImpl.unbind(new CompositeName("test"));
        assertTrue(connection.isUnbind());
    }

    public void testUnbindString() throws InvalidNameException, NamingException {
        namingContextImpl.unbind(new CompositeName("test"));
        assertTrue(connection.isUnbind());
    }

}
