/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MockJNDIConnection.java 11:12:31 AM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.kernel.registry.jndi.mock;

import javax.naming.Binding;
import javax.naming.NameClassPair;

import org.objectweb.petals.kernel.registry.jndi.JNDIConnection;
import org.objectweb.petals.kernel.registry.msg.request.RegistryRequest;
import org.objectweb.petals.kernel.registry.msg.request.RegistryRequest.RequestType;

/**
 * Mock of the JNDIConnection
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class MockJNDIConnection extends JNDIConnection {

    public MockJNDIConnection(int port, String host) {
        super(port, host);
    }

    /**
     * 
     */
    private static final long serialVersionUID = 1L;

    protected boolean isBind;

    protected boolean isCreateSubcontext;

    protected boolean isDestroySubcontext;

    protected boolean isList;

    protected boolean isListBindings;

    protected boolean isLookup;

    protected boolean isLookupLink;

    protected boolean isRebind;

    protected boolean isRename;

    protected boolean isUnbind;

    public boolean isUnbind() {
        return isUnbind;
    }

    public void setUnbind(boolean isUnbind) {
        this.isUnbind = isUnbind;
    }

    public boolean isDestroySubcontext() {
        return isDestroySubcontext;
    }

    public void setDestroySubcontext(boolean isDestroySubcontext) {
        this.isDestroySubcontext = isDestroySubcontext;
    }

    public Object send(RegistryRequest registryRequest) {
        Object out = null;
        if (registryRequest.getType().equals(RequestType.bind)) {
            isBind = true;
        } else if (registryRequest.getType().equals(
            RequestType.createSubcontext)) {
            isCreateSubcontext = true;
        } else if (registryRequest.getType().equals(
            RequestType.destroySubcontext)) {
            isDestroySubcontext = true;
        } else if (registryRequest.getType().equals(RequestType.list)) {
            isList = true;
            out = new NameClassPair[0];
        } else if (registryRequest.getType().equals(RequestType.listBindings)) {
            isListBindings = true;
            out = new Binding[0];
        } else if (registryRequest.getType().equals(RequestType.lookup)) {
            isLookup = true;
        } else if (registryRequest.getType().equals(RequestType.lookupLink)) {
            isLookupLink = true;
        } else if (registryRequest.getType().equals(RequestType.rebind)) {
            isRebind = true;
        } else if (registryRequest.getType().equals(RequestType.rename)) {
            isRename = true;
        } else if (registryRequest.getType().equals(RequestType.unbind)) {
            isUnbind = true;
        }
        return out;
    }

    public boolean isBind() {
        return isBind;
    }

    public void setBind(boolean isBind) {
        this.isBind = isBind;
    }

    public boolean isCreateSubcontext() {
        return isCreateSubcontext;
    }

    public void setCreateSubcontext(boolean isCreateSubcontext) {
        this.isCreateSubcontext = isCreateSubcontext;
    }

    public boolean isList() {
        return isList;
    }

    public void setList(boolean isList) {
        this.isList = isList;
    }

    public boolean isListBindings() {
        return isListBindings;
    }

    public void setListBindings(boolean isListBindings) {
        this.isListBindings = isListBindings;
    }

    public boolean isLookup() {
        return isLookup;
    }

    public void setLookup(boolean isLookup) {
        this.isLookup = isLookup;
    }

    public boolean isLookupLink() {
        return isLookupLink;
    }

    public void setLookupLink(boolean isLookupLink) {
        this.isLookupLink = isLookupLink;
    }

    public boolean isRebind() {
        return isRebind;
    }

    public void setRebind(boolean isRebind) {
        this.isRebind = isRebind;
    }

    public boolean isRename() {
        return isRename;
    }

    public void setRename(boolean isRename) {
        this.isRename = isRename;
    }

}
