/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: TribeThreadTest.java 1:43:39 PM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.kernel.registry.thread;

import javax.naming.NamingException;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.petals.kernel.registry.RegistryServer;
import org.objectweb.petals.kernel.registry.msg.request.BindRequest;
import org.objectweb.petals.kernel.registry.msg.request.CreateSubcontextRequest;
import org.objectweb.petals.kernel.registry.msg.request.DestroySubcontextRequest;
import org.objectweb.petals.kernel.registry.msg.request.PingRequest;
import org.objectweb.petals.kernel.registry.msg.request.RebindRequest;
import org.objectweb.petals.kernel.registry.msg.request.RenameRequest;
import org.objectweb.petals.kernel.registry.msg.request.UnbindRequest;
import org.objectweb.petals.kernel.registry.msg.response.MyStartTimeResponse;
import org.objectweb.tribe.channel.ReliableGroupChannelWithGms;
import org.objectweb.tribe.exceptions.ChannelException;
import org.objectweb.tribe.exceptions.NotConnectedException;

/**
 * Tests of the TribeThread
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class TribeThreadTest extends TestCase {

    public void testRunNullMessage() throws ChannelException,
        NotConnectedException {
        RegistryServer registryServer = EasyMock
            .createMock(RegistryServer.class);
        ReliableGroupChannelWithGms channel = EasyMock
            .createMock(ReliableGroupChannelWithGms.class);
        MockLoggingUtil mockLoggingUtil = new MockLoggingUtil();

        EasyMock.expect(registryServer.getStartTime()).andReturn((long) 1)
            .anyTimes();
        EasyMock.expect(registryServer.getLog()).andReturn(mockLoggingUtil)
            .anyTimes();
        EasyMock.expect(registryServer.getChannel()).andReturn(channel)
            .anyTimes();
        EasyMock.expect(channel.receive()).andReturn(null).anyTimes();

        EasyMock.replay(registryServer);
        EasyMock.replay(channel);

        TribeThread tribeThread = new TribeThread(registryServer);
        tribeThread.start();
        try {
            Thread.sleep(500);
        } catch (InterruptedException e) {
            // do nothing
        }
        tribeThread.terminate();
        assertTrue(mockLoggingUtil.isWarning());
    }

    public void testRunNotRegistryMessage() throws ChannelException,
        NotConnectedException {
        RegistryServer registryServer = EasyMock
            .createMock(RegistryServer.class);
        ReliableGroupChannelWithGms channel = EasyMock
            .createMock(ReliableGroupChannelWithGms.class);
        MockLoggingUtil mockLoggingUtil = new MockLoggingUtil();

        EasyMock.expect(registryServer.getStartTime()).andReturn((long) 1)
            .anyTimes();
        EasyMock.expect(registryServer.getLog()).andReturn(mockLoggingUtil)
            .anyTimes();
        EasyMock.expect(registryServer.getChannel()).andReturn(channel)
            .anyTimes();
        EasyMock.expect(channel.receive()).andReturn("test").anyTimes();

        EasyMock.replay(registryServer);
        EasyMock.replay(channel);

        TribeThread tribeThread = new TribeThread(registryServer);
        tribeThread.start();
        try {
            Thread.sleep(500);
        } catch (InterruptedException e) {
            // do nothing
        }
        tribeThread.terminate();
        assertTrue(mockLoggingUtil.isWarning());
    }

    public void testRunException() throws ChannelException,
        NotConnectedException {
        RegistryServer registryServer = EasyMock
            .createMock(RegistryServer.class);
        ReliableGroupChannelWithGms channel = EasyMock
            .createMock(ReliableGroupChannelWithGms.class);
        MockLoggingUtil mockLoggingUtil = new MockLoggingUtil();

        EasyMock.expect(registryServer.getStartTime()).andReturn((long) 1)
            .anyTimes();
        EasyMock.expect(registryServer.getLog()).andReturn(mockLoggingUtil)
            .anyTimes();
        EasyMock.expect(registryServer.getChannel()).andReturn(channel)
            .anyTimes();
        EasyMock.expect(channel.receive()).andThrow(new RuntimeException())
            .anyTimes();

        EasyMock.replay(registryServer);
        EasyMock.replay(channel);

        TribeThread tribeThread = new TribeThread(registryServer);
        tribeThread.start();
        try {
            Thread.sleep(500);
        } catch (InterruptedException e) {
            // do nothing
        }
        tribeThread.terminate();
        assertTrue(mockLoggingUtil.isError());
    }

    public void testProcessRequestCreateSubcontextRequest()
        throws NamingException {
        MockLoggingUtil mockLoggingUtil = new MockLoggingUtil();
        CreateSubcontextRequest request = new CreateSubcontextRequest("/",
            "test", 0, 0);
        RegistryServer registryServer = EasyMock
            .createMock(RegistryServer.class);

        EasyMock.expect(registryServer.getStartTime()).andReturn((long) 1)
            .anyTimes();
        EasyMock.expect(registryServer.getLog()).andReturn(mockLoggingUtil)
            .anyTimes();
        EasyMock.expect(registryServer.processRequest(request)).andReturn(null)
            .anyTimes();

        EasyMock.replay(registryServer);

        TribeThread tribeThread = new TribeThread(registryServer);

        tribeThread.processRequest(request);
    }

    public void testProcessRequestDestroySubcontextRequest()
        throws NamingException {
        MockLoggingUtil mockLoggingUtil = new MockLoggingUtil();
        DestroySubcontextRequest request = new DestroySubcontextRequest("/",
            "test", 0, 0);
        RegistryServer registryServer = EasyMock
            .createMock(RegistryServer.class);

        EasyMock.expect(registryServer.getStartTime()).andReturn((long) 1)
            .anyTimes();
        EasyMock.expect(registryServer.getLog()).andReturn(mockLoggingUtil)
            .anyTimes();
        EasyMock.expect(registryServer.processRequest(request)).andReturn(null)
            .anyTimes();

        EasyMock.replay(registryServer);

        TribeThread tribeThread = new TribeThread(registryServer);

        tribeThread.processRequest(request);
    }

    public void testProcessRequestRebindRequest() throws NamingException {
        MockLoggingUtil mockLoggingUtil = new MockLoggingUtil();
        RebindRequest request = new RebindRequest("/", "test", null, 0, 0);
        RegistryServer registryServer = EasyMock
            .createMock(RegistryServer.class);

        EasyMock.expect(registryServer.getStartTime()).andReturn((long) 1)
            .anyTimes();
        EasyMock.expect(registryServer.getLog()).andReturn(mockLoggingUtil)
            .anyTimes();
        EasyMock.expect(registryServer.processRequest(request)).andReturn(null)
            .anyTimes();

        EasyMock.replay(registryServer);

        TribeThread tribeThread = new TribeThread(registryServer);

        tribeThread.processRequest(request);
    }

    public void testProcessRequestRenameRequest() throws NamingException {
        MockLoggingUtil mockLoggingUtil = new MockLoggingUtil();
        RenameRequest request = new RenameRequest("/", "test", null, 0, 0);
        RegistryServer registryServer = EasyMock
            .createMock(RegistryServer.class);

        EasyMock.expect(registryServer.getStartTime()).andReturn((long) 1)
            .anyTimes();
        EasyMock.expect(registryServer.getLog()).andReturn(mockLoggingUtil)
            .anyTimes();
        EasyMock.expect(registryServer.processRequest(request)).andReturn(null)
            .anyTimes();

        EasyMock.replay(registryServer);

        TribeThread tribeThread = new TribeThread(registryServer);

        tribeThread.processRequest(request);
    }

    public void testProcessRequestUnbindRequest() throws NamingException {
        MockLoggingUtil mockLoggingUtil = new MockLoggingUtil();
        UnbindRequest request = new UnbindRequest("/", "test", 0, 0);
        RegistryServer registryServer = EasyMock
            .createMock(RegistryServer.class);

        EasyMock.expect(registryServer.getStartTime()).andReturn((long) 1)
            .anyTimes();
        EasyMock.expect(registryServer.getLog()).andReturn(mockLoggingUtil)
            .anyTimes();
        EasyMock.expect(registryServer.processRequest(request)).andReturn(null)
            .anyTimes();

        EasyMock.replay(registryServer);

        TribeThread tribeThread = new TribeThread(registryServer);

        tribeThread.processRequest(request);
    }

    public void testProcessRequestAnything() throws NamingException {
        MockLoggingUtil mockLoggingUtil = new MockLoggingUtil();
        PingRequest request = new PingRequest();
        RegistryServer registryServer = EasyMock
            .createMock(RegistryServer.class);

        EasyMock.expect(registryServer.getStartTime()).andReturn((long) 1)
            .anyTimes();
        EasyMock.expect(registryServer.getLog()).andReturn(mockLoggingUtil)
            .anyTimes();
        EasyMock.expect(registryServer.processRequest(request)).andReturn(null)
            .anyTimes();

        EasyMock.replay(registryServer);

        TribeThread tribeThread = new TribeThread(registryServer);

        tribeThread.processRequest(request);
    }

    public void testProcessRequestException() throws NamingException {
        MockLoggingUtil mockLoggingUtil = new MockLoggingUtil();
        BindRequest request = new BindRequest("/", "key", "value", 0, 0);
        RegistryServer registryServer = EasyMock
            .createMock(RegistryServer.class);
        NamingException e = new NamingException("error");

        EasyMock.expect(registryServer.getStartTime()).andReturn((long) 1)
            .anyTimes();
        EasyMock.expect(registryServer.getLog()).andReturn(mockLoggingUtil)
            .anyTimes();
        EasyMock.expect(registryServer.processRequest(request)).andThrow(e)
            .anyTimes();
        EasyMock.expect(registryServer.getChannel()).andThrow(
            new RuntimeException("error")).anyTimes();

        EasyMock.replay(registryServer);

        TribeThread tribeThread = new TribeThread(registryServer);

        tribeThread.processRequest(request);
        assertTrue(mockLoggingUtil.isError());
    }

    public void testProcessResponseMyStartTimeResponseException()
        throws NamingException {
        MockLoggingUtil mockLoggingUtil = new MockLoggingUtil();
        RegistryServer registryServer = EasyMock
            .createMock(RegistryServer.class);
        MyStartTimeResponse myStartTimeResponse = new MyStartTimeResponse(
            (long) 1, 0, 0);

        EasyMock.expect(registryServer.getStartTime()).andReturn((long) 1)
            .anyTimes();
        EasyMock.expect(registryServer.getLog()).andReturn(mockLoggingUtil)
            .anyTimes();
        EasyMock.expect(registryServer.processResponse(myStartTimeResponse))
            .andReturn("0").anyTimes();
        EasyMock.expect(registryServer.getChannel()).andThrow(
            new RuntimeException()).anyTimes();

        EasyMock.replay(registryServer);

        TribeThread tribeThread = new TribeThread(registryServer);
        tribeThread.processResponse(myStartTimeResponse);
        assertTrue(mockLoggingUtil.isError());
    }

}
