/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: LauncherTest.java 11:45:12 AM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.system.launch;

import java.io.File;
import java.io.IOException;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.NameController;
import org.objectweb.util.monolog.Monolog;

import org.objectweb.petals.util.SystemUtil;

/**
 * Test of the launcher
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class LauncherTest extends TestCase {

    
    public void testInitializeKernel() throws Exception {
        Launcher launcher = new Launcher();
        launcher.initializeFractal();
        assertNotNull(launcher.petalsComposite);
    }

    public void testInitializeMonolog() throws IOException {
        Launcher launcher = new Launcher();
        String classDir = this.getClass().getResource(".").toString();
        String targetDir = classDir.substring(0, classDir
            .indexOf("test-classes"));
        targetDir = targetDir.substring(targetDir.indexOf(":") + 1);
        File tmpPH = new File(targetDir);
        new File(targetDir + File.separator + "logs").mkdir();
        SystemUtil.setPetalsInstallDirectory(tmpPH);
        launcher.initializeMonolog();
        assertEquals(Monolog.monologFactory.getClass().getName(),
            "org.objectweb.petals.util.monolog.wrapper.javalog.LoggerFactory");
    }

    

    public void testGetComponentByName() throws NoSuchInterfaceException {
        ContentController contentController = EasyMock
            .createMock(ContentController.class);
        Component component = EasyMock.createMock(Component.class);
        NameController nameController = EasyMock
            .createMock(NameController.class);

        Component[] compos = new Component[] {component};

        EasyMock.expect(contentController.getFcSubComponents()).andReturn(
            compos).anyTimes();
        EasyMock.expect(component.getFcInterface("name-controller")).andReturn(
            nameController).anyTimes();
        EasyMock.expect(nameController.getFcName()).andReturn("test")
            .anyTimes();

        EasyMock.replay(contentController);
        EasyMock.replay(component);
        EasyMock.replay(nameController);

        assertEquals(component, FractalHelper.getComponentByName(contentController,
            "test"));
    }

    public void testGetComponentByNameException()
        throws NoSuchInterfaceException {
        ContentController contentController = EasyMock
            .createMock(ContentController.class);
        Component component = EasyMock.createMock(Component.class);

        Component[] compos = new Component[] {component};

        EasyMock.expect(contentController.getFcSubComponents()).andReturn(
            compos).anyTimes();
        EasyMock.expect(component.getFcInterface("name-controller")).andThrow(
            new NoSuchInterfaceException("name-controller"));

        EasyMock.replay(contentController);
        EasyMock.replay(component);

        assertNull(FractalHelper.getComponentByName(contentController, "test"));
    }

    
    public void testRun() throws Exception {
      /*
       * TODO fix crashing test
       *   Launcher launcher = EasyMock.createMock(Launcher.class, new Method[] {
            Launcher.class.getDeclaredMethod("initializeMonolog",
                new Class[] {}),
            Launcher.class.getDeclaredMethod("getComponentByName", new Class[] {
                ContentController.class, String.class})});
        Component component = EasyMock.createMock(Component.class);
        ContentController contentController = EasyMock
            .createMock(ContentController.class);
        Component[] compos = new Component[] {component};
        NameController nameController = EasyMock
            .createMock(NameController.class);
        Component agent = EasyMock.createMock(Component.class);
        Admin admin = EasyMock.createMock(Admin.class);
        LifeCycleController lifeCycleController = EasyMock
            .createMock(LifeCycleController.class);
        AttributeController attributeController = EasyMock
            .createMock(AttributeController.class);
        AdminAttributes adminAttributes = EasyMock
            .createMock(AdminAttributes.class);
        MBeanServer mBeanServer = EasyMock.createMock(MBeanServer.class);
        Component jbi = EasyMock.createMock(Component.class);
        Component system = EasyMock.createMock(Component.class);
        SystemState systemState = EasyMock.createMock(SystemState.class);

        EasyMock.expect(component.getFcInterface("content-controller"))
            .andReturn(contentController).anyTimes();
        EasyMock.expect(component.getFcInterface("name-controller")).andReturn(
            nameController).anyTimes();
        EasyMock.expect(component.getFcInterface("lifecycle-controller"))
            .andReturn(lifeCycleController).anyTimes();

        EasyMock.expect(contentController.getFcSubComponents()).andReturn(
            compos).anyTimes();
        launcher.initializeMonolog();
        launcher.initializeFractal();
        EasyMock.expect(
            launcher.getComponentByName(contentController, "agent-jmx"))
            .andReturn(agent).anyTimes();
        EasyMock.expect(agent.getFcInterface("admin")).andReturn(admin);
        admin.expose();
        EasyMock.expect(agent.getFcInterface("attribute-controller"))
            .andReturn(adminAttributes);
        EasyMock.expect(adminAttributes.getRawMBeanServer()).andReturn(
            mBeanServer);
        EasyMock.expect(launcher.getComponentByName(contentController, "jbi"))
            .andReturn(jbi);
        EasyMock.expect(jbi.getFcInterface("content-controller")).andReturn(
            contentController);
        EasyMock.expect(
            launcher.getComponentByName(contentController, "systemstate-impl"))
            .andReturn(system);
        EasyMock.expect(system.getFcInterface("service"))
            .andReturn(systemState);
        EasyMock.expect(systemState.recoverAllEntities()).andReturn(
            Boolean.TRUE);
        EasyMock.expect(lifeCycleController.getFcState()).andReturn(
            LifeCycleController.STARTED).anyTimes();
        lifeCycleController.stopFc();

        EasyMock.replay(jbi);
        EasyMock.replay(systemState);
        EasyMock.replay(system);
        EasyMock.replay(adminAttributes);
        EasyMock.replay(lifeCycleController);
        EasyMock.replay(attributeController);
        EasyMock.replay(launcher);
        EasyMock.replay(agent);
        EasyMock.replay(admin);
        EasyMock.replay(nameController);
        EasyMock.replay(contentController);
        EasyMock.replay(component);

        launcher.petalsComposite = component;
        launcher.run(new String[0]);*/
    }

}
