/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: Loaders.java 1099 2006-09-18 16:45:13Z ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.classloader;

import java.util.HashMap;
import java.util.Map;

/**
 * This class gathers the class loaders used in Petals
 * 
 * @version $Revision: 1099 $ $Date: 2006-05-24 12:42:07 +0200 (mer., 24 mai
 *          2006) $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 * @author ddesjardins - eBMWebsourcing
 */
final class Loaders {

    public static final String COMMON_LOADER = "common";

    private static Loaders singletonInstance;

    private static SharedLibrariesClassLoader slClassLoader;

    private Map<String, PetalsClassLoader> classLoadersTable = new HashMap<String, PetalsClassLoader>();

    /**
     * Constructor. An unique SharedLibrariesClassLoader is created. It must be
     * used to reference all classloader created for shared libraries.
     * 
     */
    private Loaders() {
        super();
        slClassLoader = new SharedLibrariesClassLoader();

    }

    public static Loaders getInstance() {
        if (singletonInstance == null) {
            singletonInstance = new Loaders();
        }
        return singletonInstance;
    }

    public boolean contains(String loaderName) {
        return classLoadersTable.containsKey(loaderName);
    }

    public PetalsClassLoader get(String loaderName) {
        return classLoadersTable.get(loaderName);
    }

    public SharedLibrariesClassLoader getSharedLibrariesClassLoader() {
        return slClassLoader;
    }

    public ClassLoader put(String loaderName, PetalsClassLoader cl) {
        return classLoadersTable.put(loaderName, cl);
    }

    public PetalsClassLoader remove(String loaderName) {
        return classLoadersTable.remove(loaderName);
    }
}
