/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JarFileLocatorTest.java 11:44:45 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.classloader.locator;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

import junit.framework.TestCase;

/**
 * Test for the JarFileLocator
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class JarFileLocatorTest extends TestCase {

    private String baseDir;

    public void setUp() {
        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
    }

    public void testHasFile() throws MalformedURLException, IOException {
        JarFileLocator jarFileLocator = new JarFileLocator(new URL("file:"
            + baseDir.replace(File.separator, "/")
            + "src/test-data/petals-core-test.jar"));
        assertTrue(jarFileLocator.hasFile("META-INF/MANIFEST.MF"));
    }

    public void testHasDirectory() throws MalformedURLException, IOException {
        JarFileLocator jarFileLocator = new JarFileLocator(new URL("file:"
            + baseDir.replace(File.separator, "/")
            + "src/test-data/petals-core-test.jar"));
        assertTrue(jarFileLocator.hasDirectory("META-INF"));
    }

    public void testListContent() throws MalformedURLException, IOException {
        JarFileLocator jarFileLocator = new JarFileLocator(new URL("file:"
            + baseDir.replace(File.separator, "/")
            + "src/test-data/petals-core-test.jar"));
        assertTrue((jarFileLocator.listContent("META-INF").size() == 3));
    }

    public void testHasDirectoryFalse() throws MalformedURLException, IOException {
        JarFileLocator jarFileLocator = new JarFileLocator(new URL("file:"
            + baseDir.replace(File.separator, "/")
            + "src/test-data/petals-core-test.jar"));
        assertFalse(jarFileLocator.hasDirectory("foo"));
    }

}
