/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): OUAZZANI CHAHDI Anass
 * --------------------------------------------------------------------------
 * $Id: TimerSendRouter.java,v 1.2 2006/09/05 10:24:27 oanass Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.monitoring.router.duration;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.objectweb.petals.jbi.messaging.MessageExchangeImpl;
import org.objectweb.petals.monitoring.router.RouterMonitor;

/**
 * @version $Rev: 617 $ $Date: 2006-06-19 17:28:41 +0200 (lun, 19 jun 2006) $
 * @since Petals 1.0
 * @author oanass
 * 
 * Used to monitor the send time of messages.
 */
public class RouterTimerProbe implements RouterMonitor, Serializable {
    /**
     * 
     */
    private static final long serialVersionUID = 817620834857108002L;

    List<Marker> dates = new ArrayList<Marker>();

    /**
     * Update list of marker "dates". Store before sent date(Router) of message
     * that is sending.
     */
    public synchronized void beforeMessageSent(MessageExchangeImpl exchange) {
        long dateBefore = System.currentTimeMillis();
        Marker marker = new Marker(exchange.getExchangeId(), dateBefore);
        dates.add(marker);
    }

    /**
     * 
     * @return list of marker : id, before send time, after send time
     */
    public List<Marker> getDates() {
        return dates;
    }

    public synchronized void clear() {
        this.dates.clear();
    }

    /**
     * Update list of marker "dates". Store after Sent date(Router) of message
     * that is sending.
     */
    public synchronized void afterMessageSent(MessageExchangeImpl exchange) {
        long date = System.currentTimeMillis();
        synchronized (dates) {
            Iterator it = dates.iterator();
            Marker marker = null;
            long dateAfter = 1;
            while (dateAfter != 0 && it.hasNext()) {
                marker = ((Marker) it.next());
                if (marker.getId().equals(exchange.getExchangeId()))
                    dateAfter = marker.getDateAfter();
            }
            marker.setDateAfter(date);
        }
    }

    /**
     * nothing to do
     */
    public synchronized void beforeMessageReceived(MessageExchangeImpl message) {
    }

    /**
     * nothing to do
     */
    public synchronized void afterMessageReceived(MessageExchangeImpl message) {
    }

    /**
     * Marker 
     * Class used to store information about the time: 
     * id, before send time, after send time
     * 
     * @author oanass
     * 
     */
    public class Marker implements Serializable, Comparable {

        private static final long serialVersionUID = 6951650879117740202L;

        private String id;

        private long dateBefore = 0;

        private long dateAfter = 0;

        public String getId() {
            return id;
        }

        public Marker(String id, long dateBefore) {
            this.id = id;
            this.dateBefore = dateBefore;
        }

        public long getDateBefore() {
            return dateBefore;
        }

        public long getDateAfter() {
            return dateAfter;
        }

        public void setDateAfter(long dateAfter) {
            this.dateAfter = dateAfter;
        }

        public int compareTo(Object o) {
            return (int) (((Marker) o).getDateBefore() - this.dateBefore);
        }
    }
}
