
/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): OUAZZANI CHAHDI Anass
 * --------------------------------------------------------------------------
 * $Id: SupervisionTSendRouter.java,v 1.2 2006/09/05 10:24:27 oanass Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.monitoring.router.duration;

import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.management.MBeanServerConnection;
import javax.management.ObjectName;

import org.objectweb.fractal.jmx.agent.AdminAttributes;

import org.objectweb.petals.kernel.admin.DistributedJMXServer;
import org.objectweb.petals.kernel.admin.DistributedJMXServerFactory;
import org.objectweb.petals.kernel.admin.service.PetalsAdminServiceMBean;
import org.objectweb.petals.monitoring.router.duration.RouterTimerProbe.Marker;



/**
 * @version $Rev: 617 $ $Date: 2006-06-19 17:28:41 +0200 (lun, 19 jun 2006) $
 * @since Petals 1.0
 * @author oanass
 *
 * Reports messages sends in the router
 */
public class RouterTimerSupervisor implements RouterTimerSupervisorMBean{
    private DistributedJMXServerFactory jmxFactory;
    private AdminAttributes adminAttributes;
    private PetalsAdminServiceMBean petalsAdmin;
    
    /**
     * 
     * @return : sent router duration of all messages that were sent from all containers
     */
    @SuppressWarnings("unchecked")
    public String[] getGlobalResult() {
        List<String> stringResult = new ArrayList<String>();
        try{
            List<Marker> l = null;
            List<Marker> orderTime = new ArrayList<Marker>();
            Collection<Map<String,String>> listeContainerInformation = petalsAdmin.getAllStartedServersConfiguration();
            
            for (Map<String, String> map : listeContainerInformation) {
                    DistributedJMXServer server = jmxFactory
                    .createDistributedJMXServer(map.get("name"));
                    ObjectName result = null;
                    String pattern = "FC/Petals@*:itf=monitor-timer-send-router";
                    Set objNames;
                    ObjectName objName = new ObjectName(pattern);
                    objNames = server.queryNames(objName, null);
                    if (objNames != null && objNames.size() == 1) {
                        result = (ObjectName) objNames.iterator().next();
                    }
                    if (result != null){
                        l = (List<Marker>) server.invoke(result, "getDates", new Object[0], new String[0]);
                    }
                    if(l != null){
                        orderTime.addAll(l);
                    }
            }
            Collections.sort(orderTime);
            for (Marker marker : orderTime) {
                Date d = new Date(marker.getDateBefore());
                DateFormat dF = DateFormat.getDateInstance(DateFormat.MEDIUM);
                String date = dF.format(d);
                stringResult.add("id Message : "+marker.getId() + " StartDate : " + date +" duration : "+ (marker.getDateAfter()-marker.getDateBefore())+ "ms");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return stringResult.toArray(new String[0]);
    }
    
    /**
     * Return the duration of send, of all messages sent from this container 
     * @return an array of String "id:xxx StartDate:xxx Duration:xxx"
     */
    @SuppressWarnings("unchecked")
    public String[] getLocalResult() {
        List<String> stringResult = new ArrayList<String>();
        try{
            List<Marker> l = null;
            List<Marker> orderTime = new ArrayList<Marker>();
            MBeanServerConnection connection = adminAttributes.getRawMBeanServer();
            ObjectName result = null;
            String pattern = "FC/Petals@*:itf=monitor-timer-send-router";
            Set objNames;
            ObjectName objName = new ObjectName(pattern);
            objNames = connection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                result = (ObjectName) objNames.iterator().next();
            }
            if (result != null){
                l = (List<Marker>) connection.invoke(result, "getDates", new Object[0], new String[0]);        
            }
            if(l != null){
                orderTime.addAll(l);
            }
            Collections.sort(orderTime);
            for (Marker marker : orderTime) {
                Date d = new Date(marker.getDateBefore());
                DateFormat dF = DateFormat.getDateInstance(DateFormat.MEDIUM);
                String date = dF.format(d);
                stringResult.add("id Message : "+marker.getId() + " StartDate : " + date +" duration : "+ (marker.getDateAfter()-marker.getDateBefore())+ "ms");
            }
            }catch (Exception e) {
            e.printStackTrace();
        }
        return stringResult.toArray(new String[0]);
    }
    
    /**
     *  Reset the list which contains sent duration of all messages that were sent from all containers.
     */
    public void clear(){
        try{
            Collection<Map<String,String>> listeContainerInformation = petalsAdmin.getAllStartedServersConfiguration();
            
            for (Map<String, String> map : listeContainerInformation) {
                    DistributedJMXServer server = jmxFactory
                    .createDistributedJMXServer(map.get("name"));
                    ObjectName result = null;
                    String pattern = "FC/Petals@*:itf=monitor-timer-send-router";
                    Set objNames;
                    ObjectName objName = new ObjectName(pattern);
                    objNames = server.queryNames(objName, null);
                    if (objNames != null && objNames.size() == 1) {
                        result = (ObjectName) objNames.iterator().next();
                    }
                    if (result != null){
                        server.invoke(result, "clear", new Object[0], new String[0]);
                    }
            }
        }catch (Exception e) {
            e.printStackTrace();
        }
    }

public RouterTimerSupervisor(DistributedJMXServerFactory jmxFactory, PetalsAdminServiceMBean petalsAdmin, AdminAttributes adminAttributes) {
        this.jmxFactory = jmxFactory;
        this.petalsAdmin = petalsAdmin;
        this.adminAttributes = adminAttributes;
    }
}
