/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): OUAZZANI CHAHDI Anass
 * --------------------------------------------------------------------------
 * $Id: TimerExchangeRouter.java,v 1.2 2006/09/05 10:24:27 oanass Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.monitoring.router.exchange;


import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.objectweb.petals.jbi.messaging.MessageExchangeImpl;
import org.objectweb.petals.monitoring.router.RouterMonitor;


/**
 * @version $Rev: 617 $ $Date: 2006-06-19 17:28:41 +0200 (lun, 19 jun 2006) $
 * @since Petals 1.0
 * @author oanass
 *
 * Used to monitor the exchange time of messages.
 */
public class ExchangeProbe implements RouterMonitor,Serializable{
    
    /**
     * 
     */
    private static final long serialVersionUID = 5388186479499964038L;
    List<Marker> dates = new ArrayList< Marker>();

    public List<Marker> getDates() {
        return dates;
    }

    /**
     * 
     * Reset the list of Marker "dates".
     */

    public synchronized void clear(){
        this.dates.clear();
    }
    
    /**
     * Update list of marker "dates".
     * If a message which contains the same id that was sent before, we don't modify anything.
     * Else a new Maker that contains the message id and time of bigining sent, of message that we are sending, 
     * is added to the list "dates".
     */
    public synchronized void beforeMessageSent(MessageExchangeImpl _exchange) {
        long date= System.currentTimeMillis();
        boolean found = false;
        Marker marker = null;
        
        Iterator it = dates.iterator();        
        while(it.hasNext() && found == false){
            marker = (Marker) it.next();
            if(_exchange.getExchangeId().equals(marker.getId())){
                found = true;
            }
        }
        if(found == false){
                marker = new Marker(_exchange.getExchangeId());
                marker.setBeforefirstSend(date);
                dates.add(marker); 
       }              
    }

    public synchronized void afterMessageSent(MessageExchangeImpl _exchange) {
        //nothing to do
    }

    public synchronized void beforeMessageReceived(MessageExchangeImpl _exchange) {
        //nothing to do
    }

    /**
     * Update list of marker "dates".
     * Store dateAfterReceive which is condidat to be end date of exchange. 
     */
    public synchronized void afterMessageReceived(MessageExchangeImpl _exchange) {
        long date= System.currentTimeMillis();
        boolean found = false;
        Marker marker = null;
        
        Iterator it = dates.iterator();        
        while(it.hasNext() && found == false){
            marker = (Marker) it.next();
            if(_exchange.getExchangeId().equals(marker.getId())){
                found = true;
            }
        }
        if(found == false){
            marker = new Marker(_exchange.getExchangeId());
            dates.add(marker);
        }
        marker.setAfterlastReceive(date);
    }


    /**
     * Represents a Message Time Exchange :
     * id, before exchange date, after exchange date
     * @author oanass
     * 
     */
    public class Marker implements Serializable, Comparable{
        
        private static final long serialVersionUID = 1021134084527427876L;
        private String id;
        private long beforefirstSend = 0;
        private long afterlastReceive;
        
        public Marker(String id) {
            this.id = id;
        }
        public long getAfterlastReceive() {
            return afterlastReceive;
        }
        public void setAfterlastReceive(long afterlastReceive) {
            this.afterlastReceive = afterlastReceive;
        }
        public long getBeforefirstSend() {
            return beforefirstSend;
        }
        public void setBeforefirstSend(long beforefirstSend) {
            this.beforefirstSend = beforefirstSend;
        }
        public String getId() {
            return id;
        }
        public int compareTo(Object o) {
            return (int) (((Marker) o).getBeforefirstSend() - this.beforefirstSend);
        }
    }
    
    
}


