/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): OUAZZANI CHAHDI Anass
 * --------------------------------------------------------------------------
 * $Id: SupervisionTExchangeRouter.java,v 1.2 2006/09/05 10:24:27 oanass Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.monitoring.router.exchange;

import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.management.ObjectName;

import org.objectweb.fractal.jmx.agent.AdminAttributes;

import org.objectweb.petals.kernel.admin.DistributedJMXServer;
import org.objectweb.petals.kernel.admin.DistributedJMXServerFactory;
import org.objectweb.petals.kernel.admin.service.PetalsAdminServiceMBean;
import org.objectweb.petals.monitoring.router.exchange.ExchangeProbe.Marker;

/**
 * Reports exchange time
 * 
 * @version $Rev: 617 $ $Date: 2006-06-19 17:28:41 +0200 (lun, 19 jun 2006) $
 * @since Petals 1.0
 * @author oanass
 */
public class ExchangeSupervisor implements ExchangeSupervisorMBean{
    private DistributedJMXServerFactory jmxFactory;
    @SuppressWarnings("unused")
    private AdminAttributes adminAttributes;
    private PetalsAdminServiceMBean petalsAdmin;
    List<Marker> listTime = new ArrayList<Marker>();
    
    /**
     * 
     * @return : exchange duration of all messages that were sent from all containers
     */
    @SuppressWarnings("unchecked")
    public List<String> getGlobalResult() {
        List<String> stringResult= new ArrayList<String>();
        try{
            List<Marker> markers = new ArrayList<Marker>();
            Collection<Map<String,String>> listeContainerInformation = petalsAdmin.getAllStartedServersConfiguration();
            
            for (Map<String, String> map : listeContainerInformation) {
                    DistributedJMXServer server = jmxFactory
                    .createDistributedJMXServer(map.get("name"));
                    ObjectName result = null;
                    String pattern = "FC/Petals@*:itf=monitor-timer-exchange-router";
                    Set objNames;
                    ObjectName objName = new ObjectName(pattern);
                    objNames = server.queryNames(objName, null);
                    if (objNames != null && objNames.size() == 1) {
                        result = (ObjectName) objNames.iterator().next();
                    }
                    if (result != null){
                        markers = (List<Marker>) server.invoke(result, "getDates", new Object[0], new String[0]);                
                    }
                    addMarkers(markers);
                   
            }
            Collections.sort(listTime);
            for (Marker marker : listTime) {
                Date d = new Date(marker.getBeforefirstSend());
                DateFormat dF = DateFormat.getDateInstance(DateFormat.MEDIUM);
                String date = dF.format(d);
                String s = "id = " + marker.getId() + " StartDate : " + date +" duration : "+(marker.getAfterlastReceive() -marker.getBeforefirstSend() + "ms");
                stringResult.add(s);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }      
        return stringResult;
    }
    
    /**
     *  Reset the list which contains the exchange duration of all messages that were sent from all containers.
     */
    public void clear(){
        try{
            this.listTime.clear();
            Collection<Map<String,String>> listeContainerInformation = petalsAdmin.getAllStartedServersConfiguration();
            
            for (Map<String, String> map : listeContainerInformation) {
                    DistributedJMXServer server = jmxFactory
                    .createDistributedJMXServer(map.get("name"));
                    ObjectName result = null;
                    String pattern = "FC/Petals@*:itf=monitor-timer-exchange-router";
                    Set objNames;
                    ObjectName objName = new ObjectName(pattern);
                    objNames = server.queryNames(objName, null);
                    if (objNames != null && objNames.size() == 1) {
                        result = (ObjectName) objNames.iterator().next();
                    }
                    if (result != null){
                        server.invoke(result, "clear", new Object[0], new String[0]);
                    }
                
            }
        }catch (Exception e) {
            e.printStackTrace();
        }
    }

    public ExchangeSupervisor(DistributedJMXServerFactory jmxFactory, PetalsAdminServiceMBean petalsAdmin, AdminAttributes adminAttributes) {
        this.jmxFactory = jmxFactory;
        this.petalsAdmin= petalsAdmin;
        this.adminAttributes = adminAttributes;
    }
    
    /**
     * Each Marker contains an id with a time beforeSent and afterReceive 
     *     of a message which have this id.
     * This method update the list of marker "listTime" to obtain a list
     *     which contain a date of beginiing and end of the exchanges.
     * 
     * @param listToAdd : list which contains Markers.
     */
    public void addMarkers(List<Marker> listToAdd){
        for (Marker marker : listToAdd) {
            boolean found = false;
            for (Marker marker1 : this.listTime) {
                if(marker.getId().equals(marker1.getId())){
                    found = true;
                        long timeBefore = marker.getBeforefirstSend();
                        long timeAfter = marker.getAfterlastReceive();
                        if(timeBefore != 0 && timeBefore < marker1.getBeforefirstSend())
                            marker1.setBeforefirstSend(timeBefore);
                        if(timeAfter > marker1.getAfterlastReceive())
                            marker1.setAfterlastReceive(timeAfter);
                }
            }
            if (found == false)
               this.listTime.add(marker);
            }
        
    }
}
