/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): OUAZZANI CHAHDI Anass
 * --------------------------------------------------------------------------
 * $Id: SupervisionContentMsg.java,v 1.2 2006/09/05 10:24:27 oanass Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.monitoring.router.volume;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.management.MBeanServerConnection;
import javax.management.ObjectName;

import org.objectweb.fractal.jmx.agent.AdminAttributes;

import org.objectweb.petals.kernel.admin.DistributedJMXServer;
import org.objectweb.petals.kernel.admin.DistributedJMXServerFactory;
import org.objectweb.petals.kernel.admin.service.PetalsAdminServiceMBean;
import org.objectweb.petals.monitoring.router.volume.VolumeProbe.Marker;

/**
 * @version $Rev: 617 $ $Date: 2006-06-19 17:28:41 +0200 (lun, 19 jun 2006) $
 * @since Petals 1.0
 * @author oanass
 * 
 * Reports messages volumes FIXME the size of the exchange is not well
 * calculated (pb with Source)
 */

public class VolumeSupervisor implements VolumeSupervisorMBean {

    private DistributedJMXServerFactory jmxFactory;

    private AdminAttributes adminAttributes;

    private PetalsAdminServiceMBean petalsAdmin;

    /**
     * Global message volume calculation
     * 
     * @return a String list representing id and size of all messages sent in
     *         the containers. Format : "id:[id] size:[size]byte"
     */
    @SuppressWarnings("unchecked")
    public List<String> getGlobalSendSize() {
        List<String> marker = new ArrayList<String>();
        try {
            // Retrieve the ContainerInformation for all the launched
            // containers.
            Collection<Map<String, String>> listeContainerInformation = petalsAdmin
                .getAllStartedServersConfiguration();

            for (Map<String, String> map : listeContainerInformation) {
                DistributedJMXServer server = jmxFactory
                    .createDistributedJMXServer(map.get("name"));
                ObjectName result = null;
                String pattern = "FC/Petals@*:itf=monitor-contentMsg";
                Set objNames;
                // Retrieve objectName of the this monitor.
                ObjectName objName = new ObjectName(pattern);
                objNames = server.queryNames(objName, null);
                if (objNames != null && objNames.size() == 1) {
                    result = (ObjectName) objNames.iterator().next();
                }
                if (result != null) {
                    // Retrieve the list of stored information in this
                    // monitor.
                    List<Marker> list = (List<Marker>) server.invoke(result,
                        "getMarkers", new Object[0], new String[0]);
                    for (Marker contentMsg : list) {
                        marker.add("id : " + contentMsg.getId() + " size : "
                            + contentMsg.getSize() + " bytes");
                    }
                }

            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return marker;
    }

    @SuppressWarnings("unchecked")
    /**
     * Global Exchange volume calculation
     * 
     * @return a String list representing id and size of all exchanges sent in
     *         the containers. Format : "id:[id] size:[size]byte"
     */
    public List<String> getGlobalExchangeSize() {
        List<MarkerSize> mapTime = new ArrayList<MarkerSize>();
        List<String> resultTime = new ArrayList<String>();
        List<Marker> markers = new ArrayList<Marker>();
        try {
            Collection<Map<String, String>> listeContainerInformation = petalsAdmin
                .getAllStartedServersConfiguration();

            for (Map<String, String> map : listeContainerInformation) {
                DistributedJMXServer server = jmxFactory
                    .createDistributedJMXServer(map.get("name"));
                ObjectName result = null;
                String pattern = "FC/Petals@*:itf=monitor-contentMsg";
                Set objNames;
                ObjectName objName = new ObjectName(pattern);
                objNames = server.queryNames(objName, null);
                if (objNames != null && objNames.size() == 1) {
                    result = (ObjectName) objNames.iterator().next();
                }
                if (result != null) {
                    List<Marker> list = (List<Marker>) server.invoke(result,
                        "getMarkers", new Object[0], new String[0]);
                    markers.addAll(list);
                }

            }
            for (Marker marker : markers) {
                boolean found = false;
                for (MarkerSize markerSize : mapTime) {
                    if (markerSize.getId().equals(marker.getId())) {
                        markerSize.addSize(marker.getSize());
                        markerSize.setDate(marker.date);
                        found = true;
                    }
                }
                if (!found)
                    mapTime.add(new MarkerSize(marker.getId(),
                        marker.getSize(), marker.date));
            }
            Collections.sort(mapTime);
            for (MarkerSize marker : mapTime) {
                resultTime.add("id : " + marker.getId() + " size : "
                    + marker.getSize() + "byte");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return resultTime;
    }

    /**
     * Local message volume calculation
     * 
     * @return a String list representing id and size of all messages sent in
     *         this container. Format : "id:[id] size:[size]byte"
     */
    @SuppressWarnings("unchecked")
    public List<String> getLocalSendSize() {
        List<String> marker = new ArrayList<String>();
        try {
            MBeanServerConnection connection = adminAttributes
                .getRawMBeanServer();
            ObjectName result = null;
            String pattern = "FC/Petals@*:itf=monitor-contentMsg";
            Set objNames;
            ObjectName objName = new ObjectName(pattern);
            // Retrieve objectName of the this monitor.
            objNames = connection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                result = (ObjectName) objNames.iterator().next();
            }
            if (result != null) {
                // Retrieve the list of stored information in this monitor.
                List<Marker> list = (List<Marker>) connection.invoke(result,
                    "getMarkers" + "", new Object[0], new String[0]);
                for (Marker contentMsg : list) {
                    marker.add("id : " + contentMsg.getId() + " size : "
                        + contentMsg.getSize() + "byte");
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return marker;
    }

    /**
     * 
     */
    public void clear() {
        try {
            // Retrieve the ContainerInformation for all the launched
            // containers.
            Collection<Map<String, String>> listeContainerInformation = petalsAdmin
                .getAllStartedServersConfiguration();

            for (Map<String, String> map : listeContainerInformation) {
                DistributedJMXServer server = jmxFactory
                    .createDistributedJMXServer(map.get("name"));
                ObjectName result = null;
                String pattern = "FC/Petals@*:itf=monitor-contentMsg";
                Set objNames;
                // Retrieve objectName of the this monitor.
                ObjectName objName = new ObjectName(pattern);
                objNames = server.queryNames(objName, null);
                if (objNames != null && objNames.size() == 1) {
                    result = (ObjectName) objNames.iterator().next();
                }
                if (result != null) {
                    server
                        .invoke(result, "clear", new Object[0], new String[0]);
                }

            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public VolumeSupervisor(DistributedJMXServerFactory jmxFactory,
        AdminAttributes adminAttributes, PetalsAdminServiceMBean petalsAdmin) {
        this.jmxFactory = jmxFactory;
        this.adminAttributes = adminAttributes;
        this.petalsAdmin = petalsAdmin;
    }

    /**
     * Represents a Message Size : id, send date, size (in byte)
     * 
     * @author oanass
     * 
     */
    private class MarkerSize implements Comparable {
        private String id;

        private int size;

        private long date;

        public MarkerSize(String id, int size, long date) {
            this.id = id;
            this.size = size;
            this.date = date;
        }

        public String getId() {
            return id;
        }

        public void setDate(long date) {
            if (date < this.date)
                this.date = date;
        }

        public int getSize() {
            return size;
        }

        public void addSize(long size) {
            this.size += size;
        }

        public int compareTo(Object o) {
            return (int) (((MarkerSize) o).date - this.date);
        }

    }
}
