/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): OUAZZANI CHAHDI Anass
 * --------------------------------------------------------------------------
 * $Id: TransporterInterceptorImpl.java,v 1.2 2006/09/05 10:24:27 oanass Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.monitoring.transporter;

import java.util.ArrayList;
import java.util.Collection;

import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.fraclet.annotation.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.Interface;
import org.objectweb.fractal.fraclet.annotation.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.LifeCycleType;
import org.objectweb.fractal.fraclet.annotation.Provides;
import org.objectweb.fractal.fraclet.annotation.Requires;

import org.objectweb.petals.jbi.messaging.MessageExchangeImpl;
import org.objectweb.petals.jbi.transport.TransportException;
import org.objectweb.petals.jbi.transport.Transporter;

/**
 * @version $Rev: 617 $ $Date: 2006-06-19 17:28:41 +0200 (lun, 19 jun 2006) $
 * @since Petals 1.0
 * @author oanass
 * 
 * This class is used to intercept the message that we will send or receive and
 * to start a monitoring-operation in a new Threads.
 */
@FractalComponent
@Provides(interfaces={@Interface(name="monitoring-service",signature=org.objectweb.petals.monitoring.transporter.TransporterInterceptor.class),@Interface(name="service",signature=org.objectweb.petals.jbi.transport.Transporter.class)})
public class TransporterInterceptorImpl implements Transporter,
    TransporterInterceptor {

    public static final int beforeSent = 0;

    public static final int afterSent = 1;

    @Requires(name="transporter",signature=org.objectweb.petals.jbi.transport.Transporter.class)
    private Transporter transporter;

    private Collection<TransporterMonitor> monitors = new ArrayList<TransporterMonitor>();

    /**
     * 
     * Send the message to the real Transporter. Monitors are called before and
     * after this call (there are launched in new Threads).
     * 
     * @param exchange
     * @param containerName
     */
    public void send(MessageExchangeImpl exchange, String containerName,
        long timeToLive) throws TransportException {
        try {
            MessageExchangeImpl message = (MessageExchangeImpl) (exchange
                .clone());

            for (TransporterMonitor monitor : monitors) {
                MessageExchangeImpl msg = (MessageExchangeImpl) (message
                    .clone());
                Thread tm = new ThreadMonitor(msg, monitor,
                    TransporterInterceptorImpl.beforeSent);
                tm.start();
            }

            transporter.send(exchange, containerName, timeToLive);

            for (TransporterMonitor monitor : monitors) {
                MessageExchangeImpl msg = (MessageExchangeImpl) (exchange
                    .clone());
                Thread tm = new ThreadMonitor(msg, monitor,
                    TransporterInterceptorImpl.afterSent);
                tm.start();
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("an error hapened, the message doesn't be sent");
        }
    }

    /**
     * Add new monitor to the monitor list for transporter
     * 
     * @param monitor :
     *            the monitor to add.
     */
    public boolean addMonitor(TransporterMonitor monitor) {
        boolean add = this.monitors.add(monitor);
        return add;
    }

    /**
     * Nothing to do
     */
    public void startListening() throws TransportException {
        transporter.startListening();
    }

    public void shutdown() throws Exception {
        transporter.shutdown();
    }

    /**
     * @see org.objectweb.fractal.api.control.LifeCycleController#stopFc()
     */
    @LifeCycle(on=LifeCycleType.STOP)
    public void stop() throws IllegalLifeCycleException {
        try {
            transporter.stop();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}
