/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): OUAZZANI CHAHDI Anass
 * --------------------------------------------------------------------------
 * $Id: TimerSendTransporter.java,v 1.2 2006/09/05 10:24:27 oanass Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.monitoring.transporter.duration;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.objectweb.petals.jbi.messaging.MessageExchangeImpl;
import org.objectweb.petals.monitoring.transporter.TransporterMonitor;

/**
 * @version $Rev: 617 $ $Date: 2006-06-19 17:28:41 +0200 (lun, 19 jun 2006) $
 * @since Petals 1.0
 * @author oanass
 *
 * Used to monitor the transport send time of messages.
 */
public class TransporterTimerProbe implements TransporterMonitor,Serializable {

    /**
     * 
     */
    private static final long serialVersionUID = 3195037533857228322L;
    List<Marker> dates = new ArrayList<Marker>();
    
    /**
     * Update list of marker "dates".
     * Store date before Sent (Transporter) of message that is sending. 
     */
    public synchronized void beforeMessageSent(MessageExchangeImpl exchange) {
        long dateBefore = System.currentTimeMillis();
        Marker marker = new Marker(exchange.getExchangeId(), dateBefore);
        dates.add(marker);
    }
    
    public List<Marker> getDates() {
        return dates;
    }
    
    public synchronized void clear(){
        this.dates.clear();
    }

    /**
     * Update list of marker "dates".
     * Store date after Sent (Transporter) of message that is sending. 
     */
    public synchronized void afterMessageSent(MessageExchangeImpl exchange) {
        long date = System.currentTimeMillis();
        synchronized (dates) {
            Iterator it = dates.iterator();
            Marker marker = null;
            long dateAfter = 1;
            while (dateAfter != 0 && it.hasNext()){
                marker = ((Marker) it.next());
                if(marker.getId().equals(exchange.getExchangeId()))
                    dateAfter = marker.getDateAfter();
            }
            marker.setDateAfter(date);
        }
    }

    public synchronized void msgReceivedBefore(MessageExchangeImpl message) {
//      nothing to do
    }

    public synchronized void msgReceivedAfter(MessageExchangeImpl message) {
//      nothing to do
    }
    
    public class Marker implements Serializable, Comparable{
        
        /**
         * 
         */
        private static final long serialVersionUID = 1120777515071635133L;
        private String id;
        private long dateBefore = 0;
        private long dateAfter = 0;        
        public String getId() {
            return id;
        }
        
       /**
        * 
        * @param id
        * @param dateBefore
        */
        public Marker(String id, long dateBefore) {
            this.id = id;
            this.dateBefore = dateBefore;
        }
        /**
         * 
         * @return : date before Sent
         */
        public long getDateBefore() {
            return dateBefore;
        }
        
        /**
         * 
         * @return : date after received
         */
        public long getDateAfter() {
            return dateAfter;
        }

        public void setDateAfter(long dateAfter) {
            this.dateAfter = dateAfter;
        }
        
        /**
         * used to set an order betwen the markers
         */
        public int compareTo(Object o) {
            return (int) (((Marker) o).getDateBefore() -  this.dateBefore );
        }
    }
}
