/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): OUAZZANI CHAHDI Anass
 * --------------------------------------------------------------------------
 * $Id: Supervision.java,v 1.2 2006/09/05 10:24:27 oanass Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.monitoringConsole;

import javax.management.ObjectName;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.fraclet.annotation.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.Interface;
import org.objectweb.fractal.fraclet.annotation.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.LifeCycleType;
import org.objectweb.fractal.fraclet.annotation.Provides;
import org.objectweb.fractal.fraclet.annotation.Requires;
import org.objectweb.fractal.jmx.agent.AdminAttributes;
import org.objectweb.fractal.jmx.agent.Introspector;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.petals.kernel.admin.DistributedJMXServerFactory;
import org.objectweb.petals.kernel.admin.service.PetalsAdminServiceMBean;
import org.objectweb.petals.monitoring.router.RouterInterceptor;
import org.objectweb.petals.monitoring.router.duration.RouterTimerProbe;
import org.objectweb.petals.monitoring.router.duration.RouterTimerSupervisor;
import org.objectweb.petals.monitoring.router.duration.RouterTimerSupervisorMBean;
import org.objectweb.petals.monitoring.router.exchange.ExchangeProbe;
import org.objectweb.petals.monitoring.router.exchange.ExchangeSupervisor;
import org.objectweb.petals.monitoring.router.exchange.ExchangeSupervisorMBean;
import org.objectweb.petals.monitoring.router.meter.MeterProbe;
import org.objectweb.petals.monitoring.router.meter.MeterSupervisor;
import org.objectweb.petals.monitoring.router.meter.MeterSupervisorMBean;
import org.objectweb.petals.monitoring.router.volume.VolumeProbe;
import org.objectweb.petals.monitoring.router.volume.VolumeSupervisor;
import org.objectweb.petals.monitoring.router.volume.VolumeSupervisorMBean;
import org.objectweb.petals.monitoring.transporter.TransporterInterceptor;
import org.objectweb.petals.monitoring.transporter.duration.TransporterTimerProbe;
import org.objectweb.petals.monitoring.transporter.duration.TransporterTimerSupervisor;
import org.objectweb.petals.monitoring.transporter.duration.TransporterTimerSupervisorMBean;

/**
 * @version $Rev: 617 $ $Date: 2006-06-19 17:28:41 +0200 (lun, 19 jun 2006) $
 * @since Petals 1.0
 * @author oanass
 * 
 * This class is used to add a new monitor.
 */
@FractalComponent
@Provides(interfaces=@Interface(name="supervision",signature=org.objectweb.petals.monitoringConsole.SupervisionMBean.class))
public class Supervision implements SupervisionMBean {

    @Requires(name="dist-jmx-server",signature=org.objectweb.petals.kernel.admin.DistributedJMXServerFactory.class)
	private DistributedJMXServerFactory jmxFactory;

    @Requires(name="adminAtt",signature=org.objectweb.fractal.jmx.agent.AdminAttributes.class)
    private AdminAttributes adminAttributes;

    @Requires(name="interceptorTransporter",signature=org.objectweb.petals.monitoring.transporter.TransporterInterceptor.class)
    private TransporterInterceptor iTService;

    @Requires(name="interceptorRouter",signature=org.objectweb.petals.monitoring.router.RouterInterceptor.class)
    private RouterInterceptor iRService;
    
    @Requires(name="petalsadmin-service",signature=org.objectweb.petals.kernel.admin.service.PetalsAdminServiceMBean.class)
    private PetalsAdminServiceMBean petalsAdmin;

    /**
     * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
     */
    @LifeCycle(on=LifeCycleType.START)
    public void start() throws IllegalLifeCycleException {
        setMeter();
        setVolume();
        setExchange();
        setDurationRouter();
        setDurationTransporter();
    }

    /**
     * @see org.objectweb.petals.monitoringConsole.SupervisionMBean
     */
    public String setMeter() {
        String result = null;
        try {

            MeterProbe monitorMeter = new MeterProbe();
            adminAttributes.getRawMBeanServer().registerMBean(
                Introspector.createMBean(monitorMeter),
                new ObjectName("FC/Petals@" + ":itf=monitor-meter"));

            MeterSupervisorMBean supervisionMeter = new MeterSupervisor(
                jmxFactory,petalsAdmin, this.adminAttributes);

            adminAttributes.getRawMBeanServer()
                .registerMBean(
                    Introspector.createMBean(supervisionMeter,
                        MeterSupervisorMBean.class),
                    new ObjectName(
                        "Petals:type=monitoring,name=meter-supervision"));

            iRService.addMonitor(monitorMeter);
            result = "monitor-meter is added to the list of the components of supervision ";
        } catch (Exception e) {
            result = "this component of supervision already exist";
        }
        return result;
    }

    /**
     * FIXME do not use, problem with Source size calculation of Exchanges
     * 
     * @see org.objectweb.petals.monitoringConsole.SupervisionMBean
     */
    public String setVolume() {
        String result = null;
        try {

            VolumeProbe monitorContentMsg = new VolumeProbe();
            adminAttributes.getRawMBeanServer().registerMBean(
                Introspector.createMBean(monitorContentMsg),
                new ObjectName("FC/Petals@" + ":itf=monitor-contentMsg"));

            VolumeSupervisorMBean supervisionContentMsg = new VolumeSupervisor(
                jmxFactory, this.adminAttributes,petalsAdmin);
            adminAttributes.getRawMBeanServer().registerMBean(
                Introspector.createMBean(supervisionContentMsg,
                    VolumeSupervisorMBean.class),
                new ObjectName(
                    "Petals:type=monitoring,name=payload-volume-supervision"));

            iRService.addMonitor(monitorContentMsg);
            result = "monitor-contentMsg is added to the list of the components of supervision ";
        } catch (Exception e) {
            result = "this component of supervision already exist";
        }
        return result;
    }

    /**
     * @see org.objectweb.petals.monitoringConsole.SupervisionMBean
     */
    public String setExchange() {
        String result = null;
        try {

            ExchangeProbe monitorTExchangeRouter = new ExchangeProbe();
            adminAttributes.getRawMBeanServer().registerMBean(
                Introspector.createMBean(monitorTExchangeRouter),
                new ObjectName("FC/Petals@"
                    + ":itf=monitor-timer-exchange-router"));

            ExchangeSupervisorMBean supervisionTExchangeRouter = new ExchangeSupervisor(
                this.jmxFactory, petalsAdmin, this.adminAttributes);

            adminAttributes.getRawMBeanServer().registerMBean(
                Introspector.createMBean(supervisionTExchangeRouter,
                    ExchangeSupervisorMBean.class),
                new ObjectName(
                    "Petals:type=monitoring,name=exchange-time-supervision"));

            iRService.addMonitor(monitorTExchangeRouter);
            result = "monitor-timer-exchange-router is added to the list of the components of supervision ";
        } catch (Exception e) {
            result = "this component of supervision already exist";
        }
        return result;
    }

    /**
     * @see org.objectweb.petals.monitoringConsole.SupervisionMBean
     */
    public String setDurationRouter() {
        String result = null;
        try {

            RouterTimerProbe monitorTSendRouter = new RouterTimerProbe();
            adminAttributes.getRawMBeanServer()
                .registerMBean(
                    Introspector.createMBean(monitorTSendRouter),
                    new ObjectName("FC/Petals@"
                        + ":itf=monitor-timer-send-router"));

            RouterTimerSupervisorMBean supervisionTSendRouter = new RouterTimerSupervisor(
                this.jmxFactory,petalsAdmin, this.adminAttributes);

            adminAttributes.getRawMBeanServer().registerMBean(
                Introspector.createMBean(supervisionTSendRouter,
                    RouterTimerSupervisorMBean.class),
                new ObjectName(
                    "Petals:type=monitoring,name=routing-time-supervision"));

            iRService.addMonitor(monitorTSendRouter);
            result = "monitor-timer-send-router is added to the list of the components of supervision ";
        } catch (Exception e) {
            result = "this component of supervision already exist";
        }
        return result;
    }

    /**
     * @see org.objectweb.petals.monitoringConsole.SupervisionMBean
     */
    public String setDurationTransporter() {
        String result = null;
        try {
            TransporterTimerProbe monitorTSendTransporter = new TransporterTimerProbe();
            adminAttributes.getRawMBeanServer().registerMBean(
                Introspector.createMBean(monitorTSendTransporter),
                new ObjectName("FC/Petals@"
                    + ":itf=monitor-timer-send-transporter"));

            TransporterTimerSupervisorMBean supervisionTSendTransporter = new TransporterTimerSupervisor(
                this.jmxFactory,petalsAdmin, this.adminAttributes);

            adminAttributes.getRawMBeanServer().registerMBean(
                Introspector.createMBean(supervisionTSendTransporter,
                    TransporterTimerSupervisorMBean.class),
                new ObjectName(
                    "Petals:type=monitoring,name=transport-time-supervision"));

            iTService.addMonitor(monitorTSendTransporter);
            result = "monitor-timer-send-transporter is added to the list of the components of supervision ";

        } catch (Exception e) {
            result = "this component of supervision already exist";
        }
        return result;
    }

    /**
     * A utility function allowing to get a component from a content controller
     * 
     * @param cc
     *            ContentController
     * @param name
     *            component name
     * @return
     */
    protected Component getComponentByName(ContentController contentController,
        String name) {
        // The component to be returned
        Component c = null;
        // List content controller subcomponents
        Component[] comps = contentController.getFcSubComponents();
        for (int i = 0; i < comps.length; i++) {
            Component component = comps[i];
            try {
                String componentName = Fractal.getNameController(component)
                    .getFcName();
                if (componentName.equals(name)) {
                    c = component;
                    break;
                }
            } catch (NoSuchInterfaceException e) {
                // Do nothing
            }
        }
        return c;
    }

}
