/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: WebAdminServlet.java 16:35:33 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.webadmin;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.apache.commons.fileupload.servlet.ServletRequestContext;
import org.objectweb.petals.tools.webadmin.bean.JBIEntity;
import org.objectweb.petals.tools.webadmin.exception.WebAdminException;
import org.objectweb.petals.tools.webadmin.util.JMXUtil;

/**
 * Web Administration servlet for Petals
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class WebAdminServlet extends HttpServlet {

    private static final long serialVersionUID = 1L;

    /**
     * Map of the deployed components
     */
    protected Map<String, JBIEntity> components = new HashMap<String, JBIEntity>();

    /**
     * Map of the deployed shared libraries
     */
    // private Map<String, String> sharedLibraries = new HashMap<String,
    // String>();
    /**
     * Jmx Util
     */
    protected JMXUtil jmxUtil;

    /**
     * Map of the deployed service assemblies
     */
    protected Map<String, JBIEntity> serviceAssemblies = new HashMap<String, JBIEntity>();

    public void doGet(HttpServletRequest req, HttpServletResponse res)
        throws IOException {
        String action = req.getParameter("ACTION");
        if (ServletFileUpload
            .isMultipartContent(new ServletRequestContext(req))) {
            doInstallEntity(req, res);
        }
        if (action == null) {
            action = (String) req.getAttribute("ACTION");
            if (action == null) {
                displayDefaultPage(req, res);
            }
        } else if ("detail".equals(action)) {
            String type = req.getParameter("TYPE");
            String name = req.getParameter("NAME");
            if ("component".equals(type)) {
                doDetailComponent(req, res, name);
            } else if ("serviceassembly".equals(type)) {
                doDetailServiceAssembly(req, res, name);
            }
        } else if ("trash".equals(action)) {
            String type = req.getParameter("TYPE");
            String name = req.getParameter("NAME");
            if ("component".equals(type)) {
                doTrashComponent(req, res, name);
            } else if ("serviceassembly".equals(type)) {
                doTrashServiceAssembly(req, res, name);
            }
        } else if ("start".equals(action)) {
            String type = req.getParameter("TYPE");
            String name = req.getParameter("NAME");
            if ("component".equals(type)) {
                if (components.containsKey(name)) {
                    JBIEntity entity = components.get(name);
                    doChangeComponentState(req, res, name, entity, "start");
                }
            } else if ("serviceassembly".equals(type)
                && serviceAssemblies.containsKey(name)) {
                JBIEntity entity = serviceAssemblies.get(name);
                doChangeServiceAssemblyState(req, res, name, entity, "start");
            }
        } else if ("stop".equals(action)) {
            String type = req.getParameter("TYPE");
            String name = req.getParameter("NAME");
            if ("component".equals(type)) {
                if (components.containsKey(name)) {
                    JBIEntity entity = components.get(name);
                    doChangeComponentState(req, res, name, entity, "stop");
                }
            } else if ("serviceassembly".equals(type)
                && serviceAssemblies.containsKey(name)) {
                JBIEntity entity = serviceAssemblies.get(name);
                doChangeServiceAssemblyState(req, res, name, entity, "stop");
            }
        } else if ("shutdown".equals(action)) {
            String type = req.getParameter("TYPE");
            String name = req.getParameter("NAME");
            if ("component".equals(type)) {
                if (components.containsKey(name)) {
                    JBIEntity entity = components.get(name);
                    doChangeComponentState(req, res, name, entity, "shutDown");
                }
            } else if ("serviceassembly".equals(type)
                && serviceAssemblies.containsKey(name)) {
                JBIEntity entity = serviceAssemblies.get(name);
                doChangeServiceAssemblyState(req, res, name, entity, "shutDown");
            }
        } else if ("chJMX".equals(action)) {
            doDisplayJMXProperties(req, res);
        } else if ("validJMX".equals(action)) {
            String host = (String) req.getParameter("host");
            String port = (String) req.getParameter("port");
            String username = (String) req.getParameter("username");
            String password = (String) req.getParameter("password");
            if (host != null) {
                jmxUtil.setHost(host);
            }
            if (port != null) {
                jmxUtil.setPort(port);
            }
            if (username != null) {
                jmxUtil.setUsername(username);
            }
            if (password != null) {
                jmxUtil.setPassword(password);
            }
            displayDefaultPage(req, res);
        }
        if (!res.isCommitted()) {
            displayDefaultPage(req, res);
        }
    }

    public void doPost(HttpServletRequest req, HttpServletResponse res)
        throws IOException {
        doGet(req, res);
    }

    public void init() {
        String path = this.getServletContext().getRealPath("WEB-INF/web.xml");
        String servletPath = path.replace("WEB-INF/web.xml", "");
        InputStream stream = null;
        try {
            stream = new FileInputStream(servletPath
                + "jmxconnection.properties");
        } catch (FileNotFoundException e) {
            // Do nothing
        }
        jmxUtil = new JMXUtil(stream);
    }

    /**
     * Refresh the lists
     * 
     * @throws WebAdminException
     */
    protected void refreshLists() throws WebAdminException {
        components = jmxUtil.listComponents();
        serviceAssemblies = jmxUtil.listServiceAssemblies();
    }

    /**
     * Display the home page
     * 
     * @param req
     *            HttpServletRequest
     * @param res
     *            HttpServletResponse
     * @throws IOException
     */
    private void displayDefaultPage(HttpServletRequest req,
        HttpServletResponse res) throws IOException {
        RequestDispatcher rd = null;
        req.setAttribute("ACTION", "");
        try {
            refreshLists();
            req.setAttribute("components", components);
            req.setAttribute("serviceassemblies", serviceAssemblies);
        } catch (WebAdminException e1) {
            req.setAttribute("error",
                "Problem while trying to list the deployed JBI components\n"
                    + e1.getMessage());
        }
        rd = getServletContext().getRequestDispatcher("/jsp/home.jsp");
        try {
            rd.forward(req, res);
        } catch (ServletException e) {
            e.printStackTrace();
        }
    }

    /**
     * Change state of a component
     * 
     * @param req
     *            HttpServletRequest
     * @param res
     *            HttpServletResponse
     * @param name
     *            name of the component
     * @throws IOException
     */
    private void doChangeComponentState(HttpServletRequest req,
        HttpServletResponse res, String name, JBIEntity entity, String action)
        throws IOException {
        RequestDispatcher rd = null;
        try {
            jmxUtil.changeCompoState(entity, action);
            try {
                refreshLists();
                req.setAttribute("components", components);
                req.setAttribute("serviceassemblies", serviceAssemblies);
                entity = components.get(name);
            } catch (WebAdminException e1) {
                req.setAttribute("error",
                    "Problem while trying to list the deployed JBI components");
            }
        } catch (WebAdminException e) {
            req.setAttribute("error",
                "Problem while trying to change component state");
        }
        req.setAttribute("ACTION", "detail");
        req.setAttribute("TYPE", "component");
        req.setAttribute("NAME", name);
        req.setAttribute("entity", entity);
        rd = getServletContext().getRequestDispatcher("/jsp/detailCompo.jsp");
        try {
            rd.forward(req, res);
        } catch (ServletException e) {
            e.printStackTrace();
        }
    }

    /**
     * Change state of a service assembly
     * 
     * @param req
     *            HttpServletRequest
     * @param res
     *            HttpServletResponse
     * @param name
     *            name of the service assembly
     * @throws IOException
     */
    private void doChangeServiceAssemblyState(HttpServletRequest req,
        HttpServletResponse res, String name, JBIEntity entity, String action)
        throws IOException {
        RequestDispatcher rd = null;
        if (entity != null) {
            try {
                jmxUtil.changeServiceAssemblyState(entity, action);
                try {
                    refreshLists();
                    req.setAttribute("components", components);
                    req.setAttribute("serviceassemblies", serviceAssemblies);
                    entity = serviceAssemblies.get(name);
                } catch (WebAdminException e1) {
                    req
                        .setAttribute("error",
                            "Problem while trying to list the deployed JBI components");
                }
            } catch (WebAdminException e) {
                req.setAttribute("error",
                    "Problem while trying to change service assembly state");
            }
            List<JBIEntity> compos = new ArrayList<JBIEntity>();
            try {
                for (Entry entry : jmxUtil.getServiceAssemblyInformation(name)
                    .entrySet()) {
                    JBIEntity compo = components.get(entry.getValue());
                    if (compo != null) {
                        compo.getServiceUnits().add((String) entry.getKey());
                        components.put(compo.getName(), compo);
                        compos.add(compo);
                    }
                }
            } catch (WebAdminException e1) {
                req
                    .setAttribute("error",
                        "Problem while trying to get the service assembly information");
            }
            req.setAttribute("compos", compos);
        }
        req.setAttribute("ACTION", "detail");
        req.setAttribute("TYPE", "serviceassembly");
        req.setAttribute("NAME", name);
        req.setAttribute("entity", entity);
        rd = getServletContext().getRequestDispatcher("/jsp/detailSA.jsp");
        try {
            rd.forward(req, res);
        } catch (ServletException e) {
            e.printStackTrace();
        }
    }

    /**
     * Details a component
     * 
     * @param req
     *            HttpServletRequest
     * @param res
     *            HttpServletResponse
     * @param name
     *            name of the component
     * @throws IOException
     */
    private void doDetailComponent(HttpServletRequest req,
        HttpServletResponse res, String name) throws IOException {
        RequestDispatcher rd = null;
        req.setAttribute("ACTION", "");
        req.setAttribute("TYPE", "");
        req.setAttribute("NAME", "");
        if (components.containsKey(name)) {
            JBIEntity compo = components.get(name);
            req.setAttribute("entity", compo);
            rd = getServletContext().getRequestDispatcher(
                "/jsp/detailCompo.jsp");
            try {
                rd.forward(req, res);
            } catch (ServletException e) {
                e.printStackTrace();
            }
        } else {
            displayDefaultPage(req, res);
        }
    }

    /**
     * Details a service assembly
     * 
     * @param req
     *            HttpServletRequest
     * @param res
     *            HttpServletResponse
     * @param name
     *            name of the component
     * @throws IOException
     */
    private void doDetailServiceAssembly(HttpServletRequest req,
        HttpServletResponse res, String name) throws IOException {
        RequestDispatcher rd = null;
        req.setAttribute("ACTION", "");
        req.setAttribute("TYPE", "");
        req.setAttribute("NAME", "");
        if (serviceAssemblies.containsKey(name)) {
            JBIEntity sa = serviceAssemblies.get(name);
            List<JBIEntity> compos = new ArrayList<JBIEntity>();
            try {
                for (Entry entry : jmxUtil.getServiceAssemblyInformation(name)
                    .entrySet()) {
                    JBIEntity compo = components.get(entry.getValue());
                    if (compo != null) {
                        compo.getServiceUnits().add((String) entry.getKey());
                        components.put(compo.getName(), compo);
                        compos.add(compo);
                    }
                }
            } catch (WebAdminException e1) {
                req
                    .setAttribute("error",
                        "Problem while trying to get the service assembly information");
            }
            req.setAttribute("compos", compos);
            req.setAttribute("entity", sa);
            rd = getServletContext().getRequestDispatcher("/jsp/detailSA.jsp");
            try {
                rd.forward(req, res);
            } catch (ServletException e) {
                e.printStackTrace();
            }
        } else {
            displayDefaultPage(req, res);
        }
    }

    /**
     * Display JMX properties
     * 
     * @param req
     *            HttpServletRequest
     * @param res
     *            HttpServletResponse
     * @throws IOException
     */
    private void doDisplayJMXProperties(HttpServletRequest req,
        HttpServletResponse res) throws IOException {
        RequestDispatcher rd = null;
        req.setAttribute("ACTION", "");
        req.setAttribute("host", jmxUtil.getHost());
        req.setAttribute("port", jmxUtil.getPort());
        req.setAttribute("username", jmxUtil.getUsername());
        req.setAttribute("password", jmxUtil.getPassword());

        rd = getServletContext().getRequestDispatcher("/jsp/jmxProps.jsp");
        try {
            rd.forward(req, res);
        } catch (ServletException e) {
            e.printStackTrace();
        }
    }

    /**
     * Install a JBI Entity
     * 
     * @param req
     *            HttpServletRequest
     * @param res
     *            HttpServletResponse
     * @param file
     *            archive to install
     * @throws IOException
     * @throws IOException
     */
    private void doInstallEntity(HttpServletRequest req, HttpServletResponse res)
        throws IOException {
        req.setAttribute("ACTION", "");
        try {
            String fileName = null;
            String fieldName = null;
            // Create a factory for disk-based file items
            DiskFileItemFactory factory = new DiskFileItemFactory();
            // Set factory constraints
            factory.setSizeThreshold(1024 * 1024 * 100001);
            String path = this.getServletContext().getRealPath(
                "WEB-INF" + File.separator + "web.xml");
            String servletPath = path.replace("WEB-INF" + File.separator
                + "web.xml", "");
            path = servletPath + "tmp" + File.separator;
            factory.setRepository(new File(path));
            // Create a new file upload handler
            ServletFileUpload upload = new ServletFileUpload(factory);
            // Set overall request size constraint
            upload.setSizeMax(1024 * 1024 * 100000);
            List items = new ArrayList();
            try {
                items = upload.parseRequest(req);
            } catch (FileUploadException e) {
                e.getCause();
            }
            Iterator iter = items.iterator();
            while (iter.hasNext()) {
                FileItem item = (FileItem) iter.next();
                if (!item.isFormField() && !item.getName().equals("")) {
                    fieldName = item.getFieldName();
                    fileName = item.getName();
                    fileName = servletPath + "upload" + File.separator
                        + fileName;
                    new File(servletPath + "upload" + File.separator).mkdir();
                    item.write(new File(fileName));
                }
            }
            if ("fileCompo".equals(fieldName)) {
                jmxUtil.installComponent(fileName);
            } else if ("fileSA".equals(fieldName)) {
                jmxUtil.deployServiceAssembly(fileName);
            } else if ("fileSL".equals(fieldName)) {
                jmxUtil.installSharedLibrary(fileName);
            }
            new File(fileName).delete();
        } catch (Exception e) {
            req
                .setAttribute("error",
                    "Problem while trying to install archive");
        }
        displayDefaultPage(req, res);
    }

    /**
     * Trash a component
     * 
     * @param req
     *            HttpServletRequest
     * @param res
     *            HttpServletResponse
     * @param name
     *            name of the component
     * @throws IOException
     */
    private void doTrashComponent(HttpServletRequest req,
        HttpServletResponse res, String name) throws IOException {
        req.setAttribute("ACTION", "");
        req.setAttribute("TYPE", "");
        req.setAttribute("NAME", "");
        if (components.containsKey(name)) {
            JBIEntity compo = components.get(name);
            try {
                jmxUtil.uninstallCompo(compo);
                try {
                    refreshLists();
                    req.setAttribute("components", components);
                    req.setAttribute("serviceassemblies", serviceAssemblies);
                } catch (WebAdminException e1) {
                    req.setAttribute("error",
                        "Problem while trying to list the deployed JBI components\n"
                            + e1.getMessage());
                }
            } catch (WebAdminException e) {
                req.setAttribute("error",
                    "Problem while trying to uninstall a component");
            }
        }
        displayDefaultPage(req, res);
    }

    /**
     * Trash a service assembly
     * 
     * @param req
     *            HttpServletRequest
     * @param res
     *            HttpServletResponse
     * @param name
     *            name of the component
     * @throws IOException
     */
    private void doTrashServiceAssembly(HttpServletRequest req,
        HttpServletResponse res, String name) throws IOException {
        req.setAttribute("ACTION", "");
        req.setAttribute("TYPE", "");
        req.setAttribute("NAME", "");
        if (serviceAssemblies.containsKey(name)) {
            try {
                jmxUtil.undeployServiceAssembly(name);
                try {
                    refreshLists();
                    req.setAttribute("components", components);
                    req.setAttribute("serviceassemblies", serviceAssemblies);
                } catch (WebAdminException e1) {
                    req.setAttribute("error",
                        "Problem while trying to list the deployed JBI components\n"
                            + e1.getMessage());
                }
            } catch (WebAdminException e) {
                req.setAttribute("error",
                    "Problem while trying to undeploy a service assembly\n"
                        + e.getMessage());
            }
        }
        displayDefaultPage(req, res);
    }

}
