/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: WebAdminServletTest.java 09:53:35 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.webadmin;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Properties;

import javax.management.ObjectName;
import javax.management.remote.JMXConnector;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.petals.tools.webadmin.bean.JBIEntity;
import org.objectweb.petals.tools.webadmin.exception.WebAdminException;
import org.objectweb.petals.tools.webadmin.mock.HttpServletRequestMock;
import org.objectweb.petals.tools.webadmin.mock.HttpServletResponseMock;
import org.objectweb.petals.tools.webadmin.mock.MockMBeanConnection;
import org.objectweb.petals.tools.webadmin.mock.ServletContextMock;
import org.objectweb.petals.tools.webadmin.util.JMXUtil;

import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;

/**
 * Test of the WebAdminServlet
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class WebAdminServletTest extends TestCase {

    private String baseDir;

    private WebAdminServlet webAdminServlet;

    public void setUp() throws NumberFormatException, Exception {
        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
        FileInputStream fileInputStream = new FileInputStream(baseDir
            + "src" + File.separator + "test-data" + File.separator
            + "jmx.properties");
        Properties properties = new Properties();
        properties.load(fileInputStream);
        fileInputStream = new FileInputStream(baseDir
            + "src" + File.separator + "test-data" + File.separator
            + "jmx.properties");

        JMXUtil util = null;

        util = EasyMock.createMock(JMXUtil.class, new Method[] {JMXUtil.class
            .getDeclaredMethod("retrieveConnector", new Class[] {})});
        JMXConnector connector = EasyMock.createMock(JMXConnector.class);
        MockMBeanConnection connection = new MockMBeanConnection();

        EasyMock.expect(util.retrieveConnector()).andReturn(connector)
            .anyTimes();
        EasyMock.expect(connector.getMBeanServerConnection()).andReturn(
            connection).anyTimes();
        connector.close();
        connector.close();
        connector.close();

        EasyMock.replay(util);
        EasyMock.replay(connector);

        util.setHost("127.0.0.1");
        util.setPort("9876");

        Method method1 = null;

        ServletContextMock servletContextMock = new ServletContextMock();

        try {
            method1 = WebAdminServlet.class.getMethod("getServletContext",
                new Class[] {});
        } catch (SecurityException e1) {
            e1.printStackTrace();
        } catch (NoSuchMethodException e1) {
            e1.printStackTrace();
        }
        Method[] mockedMethods = new Method[] {method1};

        webAdminServlet = createMock(WebAdminServlet.class, mockedMethods);

        expect(webAdminServlet.getServletContext()).andReturn(
            servletContextMock).anyTimes();
        replay(webAdminServlet);

        webAdminServlet.jmxUtil = util;

        webAdminServlet.components = new HashMap<String, JBIEntity>();
        webAdminServlet.serviceAssemblies = new HashMap<String, JBIEntity>();
    }

    public void testDoGet() throws IOException {
        HttpServletRequest request = new HttpServletRequestMock();
        HttpServletResponse response = new HttpServletResponseMock();
        webAdminServlet.doGet(request, response);
    }

    public void testDoGetDoChangeComponentStateShutdown() throws Exception {

        JBIEntity entity = new JBIEntity();
        entity.setName("foo001");
        entity.setObjectName(new ObjectName(
            "org.objectweb.petals:type=engine,name=foo001"));

        webAdminServlet.components.put("foo001", entity);

        HttpServletRequestMock request = new HttpServletRequestMock();
        request.getParameters().put("ACTION", "shutdown");
        request.getParameters().put("TYPE", "component");
        request.getParameters().put("NAME", "foo001");
        HttpServletResponseMock response = new HttpServletResponseMock();
        webAdminServlet.doGet(request, response);

        JBIEntity entity2 = webAdminServlet.components.get("foo001");
        assertEquals(entity2.getState(), "Shutdown");

    }

    public void testDoGetDoChangeComponentStateStart() throws Exception {
        JBIEntity entity = new JBIEntity();
        entity.setName("foo001");
        entity.setObjectName(new ObjectName(
            "org.objectweb.petals:type=engine,name=foo001"));

        webAdminServlet.components.put("foo001", entity);

        HttpServletRequestMock request = new HttpServletRequestMock();
        request.getParameters().put("ACTION", "start");
        request.getParameters().put("TYPE", "component");
        request.getParameters().put("NAME", "foo001");
        HttpServletResponseMock response = new HttpServletResponseMock();
        webAdminServlet.doGet(request, response);

        JBIEntity entity2 = webAdminServlet.components.get("foo001");
        assertEquals(entity2.getState(), "Started");

    }

    public void testDoGetDoChangeComponentStateStop() throws Exception {
        JBIEntity entity = new JBIEntity();
        entity.setName("foo001");
        entity.setObjectName(new ObjectName(
            "org.objectweb.petals:type=engine,name=foo001"));

        webAdminServlet.components.put("foo001", entity);

        HttpServletRequestMock request = new HttpServletRequestMock();
        request.getParameters().put("ACTION", "stop");
        request.getParameters().put("TYPE", "component");
        request.getParameters().put("NAME", "foo001");
        HttpServletResponseMock response = new HttpServletResponseMock();
        webAdminServlet.doGet(request, response);

        JBIEntity entity2 = webAdminServlet.components.get("foo001");
        assertEquals(entity2.getState(), "Stopped");

    }

    public void testDoGetDoChangeServiceAssemblyStateShutdown()
        throws Exception {

        JBIEntity entity = new JBIEntity();
        entity.setName("foo0002");

        webAdminServlet.serviceAssemblies.put("foo0002", entity);

        HttpServletRequestMock request = new HttpServletRequestMock();
        request.getParameters().put("ACTION", "shutdown");
        request.getParameters().put("TYPE", "serviceassembly");
        request.getParameters().put("NAME", "foo0002");
        HttpServletResponseMock response = new HttpServletResponseMock();
        webAdminServlet.doGet(request, response);

        JBIEntity entity2 = webAdminServlet.serviceAssemblies.get("foo0002");
        assertEquals(entity2.getState(), "Shutdown");

    }

    public void testDoGetDoChangeServiceAssemblyStateStart() throws Exception {

        JBIEntity entity = new JBIEntity();
        entity.setName("foo0002");

        webAdminServlet.serviceAssemblies.put("foo0002", entity);

        HttpServletRequestMock request = new HttpServletRequestMock();
        request.getParameters().put("ACTION", "start");
        request.getParameters().put("TYPE", "serviceassembly");
        request.getParameters().put("NAME", "foo0002");
        HttpServletResponseMock response = new HttpServletResponseMock();
        webAdminServlet.doGet(request, response);

        JBIEntity entity2 = webAdminServlet.serviceAssemblies.get("foo0002");
        assertEquals(entity2.getState(), "Started");

    }

     public void testDoGetDoChangeServiceAssemblyStateStop() throws Exception {

        JBIEntity entity = new JBIEntity();
        entity.setName("foo0002");

        webAdminServlet.serviceAssemblies.put("foo0002", entity);

        HttpServletRequestMock request = new HttpServletRequestMock();
        request.getParameters().put("ACTION", "stop");
        request.getParameters().put("TYPE", "serviceassembly");
        request.getParameters().put("NAME", "foo0002");
        HttpServletResponseMock response = new HttpServletResponseMock();
        webAdminServlet.doGet(request, response);

        JBIEntity entity2 = webAdminServlet.serviceAssemblies.get("foo0002");
        assertEquals(entity2.getState(), "Stopped");

    }

    public void testDoGetDoDetailComponent() throws Exception {

        JBIEntity entity = new JBIEntity();
        entity.setName("foo001");
        entity.setObjectName(new ObjectName(
            "org.objectweb.petals:type=binding,name=foo001"));

        webAdminServlet.components.put("foo001", entity);

        HttpServletRequestMock request = new HttpServletRequestMock();
        request.getParameters().put("ACTION", "detail");
        request.getParameters().put("TYPE", "component");
        request.getParameters().put("NAME", "foo001");
        HttpServletResponseMock response = new HttpServletResponseMock();
        webAdminServlet.doGet(request, response);

        JBIEntity entity2 = (JBIEntity) request.getAttribute("entity");
        assertEquals(entity, entity2);

    }

    public void testDoGetDoDetailServiceAssembly() throws Exception {

        JBIEntity entity = new JBIEntity();
        entity.setName("foo0002");

        webAdminServlet.serviceAssemblies.put("foo0002", entity);

        HttpServletRequestMock request = new HttpServletRequestMock();
        request.getParameters().put("ACTION", "detail");
        request.getParameters().put("TYPE", "serviceassembly");
        request.getParameters().put("NAME", "foo0002");
        HttpServletResponseMock response = new HttpServletResponseMock();
        webAdminServlet.doGet(request, response);

        JBIEntity entity2 = (JBIEntity) request.getAttribute("entity");
        assertEquals(entity, entity2);

    }

    public void testDoGetDoDisplayJMXProperties() throws IOException {
        HttpServletRequestMock request = new HttpServletRequestMock();
        request.getParameters().put("ACTION", "chJMX");
        HttpServletResponseMock response = new HttpServletResponseMock();
        webAdminServlet.doGet(request, response);

        assertEquals(request.getAttribute("host"), "127.0.0.1");
        assertEquals(request.getAttribute("port"), "9876");
    }

    public void testDoGetDoTrashComponent() throws Exception {

        JBIEntity entity = new JBIEntity();
        entity.setName("foo001");
        entity.setObjectName(new ObjectName(
            "org.objectweb.petals:type=binding,name=foo001"));

        webAdminServlet.components.put("foo001", entity);

        HttpServletRequestMock request = new HttpServletRequestMock();
        request.getParameters().put("ACTION", "trash");
        request.getParameters().put("TYPE", "component");
        request.getParameters().put("NAME", "foo001");
        HttpServletResponseMock response = new HttpServletResponseMock();
        webAdminServlet.doGet(request, response);

        assertNull(webAdminServlet.components.get("foo001"));

    }

    public void testDoGetValidJMX() throws IOException {
        HttpServletRequestMock request = new HttpServletRequestMock();
        request.getParameters().put("ACTION", "validJMX");
        request.setParameter("host", "localhost");
        request.setParameter("port", "9988");
        request.setParameter("username", "");
        request.setParameter("password", "");

        HttpServletResponseMock response = new HttpServletResponseMock();
        webAdminServlet.doGet(request, response);

        request = new HttpServletRequestMock();
        request.getParameters().put("ACTION", "chJMX");
        webAdminServlet.doGet(request, response);

        assertEquals(request.getAttribute("host"), "localhost");
    }

    public void testDoPost() throws IOException {
        HttpServletRequestMock request = new HttpServletRequestMock();
        request.getParameters().put("ACTION", "toto");
        HttpServletResponseMock response = new HttpServletResponseMock();
        webAdminServlet.doPost(request, response);

        assertEquals(request.getAttribute("ACTION"), "");
    }

    public void testInit() {
        webAdminServlet.init();
    }

    public void testRefreshLists() throws WebAdminException {
        webAdminServlet.refreshLists();
    }

}
