/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JMXUtilTest.java 15:09:16 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.webadmin.util;

import java.io.File;
import java.io.FileInputStream;
import java.lang.reflect.Method;
import java.util.Map;
import java.util.Properties;

import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.remote.JMXConnector;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.petals.tools.webadmin.bean.JBIEntity;
import org.objectweb.petals.tools.webadmin.exception.WebAdminException;
import org.objectweb.petals.tools.webadmin.mock.MockMBeanConnection;

/**
 * Test of the JMXUtil
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class JMXUtilTest extends TestCase {

    private JMXUtil util;

    private String baseDir;

    public void setUp() throws Exception {
        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
        FileInputStream fileInputStream = new FileInputStream(baseDir
            + "src" + File.separator + "test-data" + File.separator
            + "jmx.properties");
        Properties properties = new Properties();
        properties.load(fileInputStream);
        fileInputStream = new FileInputStream(baseDir
            + "src" + File.separator + "test-data" + File.separator
            + "jmx.properties");

        util = new JMXUtil(fileInputStream);

        util = EasyMock.createMock(JMXUtil.class, new Method[] {JMXUtil.class
            .getDeclaredMethod("retrieveConnector", new Class[] {})});
        JMXConnector connector = EasyMock.createMock(JMXConnector.class);
        MockMBeanConnection connection = new MockMBeanConnection();

        EasyMock.expect(util.retrieveConnector()).andReturn(connector)
            .anyTimes();
        EasyMock.expect(connector.getMBeanServerConnection()).andReturn(
            connection).anyTimes();
        connector.close();

        EasyMock.replay(util);
        EasyMock.replay(connector);
    }

    public void testInstallComponent() throws WebAdminException {
        util.installComponent("file:"
            + baseDir + "src" + File.separator + "test-data" + File.separator
            + "clock.zip");
    }

    public void testChangeCompoState() throws WebAdminException,
        MalformedObjectNameException, NullPointerException {
        testInstallComponent();
        JBIEntity entity = new JBIEntity();
        entity.setName("foo001");
        entity.setObjectName(new ObjectName(
            "org.objectweb.petals:type=binding,name=foo001"));
        util.changeCompoState(entity, "start");
    }

    public void testDeployServiceAssembly() throws WebAdminException {
        util.deployServiceAssembly("file:"
            + baseDir + "src" + File.separator + "test-data" + File.separator
            + "sa.zip");
    }

    public void testChangeServiceAssemblyState() throws WebAdminException {
        testDeployServiceAssembly();
        JBIEntity entity = new JBIEntity();
        entity.setName("foo002");
        util.changeServiceAssemblyState(entity, "start");
    }

    public void testGetServiceAssemblyInformation() throws WebAdminException {
        testDeployServiceAssembly();
        Map<String, String> infos = util
            .getServiceAssemblyInformation("foo002");
        // The sa has no service units so it is empty
        assertEquals(infos.size(), 0);
    }

    public void testInstallSharedLibrary() throws WebAdminException {
        util.installSharedLibrary("file:"
            + baseDir + "src" + File.separator + "test-data" + File.separator
            + "sa.zip");
    }

    public void testListComponents() throws WebAdminException {
        Map<String, JBIEntity> compos = util.listComponents();
        assertEquals(compos.values().iterator().next().getName(), "foo001");
    }

    public void testListServiceAssemblies() throws WebAdminException {
        Map<String, JBIEntity> sas = util.listServiceAssemblies();
        assertEquals(sas.values().iterator().next().getName(), "foo0002");
    }

    public void testUndeployServiceAssembly() throws WebAdminException {
        util.undeployServiceAssembly("foo0002");
        Map<String, JBIEntity> sas = util.listServiceAssemblies();
        assertEquals(sas.size(), 0);
    }

    public void testUninstallCompo() throws WebAdminException,
        MalformedObjectNameException, NullPointerException {
        testInstallComponent();
        JBIEntity entity = new JBIEntity();
        entity.setName("foo001");
        entity.setObjectName(new ObjectName(
            "org.objectweb.petals:type=binding,name=foo001"));
        util.uninstallCompo(entity);
        Map<String, JBIEntity> compos = util.listComponents();
        assertEquals(compos.size(), 0);
    }

    public void testGetHost() {
        util.setHost("localhost");
        assertEquals(util.getHost(), "localhost");
    }

    public void testGetPort() {
        util.setPort("9988");
        assertEquals(util.getPort(), "9988");
    }

    public void testGetPassword() {
        util.setPassword("");
        assertEquals(util.getPassword(), "");
    }

    public void testGetUsername() {
        util.setUsername("");
        assertEquals(util.getUsername(), "");
    }

}
