/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: DeployServiceAssemblyTask.java,v 1.2 2005/07/22 10:24:27 ddesjardins Exp $
 * --------------------------------------------------------------------------  
 */
package org.objectweb.petals.tools.ant;

import java.net.URL;

import javax.management.MBeanServerConnection;
import javax.management.remote.JMXConnector;

import org.apache.tools.ant.BuildException;
import org.objectweb.petals.tools.ant.managers.InstallerAntTaskAbstract;
import org.objectweb.petals.tools.ant.util.JBIJMXConnectorUtil;

/**
 * Task used to deploy a service asembly
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class DeployServiceAssemblyTask extends InstallerAntTaskAbstract {

    public DeployServiceAssemblyTask() {
        super();
    }

    /**
     * Execute the ant task
     * 
     * @throws IOException
     */
    public void execute() throws BuildException {
        if (file == null) {
            throw new BuildException("Required attribute : file");
        }
        try {
            // Test if the provided file name is a correct URL
            new URL(file);
            JMXConnector connector = JBIJMXConnectorUtil.getConnection(host,
                    port, username, password);
            MBeanServerConnection connection = connector
                    .getMBeanServerConnection();
            Object[] objects = new Object[1];
            objects[0] = file;
            String[] strings = new String[1];
            strings[0] = "java.lang.String";
            Object result = connection.invoke(JBIJMXConnectorUtil
                    .getDeploymentServiceMBeanName(connection), "deploy",
                    objects, strings);
            if (result instanceof String) {
                // Extract the service assembly name
                String xml = (String) result;
                if (xml.indexOf("Successfully") > -1) {
                    String saName = "";
                    saName = xml.substring(xml.indexOf("<loc-param>") + 11, xml
                            .indexOf("</loc-param>"));
                    try {
                        log("Deployed service assembly name : " + saName);
                    } catch (NullPointerException e) {
                        // Exception thrown when execute outisde of the ant
                        // context
                    }
                } else {
                    try {
                        log("Problem during deployement :\n" + (String) result);
                    } catch (NullPointerException e) {
                        // Exception thrown when execute outisde of the ant
                        // context
                    }
                }
            }
            connector.close();
        } catch (Exception e) {
            if (Boolean.parseBoolean(failOnError)) {
                throw new BuildException(e.getMessage(), e.getCause());
            }
        }
    }
}
