/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 */
package org.objectweb.petals.tools.ant;

import javax.management.MBeanServerConnection;
import javax.management.remote.JMXConnector;

import org.apache.tools.ant.BuildException;
import org.objectweb.petals.tools.ant.managers.JBIAntTaskAbstract;
import org.objectweb.petals.tools.ant.util.JBIJMXConnectorUtil;

/**
 * Task used to uninstall a shared library
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class UninstallSharedLibTask extends JBIAntTaskAbstract {

    /**
     * Name of the shared library
     */
    private String name;

    public UninstallSharedLibTask() {
        super();
    }

    /**
     * Execute the ant task
     * 
     * @throws IOException
     */
    public void execute() throws BuildException {
        if (name == null) {
            throw new BuildException("Required attribute : name");
        }
        try {
            JMXConnector connector = JBIJMXConnectorUtil.getConnection(host,
                    port, username, password);
            MBeanServerConnection connection = connector
                    .getMBeanServerConnection();
            Object[] objects = new Object[1];
            objects[0] = name;
            String[] strings = new String[1];
            strings[0] = "java.lang.String";
            Object result = connection.invoke(JBIJMXConnectorUtil
                    .getInstallationServiceMBeanName(connection),
                    "uninstallSharedLibrary", objects, strings);
            if (result != null) {
                try {
                    log("Uninstalled shared library name : " + name);
                } catch (NullPointerException e) {
                    // Exception thrown when execute outisde of the ant context
                }
            }
            connector.close();
        } catch (Exception e) {
            if (Boolean.parseBoolean(failOnError)) {
                throw new BuildException(e.getMessage(), e.getCause());
            }
        }
    }

    public void setName(String name) {
        this.name = name;
    }
}
