package org.objectweb.petals.tools.ant;

import java.io.File;
import java.io.IOException;

import junit.framework.TestCase;

import org.objectweb.petals.tools.ant.InstallComponentTask;
import org.objectweb.petals.tools.ant.ListBindingComponentsTask;
import org.objectweb.petals.tools.ant.ListServiceEnginesTask;
import org.objectweb.petals.tools.ant.ShutdownComponentTask;
import org.objectweb.petals.tools.ant.StartComponentTask;
import org.objectweb.petals.tools.ant.StopComponentTask;
import org.objectweb.petals.tools.ant.UninstallComponentTask;
import org.objectweb.petals.tools.ant.util.ServerUtil;

/**
 * Test the component the ant tasks
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class ComponentTasksTest extends TestCase {

    /**
     * Port of the JMX server
     */
    private static int port = 9988;

    public void setUp() throws Exception {
        ServerUtil.start(port);
    }

    public void tearDown() throws IOException {
        ServerUtil.stop();
    }

    /**
     * Test the exception throwing
     * 
     * @throws Exception
     * 
     */
    public void testExceptionCompoUnsinstalled() throws Exception {
        testInstall();
        StartComponentTask startComponentTask = new StartComponentTask();
        startComponentTask.setPort("" + port);
        startComponentTask.setName("uninstalledCompo");
        try {
            startComponentTask.execute();
            fail();
        } catch (Exception e) {

        }
    }

    /**
     * Test the exception throwing
     * 
     */
    public void testExceptionFileInvalid() {
        InstallComponentTask installComponentTask = new InstallComponentTask();
        try {
            installComponentTask.setFile("foo");
            installComponentTask.execute();
            fail();
        } catch (Exception e) {
        }
    }

    /**
     * Test the exception throwing
     * 
     */
    public void testExceptionNameNotSet() {
        InstallComponentTask installComponentTask = new InstallComponentTask();
        try {
            installComponentTask.execute();
            fail();
        } catch (Exception e) {
        }
    }

    /**
     * Test the exception throwing
     * 
     * @throws Exception
     * 
     */
    public void testExceptionNameNull() throws Exception {
        testInstall();
        StartComponentTask startComponentTask = new StartComponentTask();
        startComponentTask.setPort("" + port);
        try {
            startComponentTask.execute();
            fail();
        } catch (Exception e) {

        }
    }

    /**
     * Test the exception throwing
     * 
     * @throws Exception
     * 
     */
    public void testExceptionNameNull1() throws Exception {
        testInstall();
        UninstallComponentTask uninstallComponentTask = new UninstallComponentTask();
        uninstallComponentTask.setPort("" + port);
        try {
            uninstallComponentTask.execute();
            fail();
        } catch (Exception e) {

        }
    }

    /**
     * Test the exception throwing
     * 
     * @throws Exception
     * 
     */
    public void testExceptionServerJMX2() throws Exception {
        testInstall();
        StartComponentTask startComponentTask = new StartComponentTask();
        startComponentTask.setPort("" + port);
        startComponentTask.setName("foo001");
        try {
            tearDown();
            startComponentTask.execute();
            fail();
        } catch (Exception e) {

        }
    }
    
    /**
     * Test the exception throwing
     * 
     * @throws Exception
     * 
     */
    public void testExceptionServerJMX3() throws Exception {
        testInstall();
        UninstallComponentTask uninstallComponentTask = new UninstallComponentTask();
        uninstallComponentTask.setPort("" + port);
        uninstallComponentTask.setName("foo001");
        try {
            tearDown();
            uninstallComponentTask.execute();
            fail();
        } catch (Exception e) {

        }
    }

    /**
     * Test the exception throwing
     * 
     */
    public void testExceptionServerNotStarted() {
        InstallComponentTask installComponentTask = new InstallComponentTask();
        try {
            tearDown();
            installComponentTask.setFile("foo");
            installComponentTask.execute();
            fail();
        } catch (Exception e) {
        }
    }

    /**
     * Test the exception throwing
     * 
     * @throws Exception
     * 
     */
    public void testExceptionServiceAssemblyNotDeployed() throws Exception {
        ListBindingComponentsTask listBindingComponentsTask = new ListBindingComponentsTask();
        listBindingComponentsTask.setPort("" + port);
        listBindingComponentsTask.setServiceAssemblyName("foo");
        listBindingComponentsTask.setBindingComponentName("bar");
//        listBindingComponentsTask.setSharedLibraryName("test");
        testInstall();
        listBindingComponentsTask.execute();
    }

    /**
     * Test the exception throwing
     * 
     */
    public void testExceptionStateInvalid() {
        ListBindingComponentsTask listBindingComponentsTask = new ListBindingComponentsTask();
        listBindingComponentsTask.setPort("" + port);
        listBindingComponentsTask.setState("fooState");
        try {
            testInstall();
            listBindingComponentsTask.execute();
            fail();
        } catch (Exception e) {
        }
    }

    /**
     * Test the exception throwing
     * 
     */
    public void testExceptionStateInvalid1() {
        ListServiceEnginesTask listServiceEnginesTask = new ListServiceEnginesTask();
        listServiceEnginesTask.setPort("" + port);
        listServiceEnginesTask.setState("InvalidState");
        try {
            testInstall();
            listServiceEnginesTask.execute();
            fail();
        } catch (Exception e) {
        }
    }

    /**
     * Test the component installation task
     * 
     * @throws Exception
     * @throws Exception
     * @throws IOException
     */
    public void testInstall() throws Exception {
        // port = ServerUtil.start(port);
        String baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":")+1);
        InstallComponentTask installComponentTask = new InstallComponentTask();
        installComponentTask.setFile("file:" + baseDir + "src" + File.separator
                + "test-data" + File.separator + "clock.zip");
        installComponentTask.setPort("" + port);
        installComponentTask.execute();
    }

    /**
     * Test the list of bindings component task
     * 
     * @throws Exception
     */
    public void testListBC() throws Exception {
        testInstall();
        ListBindingComponentsTask listBindingComponentsTask = new ListBindingComponentsTask();
        listBindingComponentsTask.setPort("" + port);
        listBindingComponentsTask.execute();
    }

    /**
     * Test the list of service engine task
     */
    public void testListSE() {
        ListServiceEnginesTask listServiceEnginesTask = new ListServiceEnginesTask();
        listServiceEnginesTask.setPort("" + port);
        listServiceEnginesTask.execute();
    }

    /**
     * Test the list of service engine task
     * 
     * @throws Exception
     */
    public void testListSE1() throws Exception {
        testInstall();
        ListServiceEnginesTask listServiceEnginesTask = new ListServiceEnginesTask();
        listServiceEnginesTask.setPort("" + port);
        listServiceEnginesTask.setServiceAssemblyName("test");
        listServiceEnginesTask.setXmlOutput("test");
//        listServiceEnginesTask.setSharedLibraryName("foo");
        listServiceEnginesTask.setServiceEngineName("bar");
        listServiceEnginesTask.execute();
    }

    /**
     * Test the list with xml output
     * 
     * @throws Exception
     * 
     */
    public void testListXMLOutput() throws Exception {
        ListBindingComponentsTask listBindingComponentsTask = new ListBindingComponentsTask();
        listBindingComponentsTask.setPort("" + port);
        listBindingComponentsTask.setXmlOutput("output");
        testInstall();
        listBindingComponentsTask.execute();
    }

    /**
     * Test the component shutdown task
     * 
     * @throws Exception
     */
    public void testShutdown() throws Exception {
        testInstall();
        ShutdownComponentTask shutdownComponentTask = new ShutdownComponentTask();
        shutdownComponentTask.setName("foo001");
        shutdownComponentTask.setPort("" + port);
        shutdownComponentTask.execute();
    }

    /**
     * Test the component starting task
     * 
     * @throws Exception
     */
    public void testStart() throws Exception {
        testInstall();
        StartComponentTask startComponentTask = new StartComponentTask();
        startComponentTask.setName("foo001");
        startComponentTask.setPort("" + port);
        startComponentTask.execute();
    }

    /**
     * Test the component stoping task
     * 
     * @throws Exception
     */
    public void testStop() throws Exception {
        testInstall();
        StopComponentTask stopComponentTask = new StopComponentTask();
        stopComponentTask.setName("foo001");
        stopComponentTask.setPort("" + port);
        stopComponentTask.execute();
    }

    /**
     * Test the component uninstallation task
     * 
     * @throws Exception
     */
    public void testUninstall() throws Exception {
        testInstall();
        UninstallComponentTask uninstallComponentTask = new UninstallComponentTask();
        uninstallComponentTask.setName("foo001");
        uninstallComponentTask.setPort("" + port);
        uninstallComponentTask.execute();
    }
    
    /**
     * Test the component starting task
     * 
     * @throws Exception
     */
    public void testStart1() throws Exception {
        testInstall();
        StartComponentTask startComponentTask = new StartComponentTask();
        startComponentTask.setName("foo001");
        startComponentTask.setPort("" + port);
        startComponentTask.setHost("localhost");
        startComponentTask.setFailOnError("true");
        startComponentTask.setUsername("");
        startComponentTask.setPassword("");
        startComponentTask.execute();
    }

}