package org.objectweb.petals.tools.ant;

import java.io.File;
import java.io.IOException;

import junit.framework.TestCase;

import org.objectweb.petals.tools.ant.DeployServiceAssemblyTask;
import org.objectweb.petals.tools.ant.ListServiceAssembliesTask;
import org.objectweb.petals.tools.ant.ShutdownServiceAssemblyTask;
import org.objectweb.petals.tools.ant.StartServiceAssemblyTask;
import org.objectweb.petals.tools.ant.StopServiceAssemblyTask;
import org.objectweb.petals.tools.ant.UndeployServiceAssemblyTask;
import org.objectweb.petals.tools.ant.util.ServerUtil;

/**
 * Test the service assemlies ant tasks
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class ServiceAssembliesTasksTest extends TestCase {

    /**
     * Port of the JMX server
     */
    private static int port = 9988;

    public void setUp() throws Exception {
        ServerUtil.start(port);
    }

    public void tearDown() throws IOException {
        ServerUtil.stop();
    }

    /**
     * Test the deploy of a service assembly task
     * 
     * @throws Exception
     */
    public void testDeploy() {
        String baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":")+1);
        DeployServiceAssemblyTask deployServiceAssemblyTask = new DeployServiceAssemblyTask();
        deployServiceAssemblyTask.setFile("file:" + baseDir + "src"
                + File.separator + "test-data" + File.separator + "sa.zip");
        deployServiceAssemblyTask.setPort("" + port);
        deployServiceAssemblyTask.execute();
    }

    /**
     * Test the exception throwing
     * 
     */
    public void testExceptionDeployUnsuccessfull() {
        DeployServiceAssemblyTask deployServiceAssemblyTask = new DeployServiceAssemblyTask();
        String baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":")+1);
        deployServiceAssemblyTask.setFile("file:" + baseDir + "src"
                + File.separator + "test-data" + File.separator
                + "sa-unsuccess.zip");
        deployServiceAssemblyTask.setPort("" + port);
        deployServiceAssemblyTask.execute();
    }

    /**
     * Test the exception throwing
     * 
     */
    public void testExceptionFileNull() {
        DeployServiceAssemblyTask deployServiceAssemblyTask = new DeployServiceAssemblyTask();
        deployServiceAssemblyTask.setPort("" + port);
        try {
            deployServiceAssemblyTask.execute();
            fail();
        } catch (Exception e) {

        }
    }

    /**
     * Test the exception throwing
     * 
     */
    public void testExceptionInvalidState() {
        ListServiceAssembliesTask listServiceAssembliesTask = new ListServiceAssembliesTask();
        listServiceAssembliesTask.setPort("" + port);
        listServiceAssembliesTask.setState("InvalidState");
        try {
            listServiceAssembliesTask.execute();
            fail();
        } catch (Exception e) {
        }
    }

    /**
     * Test the exception throwing
     * 
     */
    public void testExceptionProblemServerJMX() {
        DeployServiceAssemblyTask deployServiceAssemblyTask = new DeployServiceAssemblyTask();
        String baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":")+1);
        deployServiceAssemblyTask.setFile("file:" + baseDir + "src"
                + File.separator + "test-data" + File.separator + "sa.zip");
        deployServiceAssemblyTask.setPort("" + port);
        try {
            tearDown();
            deployServiceAssemblyTask.execute();
            fail();
        } catch (Exception e) {

        }
    }

    /**
     * Test the exception throwing
     * 
     */
    public void testExceptionXMLOutput() {
        ListServiceAssembliesTask listServiceAssembliesTask = new ListServiceAssembliesTask();
        listServiceAssembliesTask.setPort("" + port);
        listServiceAssembliesTask.setXmlOutput("output");
        listServiceAssembliesTask.execute();
    }

    /**
     * Test the exception throwing
     * 
     */
    public void testExceptionNameNull() {
        StartServiceAssemblyTask startServiceAssemblyTask = new StartServiceAssemblyTask();
        startServiceAssemblyTask.setPort("" + port);
        try {
            startServiceAssemblyTask.execute();
            fail();
        } catch (Exception e) {

        }
    }

    /**
     * Test the exception throwing
     * 
     */
    public void testExceptionServerJMX() {
        testDeploy();
        StartServiceAssemblyTask startServiceAssemblyTask = new StartServiceAssemblyTask();
        startServiceAssemblyTask.setPort("" + port);
        startServiceAssemblyTask.setName("unsuccessSA");
        startServiceAssemblyTask.execute();
    }

    /**
     * Test the list of service assemblies task
     */
    public void testList() {
        testDeploy();
        ListServiceAssembliesTask listServiceAssembliesTask = new ListServiceAssembliesTask();
        listServiceAssembliesTask.setPort("" + port);
        listServiceAssembliesTask.execute();
    }

    /**
     * Test the list of service assemblies task
     */
    public void testList1() {
        testDeploy();
        ListServiceAssembliesTask listServiceAssembliesTask = new ListServiceAssembliesTask();
        listServiceAssembliesTask.setPort("" + port);
        listServiceAssembliesTask.setComponentName("foo");
        listServiceAssembliesTask.execute();
    }

    /**
     * Test the shutdown of a service assembly task
     */
    public void testShutdown() {
        testDeploy();
        ShutdownServiceAssemblyTask shutdownServiceAssemblyTask = new ShutdownServiceAssemblyTask();
        shutdownServiceAssemblyTask.setName("foo0002");
        shutdownServiceAssemblyTask.setPort("" + port);
        shutdownServiceAssemblyTask.execute();
    }

    /**
     * Test the start of a service assembly task
     */
    public void testStart() {
        testDeploy();
        StartServiceAssemblyTask startServiceAssemblyTask = new StartServiceAssemblyTask();
        startServiceAssemblyTask.setName("foo0002");
        startServiceAssemblyTask.setPort("" + port);
        startServiceAssemblyTask.execute();
    }

    /**
     * Test the stop of a service assembly task
     */
    public void testStop() {
        testDeploy();
        StopServiceAssemblyTask stopServiceAssemblyTask = new StopServiceAssemblyTask();
        stopServiceAssemblyTask.setName("foo0002");
        stopServiceAssemblyTask.setPort("" + port);
        stopServiceAssemblyTask.execute();
    }

    /**
     * Test the undeployment of a service assembly task
     */
    public void testUndeploy() {
        testDeploy();
        UndeployServiceAssemblyTask undeployServiceAssemblyTask = new UndeployServiceAssemblyTask();
        undeployServiceAssemblyTask.setName("foo0002");
        undeployServiceAssemblyTask.setPort("" + port);
        undeployServiceAssemblyTask.execute();
    }
}
