/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ServerUtil.java 13:57:33 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.ant.util;

import java.io.IOException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.util.Hashtable;

import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.ObjectName;
import javax.management.remote.JMXConnectorServer;
import javax.management.remote.JMXConnectorServerFactory;
import javax.management.remote.JMXServiceURL;
import javax.management.remote.rmi.RMIConnectorServer;

/**
 * Start a JMX server
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class ServerUtil {
    
    private final static String PETALS_DOMAIN = "Petals";
    
    private final static String TYPE = "type";
    
    private final static String NAME = "name";
    
    private final static String SERVICE_TYPE = "service";
    
    private final static String ADMIN_NAME = "Admin";
    
    private final static String INSTALLATION_NAME = "Installation";
    
    private final static String DEPLOYMENT_NAME = "Deployment";

	/**
     * Boolean to know if the server is started
     */
	public static boolean isStarted = false;

	/**
     * JMX connector
     */
	private static JMXConnectorServer cs;

	/**
     * Start the JMX server
     * 
     * @param port port of the server
     * @throws Exception
     */
	public static int start(int port) throws Exception {
		if (isStarted == false) {
			Registry registry = LocateRegistry.getRegistry(port);
			try {
				registry.list();
			}
			catch (Exception e){
				LocateRegistry.createRegistry(port);
			}            
			MBeanServer beanServer = MBeanServerFactory.newMBeanServer();
            
            Hashtable<String, String> attributes = new Hashtable<String, String>();
            attributes.put(NAME, DEPLOYMENT_NAME);
            attributes.put(TYPE, SERVICE_TYPE);
			beanServer.registerMBean(new DeploymentServiceMoc(beanServer), new ObjectName(PETALS_DOMAIN, attributes));
            
            attributes.clear();
            attributes.put(NAME, INSTALLATION_NAME);
            attributes.put(TYPE, SERVICE_TYPE);
			beanServer.registerMBean(new InstallationServiceMoc(beanServer), new ObjectName(PETALS_DOMAIN, attributes));
            
            attributes.clear();
            attributes.put(NAME, ADMIN_NAME);
            attributes.put(TYPE, SERVICE_TYPE);
			beanServer.registerMBean(new AdminServiceMoc(beanServer), new ObjectName(PETALS_DOMAIN, attributes));
            
			String sJmxURL = "service:jmx:rmi:///jndi/rmi://127.0.0.1:" + port + "/management/rmi-jmx-connector";
			JMXServiceURL jmxUrl = new JMXServiceURL(sJmxURL);
			RMIConnectorServer connectorServer = (RMIConnectorServer) JMXConnectorServerFactory.newJMXConnectorServer(jmxUrl, null, beanServer);
			cs = connectorServer;
			cs.start();
			// Start the server
			isStarted = true;
		}
		return port;
	}

	/**
     * Stop the JMX
     * 
     * @throws IOException Server
     * 
     */
	public static void stop() throws IOException {
		if (isStarted) {
			isStarted = false;
			cs.stop();
		}
	}
}
