/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: PackageAssemblyMojo.java 16:56:43 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.coberturareportplugin;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import org.apache.maven.doxia.siterenderer.Renderer;
import org.apache.maven.project.MavenProject;
import org.apache.maven.reporting.AbstractMavenReport;
import org.apache.maven.reporting.MavenReportException;

/**
 * Generate a summary of cobertura ser files
 * 
 * @goal report
 * @phase verify
 * @description Generate a Cobertura report
 * @aggregator false
 * @requiresOnline false
 * @author ddesjardins - eBMWebsourcing
 */
public class CoberturaReportPluginReport extends AbstractMavenReport {

    /**
     * Base directory
     * 
     * @parameter expression="${basedir}"
     * @required
     */
    protected String baseDir;

    /**
     * Datafile
     * 
     * @parameter expression="${basedir}/target/cobertura.ser"
     * @required
     * @editable false
     */
    protected String dataFile;

    /**
     * Destination directory
     * 
     * @parameter expression="${project.reporting.outputDirectory}/cobertura-sum"
     * @required
     * @editable false
     */
    protected String destination;

    /**
     * The Maven Project.
     * 
     * @parameter expression="${project}"
     * @required
     * @readonly
     */
    protected MavenProject project;

    /**
     * Specifies the directory where the report will be generated
     * 
     * @parameter default-value="${project.reporting.outputDirectory}"
     * @required
     */
    protected File outputDirectory;

    /**
     * @component
     * @required
     * @readonly
     */
    protected Renderer siteRenderer;

    public String getDescription(Locale arg0) {
        return "Cobertura Summary Report Generator Task";
    }

    public String getName(Locale arg0) {
        return "CSRG Task";
    }

    public String getOutputName() {
        return "CSRG Task";
    }

    protected void executeReport(Locale arg0) throws MavenReportException {
        String destination = outputDirectory.getAbsolutePath() + File.separator
            + "cobertura-sum";
        String dataFile = baseDir + File.separator + "cobertura.ser";

        if (project.getModules().size() > 0) {
            new File(dataFile).delete();
            List<String> args = new ArrayList<String>();
            args.add("--datafile");
            args.add(dataFile);
            args.add("--basedir");
            args.add(baseDir);
            for (Object obj : project.getModules()) {
                args.add((String) obj + File.separator + "cobertura.ser");
            }
            System.out.println("Cobertura merge report to " + dataFile);
            try {
                net.sourceforge.cobertura.merge.Main.main(args
                    .toArray(new String[0]));
            } catch (Exception e) {
                // Do nothing
            }
            System.out.println("Merge done.");
            if (new File(dataFile).exists()) {
                System.out.println("Generate Cobertura report " + dataFile
                    + " to " + destination);
                args = new ArrayList<String>();
                args.add("--datafile");
                args.add(dataFile);
                args.add("--destination");
                args.add(destination);
                args.add("--format");
                args.add("html");
                for (Object obj : project.getModules()) {
                    String srcDir = baseDir + File.separator + (String) obj
                        + File.separator + "src" + File.separator + "java";
                    args.add(srcDir);
                }
                try {
                    net.sourceforge.cobertura.reporting.Main.main(args
                        .toArray(new String[0]));
                } catch (Exception e) {
                    // Do nothing
                }
                System.out.println("Generate done.");
            }
        }
    }

    protected String getOutputDirectory() {
        return outputDirectory.getAbsolutePath();
    }

    protected MavenProject getProject() {
        return project;
    }

    protected Renderer getSiteRenderer() {
        return (Renderer) siteRenderer;
    }

}
