/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: CoberturaReportPluginReportTest.java 11:31:42 AM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.coberturareportplugin;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import org.apache.commons.io.FileUtils;
import org.apache.maven.project.MavenProject;
import org.apache.maven.reporting.MavenReportException;
import org.easymock.classextension.EasyMock;

import junit.framework.TestCase;

/**
 * Tests of CoberturaReportPluginReport
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class CoberturaReportPluginReportTest extends TestCase {

	public void testExecuteReport() throws MavenReportException, IOException {
		CoberturaReportPluginReport pluginReport = new CoberturaReportPluginReport();

		String baseDir = this.getClass().getResource(".").toString();
		baseDir = baseDir.substring(0, baseDir.indexOf("target"));
		baseDir = baseDir.substring(baseDir.indexOf(":") + 1);

		MavenProject project = EasyMock.createMock(MavenProject.class);
		List<String> modules = new ArrayList<String>();
		modules.add("test");

		EasyMock.expect(project.getModules()).andReturn(modules).anyTimes();

		EasyMock.replay(project);

		new File(baseDir + "target" + File.separator + "test").mkdir();
		FileUtils.copyFileToDirectory(new File(baseDir + "src" + File.separator
				+ "test-data" + File.separator + "cobertura.ser"), new File(
				baseDir + "target" + File.separator + "test"));

		pluginReport.outputDirectory = new File(baseDir + "target");
		pluginReport.project = project;
		pluginReport.baseDir = baseDir + "target";
		pluginReport.executeReport(Locale.FRANCE);

		assertTrue(new File(baseDir + "target" + File.separator
				+ "cobertura-sum" + File.separator + "index.html").exists());
		assertEquals(pluginReport.getProject(), project);
	}

	public void testGetters() {
		CoberturaReportPluginReport pluginReport = new CoberturaReportPluginReport();

		String baseDir = this.getClass().getResource(".").toString();
		baseDir = baseDir.substring(0, baseDir.indexOf("target"));
		baseDir = baseDir.substring(baseDir.indexOf(":") + 1);

		pluginReport.outputDirectory = new File(baseDir + "target");

		assertEquals(pluginReport.getDescription(Locale.FRANCE),
				"Cobertura Summary Report Generator Task");
		assertEquals(pluginReport.getName(Locale.FRANCE), "CSRG Task");
		assertEquals(pluginReport.getOutputName(), "CSRG Task");
		assertEquals(pluginReport.getOutputDirectory(), new File(baseDir
				+ "target").getAbsolutePath());
		assertNull(pluginReport.getSiteRenderer());
	}
}
