/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.jbicommon.descriptor;

import java.net.URI;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.w3c.dom.DocumentFragment;

/**
 * 
 * @author ofabre
 * 
 */
public class Extensions {

    private DocumentFragment documentFragment;

    private Map<URI, PetalsExtension> petalsExtensions;

    private Map<URI, UnknownExtension> unknownsExtensions;

    public Extensions() {
        super();
        petalsExtensions = new HashMap<URI, PetalsExtension>();
        unknownsExtensions = new HashMap<URI, UnknownExtension>();
    }

    public void addPetalsExtension(final URI uri,
            final PetalsExtension extension) {
        petalsExtensions.put(uri, extension);
    }

    public void addUnknownExtension(final URI uri,
            final UnknownExtension extension) {
        unknownsExtensions.put(uri, extension);
    }

    @Override
    public boolean equals(final Object other) {
        if (!(other instanceof Extensions)) {
            return false; // NOPMD by gblondelle
        }
        Extensions castOther = (Extensions) other;
        return new EqualsBuilder().append(petalsExtensions,
                castOther.petalsExtensions).append(unknownsExtensions,
                castOther.unknownsExtensions).isEquals();
    }

    public DocumentFragment getDocumentFragment() {
        return documentFragment;
    }

    public Map<URI, PetalsExtension> getPetalsExtensions() {
        return petalsExtensions;
    }

    public Map<URI, UnknownExtension> getUnknownsExtensions() {
        return unknownsExtensions;
    }

    @Override
    public int hashCode() {
        return new HashCodeBuilder().append(petalsExtensions).append(
                unknownsExtensions).toHashCode();
    }

    public void setDocumentFragment(final DocumentFragment documentFragment) {
        this.documentFragment = documentFragment;
    }

    public void setPetalsExtensions(
            final Map<URI, PetalsExtension> petalsExtensions) {
        this.petalsExtensions = petalsExtensions;
    }

    public void setUnknownsExtensions(
            final Map<URI, UnknownExtension> unknownsExtensions) {
        this.unknownsExtensions = unknownsExtensions;
    }

    @Override
    public String toString() {
        return new ToStringBuilder(this).append("petalsExtensions",
                petalsExtensions).append("unknownsExtensions",
                unknownsExtensions).toString();
    }

}
