/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.jbicommon.descriptor;

import java.net.URI;
import java.net.URISyntaxException;

/**
 * Constants for Petals extensions uris.
 * 
 * @author ofabre
 * 
 */
public enum PetalsExtensionsUris {
    PETALS_EXTENSIONS("http://petals.objectweb.org/extensions/"),
    PETALS_EXTENSION_KEY_VALUE(
            "http://petals.objectweb.org/extensions/key-value/");

    private final String nameSpace;

    private PetalsExtensionsUris(String nameSpace) {
        this.nameSpace = nameSpace;
    }

    /**
     * Returns an URI matching the related {@link PetalsExtensionsUris}
     * 
     * @return the URI linked to the related {@link PetalsExtensionsUris}
     */
    public URI value() {
        URI extURI = null;
        try {
            extURI = new URI(nameSpace);
        } catch (URISyntaxException e) {
            throw new Error("Unexpected Error in JBI URI namespace syntax", e); // NOPMD
        }
        return extURI;
    }

    /**
     * Returns a {@link PetalsExtensionsUris} matching the given URI pattern
     * 
     * @param pattern
     *            the pattern to identify
     * @return the {@link PetalsExtensionsUris} matching the given pattern or
     *         null.
     */
    public static PetalsExtensionsUris valueOf(final URI pattern) {
        PetalsExtensionsUris result = null;
        for (PetalsExtensionsUris extURI : PetalsExtensionsUris.values()) {
            if (extURI.nameSpace.equals(pattern.toString())) {
                result = extURI;
            }
        }
        return result;
    }
}
