/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JBIDescriptorBuilderTest.java 632 2006-06-22 07:28:26Z ddesjardins $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.jbicommon.descriptor;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Method;
import java.net.URI;
import java.util.logging.Logger;

import javax.xml.transform.TransformerException;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.petals.tools.jbicommon.util.XMLUtil;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

/**
 * @version $Rev: 632 $ $Date: 2006-06-22 07:28:26Z $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 * @author ddesjardins - eBMWebsourcing
 */
public class JBIDescriptorBuilderTest extends TestCase { // NOPMD by ofabre

    private static final String TARGET = "target";

    private String baseDir;

    public void setUp() {
        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf(TARGET));
        baseDir = baseDir.substring(baseDir.indexOf(':') + 1);
    }

    // public void testExtractXmlAttributeQName() throws SecurityException,
    // NoSuchMethodException {
    // JBIDescriptorBuilder builder = EasyMock.createMock(
    // JBIDescriptorBuilder.class,
    // new Method[] {JBIDescriptorBuilder.class.getDeclaredMethod(
    // "buildDescriptor", new Class[] {Document.class})});
    //
    // Node node = EasyMock.createMock(Node.class);
    // NamedNodeMap nodeMap = EasyMock.createMock(NamedNodeMap.class);
    // Node outNode = EasyMock.createMock(Node.class);
    //
    // EasyMock.expect(node.getAttributes()).andReturn(nodeMap);
    // EasyMock.expect(nodeMap.getNamedItem("test")).andReturn(outNode);
    // EasyMock.expect(outNode.getNodeValue()).andReturn("out");
    //
    // EasyMock.replay(nodeMap);
    // EasyMock.replay(outNode);
    // EasyMock.replay(node);
    //
    // assertEquals(QName.valueOf("out"), builder.extractXmlAttributeQName(
    // node, "test"));
    // }

    // public void testExtractXmlAttributeQNameWithNS() throws
    // SecurityException,
    // NoSuchMethodException {
    // JBIDescriptorBuilder builder = EasyMock.createMock(
    // JBIDescriptorBuilder.class,
    // new Method[] {JBIDescriptorBuilder.class.getDeclaredMethod(
    // "buildDescriptor", new Class[] {Document.class})});
    //
    // Node node = EasyMock.createMock(Node.class);
    // NamedNodeMap nodeMap = EasyMock.createMock(NamedNodeMap.class);
    // Node outNode = EasyMock.createMock(Node.class);
    // Node nsNode = EasyMock.createMock(Node.class);
    //
    // EasyMock.expect(node.getAttributes()).andReturn(nodeMap).anyTimes();
    // EasyMock.expect(nodeMap.getNamedItem("test")).andReturn(outNode);
    // EasyMock.expect(outNode.getNodeValue()).andReturn("ns:out");
    // EasyMock.expect(node.getParentNode()).andReturn(null).anyTimes();
    // EasyMock.expect(nodeMap.getLength()).andReturn(1);
    // EasyMock.expect(nodeMap.item(0)).andReturn(nsNode);
    // EasyMock.expect(nsNode.getNodeName()).andReturn("xmlns:ns");
    // EasyMock.expect(nsNode.getNodeValue()).andReturn("outvalue");
    //
    // EasyMock.replay(nsNode);
    // EasyMock.replay(nodeMap);
    // EasyMock.replay(outNode);
    // EasyMock.replay(node);
    //
    // assertEquals(QName.valueOf("{outvalue}out"), builder
    // .extractXmlAttributeQName(node, "test"));
    // }

    public void testParseJbiXml() throws SecurityException,
            NoSuchMethodException, SAXException, IOException {
        File jbiFile = new File(baseDir + "src" + File.separator + "test-data"
                + File.separator + "descriptors" + File.separator
                + "emptyDescriptor.xml");

        JBIDescriptorBuilder builder = EasyMock.createMock(
                JBIDescriptorBuilder.class, new Method[] {
                        JBIDescriptorBuilder.class.getDeclaredMethod(
                                "validateJbiXml", new Class[] { URI.class }),
                        JBIDescriptorBuilder.class.getDeclaredMethod(
                                "buildDescriptor",
                                new Class[] { Document.class }) });

        try {
            assertNotNull(builder.parseJbiXml(jbiFile.toURI()));
        } catch (JBIDescriptorException e) {
            e.printStackTrace();
        }
    }

    public void testValidateJbiXml() throws SecurityException,
            NoSuchMethodException, SAXException, IOException {
        File jbiFile = new File(baseDir + "src" + File.separator + "test-data"
                + File.separator + "descriptors" + File.separator
                + "componentInstallationDescriptor.xml");

        JBIDescriptorBuilder builder = EasyMock.createMock(
                JBIDescriptorBuilder.class,
                new Method[] { JBIDescriptorBuilder.class.getDeclaredMethod(
                        "buildDescriptor", new Class[] { Document.class }) });
        Logger logger = EasyMock.createMock(Logger.class);

        EasyMock.replay(logger);

        builder.setLog(logger);

        assertTrue(builder.validateJbiXml(jbiFile.toURI()));
    }

    public void testValidateJbiXmlException() throws SecurityException,
            NoSuchMethodException, SAXException, IOException {
        File jbiFile = new File(baseDir + "src" + File.separator + "test-data"
                + File.separator + "descriptors" + File.separator
                + "badDescriptor.xml");

        JBIDescriptorBuilder builder = EasyMock.createMock(
                JBIDescriptorBuilder.class,
                new Method[] { JBIDescriptorBuilder.class.getDeclaredMethod(
                        "buildDescriptor", new Class[] { Document.class }) });
        Logger logger = EasyMock.createMock(Logger.class);

        builder.setLog(logger);

        assertFalse(builder.validateJbiXml(jbiFile.toURI()));
    }

    public void testComponentDescriptorBuild() {
        File jbiFile = new File(baseDir + "src" + File.separator + "test-data"
                + File.separator + "descriptors" + File.separator
                + "componentInstallationDescriptor.xml");
        Logger logger = EasyMock.createMock(Logger.class);
        JBIDescriptorBuilder descriptorBuilder = new JBIDescriptorBuilder(
                jbiFile.toURI(), logger);

        try {
            JBIDescriptor generatedDescriptor = descriptorBuilder.build();
            System.out.println("###### A Component Descriptor");
            System.out.println(generatedDescriptor);
            System.out.println(generatedDescriptor.getComponent()
                    .getExtensions());
            System.out.println(generatedDescriptor.getComponent()
                    .getIdentification().getExtensions());
            System.out.println(XMLUtil
                    .createStringFromDOMDocument(generatedDescriptor
                            .getComponent().getExtensions()
                            .getDocumentFragment()));
        } catch (JBIDescriptorException e) {
            e.printStackTrace();
        } catch (TransformerException e) {
            e.printStackTrace();
        }

    }

    public void testSharedLibDescriptorBuild() {
        File jbiFile = new File(baseDir + "src" + File.separator + "test-data"
                + File.separator + "descriptors" + File.separator
                + "sharedLibraryInstallationDescriptor.xml");
        Logger logger = EasyMock.createMock(Logger.class);
        JBIDescriptorBuilder descriptorBuilder = new JBIDescriptorBuilder(
                jbiFile.toURI(), logger);

        try {
            JBIDescriptor generatedDescriptor = descriptorBuilder.build();
            System.out.println("###### A Shared Lib Descriptor");
            System.out.println(generatedDescriptor);
            System.out.println(generatedDescriptor.getSharedLibrary()
                    .getIdentification().getExtensions());
        } catch (JBIDescriptorException e) {
            e.printStackTrace();
        }

    }

    public void testServiceAssemblyDescriptorBuild() {
        File jbiFile = new File(baseDir + "src" + File.separator + "test-data"
                + File.separator + "descriptors" + File.separator
                + "serviceAssemblyDescriptor.xml");
        Logger logger = EasyMock.createMock(Logger.class);
        JBIDescriptorBuilder descriptorBuilder = new JBIDescriptorBuilder(
                jbiFile.toURI(), logger);

        try {
            JBIDescriptor generatedDescriptor = descriptorBuilder.build();
            System.out.println("###### A Service Assembly Descriptor");
            System.out.println(generatedDescriptor);
            System.out.println(generatedDescriptor.getServiceAssembly()
                    .getExtensions());
            System.out.println(generatedDescriptor.getServiceAssembly()
                    .getServiceUnits().get(1).getExtensions());
            System.out.println(generatedDescriptor.getServiceAssembly()
                    .getIdentification().getExtensions());
        } catch (JBIDescriptorException e) {
            e.printStackTrace();
        }
    }

    public void testServicesDescriptorBuild() {
        File jbiFile = new File(baseDir + "src" + File.separator + "test-data"
                + File.separator + "descriptors" + File.separator
                + "servicesDescriptor.xml");
        Logger logger = EasyMock.createMock(Logger.class);
        JBIDescriptorBuilder descriptorBuilder = new JBIDescriptorBuilder(
                jbiFile.toURI(), logger);

        try {
            JBIDescriptor generatedDescriptor = descriptorBuilder.build();
            System.out.println("###### A Services Descriptor");
            System.out.println(generatedDescriptor);
            System.out.println(generatedDescriptor.getServices().getConsumes()
                    .get(0).getExtensions());
            System.out.println(generatedDescriptor.getServices().getProvides()
                    .get(0).getExtensions());
        } catch (JBIDescriptorException e) {
            e.printStackTrace();
        }
    }
}
