/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: XMLUtilTest.java 17:13:00 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.util;

import java.io.File;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import junit.framework.TestCase;

import org.objectweb.petals.tools.jbicommon.util.XMLUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * Test the XMLUtil
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class XMLUtilTest extends TestCase {

    private Document document;

    public void setUp() throws Exception {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder = factory.newDocumentBuilder();
        String baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":")+1);
        document = builder.parse(new File(baseDir + "src" + File.separator
                + "test-data" + File.separator + "Axis2BCService.wsdl"));
    }

    public void testCreateAttribute() {
        Node att = XMLUtil.createAttribute(document, "test", "test");
        assertNotNull(att);
    }

    public void testCreateNode() {
        Node att = XMLUtil.createNode(document, "test", "test-att", "test-val");
        assertNotNull(att);
    }

    public void testGetAttributeValue() {
        Node att = XMLUtil.createNode(document, "test", "test-att", "test-val");
        String attVal = XMLUtil.getAttributeValue(att, "test-att");
        assertNotNull(attVal);
    }

    public void testGetAttributeValue1() {
        Node att = XMLUtil.createNode(document, "test", "test-att", "test-val");
        String attVal = XMLUtil.getAttributeValue(att, "test");
        assertNull(attVal);
    }

    public void testGetAttributeValue2() {
        String attVal = XMLUtil.getAttributeValue(null, "test");
        assertNull(attVal);
    }

    public void testGetFirstChild() {
        Node fc = XMLUtil.getFirstChild(document.getFirstChild());
        assertNotNull(fc);
    }

    public void testGetNextSibling() {
        Node ns = XMLUtil.getNextSibling(document.getFirstChild()
                .getFirstChild());
        assertNotNull(ns);
    }

//    public void testGetNode() {
//        Node ns = XMLUtil.getNode(document, "*/types");
//        assertNotNull(ns);
//    }
//    
//    public void testGetNode1() {
//        Node ns = XMLUtil.getNode(document, "*/test");
//        assertNull(ns);
//    }

    public void testGetTextContent() {
        Node ns = XMLUtil.getNextSibling(document.getFirstChild()
                .getFirstChild());
        String text = XMLUtil.getTextContent(ns);
        assertNotNull(text);
    }


    public void testGetTextContents() {
        List<String> texts = XMLUtil.getTextContents(document.getChildNodes());
        assertNotNull(texts);
    }

}
