/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: PackageAssemblyMojo.java 16:56:43 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.jbiplugin;

import javax.jbi.management.LifeCycleMBean;
import javax.management.MBeanException;
import javax.management.MBeanServerConnection;
import javax.management.remote.JMXConnector;

import org.objectweb.petals.tools.jbiplugin.util.JBIJMXConnectorUtil;

/**
 * This class gathers the action on a service assembly
 * 
 * @author cdeneux - Capgemini Sud
 */
public abstract class JMXServiceAssemblyAbstractMojo extends JMXTaskAbstractMojo {

	/**
     * Action : shutdown a service assembly
     */
    protected static final String SHUTDOWN = "shutDown";

    /**
     * Action : start a service assembly
     */
    protected static final String START = "start";

    /**
     * Action : stop a service assembly
     */
    protected static final String STOP = "stop";

    /**
     * Action : deploy a service assembly
     */
    protected static final String DEPLOY = "deploy";

    /**
     * Action : undeploy a service assembly
     */
    protected static final String UNDEPLOY = "undeploy";

    /**
	 * Action : get state of a component
	 */
	protected static final String GET_STATE = "getState";

	/**
	 * Get state of a service assembly
	 * 
	 * @param serviceAssemblyName
	 *            service assembly
	 */
	protected String getState(String serviceAssemblyName) throws Exception {
		JMXConnector connector = null;
		String state = null;
		try {
			connector = JBIJMXConnectorUtil.getConnection(host, port, username,
					password);
			MBeanServerConnection connection = connector
					.getMBeanServerConnection();
			Object[] objects = new Object[1];
			objects[0] = serviceAssemblyName;
			String[] strings = new String[1];
			strings[0] = "java.lang.String";
			Object result = connection.invoke(JBIJMXConnectorUtil
					.getDeploymentServiceMBeanName(connection),
					JMXServiceAssemblyAbstractMojo.GET_STATE, objects,
					strings);
			if (result instanceof String) {
				state = (String)result;
			} else if (result instanceof Exception) {
				state = LifeCycleMBean.UNKNOWN;
			}
			
		} catch (MBeanException e) {
			state = LifeCycleMBean.UNKNOWN;
			
		} catch (Exception e) {
			if (failOnError) {
				throw e;
			}
		} finally {
			connector.close();
		}
		
		System.out.println("Service Assembly: "
				+ serviceAssemblyName + " is in state: " + state);
		return state;
		
	}
	
	/**
     * Perform an action on a service assembly
     * @param action
     */
    protected void performAction(String action, String serviceAssembly) throws Exception {
        try {
        	if (verbose) {
                System.out.println("Start operation [" + action + "] on JBI Service Assembly archive: "
                    + serviceAssembly);
            }
        	
        	JMXConnector connector = JBIJMXConnectorUtil.getConnection(host,
                    port, username, password);
            MBeanServerConnection connection = connector
                    .getMBeanServerConnection();
            Object[] objects = new Object[1];
            objects[0] = serviceAssembly;
            String[] strings = new String[1];
            strings[0] = "java.lang.String";
            Object result = connection.invoke(JBIJMXConnectorUtil
                    .getDeploymentServiceMBeanName(connection), action,
                    objects, strings);
            if (result instanceof String) {
                // Extract the service assembly name
                String xml = (String) result;
                if (xml.indexOf("Successfully") > -1) {
                    String saName = "";
                    saName = xml.substring(xml.indexOf("<loc-param>") + 11, xml
                            .indexOf("</loc-param>"));
                    System.out.println("Operation succeeded on service assembly named : " + saName);
                } else {
                    System.out.println("Problem during the " + action + " :\n"
                                + (String) result);
                }
            }
            connector.close();
        } catch (Exception e) {
            if (failOnError) {
                throw e;
            }
        }
    }
}
