package petalsclipse.builder;

import java.io.File;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.console.MessageConsole;

import petalsclipse.builder.util.BuildJob;
import petalsclipse.builder.util.ConsoleFinderUtil;

public class BuildWithTestsAction implements IObjectActionDelegate {

    private ISelection selection;

    public static final String BUILD_WITH_TESTS = "BuildWithTestsAction";

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
     */
    public void run(IAction action) {
        if (selection instanceof IStructuredSelection) {
            for (Iterator it = ((IStructuredSelection) selection).iterator(); it
                    .hasNext();) {
                Object element = it.next();
                IProject project = null;
                if (element instanceof IProject) {
                    project = (IProject) element;
                } else if (element instanceof IAdaptable) {
                    project = (IProject) ((IAdaptable) element)
                            .getAdapter(IProject.class);
                }
                if (project != null) {
                    buildWithTests(project);
                }
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction,
     *      org.eclipse.jface.viewers.ISelection)
     */
    public void selectionChanged(IAction action, ISelection selection) {
        this.selection = selection;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IObjectActionDelegate#setActivePart(org.eclipse.jface.action.IAction,
     *      org.eclipse.ui.IWorkbenchPart)
     */
    public void setActivePart(IAction action, IWorkbenchPart targetPart) {
    }

    /**
     * Toggles sample nature on a project
     * 
     * @param project
     *            to have sample nature added or removed
     */
    private void buildWithTests(IProject project) {
        MessageConsole messageConsole = ConsoleFinderUtil
                .findConsole("Petals : " + project.getName()
                        + " -> Build with tests (" + new Date() + ")");
        messageConsole.activate();
        try {
            String command = "mvn";
            String os = System.getProperty("os.name");
            if (os.toLowerCase().startsWith("windows")) {
                command = "mvn.bat";
            }
            ProcessBuilder processBuilder = new ProcessBuilder(command, "-B");
            Map<String, String> env = processBuilder.environment();
            if (env.get("JAVA_HOME") == null) {
                String javaHome = System.getProperty("java.home");
                if (os.toLowerCase().startsWith("windows")) {
                    if (javaHome.indexOf("jre") > -1) {
                        javaHome = javaHome.replace("jre", "jdk");
                    }
                }
                env.put("JAVA_HOME", javaHome);
            }
            processBuilder.directory(new File(project.getLocation().toFile()
                    .getAbsolutePath()
                    + File.separator));

            BuildJob compileJob = new BuildJob(processBuilder, messageConsole,
                    100000000, BUILD_WITH_TESTS);
            compileJob.schedule();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}
