/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: PropertyUtil.java 14:47:22 ddesjardins $
 * -------------------------------------------------------------------------
 */
package petalsclipse.builder.util;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.util.Date;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.ui.console.IOConsoleOutputStream;
import org.eclipse.ui.console.MessageConsole;

import petalsclipse.builder.BuildWithTestsAction;
import petalsclipse.builder.BuildWithoutTestsAction;
import petalsclipse.builder.CleanAction;

/**
 * 
 * @author ddesjardins - eBMWebSourcing
 * 
 */
public class BuildJob extends Job {

    private ProcessBuilder processBuilder;

    private MessageConsole messageConsole;

    private int timeOut;

    private String command;

    public IStatus run(IProgressMonitor monitor) {
        IStatus status = null;
        try {
            StringBuffer output = new StringBuffer();
            monitor.beginTask(command, 100);
            long startTime = new Date().getTime();
            boolean test = true;
            int exitValue = -1;
            IOConsoleOutputStream consoleOutputStream = null;
            messageConsole.activate();
            Process process = null;
            try {
                monitor.subTask("Starting build process");
                monitor.worked(5);
                process = processBuilder.start();
            } catch (IOException e1) {
                e1.printStackTrace();
            }
            while (test) {
                if (monitor.isCanceled()) {
                    test = false;
                    status = Status.CANCEL_STATUS;
                    consoleOutputStream = messageConsole.newOutputStream();
                    consoleOutputStream.setActivateOnWrite(true);
                    consoleOutputStream.write("\n\n TASK CANCELED \n\n");
                    consoleOutputStream.flush();
                    consoleOutputStream.close();
                }
                if (new Date().getTime() - startTime > timeOut) {
                    test = false;
                }
                try {
                    exitValue = process.exitValue();
                } catch (Exception e) {
                    e.printStackTrace();
                }
                try {
                    BufferedInputStream bufferedInputStream = ((BufferedInputStream) process
                            .getInputStream());
                    byte[] bytes = new byte[bufferedInputStream.available()];
                    bufferedInputStream.read(bytes);
                    String out = new String(bytes);
                    consoleOutputStream = messageConsole.newOutputStream();
                    consoleOutputStream.write(out);
                    consoleOutputStream.flush();
                    consoleOutputStream.close();
                    output.append(out);
                    // Change task progress
                    if (command
                            .equals(BuildWithoutTestsAction.BUILD_WITHOUT_TESTS)) {
                        if (out.indexOf("Scanning for projects") > -1) {
                            monitor.subTask("Scanning for projects");
                            monitor.worked(10);
                        }
                        if (out.indexOf("[resources:resources]") > -1) {
                            monitor.subTask("Preparing resources");
                            monitor.worked(20);
                        } else if (out.indexOf("[compiler:compile]") > -1) {
                            monitor.subTask("Compiling classes");
                            monitor.worked(30);
                        } else if (out.indexOf("[compiler:testCompile]") > -1) {
                            monitor.subTask("Compiling test classes");
                            monitor.worked(50);
                        } else if (out.indexOf("[install:install]") > -1) {
                            monitor.subTask("Installing");
                            monitor.worked(90);
                        }
                    } else if (command.equals(CleanAction.CLEAN)) {
                        if (out.indexOf("[clean:clean]") > -1) {
                            monitor.subTask("Cleaning project");
                            monitor.worked(30);
                        } else if (out.indexOf("[cobertura:clean") > -1) {
                            monitor.subTask("Cleaning cobertura files");
                            monitor.worked(70);
                        }
                    } else if (command
                            .equals(BuildWithTestsAction.BUILD_WITH_TESTS)) {
                        if (out.indexOf("Scanning for projects") > -1) {
                            monitor.subTask("Scanning for projects");
                            monitor.worked(10);
                        }
                        if (out.indexOf("[resources:resources]") > -1) {
                            monitor.subTask("Preparing resources");
                            monitor.worked(20);
                        } else if (out.indexOf("[compiler:compile]") > -1) {
                            monitor.subTask("Compiling classes");
                            monitor.worked(30);
                        } else if (out.indexOf("[compiler:testCompile]") > -1) {
                            monitor.subTask("Compiling test classes");
                            monitor.worked(40);

                        } else if (out.indexOf("[surefire:test]") > -1) {
                            monitor.subTask("Running tests");
                            monitor.worked(50);
                        } else if (out.indexOf("[install:install]") > -1) {
                            monitor.subTask("Installing");
                            monitor.worked(90);
                        }
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
                try {
                    Thread.sleep(100);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
                if (exitValue != -1) {
                    test = false;
                }
            }
            try {
                BufferedInputStream bufferedInputStream = ((BufferedInputStream) process
                        .getInputStream());
                byte[] bytes = new byte[bufferedInputStream.available()];
                bufferedInputStream.read(bytes);
                String out = new String(bytes);
                consoleOutputStream = messageConsole.newOutputStream();
                consoleOutputStream.setActivateOnWrite(true);
                consoleOutputStream.write(out);
                consoleOutputStream.flush();
                consoleOutputStream.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
            // done and cleaning
            monitor.worked(100);
            monitor.done();
        } catch (Exception t) {
            t.printStackTrace();
        }
        status = Status.OK_STATUS;
        return status;
    }

    public BuildJob(ProcessBuilder processBuilder,
            MessageConsole messageConsole, int timeOut, String command) {
        super(command);
        this.processBuilder = processBuilder;
        this.messageConsole = messageConsole;
        this.timeOut = timeOut;
        this.command = command;
    }
}
