package petalsclipse.wizards;

import java.io.File;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IResource;
import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * The "New" wizard page allows setting the container for the new file as well
 * as the file name. The page will only accept file name without the extension
 * OR with the extension that matches the expected one (mpe).
 */

public class NewPetalsProjectWizardPage extends WizardPage {
	private Text parentProjectText;

	private Text projectNameText;
	
	private Text projectFolderNameText;
	
	private Text projectCompleteNameText;

	private Combo projectType;

	private ISelection selection;

	public final static String JBI_PROJECT = "JBI component project";

	public final static String WAR_PROJECT = "War project";

	public final static String SIMPLE_PROJECT = "Simple project";

	public final static String MODULE_PROJECT = "Module project";

	/**
	 * Constructor for SampleNewWizardPage.
	 * 
	 * @param pageName
	 */
	public NewPetalsProjectWizardPage(ISelection selection) {
		super("wizardPage");
		setTitle("Petals Project");
		setDescription("This wizard creates a Petals sub-project.");
		this.selection = selection;
	}

	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	public void createControl(Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.numColumns = 3;
		layout.verticalSpacing = 9;
		Label label = new Label(container, SWT.NULL);
		label.setText("&Parent project pom file:");

		parentProjectText = new Text(container, SWT.BORDER | SWT.SINGLE);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		parentProjectText.setLayoutData(gd);
		parentProjectText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				dialogChanged();
			}
		});

		Button button = new Button(container, SWT.PUSH);
		button.setText("Browse...");
		button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleBrowse();
			}
		});

		label = new Label(container, SWT.NULL);
		label.setText("&Project type:");

		projectType = new Combo(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		projectType.setLayoutData(gd);
		projectType.setItems(new String[] { JBI_PROJECT, WAR_PROJECT,
				SIMPLE_PROJECT, MODULE_PROJECT });
		projectType.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				dialogChanged();
			}
		});

		label = new Label(container, SWT.NULL);
		label.setText("");

		label = new Label(container, SWT.NULL);
		label.setText("&Project artifact name:");

		projectNameText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		projectNameText.setLayoutData(gd);
		projectNameText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				dialogChanged();
			}
		});
		
		label = new Label(container, SWT.NULL);
		label.setText("");

		label = new Label(container, SWT.NULL);
		label.setText("&Project folder name:");

		projectFolderNameText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		projectFolderNameText.setLayoutData(gd);
		projectFolderNameText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				dialogChanged();
			}
		});
		
		label = new Label(container, SWT.NULL);
		label.setText("");

		label = new Label(container, SWT.NULL);
		label.setText("&Project complete name:");

		projectCompleteNameText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		projectCompleteNameText.setLayoutData(gd);
		projectCompleteNameText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				dialogChanged();
			}
		});

		initialize();
		dialogChanged();
		setControl(container);
	}

	/**
	 * Tests if the current workbench selection is a suitable container to use.
	 */

	private void initialize() {
		if (selection != null && selection.isEmpty() == false
				&& selection instanceof IStructuredSelection) {
			IStructuredSelection ssel = (IStructuredSelection) selection;
			if (ssel.size() > 1)
				return;
			Object obj = ssel.getFirstElement();
			if (obj instanceof IResource) {
				IContainer container;
				if (obj instanceof IContainer)
					container = (IContainer) obj;
				else
					container = ((IResource) obj).getParent();
				parentProjectText.setText(container.getFullPath().toString());
			}
		}
		projectNameText.setText("project-name");
	}

	/**
	 * Uses the standard container selection dialog to choose the new value for
	 * the container field.
	 */

	private void handleBrowse() {
		FileDialog dialog = new FileDialog(getShell(), 1);
		dialog.setFilterNames(new String[] { "pom.xml" });
		dialog.setFilterExtensions(new String[] { "pom.xml" });

		String result = dialog.open();
		if (result != null) {
			parentProjectText.setText(result);
		}
	}

	/**
	 * Ensures that both text fields are set.
	 */

	private void dialogChanged() {
		String parentName = getParentProjectName();
		String projectName = getProjectName();
		String projectType = getProjectType();
		String projectCompleteName = getProjectCompleteName();
		String projectFolderName = getProjectFolderName();

		if (parentName.length() == 0) {
			updateStatus("Parent project pom.xml file must be selected");
			return;
		}
		if (!checkParentPackaging()){
			updateStatus("The parent project packaging must be pom");
			return;
		}
		if (projectName.length() == 0) {
			updateStatus("Project name must be specified");
			return;
		}
		if (projectName.indexOf('/') > -1
				|| !projectName.equals(projectName.toLowerCase())) {
			updateStatus("Project name must be valid");
			return;
		}
		if (projectFolderName.length() == 0) {
			updateStatus("Project complete name must be specified");
			return;
		}
		if (projectCompleteName.length() == 0) {
			updateStatus("Project complete name must be specified");
			return;
		}
		if (projectType.length() == 0) {
			updateStatus("You must choose a project type");
			return;
		}
		if (!"project-name".equals(projectName)) {
			// Check if a project with the same name does not exists
			boolean alreadyExists = false;
			File parentDir = new File(new File(getParentProjectName())
					.getParent());
			for (File file : parentDir.listFiles()) {
				if (file.getName().equals(projectName)) {
					alreadyExists = true;
					break;
				}
			}
			if (alreadyExists) {
				updateStatus("Project name not already exist");
				return;
			}
		} else {
			updateStatus("You must change project name");
			return;
		}

		updateStatus(null);
	}

	private void updateStatus(String message) {
		setErrorMessage(message);
		setPageComplete(message == null);
	}

	public String getParentProjectName() {
		return parentProjectText.getText();
	}

	public String getProjectName() {
		return projectNameText.getText();
	}
	
	public String getProjectFolderName() {
		return projectFolderNameText.getText();
	}
	
	public String getProjectCompleteName() {
		return projectCompleteNameText.getText();
	}

	public String getProjectType() {
		return projectType.getText();
	}
	
	public boolean checkParentPackaging(){
		boolean isPom = true;
		try {
			DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			// lecture du contenu d'un fichier XML avec DOM
			File xml = new File(getParentProjectName());
			Document document = builder.parse(xml);
			Element element = document.getDocumentElement();
			NodeList packs = element.getElementsByTagName("packaging");
			Element packaging = (Element)packs.item(0);
			if (!"pom".equals(packaging.getTextContent())){
				isPom = false;
			}
		} catch (Exception e) {
			// Do nothing
		}
		return isPom;
	}
	
}