/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: XlinkerWsdl2JavaMojo.java 16:56:43 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.xlinkerplugin;

import org.act.soap.wsdl.wsdl2java.WSDL2Java;
import org.apache.maven.artifact.repository.ArtifactRepository;
import org.apache.maven.artifact.resolver.ArtifactResolver;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.MavenProject;

/**
 * Build an xlinker stub from a wsdl file
 * 
 * @goal wsdl2java
 * @description Build an xlinker stub from a wsdl file
 * @aggregator
 * @requiresDependencyResolution runtime
 * 
 * @author ofabre - EBM Websourcing
 */
public class XlinkerWsdl2JavaMojo extends AbstractMojo {

    /**
     * @component
     */
    protected ArtifactResolver artifactResolver;

    /**
     * Local maven repository.
     * 
     * @parameter expression="${localRepository}"
     * @required
     * @readonly
     */
    protected ArtifactRepository localRepository;

    /**
     * The Maven Project.
     * 
     * @parameter expression="${project}"
     * @required
     * @readonly
     */
    protected MavenProject project;

    /**
     * The location of the input wsdl file. For exemple :
     * "${basedir}/src/conf/wsdl/Echo.wsdl"
     * 
     * @parameter expression="${basedir}/src/conf/wsdl/Echo.wsdl"
     * @required
     */
    protected String wsdlLocation;

    /**
     * Output directory for generated artefacts (ws interface, stub and java
     * beans for complex types)
     * 
     * @parameter expression="${basedir}/src/java"
     * @required
     */
    protected String outputDirectory;

    /**
     * Execute the plugin
     */
    public void execute() throws MojoExecutionException {
        try {

            // Build a new array of String parameters
            String[] parameters = {wsdlLocation, outputDirectory};

            // Call the WSDL to JAVA generator
            WSDL2Java.main(parameters);

        } catch (Exception e) {
            throw new MojoExecutionException(e.getLocalizedMessage(), e);
        }
    }
}
